/***************************************************************************
                          add.cpp  -  description
                             -------------------
    begin                : Wed Apr 14 2004
    copyright            : (C) 2004 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "add.h"
#include "addnode.h"
#include "addfunction.h"
#include "addgroup.h"
#include "addvarorder.h"
#include "addoutput.h"
#include "../common/varorder.h"
#include "../common/ptrhashmap.h"
#include "../common/reordering.h"
#include "../common/blifinput.h"
#include "../common/matrixinput.h"
#include "../common/configreader.h"

#include "vereofy/common/tracing.hpp"
#include "vereofy/common/interruptible.hpp"

#include <stack>

using namespace std;

ConfigReader ADDConfigReader("");

//init number of dead nodes
unsigned long ADD::deadNodes=0;
//init garbage-collection delay
unsigned long ADD::garbageCollectionDelay=ADDConfigReader.getULong("garbage collection delay",GARBAGE_COLLECTION_DELAY_DEFAULT);
//init variableorder
ADDVariableOrdering ADD::variableOrdering=ADDVariableOrdering();
//init hashmaps
PtrHashMap2 ADD::ArithmeticComputedTable=PtrHashMap2(ADDConfigReader.getULong("init arithmetic hash size",INIT_HASHMAP_SIZE),ADDConfigReader.getULong("max arithmetic hash size",MAX_HASHMAP_SIZE),ADDConfigReader.getDouble("arithmetic resize ratio",HASHMAP_RESIZE_RATIO));
PtrHashMap2 ADD::ExtremaComputedTable=PtrHashMap2(ADDConfigReader.getULong("init extrema hash size",INIT_HASHMAP_SIZE),ADDConfigReader.getULong("max extrema hash size",MAX_HASHMAP_SIZE),ADDConfigReader.getDouble("extrema resize ratio",HASHMAP_RESIZE_RATIO));
PtrHashMap2 ADD::AbstractComputedTable=PtrHashMap2(ADDConfigReader.getULong("init abstract hash size",INIT_HASHMAP_SIZE),ADDConfigReader.getULong("max abstract hash size",MAX_HASHMAP_SIZE),ADDConfigReader.getDouble("abstract resize ratio",HASHMAP_RESIZE_RATIO));
PtrHashMap2 ADD::CompareComputedTable=PtrHashMap2(ADDConfigReader.getULong("init compare hash size",INIT_HASHMAP_SIZE),ADDConfigReader.getULong("max compare hash size",MAX_HASHMAP_SIZE),ADDConfigReader.getDouble("compare resize ratio",HASHMAP_RESIZE_RATIO));
PtrHashMap3 ADD::MatrixComputedTable=PtrHashMap3(ADDConfigReader.getULong("init matrix hash size",INIT_HASHMAP_SIZE),ADDConfigReader.getULong("max matrix hash size",MAX_HASHMAP_SIZE),ADDConfigReader.getDouble("matrix resize ratio",HASHMAP_RESIZE_RATIO));
PtrHashMap3 ADD::BoolComputedTable=PtrHashMap3(ADDConfigReader.getULong("init bool hash size",INIT_HASHMAP_SIZE),ADDConfigReader.getULong("max bool hash size",MAX_HASHMAP_SIZE),ADDConfigReader.getDouble("bool resize ratio",HASHMAP_RESIZE_RATIO));
PtrHashMap3 ADD::MoveComputedTable=PtrHashMap3(ADDConfigReader.getULong("init move hash size",INIT_HASHMAP_SIZE),ADDConfigReader.getULong("max move hash size",MAX_HASHMAP_SIZE),ADDConfigReader.getDouble("move resize ratio",HASHMAP_RESIZE_RATIO));
PtrHashMap3 ADD::SpecialComputedTable=PtrHashMap3(ADDConfigReader.getULong("init equiv hash size",INIT_HASHMAP_SIZE),ADDConfigReader.getULong("max equiv hash size",MAX_HASHMAP_SIZE),ADDConfigReader.getDouble("equiv resize ratio",HASHMAP_RESIZE_RATIO));
//init reordering
Reordering<ADDVarOrder,ADDGroup,ADD>* ADD::reordering=0;
unsigned long ADD::reorderingThreshold=ADDConfigReader.getULong("reordering threshold",REORDERING_THRESHOLD);
unsigned long ADD::operatorCounter=0;
unsigned long ADD::lastReorderSize=0;
bool ADD::dynamicReordering=false;
float ADD::dynamicReorderingMaxGrowth=MAX_GROWTH_DEFAULT;
unsigned int ADD::dynamicReorderingTimeout=0;
//init add
ADD* ADD::BDD=0;
typedef ADDNode* (ADD::*functionPtr)(ADDNode*, ADDNode*);
functionPtr ADD::currentAbstractFunction=0;
short ADD::currentAbstractFunctionID=0;
unsigned long ADD::swapcount=0;
//init drains
ADDNode* ADD::zeroDrain=0;
ADDNode* ADD::oneDrain=0;
//create ADD
ADD add;

inline ADDNode* calcPtr(const ADDNode* node, unsigned long op){return (ADDNode*) ((unsigned long) node+op);}
inline void swapPointer(ADDNode** u, ADDNode** v){
  //swap nodes for more computed table hits
  if((*u)>(*v)){
    ADDNode* temp=*u;
    *u=*v;
    *v=temp;
  }
}

//**********************************public**********************************
ADD::ADD(void){
  if(!ADD::BDD){
    //init hash unique table sizes
    ADDUniqueTable::setInitHashSize(ADDConfigReader.getULong("init unique table hash size",INIT_UNIQUE_TABLE_SIZE_DEFAULT));
    ADDUniqueTable::setMaxHashSize(ADDConfigReader.getULong("max unique table hash size",MAX_UNIQUE_TABLE_SIZE_DEFAULT));
    //create reordering object
    reordering=new Reordering<ADDVarOrder,ADDGroup,ADD>(&ADD::swap,this);

    //create drains and save it (garbage-collection)
    //zero drain cannot be created with findOrAddDrain, because a value close to zero returns the zeroDrain and thus
    //no node would be created
    zeroDrain=ADDNode::getNewNode();
	  zeroDrain->setValue(0.0);
	  zeroDrain->setDrainTable(variableOrdering.getDrainData());
    variableOrdering.getDrainData()->addValue(zeroDrain);
    zeroDrain->incNodeCount();
    oneDrain=findOrAddDrain(1.0);
    oneDrain->incNodeCount();

    //init first ADD
    ADD::BDD=this;

    //give objects the information which ADD to use
    ADDNode::BDD=this;
    ADDFunction::BDD=this;
    ADDGroup::BDD=this;
    ADDGroup::varOrder=&variableOrdering;
    ADDVarOrder::varOrder=&variableOrdering;
  }
}

ADD::~ADD(void){
  if(ADD::BDD==this){
    //garbageCollect();

    delete reordering;
  }
}

bool ADD::addVariable(const string& variable){
  return ADDVarOrder::addVariable(variable);
}

bool ADD::clear(void){
  //garbage collection
  garbageCollect();

  //count number of inner nodes
  unsigned long innerNodes=0;
  for(unsigned long i=0;i<variableOrdering.size();++i) innerNodes+=variableOrdering.getData(i)->size();
  
  if(innerNodes==0){
    variableOrdering.clear();

    return true;
  } else {
    bool ret=false;
    for(unsigned long i=variableOrdering.size();i>0;--i) ret|=variableOrdering.removeVariable(i-1);
    variableOrdering.clearGroups();

    return ret;
  }
}

// do a full reset of the ADD, delete all nodes and clear the variable ordering
void ADD::reset() {
  CompareComputedTable.clear();
  ArithmeticComputedTable.clear();
  AbstractComputedTable.clear();
  ExtremaComputedTable.clear();
  MatrixComputedTable.clear();
  BoolComputedTable.clear();
  MoveComputedTable.clear();
  SpecialComputedTable.clear();
  
  ADDUniqueTable* uniqueT;
  for(unsigned long i=0;i<variableOrdering.size();++i){
    uniqueT=variableOrdering.getData(i);
    
    ADDNode* currentTable=(ADDNode*) uniqueT->getFirst();
    ADDNode* currentNode;
    ADDNode* tempNode;
    while(currentTable){
      currentNode=currentTable;
      while(currentNode){
        tempNode=currentNode->getNext();
        BDD->deleteSubTree(currentNode);
        currentNode=tempNode;
      }
      currentTable=(ADDNode*) uniqueT->next();
    }
  }
  deadNodes=0;
  variableOrdering.clear();
}

/**
* \brief This function returns the number of nodes (including dead nodes).
*
* The returned number of nodes includes the number of dead nodes. To get the real number of
* nodes proceed a garbage collection before calling this function.
*/
unsigned long ADD::size(void){
  unsigned long ret=0;

  for(unsigned long i=0;i<variableOrdering.size();++i) ret+=variableOrdering.getData(i)->size();
  ret+=variableOrdering.getDrainData()->size();

  //do not count zero or one drain if they have no predecessors
  if(zeroDrain->preNodeCount==1) --ret;
  if(oneDrain->preNodeCount==1) --ret;

  return ret;
}

/**
* \brief This function returns the number of dead nodes
*/
unsigned long ADD::countDeadNodes(void){
  return deadNodes;
}

/**
* \brief This function returns the peak number of nodes.
*
* The returned number of peak nodes does contain the number of drains.
*/
unsigned long ADD::PEAK(void){
  return ADDNode::PEAK();
}

void ADD::printStat(void){
  ArithmeticComputedTable.printStat("Arithmetic Table");
  AbstractComputedTable.printStat("Abstract Table");
  CompareComputedTable.printStat("Compare Table");
  MatrixComputedTable.printStat("Matrix Table");
  BoolComputedTable.printStat("Bool Table");
  MoveComputedTable.printStat("Move Table");
  SpecialComputedTable.printStat("Special Table");
}

ADDFunction ADD::loadFunctionBLIF(const string& filename, const string& outputname){
  return BLIFInput<ADD,ADDFunction,ADDFunctionContainer>::getOutput(filename,outputname);
}

ADDFunctionContainer ADD::loadFunctionsBLIF(const string& filename){
  return BLIFInput<ADD,ADDFunction,ADDFunctionContainer>::getAllOutputs(filename);
}

ADDFunction ADD::loadMatrix(const string& filename, const unsigned short format){
  return MatrixInput<ADDFunction,ADDVarOrder>::loadMatrix(filename,format);
}

bool ADD::storeFunction(const string& filename, const ADDFunction& function){
  return ADDOutput::storeFunction(filename,function);
}

bool ADD::storeFunctions(const string& filename, const ADDFunctionContainer& container){
  return ADDOutput::storeFunctions(filename,container);
}

void ADD::sifting(const float maxGrowth){
  ADD::variableSifting(maxGrowth);
  ADD::groupSifting(maxGrowth);
}

void ADD::variableSifting(const float maxGrowth){
  reordering->variableSifting(maxGrowth);
}

void ADD::groupSifting(const float maxGrowth){
  reordering->groupSifting(maxGrowth, dynamicReorderingTimeout);
}


void ADD::groupExhaustive(ADD::CallBack& call_back) {
  reordering->groupExhaustive(call_back);
}

void ADD::windowPermutation2(void){
  reordering->windowPermutation2();
}

void ADD::windowPermutation3(void){
  reordering->windowPermutation3();
}

void ADD::windowPermutation4(void){
  reordering->windowPermutation4();
}

void ADD::geneticMinimize(void){
  reordering->geneticMinimize();
}

void ADD::enableDynamicReordering(void){
  dynamicReordering=true;
}

void ADD::disableDynamicReordering(void){
  dynamicReordering=false;
}

void ADD::setDynamicReorderingMaxGrowth(float maxGrowth) {
  dynamicReorderingMaxGrowth=maxGrowth;
}

void ADD::setDynamicReorderingTimeout(unsigned int timeout_ms) {
  dynamicReorderingTimeout=timeout_ms;
}

void ADD::setUniqueTableMaxLoadFactor(float load_factor) {
  ADDUniqueTable::max_load_factor=load_factor;
}


void ADD::garbageCollect(void){
  TRACE("ADD", "garbageCollect", cout << "Garbage Collect, " << deadNodes<< " dead nodes, BDD Size: " << ADD::size() << "\n");

  CompareComputedTable.clear();
  ArithmeticComputedTable.clear();
  AbstractComputedTable.clear();
  ExtremaComputedTable.clear();
  MatrixComputedTable.clear();
  BoolComputedTable.clear();
  MoveComputedTable.clear();
  SpecialComputedTable.clear();
  
  ADDUniqueTable* uniqueT;
  for(unsigned long i=0;(i<=variableOrdering.size()) && (deadNodes>0);++i){
    if(i==variableOrdering.size()) uniqueT=variableOrdering.getDrainData();
    else uniqueT=variableOrdering.getData(i);
    if(uniqueT->getDeadNodes()>0){
      unsigned long remainingDeadNodes=uniqueT->getDeadNodes();
      ADDNode* currentTable=(ADDNode*) uniqueT->getFirst();
      ADDNode* currentNode;
      ADDNode* tempNode;
      while(remainingDeadNodes){
        currentNode=currentTable;
        while(currentNode){
          tempNode=currentNode->getNext();
          if(currentNode->preNodeCount==ULONG_MAX){
            BDD->deleteSubTree(currentNode);
            --remainingDeadNodes;
          }
          currentNode=tempNode;
        }
        currentTable=(ADDNode*) uniqueT->next();
      }
      deadNodes-=uniqueT->getDeadNodes();
      uniqueT->resetDeadNodeCount();
    }
  }
  TRACE("ADD", "garbageCollect", cout << "After garbage collect, " << deadNodes<< " dead nodes, BDD Size: " << ADD::size() << "\n");
  deadNodes=0;
}

unsigned long ADD::setGarbageCollectionDelay(void){
  return garbageCollectionDelay;
}

unsigned long ADD::setGarbageCollectionDelay(const unsigned long newSize){
  if(newSize==garbageCollectionDelay) return garbageCollectionDelay;

  unsigned long oldSize=garbageCollectionDelay;
  garbageCollectionDelay=newSize;
  if(deadNodes>=garbageCollectionDelay) garbageCollect();

  return oldSize;
}

ADDFunction ADD::projection(const string& variable){
  //append label if variableOrdering has no such variable
  unsigned long index=variableOrdering[variable];
  if(index==ULONG_MAX){
    if(variable.find_first_of(badChars)!=string::npos) return ADDFunction::ERROR_FUNCTION();
    if(!variableOrdering.appendVariable(variable)) return ADDFunction::ERROR_FUNCTION();
    index=variableOrdering.size()-1;
  }

  return createFunction(findOrAdd(variableOrdering.getData(index),oneDrain,zeroDrain));
}

ADDFunction ADD::projection(const unsigned long index){
  return (index>=variableOrdering.size())?ADDFunction::ERROR_FUNCTION():createFunction(findOrAdd(variableOrdering.getData(index),oneDrain,zeroDrain));
}

ADDFunction ADD::negate(const ADDFunction& function){
  return (!function.rootNode)?ADDFunction::ERROR_FUNCTION():createFunction(negate(function.rootNode));
}

ADDFunction ADD::addition(const ADDFunction& function1, const ADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode))?ADDFunction::ERROR_FUNCTION():createFunction(addition(function1.rootNode,function2.rootNode));
}

ADDFunction ADD::sum(const ADDFunction& function, const ADDFunction& varSet){
  if((!function.rootNode)||(!varSet.rootNode)) return ADDFunction::ERROR_FUNCTION();
  
  currentAbstractFunction=&ADD::addition;
  currentAbstractFunctionID=SUM_OP;
  
  return createFunction(abstract(function.rootNode,varSet.rootNode));
}

ADDFunction ADD::subtraction(const ADDFunction& function1, const ADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode))?ADDFunction::ERROR_FUNCTION():createFunction(subtraction(function1.rootNode,function2.rootNode));
}

ADDFunction ADD::multiplication(const ADDFunction& function1, const ADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode))?ADDFunction::ERROR_FUNCTION():createFunction(multiplication(function1.rootNode,function2.rootNode));
}

ADDFunction ADD::product(const ADDFunction& function, const ADDFunction& varSet){
  if((!function.rootNode)||(!varSet.rootNode)) return ADDFunction::ERROR_FUNCTION();
  
  currentAbstractFunction=&ADD::multiplication;
  currentAbstractFunctionID=PRO_OP;
  
  return createFunction(abstract(function.rootNode,varSet.rootNode));
}

ADDFunction ADD::MM(const ADDFunction& A, const ADDFunction& B, const ADDFunction& varSet){
  if((!A.rootNode)||(!B.rootNode)||(!varSet.rootNode)) return ADDFunction::ERROR_FUNCTION();
  
  unsigned long m=varSet.isConstant()?0:varSet.size()-2;
  
  return createFunction(MM(A.rootNode,B.rootNode,varSet.rootNode,m));
}

ADDFunction ADD::MV(const ADDFunction& A, const ADDFunction& b, const ADDFunction& varSet){
  if((!A.rootNode)||(!b.rootNode)||(!varSet.rootNode)) return ADDFunction::ERROR_FUNCTION();
  
  unsigned long m=varSet.isConstant()?0:varSet.size()-2;

  return createFunction(MV(A.rootNode,b.rootNode,varSet.rootNode,m));
}

ADDFunction ADD::VM(const ADDFunction& b, const ADDFunction& A, const ADDFunction& varSet){
  if((!A.rootNode)||(!b.rootNode)||(!varSet.rootNode)) return ADDFunction::ERROR_FUNCTION();
  
  unsigned long m=varSet.isConstant()?0:varSet.size()-2;
	
  return createFunction(VM(b.rootNode,A.rootNode,varSet.rootNode,m));
}

ADDFunction ADD::equivalenceClasses(const ADDFunction& u, const ADDFunction& e, const ADDFunction& c){
  return ((!u.rootNode)||(!e.rootNode)||(!c.rootNode))?ADDFunction::ERROR_FUNCTION():createFunction(equivalenceClasses(u.rootNode,e.rootNode,c.rootNode));
}

ADDFunction ADD::constrain(const ADDFunction& f, const ADDFunction& g){
  return ((!f.rootNode)||(!g.rootNode))?ADDFunction::ERROR_FUNCTION():createFunction(constrain(f.rootNode,g.rootNode));
}

ADDFunction ADD::ITE(const ADDFunction& i, const ADDFunction& t, const ADDFunction& e){
  return ((!i.rootNode)||(!t.rootNode)||(!e.rootNode))?ADDFunction::ERROR_FUNCTION():createFunction(ITE(i.rootNode,t.rootNode,e.rootNode));
}

ADDFunction ADD::NOT(const ADDFunction& function){
  return (!function.rootNode)?ADDFunction::ERROR_FUNCTION():createFunction(NOT(function.rootNode));
}

ADDFunction ADD::AND(const ADDFunction& function1, const ADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode))?ADDFunction::ERROR_FUNCTION():createFunction(AND(function1.rootNode,function2.rootNode));
}

ADDFunction ADD::OR(const ADDFunction& function1, const ADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode))?ADDFunction::ERROR_FUNCTION():createFunction(OR(function1.rootNode,function2.rootNode));
}

ADDFunction ADD::XOR(const ADDFunction& function1, const ADDFunction& function2){
  return ITE(function1,NOT(function2),function2);
}

ADDFunction ADD::EQUIV(const ADDFunction& function1, const ADDFunction& function2){
  return ITE(function1,function2,NOT(function2));
}

ADDFunction ADD::cofactor(const ADDFunction& function, const unsigned long index, const bool value){
  if ((index>=variableOrdering.size())||(!function.rootNode)) return function;
  
  //create temporary node
  ADDNode* tempVar=value?findOrAdd(variableOrdering.getData(index),oneDrain,zeroDrain):findOrAdd(variableOrdering.getData(index),zeroDrain,oneDrain);
  ADDFunction tempFunction=createFunction(cofactor(function.rootNode,tempVar));
  
  //if node was really created above delete it
  if(tempVar->preNodeCount==0) tempVar->garbageCollect();
  
  return tempFunction;
}

ADDFunction ADD::cofactor(const ADDFunction& function, const ADDFunction& assignment){
  return ((!function.rootNode)||(!assignment.rootNode))?ADDFunction::ERROR_FUNCTION():createFunction(cofactor(function.rootNode,assignment.rootNode));
}

ADDFunction ADD::change(const ADDFunction& function){
  return (!function.rootNode)?ADDFunction::ERROR_FUNCTION():createFunction(change(function.rootNode));
}

ADDFunction ADD::change(const ADDFunction& function, const unsigned long index){
  if((index>=variableOrdering.size())||(!function.rootNode)) return function;
  
  //create temporary node
  ADDNode* tempVar=findOrAdd(variableOrdering.getData(index),oneDrain,zeroDrain);
  ADDFunction tempFunction=createFunction(change(function.rootNode,tempVar));
  
  //if node was really created above delete it
  if(tempVar->preNodeCount==0) tempVar->garbageCollect();
  
  return tempFunction;
}

ADDFunction ADD::change(const ADDFunction& function, const ADDFunction& cubeSet){
  return ((!function.rootNode)||(!cubeSet.rootNode))?ADDFunction::ERROR_FUNCTION():createFunction(change(function.rootNode,cubeSet.rootNode));
}

ADDFunction ADD::exists(const ADDFunction& function, const ADDFunction& varSet){
  if((!function.rootNode)||(!varSet.rootNode)) return ADDFunction::ERROR_FUNCTION();
  return createFunction(exists(function.rootNode,varSet.rootNode));
}

ADDFunction ADD::exists(const ADDFunction& function, const unsigned long index){
  return ((index>=variableOrdering.size())||(!function.rootNode))?function:(cofactor(function,index,0)|cofactor(function,index,1));
}

ADDFunction ADD::forall(const ADDFunction& function, const ADDFunction& varSet){
  return !exists(!function,varSet);
}

ADDFunction ADD::forall(const ADDFunction& function, const unsigned long index){
  return ((index>=variableOrdering.size())||(!function.rootNode))?function:(cofactor(function,index,0)&cofactor(function,index,1));
}

ADDFunction ADD::less(const ADDFunction& function1, const ADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode))?ADDFunction::ERROR_FUNCTION():createFunction(less(function1.rootNode,function2.rootNode));
}

ADDFunction ADD::lessEqual(const ADDFunction& function1, const ADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode))?ADDFunction::ERROR_FUNCTION():createFunction(lessEqual(function1.rootNode,function2.rootNode));
}

ADDFunction ADD::equal(const ADDFunction& function1, const ADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode))?ADDFunction::ERROR_FUNCTION():createFunction(equal(function1.rootNode,function2.rootNode));
}

ADDFunction ADD::maximum(const ADDFunction& function, const ADDFunction& varSet){
  if((!function.rootNode)||(!varSet.rootNode)) return ADDFunction::ERROR_FUNCTION();
  
  currentAbstractFunction=&ADD::max;
  currentAbstractFunctionID=MAX_OP;
  
  return createFunction(idempotentAbstract(function.rootNode,varSet.rootNode));
}

ADDFunction ADD::max(const ADDFunction& function1, const ADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode))?ADDFunction::ERROR_FUNCTION():createFunction(max(function1.rootNode,function2.rootNode));
}

double ADD::max(const ADDFunction& function){
  map<ADDNode*,double> CT;

  return (!function.rootNode)?0.0:max(function.rootNode,CT);
}

ADDFunction ADD::minimum(const ADDFunction& function, const ADDFunction& varSet){
  if((!function.rootNode)||(!varSet.rootNode)) return ADDFunction::ERROR_FUNCTION();
  
  currentAbstractFunction=&ADD::min;
  currentAbstractFunctionID=MIN_OP;
  
  return createFunction(idempotentAbstract(function.rootNode,varSet.rootNode));
}

ADDFunction ADD::min(const ADDFunction& function1, const ADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode))?ADDFunction::ERROR_FUNCTION():createFunction(min(function1.rootNode,function2.rootNode));
}

double ADD::min(const ADDFunction& function){
  map<ADDNode*,double> CT;

  return (!function.rootNode)?0.0:min(function.rootNode,CT);
}

ADDFunction ADD::oneOver(const ADDFunction& function){
  return (!function.rootNode)?ADDFunction::ERROR_FUNCTION():createFunction(oneOver(function.rootNode));
}

ADDFunction ADD::moveUp(const ADDFunction& function, const unsigned int skip){
  if(!function.rootNode) return ADDFunction::ERROR_FUNCTION();
  if(function.isConstant()) return function;
  if(skip==0) return function;
  if(function.rootNode->getLevel()<skip) return function.rootNode;

  if(skip==1) return createFunction(moveUp(function.rootNode));
  else return createFunction(moveUp(function.rootNode,skip));
}

ADDFunction ADD::moveUp(const ADDFunction& function, const ADDFunction& set, const unsigned int skip){
  if((!function.rootNode)||(!set.rootNode)) return ADDFunction::ERROR_FUNCTION();
  if(function.isConstant()) return function;
  if(skip==0) return function;

  if(skip==1) return createFunction(moveUp(function.rootNode,set.rootNode));
  else return createFunction(moveUp(function.rootNode,set.rootNode,skip));
}

ADDFunction ADD::moveDown(const ADDFunction& function, const unsigned int skip){
  if(!function.rootNode) return ADDFunction::ERROR_FUNCTION();
  if(function.isConstant()) return function;
  if(skip==0) return function;

  if(skip==1) return createFunction(moveDown(function.rootNode));
  else return createFunction(moveDown(function.rootNode,skip));
}

ADDFunction ADD::moveDown(const ADDFunction& function, const ADDFunction& set, const unsigned int skip){
  if((!function.rootNode)||(!set.rootNode)) return ADDFunction::ERROR_FUNCTION();
  if(function.isConstant()) return function;
  if(skip==0) return function;

  if(skip==1) return createFunction(moveDown(function.rootNode,set.rootNode));
  else return createFunction(moveDown(function.rootNode,set.rootNode,skip));
}

ADDFunction ADD::getAssignment(const ADDFunction& function){
  if(!function.rootNode) return ADDFunction::ERROR_FUNCTION();

  return createFunction(getAssignment(0,function.rootNode));
}

void ADD::getAssignment(const ADDFunction& function, char* buffer) const {
  //initially, every variable gets the value false
  for(unsigned long i=0;i<variableOrdering.size();++i) buffer[i]='0';
  
  if(!function.rootNode) return;
  
  getAssignment(function.rootNode,buffer);
}

double ADD::countAssignments(const ADDFunction& function) const {
  if(!function.rootNode) return 0.0;

  map<ADDNode*,double> CT;
  double ret=countAssignments(function.rootNode,CT);

  unsigned long diff=function.rootNode->isDrain()?variableOrdering.size():function.rootNode->getLevel();
  while(diff){
    ret*=2.0;
    --diff;
  }

  return ret;
}

double ADD::countAssignments(const ADDFunction& function, const ADDFunction& cubeSet) const {
  if(!function.rootNode ||
     !cubeSet.rootNode) return 0.0;

  map<pair<const ADDNode*, const ADDNode*>, double> CT;
  double count=countAssignments(function.rootNode, cubeSet.rootNode, CT);

  return count;
}

ADDFunction ADD::getRandomAssignment(const ADDFunction& function, const ADDFunction& cube_set) {
  if (!function.rootNode) return ADDFunction::ERROR_FUNCTION();
  if (!cube_set.rootNode) return ADDFunction::ERROR_FUNCTION();
    
  map<pair<const ADDNode*, const ADDNode*>, double> CT;
  countAssignments(function.rootNode, cube_set.rootNode, CT);
  
  ADDNode* cube_current=cube_set.rootNode;
  ADDNode* node_current=function.rootNode;

  std::vector<ADDNode*> cube_nodes;
  std::vector<bool> choices;  // true = choose 1-successor, false = 0-successor

  while (cube_current!=ADD::oneDrain) {
    cube_nodes.push_back(cube_current);

    if (node_current->isDrain() ||
        cube_current->getLevel() < node_current->getLevel()) {
      // level is not relevant in function, 0 or 1 with 1/2 probability
      
      bool rand=RandomHelper::getBool();
      choices.push_back(rand);
    } else {
      ADDNode* w0=node_current->getSucc0();
      ADDNode* w1=node_current->getSucc1();

      if (w0->isDrain() && w0==ADD::zeroDrain) {
        choices.push_back(true);
        node_current=w1;
      } else if (w1->isDrain() && w1==ADD::zeroDrain) {
        choices.push_back(false);
        node_current=w0;
      } else {
        double count0=CT[make_pair(w0, cube_current->getSucc1())];
        double count1=CT[make_pair(w1, cube_current->getSucc1())];

        double sum=count0+count1;
        double rand=RandomHelper::getUDouble(sum);
        if (rand >= count0) {
          choices.push_back(true);
          node_current=w1;
        } else {
          choices.push_back(false);
          node_current=w0;
        }          
      }
    }

    cube_current=cube_current->getSucc1();
  }

  // construct the assignment
  ADDNode* result=ADD::oneDrain;
  for (unsigned int i=cube_nodes.size(); i>0; i--) {
    unsigned long level=cube_nodes.at(i-1)->getLevel();
    bool choice=choices.at(i-1);
    
    if (choice) {
      // succ1 chosen
      result=findOrAdd(level, result, zeroDrain);
    } else {
      // succ0 chosen
      result=findOrAdd(level, zeroDrain, result);
    }
  }

  return createFunction(result);
}


ADDFunction ADD::getPath(const ADDFunction& function){
  if(!function.rootNode) return function;  
  return createFunction(getPath(function.rootNode));
}

void ADD::getPath(const ADDFunction& function, char* buffer) const {
  //initially, every variable is a don't care symbol
  for(unsigned long i=0;i<variableOrdering.size();++i) buffer[i]='-';
  
  if(!function.rootNode) return;
  
  getPath(function.rootNode,buffer);
}

//********************************protected**********************************
bool ADD::swap(const unsigned long index){
  if(index+1>=variableOrdering.size()) return false;

  ADDNode* w00;
  ADDNode* w01;
  ADDNode* w10;
  ADDNode* w11;
  ADDNode* temp0;
  ADDNode* temp1;
  ADDNode* firstListElement=0;

  ADDUniqueTable* tempTable1=variableOrdering.getData(index);
  ADDUniqueTable* tempTable2=variableOrdering.getData(index+1);
  ADDNode* currentTable=(ADDNode*) tempTable1->getFirst();
  ADDNode* currentNode;
  ADDNode* tempNode;
  while(currentTable){
    currentNode=currentTable;
	  while(currentNode){
      tempNode=currentNode->getNext();
    
      if((currentNode->getSucc0()->getLevel()==index+1)||(currentNode->getSucc1()->getLevel()==index+1)){
        currentNode->removeFromUniqueTable();
        currentNode->setNext(firstListElement);
        firstListElement=currentNode;
      }
      currentNode=tempNode;
    }
    currentTable=(ADDNode*) tempTable1->next();
  }

  currentNode=firstListElement;
  ADDNode* w0;
  ADDNode* w1;
  while(currentNode){
    tempNode=currentNode->getNext();
    w0=currentNode->getSucc0();
	  w1=currentNode->getSucc1();

    if(w0->getLevel()==index+1){
      w00=w0->getSucc0();
      w01=w0->getSucc1();
    } else {
      w00=w01=w0;
    }

    if(w1->getLevel()==index+1){
      w10=w1->getSucc0();
      w11=w1->getSucc1();
    } else {
      w10=w11=w1;
    }
	
    if(w00==w10) temp0=w00;
    else temp0=findOrAdd(tempTable1,w10,w00);
	
    if(w01==w11) temp1=w11;
    else temp1=findOrAdd(tempTable1,w11,w01);
	
    currentNode->setNode(tempTable2,temp0,temp1);
    currentNode=tempNode;
  }

  //swap variables
  variableOrdering.swapVariables(index);

  ++swapcount;

  return true;
}

void ADD::deleteSubTree(ADDNode* node){
  if(!node->isDrain()){
    if(--node->type.children.succ0->preNodeCount==0) deleteSubTree(node->type.children.succ0);
    if(--node->type.children.succ1->preNodeCount==0) deleteSubTree(node->type.children.succ1);
  }
  
  //reset node
  node->removeFromUniqueTable();

  //free mem for node
  ADDNode::freeNode(node);
}

void ADD::deleteNode(ADDNode* node){
  --node->type.children.succ0->preNodeCount;
  --node->type.children.succ1->preNodeCount;
  node->removeFromUniqueTable();
  ADDNode::freeNode(node);
}

//**********************************private**********************************
ADDNode* ADD::abstract(ADDNode* node, ADDNode* varSet){
  INTERRUPTIBLE();

  if(varSet->isDrain()) return node;

  ADDNode* temp=(ADDNode*) AbstractComputedTable.find(calcPtr(node,currentAbstractFunctionID),varSet);
  if(temp) return temp;

  ADDNode* w0;
  ADDNode* w1;

  if(varSet->getUniqueTable()==node->getUniqueTable()){
    w0=abstract(node->getSucc0(),varSet->getSucc1());
	  w1=abstract(node->getSucc1(),varSet->getSucc1());
    temp=(this->*currentAbstractFunction)(w0,w1);
    //prevent from garbage collection
    temp->incNodeCount();
    if(w0->preNodeCount==0) w0->garbageCollect();
    if(w1->preNodeCount==0) w1->garbageCollect();
    //reverse protection
    --temp->preNodeCount;
    AbstractComputedTable.insert(calcPtr(node,currentAbstractFunctionID),varSet,temp);
	
    return temp;
  } else {
    if(varSet->getRealLevel()<node->getRealLevel()){
      w0=abstract(node,varSet->getSucc1());
      temp=(this->*currentAbstractFunction)(w0,w0);
      //prevent from garbage collection
      temp->incNodeCount();
      if(w0->preNodeCount==0) w0->garbageCollect();
      //reverse protection
      --temp->preNodeCount;
    } else {
      w0=abstract(node->getSucc0(),varSet);
      w1=abstract(node->getSucc1(),varSet);
	
      temp=(w0==w1)?w0:findOrAdd(node->getUniqueTable(),w1,w0);
	}
	AbstractComputedTable.insert(calcPtr(node,currentAbstractFunctionID),varSet,temp);
	
	return temp;
  }
}

ADDNode* ADD::idempotentAbstract(ADDNode* node, ADDNode* varSet){
  INTERRUPTIBLE();

  if(node->isDrain()) return node;

  while(varSet->getRealLevel()<node->getRealLevel()) varSet=varSet->getSucc1();

  if(varSet->isDrain()) return node;

  ADDNode* temp=(ADDNode*) AbstractComputedTable.find(calcPtr(node,currentAbstractFunctionID),varSet);
  if(temp) return temp;

  ADDNode* w0;
  ADDNode* w1;

  if(varSet->getUniqueTable()==node->getUniqueTable()){
    w0=idempotentAbstract(node->getSucc0(),varSet->getSucc1());
	  w1=idempotentAbstract(node->getSucc1(),varSet->getSucc1());
    temp=(this->*currentAbstractFunction)(w0,w1);
    //prevent from garbage collection
    temp->incNodeCount();
    if(w0->preNodeCount==0) w0->garbageCollect();
    if(w1->preNodeCount==0) w1->garbageCollect();
    //reverse protection
    --temp->preNodeCount;
    AbstractComputedTable.insert(calcPtr(node,currentAbstractFunctionID),varSet,temp);
	
    return temp;
  } else {
	  w0=idempotentAbstract(node->getSucc0(),varSet);
	  w1=idempotentAbstract(node->getSucc1(),varSet);
	
	  temp=(w0==w1)?w0:findOrAdd(node->getUniqueTable(),w1,w0);
	  AbstractComputedTable.insert(calcPtr(node,currentAbstractFunctionID),varSet,temp);
	
	  return temp;
  }
}

ADDNode* ADD::exists(ADDNode* node, ADDNode* varSet){
  INTERRUPTIBLE();

  if(node->isDrain()) return node;

  unsigned long nodeLevel=node->getRealLevel();
  while(varSet->getRealLevel()<nodeLevel) varSet=varSet->getSucc1();

  if(varSet->isDrain()) return node;

  ADDNode* temp=(ADDNode*) BoolComputedTable.find(calcPtr(node,EXI_OP),varSet);
  if(temp) return temp;

  ADDNode* w0;
  ADDNode* w1;

  if(varSet->getUniqueTable()==node->getUniqueTable()){
    w0=exists(node->getSucc0(),varSet->getSucc1());
	  w1=exists(node->getSucc1(),varSet->getSucc1());
    temp=OR(w0,w1);
    
    //prevent from garbage collection
    temp->incNodeCount();
    if(w0->preNodeCount==0) w0->garbageCollect();
    if(w1->preNodeCount==0) w1->garbageCollect();
    //reverse protection
    --temp->preNodeCount;
    
	  BoolComputedTable.insert(calcPtr(node,EXI_OP),varSet,temp);
	
    return temp;
  } else {
	  w0=exists(node->getSucc0(),varSet);
	  w1=exists(node->getSucc1(),varSet);
	
	  temp=(w0==w1)?w0:findOrAdd(node->getUniqueTable(),w1,w0);
	  BoolComputedTable.insert(calcPtr(node,EXI_OP),varSet,temp);
	
	  return temp;
  }
}

ADDNode* ADD::negate(const ADDNode* u){
  INTERRUPTIBLE();

  if(u->isDrain()) return findOrAddDrain(-(u->getValue()));

  //try to get result from computed table
  ADDNode* tempNode=(ADDNode*) ArithmeticComputedTable.find(calcPtr(zeroDrain,SUB_OP),u);
  if(tempNode) return tempNode;

  ADDNode* w0;
  ADDNode* w1;

  w0=negate(u->getSucc0());
  w1=negate(u->getSucc1());

  tempNode=(w0==w1)?w0:findOrAdd(u->getUniqueTable(),w1,w0);

  //insert result into computed table
  ArithmeticComputedTable.insert(calcPtr(zeroDrain,SUB_OP),u,tempNode);

  return tempNode;
}

ADDNode* ADD::addition(ADDNode* u, ADDNode* v){
  //case 1a: (0,f) or (*,*)
  if(u->isDrain()){
    if(u==zeroDrain) return v;
    else return constAddition(u,v);
  }
  
  //case 1b: (f,0) or (f,1)
  if(v->isDrain()){
    if(v==zeroDrain) return u;
    else return constAddition(v,u);
  }
  
  swapPointer(&u,&v);
  
  //try to get result from computed table
  ADDNode* node=(ADDNode*) ArithmeticComputedTable.find(calcPtr(u,ADD_OP),v);
  if(node) return node;

  ADDNode* w0;
  ADDNode* w1;
  ADDUniqueTable* minTable;

  //calculate minTable
  if(u->getRealLevel()<=v->getRealLevel()){
    minTable=u->getUniqueTable();
    if(v->getUniqueTable()==minTable){
      w0=addition(u->getSucc0(),v->getSucc0());
      w1=addition(u->getSucc1(),v->getSucc1());
    } else {
      w0=addition(u->getSucc0(),v);
      w1=addition(u->getSucc1(),v);
    }
  } else {
    minTable=v->getUniqueTable();
    w0=addition(u,v->getSucc0());
    w1=addition(u,v->getSucc1());
  } 
  
  node=(w0==w1)?w0:findOrAdd(minTable,w1,w0);

  //insert result into computed table
  ArithmeticComputedTable.insert(calcPtr(u,ADD_OP),v,node);

  return node;
}

ADDNode* ADD::constAddition(ADDNode* u, ADDNode* v){
  if(v->isDrain()) return findOrAddDrain(u->getValue()+v->getValue());
  
  //try to get result from computed table
  ADDNode* node=(ADDNode*) ArithmeticComputedTable.find(calcPtr(u,ADD_OP),v);
  if(node) return node;
  
  ADDNode* w0;
  ADDNode* w1;
  
  w0=constAddition(u,v->getSucc0());
  w1=constAddition(u,v->getSucc1());
  
  node=(w0==w1)?w0:findOrAdd(v->getUniqueTable(),w1,w0);
  
  //insert result into computed table
  ArithmeticComputedTable.insert(calcPtr(u,ADD_OP),v,node);
  
  return node;
}

ADDNode* ADD::subtraction(ADDNode* u, ADDNode* v){
  //case 1: (f,f)
  if(u==v) return zeroDrain;

  //case 2a: (f,0)
  if(v==zeroDrain) return u;

  //case 2b: (0,f)
  if(u==zeroDrain) return negate(v);

  //case 3: (*,*)
  if((u->isDrain())&&(v->isDrain())) return findOrAddDrain(u->getValue()-v->getValue());

  //try to get result from computed table
  ADDNode* node=(ADDNode*) ArithmeticComputedTable.find(calcPtr(u,SUB_OP),v);
  if(node) return node;

  ADDNode* w0;
  ADDNode* w1;
  ADDNode* u0;
  ADDNode* u1;
  ADDNode* v0;
  ADDNode* v1;
  ADDUniqueTable* minTable;
  
  //calculate minTable
  if(u->getRealLevel()<=v->getRealLevel()){
    minTable=u->getUniqueTable();
    u0=u->getSucc0();
    u1=u->getSucc1();
    if(minTable==v->getUniqueTable()){
      v0=v->getSucc0();
      v1=v->getSucc1();
    } else v0=v1=v;
  } else {
    minTable=v->getUniqueTable();
    v0=v->getSucc0();
    v1=v->getSucc1();
    u0=u1=u;
  }
  
  w0=subtraction(u0,v0);
  w1=subtraction(u1,v1);
  
  node=(w0==w1)?w0:findOrAdd(minTable,w1,w0);
  
  //insert result into computed table
  ArithmeticComputedTable.insert(calcPtr(u,SUB_OP),v,node);
  
  return node;
}

ADDNode* ADD::multiplication(ADDNode* u, ADDNode* v){
  //case 1a: (0,f), (1,f) or (*,*)
  if(u->isDrain()){
    if(u==zeroDrain) return zeroDrain;
	  else if(u==oneDrain) return v;
    else return constMultiplication(u,v);
  }
  
  //case 1b: (f,0) or (f,1)
  if(v->isDrain()){
    if(v==zeroDrain) return zeroDrain;
	  else if(v==oneDrain) return u;
    else return constMultiplication(v,u);
  }

  swapPointer(&u,&v);
  
  //try to get result from computed table
  ADDNode* node=(ADDNode*) ArithmeticComputedTable.find(calcPtr(u,MUL_OP),v);
  if(node) return node;

  ADDNode* w0;
  ADDNode* w1;
  ADDUniqueTable* minTable;

  //calculate minTable
  if(u->getRealLevel()<=v->getRealLevel()){
    minTable=u->getUniqueTable();
    if(v->getUniqueTable()==minTable){
      w0=multiplication(u->getSucc0(),v->getSucc0());
      w1=multiplication(u->getSucc1(),v->getSucc1());
    } else {
      w0=multiplication(u->getSucc0(),v);
      w1=multiplication(u->getSucc1(),v);
    }
  } else {
    minTable=v->getUniqueTable();
    w0=multiplication(u,v->getSucc0());
    w1=multiplication(u,v->getSucc1());
  } 

  node=(w0==w1)?w0:findOrAdd(minTable,w1,w0);

  //insert result into computed table
  ArithmeticComputedTable.insert(calcPtr(u,MUL_OP),v,node);

  return node;
}

ADDNode* ADD::constMultiplication(ADDNode* u, ADDNode* v){
  if(v->isDrain()) return findOrAddDrain(u->getValue()*v->getValue());

  //try to get result from computed table
  ADDNode* node=(ADDNode*) ArithmeticComputedTable.find(calcPtr(u,MUL_OP),v);
  if(node) return node;
  
  ADDNode* w0;
  ADDNode* w1;
  
  w0=constMultiplication(u,v->getSucc0());
  w1=constMultiplication(u,v->getSucc1());
  
  node=(w0==w1)?w0:findOrAdd(v->getUniqueTable(),w1,w0);
  
  //insert result into computed table
  ArithmeticComputedTable.insert(calcPtr(u,MUL_OP),v,node);
  
  return node;
}

ADDNode* ADD::MM(ADDNode* A, ADDNode* B, ADDNode* varSet, unsigned long m){
  if((A==zeroDrain) || (B==zeroDrain)) return zeroDrain;
  if((A->isDrain() && B->isDrain()) || (m==0)){
	  ADDNode* tempDrain=findOrAddDrain(double(1<<m));
	  ADDNode* tempNode=multiplication(multiplication(A,B),tempDrain);
    
    //prevent from garbage collection
    tempNode->incNodeCount();
    if(tempDrain->preNodeCount==0) tempDrain->garbageCollect();
    //reverse protection
    --tempNode->preNodeCount;
    
	  return tempNode;
  }
  
  ADDNode* tempNode=(ADDNode*) MatrixComputedTable.find(calcPtr(A,MMA_OP),B,varSet);
  if(tempNode) return tempNode;
  
  ADDUniqueTable* uniqueTable=varSet->getUniqueTable();
  ADDNode* A0;
  ADDNode* A1;  
  ADDNode* A00, *A01;
  ADDNode* A10, *A11;
  
  if(uniqueTable==A->getUniqueTable()){
    A0=A->getSucc0();
	  A1=A->getSucc1();
  } else {
    A0=A1=A;
  }
  if(A0->getLevel()==uniqueTable->getLevel()+1){
    A00=A0->getSucc0();
    A01=A0->getSucc1();
  } else {
    A00=A01=A0;
  }
  if(A1->getLevel()==uniqueTable->getLevel()+1){
    A10=A1->getSucc0();
    A11=A1->getSucc1();
  } else {
    A10=A11=A1;
  }
  
  ADDNode* B0;
  ADDNode* B1;  
  ADDNode* B00, *B01;
  ADDNode* B10, *B11;
  
  if(uniqueTable==B->getUniqueTable()){
    B0=B->getSucc0();
	  B1=B->getSucc1();
  } else {
    B0=B1=B;
  }
  if(B0->getLevel()==uniqueTable->getLevel()+1){
    B00=B0->getSucc0();
    B01=B0->getSucc1();
  } else {
    B00=B01=B0;
  }
  if(B1->getLevel()==uniqueTable->getLevel()+1){
    B10=B1->getSucc0();
    B11=B1->getSucc1();
  } else {
    B10=B11=B1;
  }
  
  ADDNode* newVarSet=varSet->getSucc1();
  
  ADDNode* w0000=MM(A00,B00,newVarSet,m-1);
  ADDNode* w0110=MM(A01,B10,newVarSet,m-1);
  ADDNode* w0001=MM(A00,B01,newVarSet,m-1);
  ADDNode* w0111=MM(A01,B11,newVarSet,m-1);
  ADDNode* w1000=MM(A10,B00,newVarSet,m-1);
  ADDNode* w1110=MM(A11,B10,newVarSet,m-1);
  ADDNode* w1001=MM(A10,B01,newVarSet,m-1);
  ADDNode* w1111=MM(A11,B11,newVarSet,m-1);
  
  ADDNode* c00=addition(w0000,w0110);
  ADDNode* c01=addition(w0001,w0111);
  ADDNode* c10=addition(w1000,w1110);
  ADDNode* c11=addition(w1001,w1111);
  
  ADDNode* temp0=(c00==c01)?c00:findOrAdd(uniqueTable->getLevel()+1,c01,c00);
  ADDNode* temp1=(c10==c11)?c11:findOrAdd(uniqueTable->getLevel()+1,c11,c10);
  tempNode=(temp0==temp1)?temp1:findOrAdd(uniqueTable,temp1,temp0);
  
  //prevent from garbage collection
  tempNode->incNodeCount();
  if(w0000->preNodeCount==0) w0000->garbageCollect();
  if(w0110->preNodeCount==0) w0110->garbageCollect();
  if(w0001->preNodeCount==0) w0001->garbageCollect();
  if(w0111->preNodeCount==0) w0111->garbageCollect();
  if(w1000->preNodeCount==0) w1000->garbageCollect();
  if(w1110->preNodeCount==0) w1110->garbageCollect();
  if(w1001->preNodeCount==0) w1001->garbageCollect();
  if(w1111->preNodeCount==0) w1111->garbageCollect();
  
  //reverse protection
  --tempNode->preNodeCount;
  
  MatrixComputedTable.insert(calcPtr(A,MMA_OP),B,varSet,tempNode);
  
  return tempNode;
}

ADDNode* ADD::MV(ADDNode* A, ADDNode* b, ADDNode* varSet, unsigned long m){
  if((A==zeroDrain) || (b==zeroDrain)) return zeroDrain;
  if((A->isDrain() && b->isDrain()) || (m==0)){
	  ADDNode* tempDrain=findOrAddDrain(double(1<<m));
	  ADDNode* tempNode=multiplication(multiplication(A,b),tempDrain);

    //prevent from garbage collection
    tempNode->incNodeCount();
    if(tempDrain->preNodeCount==0) tempDrain->garbageCollect();
    //reverse protection
    --tempNode->preNodeCount;
    
	  return tempNode;
  }

  ADDNode* tempNode=(ADDNode*) MatrixComputedTable.find(calcPtr(A,MAV_OP),b,varSet);
  if(tempNode) return tempNode;
  
  ADDUniqueTable* uniqueTable=varSet->getUniqueTable();
  ADDNode* A0;
  ADDNode* A1;  
  ADDNode* A00, *A01;
  ADDNode* A10, *A11;
  
  if(uniqueTable==A->getUniqueTable()){
    A0=A->getSucc0();
	  A1=A->getSucc1();
  } else {
    A0=A1=A;
  }
  if(A0->getLevel()==uniqueTable->getLevel()+1){
    A00=A0->getSucc0();
    A01=A0->getSucc1();
  } else {
    A00=A01=A0;
  }
  if(A1->getLevel()==uniqueTable->getLevel()+1){
    A10=A1->getSucc0();
    A11=A1->getSucc1();
  } else {
    A10=A11=A1;
  }
  
  ADDNode* b0, *b1;
  if(b->getLevel()==uniqueTable->getLevel()){
    b0=b->getSucc0();
	  b1=b->getSucc1();
  } else {
    b0=b1=b;
  }

  ADDNode* newVarSet=varSet->getSucc1();

  ADDNode* w00=MV(A00,b0,newVarSet,m-1);
  ADDNode* w01=MV(A01,b1,newVarSet,m-1);
  ADDNode* w10=MV(A10,b0,newVarSet,m-1);
  ADDNode* w11=MV(A11,b1,newVarSet,m-1);
  
  ADDNode* c0=addition(w00,w01);
  ADDNode* c1=addition(w10,w11);
  tempNode=((c0->getUniqueTable()==uniqueTable)||(c1->getUniqueTable()==uniqueTable)||(c0==c1))?c0:findOrAdd(uniqueTable,c1,c0);
  tempNode->incNodeCount();
  if(w00->preNodeCount==0) w00->garbageCollect();
  if(w01->preNodeCount==0) w01->garbageCollect();
  if(w10->preNodeCount==0) w10->garbageCollect();
  if(w11->preNodeCount==0) w11->garbageCollect();
  --tempNode->preNodeCount;
  MatrixComputedTable.insert(calcPtr(A,MAV_OP),b,varSet,tempNode);
  
  return tempNode;
}

ADDNode* ADD::VM(ADDNode* b, ADDNode* A, ADDNode* varSet, unsigned long m){
  if((A==zeroDrain) || (b==zeroDrain)) return zeroDrain;
  if((A->isDrain() && b->isDrain()) || (m==0)){
    ADDNode* tempDrain=findOrAddDrain(double(1<<m));
		ADDNode* tempNode=multiplication(multiplication(A,b),tempDrain);
		
    //prevent from garbage collection
    tempNode->incNodeCount();
    if(tempDrain->preNodeCount==0) tempDrain->garbageCollect();
    //reverse protection
    --tempNode->preNodeCount;
    
		return tempNode;
  }
	
  ADDNode* tempNode=(ADDNode*) MatrixComputedTable.find(calcPtr(b,VMA_OP),A,varSet);
  if(tempNode) return tempNode;
  
  ADDUniqueTable* uniqueTable=varSet->getUniqueTable();
  ADDNode* A0;
  ADDNode* A1;  
  ADDNode* A00, *A01;
  ADDNode* A10, *A11;
  
  if(uniqueTable==A->getUniqueTable()){
    A0=A->getSucc0();
		A1=A->getSucc1();
  } else {
    A0=A1=A;
  }
  if(A0->getLevel()==uniqueTable->getLevel()+1){
    A00=A0->getSucc0();
    A01=A0->getSucc1();
  } else {
    A00=A01=A0;
  }
  if(A1->getLevel()==uniqueTable->getLevel()+1){
    A10=A1->getSucc0();
    A11=A1->getSucc1();
  } else {
    A10=A11=A1;
  }
  
  ADDNode* b0, *b1;
  if(b->getLevel()==uniqueTable->getLevel()){
    b0=b->getSucc0();
		b1=b->getSucc1();
  } else {
    b0=b1=b;
  }
	
  ADDNode* newVarSet=varSet->getSucc1();
	
  ADDNode* w00=VM(b0,A00,newVarSet,m-1);
  ADDNode* w01=VM(b0,A01,newVarSet,m-1);
  ADDNode* w10=VM(b1,A10,newVarSet,m-1);
  ADDNode* w11=VM(b1,A11,newVarSet,m-1);
  
  ADDNode* c0=addition(w00,w10);
  ADDNode* c1=addition(w01,w11);
  
  tempNode=((c0->getUniqueTable()==uniqueTable)||(c1->getUniqueTable()==uniqueTable)||(c0==c1))?c0:findOrAdd(uniqueTable,c1,c0);
  tempNode->incNodeCount();
  if(w00->preNodeCount==0) w00->garbageCollect();
  if(w01->preNodeCount==0) w01->garbageCollect();
  if(w10->preNodeCount==0) w10->garbageCollect();
  if(w11->preNodeCount==0) w11->garbageCollect();
  --tempNode->preNodeCount;
  MatrixComputedTable.insert(calcPtr(b,VMA_OP),A,varSet,tempNode);
  
  return tempNode;
}

ADDNode* ADD::equivalenceClasses(ADDNode* u, ADDNode* equiv, ADDNode* cubeSet){
  INTERRUPTIBLE();

  if(u==equiv) return oneDrain;
  
  if(u->isDrain()) return zeroDrain;
  
  ADDUniqueTable* uniqueTable=u->getUniqueTable();
  unsigned long index=uniqueTable->getRealLevel();
  if(equiv->getRealLevel()<index) return zeroDrain;
  
  while(cubeSet->getRealLevel()<index){
    cubeSet=cubeSet->getSucc1();
  }
  
  ADDNode* w0;
  ADDNode* w1;
  
  ADDNode* result=(ADDNode*) SpecialComputedTable.find(calcPtr(u,EQI_OP),equiv,cubeSet);
  if(result) return result;
  
  if(uniqueTable==cubeSet->getUniqueTable()){
    ADDNode* equiv0;
    ADDNode* equiv1;
    
    if(equiv->getUniqueTable()==uniqueTable){
      equiv0=equiv->getSucc0();
      equiv1=equiv->getSucc1();
    } else equiv0=equiv1=equiv;
    
    w0=equivalenceClasses(u->getSucc0(),equiv0,cubeSet->getSucc1());
    w1=equivalenceClasses(u->getSucc1(),equiv1,cubeSet->getSucc1());
    
    result=AND(w0,w1);
      
    //protect from garbage collection
    result->incNodeCount();
    if(w0->preNodeCount==0) w0->garbageCollect();
    if(w1->preNodeCount==0) w1->garbageCollect();
    //revert protection
    --result->preNodeCount;
  } else {
    w0=equivalenceClasses(u->getSucc0(),equiv,cubeSet);
    w1=equivalenceClasses(u->getSucc1(),equiv,cubeSet);
    result=(w0==w1)?w0:findOrAdd(uniqueTable,w1,w0);
  }
  
  SpecialComputedTable.insert(calcPtr(u,EQI_OP),equiv,cubeSet,result);
  
  return result;
}

ADDNode* ADD::constrain(ADDNode* f, ADDNode* g){
  INTERRUPTIBLE();

  //if g is a drain return constrain(f,0)=0 or constrain(f,1)=f
  if(g->isDrain()) return (g==zeroDrain)?zeroDrain:f;
  
  //if f is a drain and g is not, return constrain(1,g)=1 or constrain(0,g)=0
  if(f->isDrain()) return f;

  //if v=w constrain(v,w)=1
  if(f==g) return oneDrain;
  
  ADDNode* result=(ADDNode*) SpecialComputedTable.find(calcPtr(f,CON_OP),g);
  if(result) return result;
  
  ADDUniqueTable* minTable;
  
  ADDNode* f0, *f1;
  ADDNode* g0, *g1;
  
  if(g->getRealLevel()<=f->getRealLevel()){
    minTable=g->getUniqueTable();
    g0=g->getSucc0();
    g1=g->getSucc1();
    if(f->getUniqueTable()==minTable){
      f0=f->getSucc0();
      f1=f->getSucc1();
    } else f0=f1=f;
    if(g0==zeroDrain){
      result=constrain(f1,g1);
      SpecialComputedTable.insert(calcPtr(f,CON_OP),g,result);

      return result;
    } else if(g1==zeroDrain){
      result=constrain(f0,g0);
      SpecialComputedTable.insert(calcPtr(f,CON_OP),g,result);

      return result;
    }
  } else {
    minTable=f->getUniqueTable();
    g0=g1=g;
    f0=f->getSucc0();
    f1=f->getSucc1();
  }

  ADDNode* w0, *w1;
  
  w0=constrain(f0,g0);
  w1=constrain(f1,g1);
      
  result=(w0==w1)?w0:findOrAdd(minTable,w1,w0);
  
  SpecialComputedTable.insert(calcPtr(f,CON_OP),g,result);
  
  return result;
  
}

ADDNode* ADD::ITE(ADDNode* u, ADDNode* v1, ADDNode* v2){
  INTERRUPTIBLE();

  //case 1: (h,*,*)
  if(v1==v2) return v1;
  
  unsigned long index=v1->getRealLevel()<v2->getRealLevel()?v1->getRealLevel():v2->getRealLevel();
  
  if(u->getRealLevel()<index){
    index=u->getRealLevel();
    if((u->getSucc0()==zeroDrain) && (u->getSucc1()==oneDrain)) return findOrAdd(u->getUniqueTable(),v1,v2);
    if((u->getSucc1()==zeroDrain) && (u->getSucc0()==oneDrain)) return findOrAdd(u->getUniqueTable(),v2,v1);
  }

  //case 2: (*,f,g)
  if(u->isDrain()) return (u==zeroDrain)?v2:v1;

  //case 3: (h,f,0)
  if(v2==zeroDrain) return AND(u,v1);

  //case 4: (h,1,g)
  if(v1==oneDrain) return OR(u,v2);

  //try to get result from computed table
  ADDNode* tempADDNode=(ADDNode*) BoolComputedTable.find(calcPtr(u,ITE_OP),v1,v2);
  if(tempADDNode) return tempADDNode;

  ADDNode* w0;
  ADDNode* w1;
  ADDNode* u0, *u1;
  ADDNode* v10, *v11;
  ADDNode* v20, *v21;
  ADDNode* node;

  ADDUniqueTable* minTable=0;

  if(u->getRealLevel()<=index){
    minTable=u->getUniqueTable();
    u0=u->getSucc0();
    u1=u->getSucc1();
  } else u0=u1=u;
  if(v1->getRealLevel()==index){
    minTable=v1->getUniqueTable();
    v10=v1->getSucc0();
    v11=v1->getSucc1();
  } else v10=v11=v1;
  if(v2->getRealLevel()==index) {
    minTable=v2->getUniqueTable();
    v20=v2->getSucc0();
    v21=v2->getSucc1();
  } else v20=v21=v2;

  w0=ITE(u0,v10,v20);
  w1=ITE(u1,v11,v21);

  node=(w0==w1)?w0:findOrAdd(minTable,w1,w0);

  //insert result into computed table
  BoolComputedTable.insert(calcPtr(u,ITE_OP),v1,v2,node);

  return node;
}

ADDNode* ADD::AND(ADDNode* v1, ADDNode* v2){
  INTERRUPTIBLE();

  if(v1==v2) return v1;
  
  //case 1: (*,f) or (f,*)
  if(v1->isDrain()){
    if(v1==zeroDrain) return zeroDrain;
    else return v2;
  }
  if(v2->isDrain()){
    if(v2==zeroDrain) return zeroDrain;
    else return v1;
  }
  
  swapPointer(&v1,&v2);
  
  //try to get result from computed table
  ADDNode* node=(ADDNode*) BoolComputedTable.find(calcPtr(v1,ITE_OP),v2,zeroDrain);
  if(node) return node;

  ADDNode* w0;
  ADDNode* w1;

  ADDUniqueTable* minTable;

  if(v1->getRealLevel()<=v2->getRealLevel()){
    minTable=v1->getUniqueTable();
    if(v2->getUniqueTable()==minTable){
      w0=AND(v1->getSucc0(),v2->getSucc0());
      w1=AND(v1->getSucc1(),v2->getSucc1());
    } else {
      w0=AND(v1->getSucc0(),v2);
      w1=AND(v1->getSucc1(),v2);
    }
  } else {
    minTable=v2->getUniqueTable();
    w0=AND(v1,v2->getSucc0());
    w1=AND(v1,v2->getSucc1());
  } 

  node=(w0==w1)?w0:findOrAdd(minTable,w1,w0);

  //insert result into computed table
  BoolComputedTable.insert(calcPtr(v1,ITE_OP),v2,zeroDrain,node);

  return node;
}

ADDNode* ADD::OR(ADDNode* v1, ADDNode* v2){
  INTERRUPTIBLE();

  if(v1==v2) return v1;
  
  //case 1: (*,f) or (f,*)
  if(v1->isDrain()){
    if(v1==zeroDrain) return v2;
    else return oneDrain;
  }
  if(v2->isDrain()){
    if(v2==zeroDrain) return v1;
    else return oneDrain;
  }

  swapPointer(&v1,&v2);

  //try to get result from computed table
  ADDNode* node=(ADDNode*) BoolComputedTable.find(calcPtr(v1,ITE_OP),oneDrain,v2);
  if(node) return node;

  ADDNode* w0;
  ADDNode* w1;

  ADDUniqueTable* minTable;

  if(v1->getRealLevel()<=v2->getRealLevel()){
    minTable=v1->getUniqueTable();
    if(v2->getUniqueTable()==minTable){
      w0=OR(v1->getSucc0(),v2->getSucc0());
      w1=OR(v1->getSucc1(),v2->getSucc1());
    } else {
      w0=OR(v1->getSucc0(),v2);
      w1=OR(v1->getSucc1(),v2);
    }
  } else {
    minTable=v2->getUniqueTable();
    w0=OR(v1,v2->getSucc0());
    w1=OR(v1,v2->getSucc1());
  } 

  node=(w0==w1)?w0:findOrAdd(minTable,w1,w0);

  //insert result into computed table
  BoolComputedTable.insert(calcPtr(v1,ITE_OP),oneDrain,v2,node);

  return node;
}

ADDNode* ADD::NOT(ADDNode* u){
  INTERRUPTIBLE();

  if(u->isDrain()) return (u==zeroDrain)?oneDrain:zeroDrain;

  ADDNode* tempADDNode=(ADDNode*) BoolComputedTable.find(calcPtr(u,ITE_OP),zeroDrain,oneDrain);
  if(tempADDNode) return tempADDNode;

  ADDNode* w0=NOT(u->getSucc0());
  ADDNode* w1=NOT(u->getSucc1());

  ADDNode* node=(w0==w1)?w0:findOrAdd(u->getUniqueTable(),w1,w0);

  //insert result into computed table
  BoolComputedTable.insert(calcPtr(u,ITE_OP),zeroDrain,oneDrain,node);

  return node;
}

ADDNode* ADD::cofactor(ADDNode* node, ADDNode* assignment){
  INTERRUPTIBLE();

  if(node->isDrain()) return node;

  while(assignment->getRealLevel()<node->getRealLevel()) assignment=(assignment->getSucc0()==zeroDrain)?assignment->getSucc1():assignment->getSucc0();

  if(assignment->isDrain()) return node;
  
  bool value=(assignment->getSucc0()==zeroDrain)?true:false;

  //check for computed table
  ADDNode* tempNode=(ADDNode*) BoolComputedTable.find(calcPtr(node,COF_OP),assignment);
  if(tempNode) return tempNode;
  
  //do it the hard way
  if(assignment->getRealLevel()==node->getRealLevel()) tempNode=cofactor(node->getSucc(value),assignment->getSucc(value));
  else {
    ADDNode* w0=cofactor(node->getSucc0(),assignment);
    ADDNode* w1=cofactor(node->getSucc1(),assignment);

    tempNode=(w0==w1)?w0:findOrAdd(node->getUniqueTable(),w1,w0);
  }
  
  //insert result into computed table
  BoolComputedTable.insert(calcPtr(node,COF_OP),assignment,tempNode);

  return tempNode;
}

ADDNode* ADD::change(ADDNode* node){
  INTERRUPTIBLE();

  if(node->isDrain()) return node;
  
  //check computed table
  ADDNode* tempNode=(ADDNode*) BoolComputedTable.find(calcPtr(node,CHA_OP),0);
  if(tempNode) return tempNode;
  
  //do it the hard way
  tempNode=findOrAdd(node->getUniqueTable(),change(node->getSucc0()),change(node->getSucc1()));
  
  //insert result into computed table
  BoolComputedTable.insert(calcPtr(node,CHA_OP),0,tempNode);
  
  return tempNode;
}

ADDNode* ADD::change(ADDNode* node, ADDNode* cubeSet){
  INTERRUPTIBLE();

  if(node->isDrain()) return node;

  while(cubeSet->getRealLevel()<node->getRealLevel()) cubeSet=cubeSet->getSucc1();  
  if(cubeSet->isDrain()) return node;
  
  //check computed table
  ADDNode* tempNode=(ADDNode*) BoolComputedTable.find(calcPtr(node,CHA_OP),cubeSet);
  if(tempNode) return tempNode;
  
  //do it the hard way
  if(cubeSet->getRealLevel()==node->getRealLevel()) tempNode=findOrAdd(node->getUniqueTable(),change(node->getSucc0(),cubeSet->getSucc1()),change(node->getSucc1(),cubeSet->getSucc1()));
  else {
    ADDNode* w0=change(node->getSucc0(),cubeSet);
    ADDNode* w1=change(node->getSucc1(),cubeSet);
    
    tempNode=(w0==w1)?w0:findOrAdd(node->getUniqueTable(),w1,w0);
  }
  
  //insert result into computed table
  BoolComputedTable.insert(calcPtr(node,CHA_OP),cubeSet,tempNode);
  
  return tempNode;
}

ADDNode* ADD::less(ADDNode* u, ADDNode* v){
  INTERRUPTIBLE();

  if(u==v) return zeroDrain;
  if((u->isDrain())&&(v->isDrain())) return u->getValue()<v->getValue()?oneDrain:zeroDrain;

  ADDNode* tempNode=(ADDNode*) CompareComputedTable.find(calcPtr(u,LES_OP),v);
  if(tempNode) return tempNode;

  ADDNode* w0;
  ADDNode* w1;
  ADDUniqueTable* minTable;

  //calculate minTable
  minTable=u->getRealLevel()<v->getRealLevel()?u->getUniqueTable():v->getUniqueTable();

  w0=less(u->getUniqueTable()==minTable?u->getSucc0():u,v->getUniqueTable()==minTable?v->getSucc0():v);
  w1=less(u->getUniqueTable()==minTable?u->getSucc1():u,v->getUniqueTable()==minTable?v->getSucc1():v);

  tempNode=(w0==w1)?w0:findOrAdd(minTable,w1,w0);
  CompareComputedTable.insert(calcPtr(u,LES_OP),v,tempNode);

  return tempNode;
}

ADDNode* ADD::lessEqual(ADDNode* u, ADDNode* v){
  INTERRUPTIBLE();

  if(u==v) return oneDrain;
  if((u->isDrain())&&(v->isDrain())) return (u->getValue()<=v->getValue())?oneDrain:zeroDrain;

  ADDNode* tempNode=(ADDNode*) CompareComputedTable.find(calcPtr(u,LEE_OP),v);
  if(tempNode) return tempNode;

  ADDNode* w0;
  ADDNode* w1;
  ADDUniqueTable* minTable;

  //calculate minTable
  minTable=u->getRealLevel()<v->getRealLevel()?u->getUniqueTable():v->getUniqueTable();

  w0=lessEqual(u->getUniqueTable()==minTable?u->getSucc0():u,v->getUniqueTable()==minTable?v->getSucc0():v);
  w1=lessEqual(u->getUniqueTable()==minTable?u->getSucc1():u,v->getUniqueTable()==minTable?v->getSucc1():v);

  tempNode=(w0==w1)?w0:findOrAdd(minTable,w1,w0);
  CompareComputedTable.insert(calcPtr(u,LEE_OP),v,tempNode);

  return tempNode;
}

ADDNode* ADD::equal(ADDNode* u, ADDNode* v){
  INTERRUPTIBLE();

  if(u==v) return oneDrain;
  if((u->isDrain())&&(v->isDrain())) return (u->getValue()==v->getValue())?oneDrain:zeroDrain;

  ADDNode* tempNode=(ADDNode*) CompareComputedTable.find(calcPtr(u,EQU_OP),v);
  if(tempNode) return tempNode;

  ADDNode* w0;
  ADDNode* w1;
  ADDUniqueTable* minTable;

  //calculate minTable
  minTable=u->getRealLevel()<v->getRealLevel()?u->getUniqueTable():v->getUniqueTable();

  w0=equal(u->getUniqueTable()==minTable?u->getSucc0():u,v->getUniqueTable()==minTable?v->getSucc0():v);
  w1=equal(u->getUniqueTable()==minTable?u->getSucc1():u,v->getUniqueTable()==minTable?v->getSucc1():v);

  tempNode=(w0==w1)?w0:findOrAdd(minTable,w1,w0);
  CompareComputedTable.insert(calcPtr(u,EQU_OP),v,tempNode);

  return tempNode;
}

ADDNode* ADD::max(ADDNode* u, ADDNode* v){
  if((u->isDrain())&&(v->isDrain())) return findOrAddDrain(u->getValue()>v->getValue()?u->getValue():v->getValue());

  swapPointer(&u,&v);
  ADDNode* tempNode=(ADDNode*) ExtremaComputedTable.find(calcPtr(u,MAX_OP),v);
  if(tempNode) return tempNode;

  ADDNode* w0;
  ADDNode* w1;
  ADDUniqueTable* minTable;

  //calculate minTable
  minTable=u->getRealLevel()<v->getRealLevel()?u->getUniqueTable():v->getUniqueTable();

  w0=max(u->getUniqueTable()==minTable?u->getSucc0():u,v->getUniqueTable()==minTable?v->getSucc0():v);
  w1=max(u->getUniqueTable()==minTable?u->getSucc1():u,v->getUniqueTable()==minTable?v->getSucc1():v);

  tempNode=(w0==w1)?w0:findOrAdd(minTable,w1,w0);
  ExtremaComputedTable.insert(calcPtr(u,MAX_OP),v,tempNode);
  
  return tempNode;
}

double ADD::max(ADDNode* node, map<ADDNode*,double>& CT){
  if(node->isDrain()) return node->getValue();
  
  if(CT.find(node)!=CT.end()) return CT[node];

  double w0=max(node->getSucc0(),CT);
  double w1=max(node->getSucc1(),CT);

  double m=(w0>=w1)?w0:w1;
  CT[node]=m;
  
  return m;
}

ADDNode* ADD::min(ADDNode* u, ADDNode* v){
  if((u->isDrain())&&(v->isDrain())) return findOrAddDrain(u->getValue()<v->getValue()?u->getValue():v->getValue());
  
  swapPointer(&u,&v);
  ADDNode* tempNode=(ADDNode*) ExtremaComputedTable.find(calcPtr(u,MIN_OP),v);
  if(tempNode) return tempNode;
  
  ADDNode* w0;
  ADDNode* w1;
  ADDUniqueTable* minTable;
  
  //calculate minTable
  minTable=u->getRealLevel()<v->getRealLevel()?u->getUniqueTable():v->getUniqueTable();
  
  w0=min(u->getUniqueTable()==minTable?u->getSucc0():u,v->getUniqueTable()==minTable?v->getSucc0():v);
  w1=min(u->getUniqueTable()==minTable?u->getSucc1():u,v->getUniqueTable()==minTable?v->getSucc1():v);
  
  tempNode=(w0==w1)?w0:findOrAdd(minTable,w1,w0);
  ExtremaComputedTable.insert(calcPtr(u,MIN_OP),v,tempNode);
  
  return tempNode;
}

double ADD::min(ADDNode* node, map<ADDNode*,double>& CT){
  if(node->isDrain()) return node->getValue();
  
  if(CT.find(node)!=CT.end()) return CT[node];

  double w0=min(node->getSucc0(),CT);
  double w1=min(node->getSucc1(),CT);

  double m=(w0<=w1)?w0:w1;
  CT[node]=m;
  
  return m;
}

ADDNode* ADD::oneOver(ADDNode* node){
  INTERRUPTIBLE();

  if(node->isDrain()) return (node==zeroDrain)?node:findOrAddDrain(1.0/node->getValue());

  ADDNode* tempADDNode=(ADDNode*) ArithmeticComputedTable.find(calcPtr(node,OVE_OP),0);
  if(tempADDNode) return tempADDNode;

  ADDNode* w0=oneOver(node->getSucc0());
  ADDNode* w1=oneOver(node->getSucc1());

  tempADDNode=findOrAdd(node->getUniqueTable(),w1,w0);
  ArithmeticComputedTable.insert(calcPtr(node,OVE_OP),0,tempADDNode);
  
  return tempADDNode;
}

ADDNode* ADD::moveUp(ADDNode* u){
  INTERRUPTIBLE();

  if(u->isDrain()) return u;
  
  ADDNode* tempNode=(ADDNode*) MoveComputedTable.find(calcPtr(u,MUP_OP),0,(ADDNode*) 1);
  if(tempNode) return tempNode;
  
  tempNode=findOrAdd(u->getLevel()-1,moveUp(u->getSucc1()),moveUp(u->getSucc0()));
  MoveComputedTable.insert(calcPtr(u,MUP_OP),0,(ADDNode*) 1,tempNode);
  
  return tempNode;
}

ADDNode* ADD::moveUp(ADDNode* u, const unsigned int skip){
  INTERRUPTIBLE();

  if(u->isDrain()) return u;

  ADDNode* tempNode=(ADDNode*) MoveComputedTable.find(calcPtr(u,MUP_OP),0,(ADDNode*) skip);
  if(tempNode) return tempNode;
    
  tempNode=findOrAdd(u->getLevel()-skip,moveUp(u->getSucc1(),skip),moveUp(u->getSucc0(),skip));
  MoveComputedTable.insert(calcPtr(u,MUP_OP),0,(ADDNode*) skip,tempNode);

  return tempNode;
}

ADDNode* ADD::moveUp(ADDNode* u, ADDNode* v){
  INTERRUPTIBLE();

  if(u->isDrain()) return u;

  unsigned long uLevel=u->getLevel();
  while(v->getLevel()<uLevel) v=v->getSucc1();
  
  if((uLevel<1)&&(uLevel==v->getLevel())) return u;
  if(v->isDrain()) return u;
  
  ADDNode* tempNode=(ADDNode*) MoveComputedTable.find(calcPtr(u,MUP_OP),v,(ADDNode*) 1);
  if(tempNode) return tempNode;
  
  ADDNode* w0;
  ADDNode* w1;
  if(uLevel==v->getLevel()){
    w0=moveUp(u->getSucc0(),v->getSucc1());
	  w1=moveUp(u->getSucc1(),v->getSucc1());
    
    tempNode=findOrAdd(uLevel-1,w1,w0);
  } else {
    w0=moveUp(u->getSucc0(),v);
	  w1=moveUp(u->getSucc1(),v);
    
    //check result
    if((w0->getLevel()<=uLevel)||(w1->getLevel()<=uLevel)){
      if(w0->preNodeCount==0) w0->garbageCollect();
      if(w1->preNodeCount==0) w1->garbageCollect();
      tempNode=u;
    } else tempNode=findOrAdd(uLevel,w1,w0);
  }
  
  MoveComputedTable.insert(calcPtr(u,MUP_OP),v,(ADDNode*) 1,tempNode);
  
  return tempNode;
}

ADDNode* ADD::moveUp(ADDNode* u, ADDNode* v, const unsigned int skip){
  INTERRUPTIBLE();

  if(u->isDrain()) return u;
  
  unsigned long uLevel=u->getLevel();
  while(v->getLevel()<uLevel) v=v->getSucc1();

  if((uLevel<skip)&&(uLevel==v->getLevel())) return u;
  if(v->isDrain()) return u;

  ADDNode* tempNode=(ADDNode*) MoveComputedTable.find(calcPtr(u,MUP_OP),v,(ADDNode*) skip);
  if(tempNode) return tempNode;
  
  ADDNode* w0;
  ADDNode* w1;
  if(uLevel==v->getLevel()){
    w0=moveUp(u->getSucc0(),v->getSucc1(),skip);
	  w1=moveUp(u->getSucc1(),v->getSucc1(),skip);

    tempNode=findOrAdd(uLevel-skip,w1,w0);
  } else {
    w0=moveUp(u->getSucc0(),v,skip);
	  w1=moveUp(u->getSucc1(),v,skip);

    //check result
    if((w0->getLevel()<=uLevel)||(w1->getLevel()<=uLevel)){
      if(w0->preNodeCount==0) w0->garbageCollect();
      if(w1->preNodeCount==0) w1->garbageCollect();
      tempNode=u;
    } else tempNode=findOrAdd(uLevel,w1,w0);
  }

  MoveComputedTable.insert(calcPtr(u,MUP_OP),v,(ADDNode*) skip,tempNode);

  return tempNode;
}

ADDNode* ADD::moveDown(ADDNode* u){
  INTERRUPTIBLE();

  if(u->isDrain()) return u;
  
  ADDNode* tempNode=(ADDNode*) MoveComputedTable.find(calcPtr(u,MDO_OP),0,(ADDNode*) 1);
  if(tempNode) return tempNode;
  
  ADDNode* w0=moveDown(u->getSucc0());
  ADDNode* w1=moveDown(u->getSucc1());
  
  tempNode=(u->getLevel()+1>=variableOrdering.size())?zeroDrain:findOrAdd(u->getLevel()+1,w1,w0);
  MoveComputedTable.insert(calcPtr(u,MDO_OP),0,(ADDNode*) 1,tempNode);
  
  return tempNode;
}

ADDNode* ADD::moveDown(ADDNode* u, const unsigned int skip){
  INTERRUPTIBLE();

  if(u->isDrain()) return u;

  ADDNode* tempNode=(ADDNode*) MoveComputedTable.find(calcPtr(u,MDO_OP),0,(ADDNode*) skip);
  if(tempNode) return tempNode;

  ADDNode* w0=moveDown(u->getSucc0(),skip);
  ADDNode* w1=moveDown(u->getSucc1(),skip);

  tempNode=(u->getLevel()+skip>=variableOrdering.size())?zeroDrain:findOrAdd(u->getLevel()+skip,w1,w0);
  MoveComputedTable.insert(calcPtr(u,MDO_OP),0,(ADDNode*) skip,tempNode);

  return tempNode;
}

ADDNode* ADD::moveDown(ADDNode* u, ADDNode* v){
  INTERRUPTIBLE();

  if(u->isDrain()) return u;
  
  unsigned long uLevel=u->getLevel();
  while(v->getLevel()<uLevel) v=v->getSucc1();
  
  if(v->isDrain()) return u;
  
  ADDNode* tempNode=(ADDNode*) MoveComputedTable.find(calcPtr(u,MDO_OP),v,(ADDNode*) 1);
  if(tempNode) return tempNode;
  
  ADDNode* w0;
  ADDNode* w1;
  if(uLevel==v->getLevel()){
    w0=moveDown(u->getSucc0(),v->getSucc1());
	  w1=moveDown(u->getSucc1(),v->getSucc1());
	  if((uLevel+1>=w0->getLevel())||(uLevel+1>=w1->getLevel())) tempNode=findOrAdd(uLevel,w1,w0);
	  else tempNode=findOrAdd(uLevel+1,w1,w0);
  } else {
    w0=moveDown(u->getSucc0(),v);
	  w1=moveDown(u->getSucc1(),v);
	  tempNode=findOrAdd(u->getLevel(),w1,w0);
  }
  
  MoveComputedTable.insert(calcPtr(u,MDO_OP),v,(ADDNode*) 1,tempNode);
  
  return tempNode;
}

ADDNode* ADD::moveDown(ADDNode* u, ADDNode* v, const unsigned int skip){
  INTERRUPTIBLE();

  if(u->isDrain()) return u;

  unsigned long uLevel=u->getLevel();
  while(v->getLevel()<uLevel) v=v->getSucc1();

  if(v->isDrain()) return u;
  
  ADDNode* tempNode=(ADDNode*) MoveComputedTable.find(calcPtr(u,MDO_OP),v,(ADDNode*) skip);
  if(tempNode) return tempNode;

  ADDNode* w0;
  ADDNode* w1;
  if(uLevel==v->getLevel()){
    w0=moveDown(u->getSucc0(),v->getSucc1(),skip);
	  w1=moveDown(u->getSucc1(),v->getSucc1(),skip);
	  if((uLevel+skip>=w0->getLevel())||(uLevel+skip>=w1->getLevel())) tempNode=findOrAdd(uLevel,w1,w0);
	  else tempNode=findOrAdd(uLevel+skip,w1,w0);
  } else {
    w0=moveDown(u->getSucc0(),v,skip);
	  w1=moveDown(u->getSucc1(),v,skip);
	  tempNode=findOrAdd(uLevel,w1,w0);
  }
  
  MoveComputedTable.insert(calcPtr(u,MDO_OP),v,(ADDNode*) skip,tempNode);

  return tempNode;
}

ADDNode* ADD::getAssignment(const unsigned long lastLevel, const ADDNode* node){
  if(node->isDrain()) return (node==zeroDrain)?zeroDrain:oneDrain;
  
  ADDNode* tempNode;
  unsigned long currentLevel=node->getLevel();
  
  if(node->getSucc0()!=zeroDrain){
    tempNode=findOrAdd(node->getUniqueTable(),zeroDrain,getAssignment(currentLevel+1,node->getSucc0()));
  } else {
    tempNode=findOrAdd(node->getUniqueTable(),getAssignment(currentLevel+1,node->getSucc1()),zeroDrain);
  }
  
  while(lastLevel<currentLevel){
    tempNode=findOrAdd(--currentLevel,zeroDrain,tempNode);
  }
  
  return tempNode;
}

bool ADD::getAssignment(const ADDNode* node, Assignment& assignment) const {
  if(node->isDrain()) return (node!=zeroDrain);

  //all variables are set to zero before this function
  if(!getAssignment(node->getSucc0(),assignment)){
    getAssignment(node->getSucc1(),assignment);
    assignment[variableOrdering[node->getLevel()]]=true;
  }

  return true;
}

bool ADD::getAssignment(const ADDNode* node, char* buffer) const {
  if(node->isDrain()) return (node!=zeroDrain);
  
  if(!getAssignment(node->getSucc0(),buffer)){
    getAssignment(node->getSucc1(),buffer);
    buffer[node->getLevel()]='1';
  }
  
  return true;
}

double ADD::countAssignments(const ADDNode* node, map<ADDNode*,double>& CT) const {
  if(node->isDrain()) return (node==zeroDrain)?0.0:1.0;

  unsigned long diff;
  double ret1, ret2;
  
  if(CT.find((ADDNode*) node)!=CT.end()) return CT[(ADDNode*) node];
  
  ret1=countAssignments(node->getSucc0(),CT);
  diff=(node->getSucc0()->isDrain()?variableOrdering.size():node->getSucc0()->getLevel())-node->getLevel()-1;
  ret1=(1<<diff)*ret1;

  ret2=countAssignments(node->getSucc1(),CT);
  diff=(node->getSucc1()->isDrain()?variableOrdering.size():node->getSucc1()->getLevel())-node->getLevel()-1;
  ret2=(1<<diff)*ret2;

  CT[(ADDNode*) node]=ret1+ret2;

  return ret1+ret2;
}

double ADD::countAssignments(const ADDNode* node, const ADDNode* cubeset_node, map<pair<const ADDNode*, const ADDNode*>,double>& CT) const {
  double factor=1.0;
  double count, count0, count1;
  
  if(CT.find(make_pair(node, cubeset_node))!=CT.end()) return CT[make_pair(node, cubeset_node)];
  
  const ADDNode* cubeset_cur=cubeset_node;
  while (cubeset_cur->getLevel() < node->getLevel()) {
    factor*=2.0;
    cubeset_cur=cubeset_cur->getSucc1();
  }

  if(node->isDrain()) {
    count=(node==zeroDrain)?0.0:1.0;
  } else {
    // check that node->getLevel() == cubeset_node->getLevel()
    // ....
    
    // at inner node
    count0=countAssignments(node->getSucc0(), cubeset_cur->getSucc1(), CT);
    count1=countAssignments(node->getSucc1(), cubeset_cur->getSucc1(), CT);
    
    count=(count0+count1);
  }
  
  CT[make_pair(node, cubeset_cur)]=count;
  
  double count_with_factor=count*factor;
  CT[make_pair(node, cubeset_node)]=count_with_factor;

  return count_with_factor;     
}




ADDNode* ADD::getPath(const ADDNode* node){
  if(node->isDrain()) return (node==zeroDrain)?zeroDrain:oneDrain;

  ADDNode* tempNode;
  
  if(node->getSucc0()!=zeroDrain){
    tempNode=findOrAdd(node->getUniqueTable(),zeroDrain,getPath(node->getSucc0()));
  } else {
    tempNode=findOrAdd(node->getUniqueTable(),getPath(node->getSucc1()),zeroDrain);
  }
  
  return tempNode;
}

bool ADD::getPath(const ADDNode* node, char* buffer) const {
  if(node->isDrain()) return (node!=zeroDrain);
  
  if(getPath(node->getSucc0(),buffer)){
    buffer[node->getLevel()]='0';
  } else {
    getPath(node->getSucc1(),buffer);
    buffer[node->getLevel()]='1';
  }
  
  return true;
}

ADDNode* ADD::createConstantFunction(const double value){
  return findOrAddDrain(value);
}

ADDFunction ADD::createFunction(ADDNode* node){
  if (dynamicReordering) {
    //protect node from garbage collection
    node->incNodeCount();

    reorderingHook();
    
    //reverse increasing
    --node->preNodeCount;
  }

  return ADDFunction(node);

}

void ADD::reorderingHook(bool force_reordering) {
  ///TODO: methods to change the following two values 
  unsigned long min_reord_bdd_size = 100;
  unsigned long reord_trigger_factor = 2;
  
  unsigned long bdd_size = ADD::size();
  if(force_reordering ||
     (bdd_size > min_reord_bdd_size &&
      (bdd_size > reord_trigger_factor*lastReorderSize))){
    
    ADD::garbageCollect();
    bdd_size = ADD::size();
    if (force_reordering ||
        (bdd_size > min_reord_bdd_size &&
         (bdd_size > reord_trigger_factor*lastReorderSize))) {
      
      unsigned long before=0, after=0;
      TRACE("ADD", "reorder", before=ADD::size();                    \
            cout << "Starting reordering: " << before << " nodes";   \
            cout.flush());
      
      ADD::groupSifting(dynamicReorderingMaxGrowth);
      
      TRACE("ADD", "reorder", after=ADD::size();                  \
            cout << ", afterwards " << after << " nodes, "              \
            << ((float)before-after)/(float)before*100.0 << "%\n");
      
      TRACE("ADD", "reorder", if (after>before) {throw "BDD is bigger after reordering!";});

      lastReorderSize=ADD::size();
    }
    
    operatorCounter=0;
  }
}

ADDNode* ADD::findOrAdd(const unsigned long index, ADDNode* succ1, ADDNode* succ0){
  return findOrAdd(variableOrdering.getData(index),succ1,succ0);
}

ADDNode* ADD::findOrAdd(ADDUniqueTable* uTable, ADDNode* succ1, ADDNode* succ0){
  //this functions checks if there is an isomorphic node
  //in this case the isomorphic node will be returned otherwise a new node will be created (and returned)

  return uTable->findOrAdd(succ0,succ1);
}

ADDNode* ADD::findOrAddDrain(const double value){
  ADDUniqueTable* uniqueTable=variableOrdering.getDrainData();
  if(value<0.0){
    if(value>-0.0000001) return zeroDrain;
  } else if(value<0.0000001) return zeroDrain;
  
  ADDNode node;
  node.setValue(value);

  return uniqueTable->findOrAddValue(&node);
}

ADDNode* ADD::findDrain(const double value){
  ADDUniqueTable* uniqueTable=variableOrdering.getDrainData();
  ADDNode node;
  node.setValue(value);

  return uniqueTable->getValue(&node);
}
