/***************************************************************************
                          addfunction.cpp  -  description
                             -------------------
    begin                : Thu Apr 15 2004
    copyright            : (C) 2004 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "addfunction.h"
#include "add.h"
#include "addnode.h"
#include "addvarorder.h"
#include "../common/assignment.h"
#include "../common/varset.h"
#include "../common/stringhelper.h"
#include "../common/varorder.h"
#include "../common/vsizearray.h"

#include <stack>
#include <iostream>

using namespace std;

inline unsigned long pow2(const unsigned long n){
  return (1 << n);
}

inline void addFunction(ADDNode* node){
  if(node) node->incNodeCount();
}

inline void removeFunction(ADDNode* node){
  if(node) node->deleteNode();
}

//************************ADDFunction************************
ADD* ADDFunction::BDD=0;

ADDFunction::ADDFunction(const ADDFunction& function){
  rootNode=function.rootNode;
  addFunction(rootNode);
}

ADDFunction::ADDFunction(const Assignment& assignment, const double value){
  if(assignment.size()==0){
    rootNode=BDD->zeroDrain;
    addFunction(rootNode);

    return;
  } else {
    rootNode=BDD->findOrAddDrain(value);
    if(rootNode!=BDD->zeroDrain){
      for(unsigned long i=0;i<assignment.size();++i) ADDVarOrder::addVariable(assignment[i].variable);

      for(unsigned long i=BDD->variableOrdering.size();i>0;--i){
        if(assignment.find(BDD->variableOrdering[i-1])){
          rootNode=(assignment.at(BDD->variableOrdering[i-1]))?BDD->findOrAdd(i-1,rootNode,BDD->zeroDrain):BDD->findOrAdd(i-1,BDD->zeroDrain,rootNode);
        }
      }
    }
  }

  addFunction(rootNode);
}

ADDFunction::ADDFunction(char* buffer, const double value){
	rootNode=BDD->findOrAddDrain(value);
	if(rootNode!=BDD->zeroDrain){
		for(unsigned long i=BDD->variableOrdering.size();i>0;--i){
			switch(buffer[i-1]){
				case '0': rootNode=BDD->findOrAdd(i-1,BDD->zeroDrain,rootNode);
					        break;
				case '1': rootNode=BDD->findOrAdd(i-1,rootNode,BDD->zeroDrain);
         					break;
				default: break;
			}
    }
  }
	
  addFunction(rootNode);
}

ADDFunction::ADDFunction(const VarSet& varSet){
  if(varSet.size()==0){
    rootNode=BDD->zeroDrain;
    addFunction(rootNode);

    return;
  } else {
    rootNode=BDD->oneDrain;
    for(unsigned long i=0;i<varSet.size();++i) ADDVarOrder::addVariable(varSet[i]);

    for(unsigned long i=BDD->variableOrdering.size();i>0;--i){
      if(varSet.find(BDD->variableOrdering[i-1])) rootNode=BDD->findOrAdd(i-1,rootNode,BDD->zeroDrain);
    }
  }

  addFunction(rootNode);
}

ADDFunction::ADDFunction(const string& variable){
  ADDFunction tempFunction=ADDFunction::BDD->projection(variable);

  rootNode=tempFunction.rootNode;

  addFunction(rootNode);
}

ADDFunction::ADDFunction(const double value){
  rootNode=ADD::BDD->createConstantFunction(value);

  addFunction(rootNode);
}

ADDFunction::ADDFunction(void){
  rootNode=0;
}

ADDFunction::~ADDFunction(void){
  if(rootNode) removeFunction(rootNode);
}

ADDFunction ADDFunction::ERROR_FUNCTION(void){
  return ADDFunction((ADDNode*) 0);
}

ADDFunction ADDFunction::ONE(void){
  return ADDFunction(BDD->oneDrain);
}

ADDFunction ADDFunction::ZERO(void){
  return ADDFunction(BDD->zeroDrain);
}

ADDFunction ADDFunction::createConstantFunction(const double value){
  return ADDFunction(ADDFunction::BDD->createConstantFunction(value));
}

ADDFunction ADDFunction::createStabilityFunction(const VarSet& set1, const VarSet& set2){
  if(set1.size()!=set2.size()) return ADDFunction::ERROR_FUNCTION();

  ADDFunction temp=ADDFunction::ONE();
  for(unsigned long i=0;i<set1.size();++i){
    temp&=EQUIV(ADDFunction(set1[i]),ADDFunction(set2[i]));
  }

  return temp;
}

ADDFunction ADDFunction::projection(const string& variable){
  return ADDFunction::BDD->projection(variable);
}

ADDFunction ADDFunction::projection(const unsigned long index){
  return ADDFunction::BDD->projection(index);
}

unsigned long ADDFunction::size(void) const {
  set<ADDNode*> visited;
  stack<ADDNode*> nodeStack;
  ADDNode* node;

  if(!rootNode) return 0;

  visited.insert(rootNode);
  nodeStack.push(rootNode);
  while(nodeStack.size()>0){
    node=nodeStack.top();
    nodeStack.pop();
    if(!node->isDrain()){
      if(visited.find(node->getSucc0())==visited.end()){
        visited.insert(node->getSucc0());
        nodeStack.push(node->getSucc0());
      }
      if(visited.find(node->getSucc1())==visited.end()){
        visited.insert(node->getSucc1());
        nodeStack.push(node->getSucc1());
      }
    }
  }

  return visited.size();
}

std::set<ADDNode*> ADDFunction::getNodes() const {
  set<ADDNode*> visited;
  stack<ADDNode*> nodeStack;
  ADDNode* node;

  if(!rootNode) return visited;

  visited.insert(rootNode);
  nodeStack.push(rootNode);
  while (nodeStack.size()>0){
    node=nodeStack.top();
    nodeStack.pop();
    if(!node->isDrain()){
      if(visited.find(node->getSucc0())==visited.end()){
        visited.insert(node->getSucc0());
        nodeStack.push(node->getSucc0());
      }
      if(visited.find(node->getSucc1())==visited.end()){
        visited.insert(node->getSucc1());
        nodeStack.push(node->getSucc1());
      }
    }
  }

  return visited;
}


ADDFunction& ADDFunction::operator=(const ADDFunction& function){
  //insert function
  addFunction(function.rootNode);
  //remove function
  removeFunction(rootNode);
  //copy values
  rootNode=function.rootNode;

  return *this;
}

ADDFunction ADDFunction::operator+(const ADDFunction& function) const {
  return ADDFunction::BDD->addition(*this,function);
}

ADDFunction& ADDFunction::operator+=(const ADDFunction& function){
  return *this=ADDFunction::BDD->addition(*this,function);
}

ADDFunction ADDFunction::operator-(const ADDFunction& function) const {
  return ADDFunction::BDD->subtraction(*this,function);
}

ADDFunction& ADDFunction::operator-=(const ADDFunction& function){
  return *this=ADDFunction::BDD->subtraction(*this,function);
}

ADDFunction ADDFunction::operator*(const ADDFunction& function) const {
  return ADDFunction::BDD->multiplication(*this,function);
}

ADDFunction& ADDFunction::operator*=(const ADDFunction& function){
  return *this=ADDFunction::BDD->multiplication(*this,function);
}

ADDFunction ADDFunction::operator|(const ADDFunction& function) const {
  return ADDFunction::BDD->OR(*this,function);
}

ADDFunction& ADDFunction::operator|=(const ADDFunction& function){
  return *this=ADDFunction::BDD->OR(*this,function);
}

ADDFunction ADDFunction::operator&(const ADDFunction& function) const {
  return ADDFunction::BDD->AND(*this,function);
}

ADDFunction& ADDFunction::operator&=(const ADDFunction& function){
  return *this=ADDFunction::BDD->AND(*this,function);
}

ADDFunction ADDFunction::operator^(const ADDFunction& function) const {
  return ADDFunction::BDD->XOR(*this,function);
}

ADDFunction& ADDFunction::operator^=(const ADDFunction& function){
  return *this=ADDFunction::BDD->XOR(*this,function);
}

ADDFunction ADDFunction::operator!(void) const {
  ADD add;
  return add.NOT(*this);
}

bool ADDFunction::operator==(const ADDFunction& function) const {
  return (function.rootNode==rootNode);
}

bool ADDFunction::operator!=(const ADDFunction& function) const {
  return (function.rootNode!=rootNode);
}

bool ADDFunction::operator<(const ADDFunction& function) const {
  return LESS(*this,function)==1.0;
}

bool ADDFunction::operator<=(const ADDFunction& function) const {
  return LESS_EQUAL(*this,function)==1.0;
}

bool ADDFunction::operator>(const ADDFunction& function) const {
  return LESS(function,*this)==1.0;
}

bool ADDFunction::operator>=(const ADDFunction& function) const {
  return LESS_EQUAL(function,*this)==1.0;
}

bool ADDFunction::isConstant(void) const {
  return rootNode?rootNode->isDrain():false;
}

double ADDFunction::getValue(void) const {
  return isConstant()?rootNode->getValue():0.0;
}

string ADDFunction::getFirstVariable(void) const {
  return rootNode?ADDVarOrder::getVariable(rootNode->getLevel()):"[ERROR]";
}

unsigned long ADDFunction::getFirstVariableIndex(void) const {
  return rootNode?rootNode->getLevel():ULONG_MAX;
}

ADDFunction ADDFunction::cofactor(const string& variable, const bool value) const {
  return cofactor(ADDVarOrder::getVariable(variable),value);
}

ADDFunction ADDFunction::cofactor(const unsigned long index, const bool value) const {
  return ADDFunction::BDD->cofactor(*this,index,value);
}

ADDFunction ADDFunction::cofactor(const ADDFunction& assignment) const {
  return ADDFunction::BDD->cofactor(*this,assignment);
}

ADDFunction ADDFunction::subSet(const string& variable, const bool value) const {
	if(!ADDVarOrder::findVariable(variable)) return (*this);

  ADDFunction tempVar=ADDFunction::projection(variable);
	if(!value) tempVar=!tempVar;
  
  return ((*this)*tempVar);
}

ADDFunction ADDFunction::subSet(const unsigned long index, const bool value) const {
  if((index>=ADDVarOrder::size())||(!rootNode)) return (*this);
  
  ADDFunction tempVar=ADDFunction::projection(index);
	if(!value) tempVar=!tempVar;
  
  return ((*this)*tempVar);
}

ADDFunction ADDFunction::subSet(const ADDFunction& assignment) const {
  return ((*this)*assignment);
}

ADDFunction ADDFunction::change(void) const {
  return ADDFunction::BDD->change(*this);
}

ADDFunction ADDFunction::change(const string& variable) const {
  return change(ADDVarOrder::getVariable(variable));
}

ADDFunction ADDFunction::change(const unsigned long index) const {
  return ADDFunction::BDD->change(*this,index);
}

ADDFunction ADDFunction::change(const ADDFunction& cubeSet) const {
  return ADDFunction::BDD->change(*this,cubeSet);
}

ADDFunction ADDFunction::exists(const ADDFunction& varSet) const {
  return ADDFunction::BDD->exists(*this,varSet);
}

ADDFunction ADDFunction::exists(const string& variable) const {
  return exists(ADDVarOrder::getVariable(variable));
}

ADDFunction ADDFunction::exists(const unsigned long index) const {
  return ADDFunction::BDD->exists(*this,index);
}

ADDFunction ADDFunction::forall(const ADDFunction& varSet) const {
  return ADDFunction::BDD->forall(*this,varSet);
}

ADDFunction ADDFunction::forall(const string& variable) const {
  return forall(ADDVarOrder::getVariable(variable));
}

ADDFunction ADDFunction::forall(const unsigned long index) const {
  return ADDFunction::BDD->forall(*this,index);
}

ADDFunction ADDFunction::equivalenceClasses(const ADDFunction& pattern, const ADDFunction& cubeSet) const {
  return ADDFunction::BDD->equivalenceClasses(*this,pattern,cubeSet);
}

ADDFunction ADDFunction::constrain(const ADDFunction& g) const {
	return ADDFunction::BDD->constrain(*this,g);
}

ADDFunction ADDFunction::sum(const ADDFunction& varset) const {
  return ADDFunction::BDD->sum(*this,varset);
}

ADDFunction ADDFunction::product(const ADDFunction& varset) const {
  return ADDFunction::BDD->product(*this,varset);
}

ADDFunction ADDFunction::maximum(const ADDFunction& varset) const {
  return ADDFunction::BDD->maximum(*this,varset);
}

double ADDFunction::max(void) const {
  return ADDFunction::BDD->max(*this);
}

ADDFunction ADDFunction::minimum(const ADDFunction& varset) const {
  return ADDFunction::BDD->minimum(*this,varset);
}

double ADDFunction::min(void) const {
  return ADDFunction::BDD->min(*this);
}

ADDFunction ADDFunction::oneOver(void) const {
  return ADDFunction::BDD->oneOver(*this);
}

ADDFunction ADDFunction::compose(const string& variable, const ADDFunction& function) const {
  return compose(ADDVarOrder::getVariable(variable),function);
}

ADDFunction ADDFunction::compose(const unsigned long index, const ADDFunction& function) const {
  if(index>=ADDVarOrder::size()) return (*this);
  else return ITE(function,cofactor(index,1),cofactor(index,0));
}

ADDFunction ADDFunction::moveUp(const unsigned int skip) const {
  return ADDFunction::BDD->moveUp(*this,skip);
}

ADDFunction ADDFunction::moveUp(const ADDFunction& varSet, const unsigned int skip) const {
  return ADDFunction::BDD->moveUp(*this,varSet,skip);
}

ADDFunction ADDFunction::moveDown(const unsigned int skip) const {
  return ADDFunction::BDD->moveDown(*this,skip);
}

ADDFunction ADDFunction::moveDown(const ADDFunction& varSet, const unsigned int skip) const {
  return ADDFunction::BDD->moveDown(*this,varSet,skip);
}

void ADDFunction::printEssentialVariables(void) const {
  string ret;
  set<string> variables;
  set<ADDNode*> visited;
  stack<ADDNode*> nodeStack;
  ADDNode* node;

  if(!rootNode) return;

  visited.insert(rootNode);
  nodeStack.push(rootNode);
  while(nodeStack.size()>0){
    node=nodeStack.top();
    nodeStack.pop();
    if(!node->isDrain()){
      if(variables.find(ADDVarOrder::getVariable(node->getLevel()))==variables.end()) variables.insert(ADDVarOrder::getVariable(node->getLevel()));
      if(visited.find(node->getSucc0())==visited.end()){
        visited.insert(node->getSucc0());
        nodeStack.push(node->getSucc0());
      }
      if(visited.find(node->getSucc1())==visited.end()){
        visited.insert(node->getSucc1());
        nodeStack.push(node->getSucc1());
      }
    }
  }

  for(unsigned long i=0;i<ADDVarOrder::size();++i){
    if(variables.find(ADDVarOrder::getVariable(i))!=variables.end()) ret+=ADDVarOrder::getVariable(i)+",";
  }

  cout << ret.substr(0,ret.length()-1) << endl;
}

set<string> ADDFunction::getEssentialVariables(void) const {
  set<string> variables;
  set<ADDNode*> visited;
  stack<ADDNode*> nodeStack;
  ADDNode* node;

  if(!rootNode) return variables;

  visited.insert(rootNode);
  nodeStack.push(rootNode);
  while(nodeStack.size()>0){
    node=nodeStack.top();
    nodeStack.pop();
    if(!node->isDrain()){
      if(variables.find(ADDVarOrder::getVariable(node->getLevel()))==variables.end()) variables.insert(ADDVarOrder::getVariable(node->getLevel()));
      if(visited.find(node->getSucc0())==visited.end()){
        visited.insert(node->getSucc0());
        nodeStack.push(node->getSucc0());
      }
      if(visited.find(node->getSucc1())==visited.end()){
        visited.insert(node->getSucc1());
        nodeStack.push(node->getSucc1());
      }
    }
  }

  return variables;
}

ADDFunction::operator Assignment() const {
  Assignment assignment;
	
  if(!rootNode) return assignment;
	
  //initially, every variable gets the value false
  for(unsigned long i=0;i<ADDVarOrder::size();++i) assignment[ADDVarOrder::getVariable(i)]=false;
	
	ADDFunction::BDD->getAssignment(rootNode,assignment);
	
  return assignment;
}

ADDFunction ADDFunction::getAssignment(void) const {
  return BDD->getAssignment(*this);
}

void ADDFunction::getAssignment(char* buffer) const {
  ADDFunction::BDD->getAssignment(*this,buffer);
}

double ADDFunction::countAssignments(void) const {
  return ADDFunction::BDD->countAssignments(*this);
}

double ADDFunction::countAssignments(const ADDFunction& cubeSet) const {
  return ADDFunction::BDD->countAssignments(*this, cubeSet);
}

ADDFunction ADDFunction::getRandomAssignment(const ADDFunction& cube_set) const {
  return ADDFunction::BDD->getRandomAssignment(*this, cube_set);
}

void ADDFunction::printPaths(void) const {
  VSIZEARRAY(char, minterm, ADDVarOrder::size());

  if(!rootNode) return;
  printPaths(0,rootNode,VSIZEARRAY_PTR(minterm));
}

double ADDFunction::countPaths(void) const {
  if(!rootNode) return 0.0;

  map<ADDNode*,double> cache;

  return countPaths(rootNode,&cache);
}

ADDFunction ADDFunction::getPath(void) const {
  return ADDFunction::BDD->getPath(*this);
}

void ADDFunction::getPath(char* buffer) const {
  ADDFunction::BDD->getPath(*this,buffer);
}

//****************************private**************************
ADDFunction::ADDFunction(const ADDNode* rootNode){
  this->rootNode=(ADDNode*) rootNode;

  addFunction(this->rootNode);
}

void ADDFunction::printPaths(const unsigned long lastLevel, const ADDNode* node, char* minterm) const {
  unsigned long i;

  if(!node->isDrain()){
    for(i=lastLevel;i<node->getLevel();++i) minterm[i]='-';
    minterm[node->getLevel()]='0';
    printPaths(node->getLevel()+1,node->getSucc0(),minterm);
    minterm[node->getLevel()]='1';
    printPaths(node->getLevel()+1,node->getSucc1(),minterm);
  } else {
    if(node->getValue()!=0.0){
      for(i=lastLevel;i<ADDVarOrder::size();++i) minterm[i]='-';
      for(i=0;i<ADDVarOrder::size();++i) cout << minterm[i];
      cout << endl;
    }
  }
}

double ADDFunction::countPaths(const ADDNode* node, map<ADDNode*,double>* cache) const {
  if(!node->isDrain()){
    double ret=0.0;
    if(cache->find((ADDNode*) node)==cache->end()){
      ret=countPaths(node->getSucc0(),cache);
      ret+=countPaths(node->getSucc1(),cache);
    } else return (*cache)[(ADDNode*) node];

    (*cache)[(ADDNode*) node]=ret;

    return ret;
  } else return (node->getValue()!=0.0)?1.0:0.0;
}

//********************************non class functions********************
ADDFunction operator+(const double value, const ADDFunction& function){
  return function+value;
}

ADDFunction operator-(const double value, const ADDFunction& function){
  return ADDFunction(value)-function;
}

ADDFunction operator*(const double value, const ADDFunction& function){
  return function*value;
}

ADDFunction operator-(const ADDFunction& function){
  ADD add;
  return add.negate(function);
}

ADDFunction EQUIV(const ADDFunction& function1, const ADDFunction& function2){
  return !(function1^function2);
}

ADDFunction IMPLIES(const ADDFunction& function1, const ADDFunction& function2){
  return !function1|function2;
}

ADDFunction ITE(const ADDFunction& function1, const ADDFunction& function2, const ADDFunction& function3){
  ADD add;
  return add.ITE(function1,function2,function3);
}

ADDFunction LESS(const ADDFunction& function1, const ADDFunction& function2){
  ADD add;
  return add.less(function1,function2);
}

ADDFunction LESS_EQUAL(const ADDFunction& function1, const ADDFunction& function2){
  ADD add;
  return add.lessEqual(function1,function2);
}

ADDFunction EQUAL(const ADDFunction& function1, const ADDFunction& function2){
  ADD add;
  return add.equal(function1,function2);
}

ADDFunction GREATER(const ADDFunction& function1, const ADDFunction& function2){
  ADD add;
  return add.less(function2,function1);
}

ADDFunction GREATER_EQUAL(const ADDFunction& function1, const ADDFunction& function2){
  ADD add;
  return add.lessEqual(function2,function1);
}

ADDFunction MAXIMUM(const ADDFunction& function1, const ADDFunction& function2){
  ADD add;
  return add.max(function1,function2);
}

ADDFunction MINIMUM(const ADDFunction& function1, const ADDFunction& function2){
  ADD add;
  return add.min(function1,function2);
}

ADDFunction MM(const ADDFunction& A, const ADDFunction& b, const ADDFunction& varSet){
  ADD add;
  return add.MM(A,b,varSet);
}

ADDFunction MV(const ADDFunction& A, const ADDFunction& b, const ADDFunction& varSet){
  ADD add;
  return add.MV(A,b,varSet);
}

ADDFunction VM(const ADDFunction& b, const ADDFunction& A, const ADDFunction& varSet){
  ADD add;
  return add.VM(b,A,varSet);
}
