/***************************************************************************
                          addgroup.cpp  -  description
                             -------------------
    begin                : Wed Apr 14 2004
    copyright            : (C) 2004 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "add.h"
#include "addnode.h"
#include "addgroup.h"
#include "addvarorder.h"
#include "../common/varorder.h"
#include "../common/constants.h"
#include "../common/reordering.h"

#include <climits>

using namespace std;

ADDVariableOrdering* ADDGroup::varOrder=0;
ADD* ADDGroup::BDD=0;

ADDGroup::ADDGroup(void){
  group=new Group<ADDUniqueTable>(*varOrder);
  group->incRefCount();
}

ADDGroup::ADDGroup(const string& groupName){
  group=varOrder->getGroup(groupName);
  if(!group){
    group=new Group<ADDUniqueTable>(*varOrder);
    if(groupName.find_first_of(badChars)==string::npos) group->setName(groupName);
  }
  group->incRefCount();
}

ADDGroup::ADDGroup(const ADDGroup& newGroup){
  group=newGroup.group;
  group->incRefCount();
}

ADDGroup::~ADDGroup(void){
  group->decRefCount();
  if(!group->getFirst()) varOrder->removeGroup(group->getName());
}

string ADDGroup::getName(void) const {
  return group->getName();
}

bool ADDGroup::setName(const string& name){
  return (name.find_first_of(badChars)==string::npos)?group->setName(name):false;
}

string ADDGroup::getGroupOrder(void){
  return group->getVariableOrdering();
}

bool ADDGroup::groupVariable(const string& variable){
  Variable<ADDUniqueTable>* tempVariable=varOrder->getVariable(variable);

  //check if variable exists
  if(!tempVariable) return false;
  //check if variable has a group
  if(tempVariable->getGroup()) return (tempVariable->getGroup()==group);

  //move variable to the edge of the group and group it
  if(group->getFirst()){
    unsigned long index=tempVariable->getLevel();
    //minimize swaps for variable
    if(index+1<group->getFirst()->getLevel()){
      unsigned long tempDelay=ADD::setGarbageCollectionDelay(0);
      BDD->reordering->shift(index,group->getFirst()->getLevel()-1);
      ADD::setGarbageCollectionDelay(tempDelay);
    } else if(index>group->getLast()->getLevel()+1){
      unsigned long tempDelay=ADD::setGarbageCollectionDelay(0);
      BDD->reordering->shift(index,group->getLast()->getLevel()+1);
      ADD::setGarbageCollectionDelay(tempDelay);
    }
  }
  group->group(variable);

  return true;
}

void ADDGroup::join(ADDGroup& group){
  //join two groups
  if(this->group!=group.group){
    string tempVariable;
    //while group has elements
    while(group.size()){
      //get first variable
      tempVariable=group[0];
      //ungroup variable
      group.ungroupVariable(tempVariable);
      //group it to this group
      groupVariable(tempVariable);
    }
  }
}

bool ADDGroup::ungroupVariable(const string& variable){
  Variable<ADDUniqueTable>* tempVariable=varOrder->getVariable(variable);

  //check if variable exists
  if(!tempVariable) return false;
  //check if variable is inside this group
  if(tempVariable->getGroup()!=group) return false;
  //if variable is on the edge of this group ungroup can to the work
  if((tempVariable==group->getFirst()) || (tempVariable==group->getLast())) return group->ungroup(variable);

  //prepare reordering
  unsigned long tempDelay=ADD::setGarbageCollectionDelay(0);
  unsigned long index=tempVariable->getLevel();
  //swap variable to the edge of the group and ungroup it
  if((index-group->getFirst()->getLevel())<(group->getLast()->getLevel()-index)) BDD->reordering->shift(index,group->getFirst()->getLevel());
  else BDD->reordering->shift(index,group->getLast()->getLevel());
  group->ungroup(variable);
  ADD::setGarbageCollectionDelay(tempDelay);

  return true;
}

void ADDGroup::ungroup(void){
  group->ungroup();
}

bool ADDGroup::addVariable(const string& variable){
  if(variable.find_first_of(badChars)!=string::npos) return false;
  else if(!varOrder->getVariable(variable)) return group->addVariable(variable);
  else return groupVariable(variable);
}

bool ADDGroup::removeVariable(const string& variable){
  return group->removeVariable(variable);
}

unsigned long ADDGroup::size(void) const {
  return (!group->getFirst())?0:(group->getLast()->getLevel()-group->getFirst()->getLevel()+1);
}

unsigned long ADDGroup::getIndexOfFirstElement(void) const {
  return (!group->getFirst())?ULONG_MAX:group->getFirst()->getLevel();
}

string ADDGroup::operator[](unsigned long index) const {
  return (index>=size())?"[ERROR]":varOrder->getVariable(group->getFirst()->getLevel()+index)->getName();
}

ADDGroup& ADDGroup::operator<<(const string& variable){
  addVariable(variable);

  return *this;
}

ADDGroup& ADDGroup::operator<<(ADDGroup& group){
  join(group);

  return *this;
}

ADDGroup& ADDGroup::operator>>(const string& variable){
  ungroupVariable(variable);

  return *this;
}

ADDGroup& ADDGroup::operator=(const ADDGroup& newGroup){
  if(group!=newGroup.group){
    group->decRefCount();
    group=newGroup.group;
    group->incRefCount();
  }

  return *this;
}

bool ADDGroup::operator==(const ADDGroup& group) const {
  return (group.group==this->group);
}

bool ADDGroup::operator!=(const ADDGroup& group) const {
  return (group.group!=this->group);
}

ADDGroup::ADDGroup(Group<ADDUniqueTable>* newGroup){
  group=newGroup?newGroup:new Group<ADDUniqueTable>(*varOrder);
  group->incRefCount();
}
