/***************************************************************************
                          addvarorder.cpp  -  description
                             -------------------
    begin                : Wed Apr 14 2004
    copyright            : (C) 2004 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "add/add.h"
#include "add/addnode.h"
#include "add/addgroup.h"
#include "add/addvarorder.h"
#include "../common/varorder.h"
#include "../common/constants.h"

#include <climits>

using namespace std;

ADDVariableOrdering* ADDVarOrder::varOrder=0;

void ADDVarOrder::clear(void){
  ADD::garbageCollect();
  varOrder->clear();
}

void ADDVarOrder::clearGroups(void){
  varOrder->clearGroups();
}

unsigned long ADDVarOrder::size(void){
  return varOrder->size();
}

string ADDVarOrder::getVarOrder(void){
  return varOrder->getVariableOrdering();
}

string ADDVarOrder::getVariable(const unsigned long index){
  return (*varOrder)[index];
}

bool ADDVarOrder::insertVariable(const string& label, const unsigned long index){
  return (label.find_first_of(badChars)!=string::npos)?false:varOrder->insertVariable(label,index);
}

bool ADDVarOrder::findVariable(const string& variable){
	return (varOrder->findVariable(variable));
}

bool ADDVarOrder::addVariable(const string& label){
  return (label.find_first_of(badChars)!=string::npos)?false:varOrder->appendVariable(label);
}

bool ADDVarOrder::prependVariable(const string& label){
  return (label.find_first_of(badChars)!=string::npos)?false:varOrder->prependVariable(label);
}

bool ADDVarOrder::removeVariable(const string& label){
  ADD::garbageCollect();

  return varOrder->removeVariable(label);
}

unsigned long ADDVarOrder::getVariable(const string& variable){
  Variable<ADDUniqueTable>* var=varOrder->getVariable(variable);
  return (var==0)?ULONG_MAX:var->getLevel();
}

bool ADDVarOrder::isGroupMember(const unsigned long index){
  return (index>=size())?false:varOrder->getVariable(index)->getGroup();
}

unsigned long ADDVarOrder::groupCount(void){
  return varOrder->groupCount();
}

ADDGroup ADDVarOrder::getGroup(const string& name){
  return ADDGroup(name);
}

ADDGroup ADDVarOrder::getGroup(const unsigned long index){
  Group<ADDUniqueTable>* tempGroupPtr=varOrder->getGroup(index);
  return tempGroupPtr?ADDVarOrder::getGroup(tempGroupPtr->getName()):ADDGroup();
}

ADDGroup ADDVarOrder::getNextGroup(const ADDGroup& group){
  return group.group?ADDGroup(varOrder->getNextGroup(group.group)):ADDGroup();
}

ADDGroup ADDVarOrder::getPrevGroup(const ADDGroup& group){
  return group.group?ADDGroup(varOrder->getPrevGroup(group.group)):ADDGroup();
}

string ADDVarOrder::toString() {
  return varOrder->toString();
}
