#include "addwriter.h"
#include "addnode.h"
#include "addfunction.h"
#include "addgroup.h"
#include "addvarorder.h"

using namespace std;

bool ADDWriter::storeFunction(ostream& out, const ADDFunction& function){
  return ADDWriter::storeFunctions(out,ADDFunctionContainer(function));
}

bool ADDWriter::storeFunctions(ostream& out, const ADDFunctionContainer& container){
  ADDWriter output(out);
  return output.storeBDD(container);
}

ADDWriter::ADDWriter(ostream& out_) : out(out_) {
}

bool ADDWriter::storeBDD(const ADDFunctionContainer& container){
  unsigned long i;

  //empty container?
  if(container.size()==0) return false;

  // store variables
  for (unsigned int i=0;i<ADDVarOrder::size();i++) {
    out << "var = " << ADDVarOrder::getVariable(i) << "\n";
  }

  // store drains
  ids[ADD::zeroDrain] = 0;
  ids[ADD::oneDrain] = 1;

  for(i=0;i<container.size();++i) {
    storeBDD(container.at(i).rootNode);
  }

  for(i=0;i<container.size();++i) {
    out << "function = " << ids.find(container.at(i).rootNode)->second << endl;
  }

  return true;
}

unsigned long ADDWriter::storeBDD(ADDNode* node){
  id_map::iterator id = ids.find(node);
  if (id != ids.end()) {
    return id->second;
  }

  if (!node->isDrain()) {
    unsigned long succ0 = storeBDD(node->getSucc0());
    unsigned long succ1 = storeBDD(node->getSucc1());

    unsigned long id = ids.size();
    ids[node] = id;

    out << node->getLevel() << "," << succ0 << "," << succ1 << "\n"; 
    return id;
  } else {
    if (node == ADD::zeroDrain) {
      return 0;
    } else if (node == ADD::oneDrain) {
      return 1;
    } else {
      throw "Unknown drain!";
    }
  }
}
