/***************************************************************************
                          assignment.h  -  description
                             -------------------
    begin                : Tue Feb 10 2004
    copyright            : (C) 2004 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASSIGNMENT_H
#define ASSIGNMENT_H

#include <map>
#include <vector>
#include <string>

/**
 * \file assignment.h
 * \author Joern Ossowski
 */

struct AssignmentPair{
  std::string variable;
  bool value;
};

/**
 * \brief Assignments can store an array of variables and its assigned values.
 *
 * Assignment is similar to VarSet. It is a combination of set and vector. This means that the
 * complexity of operations on Assignments are the maximum of each one.
 * Assignment can make a lot of things easier. Lets assume that we want a binary counter.
 * We start with the initialization of the Assignment.
 * \code
 * Assignment a;
 * //this initializes all variables with 0
 * a["x0"];
 * a["x1"];
 * a["x2"];
 * \endcode
 * As in the case of VarSet the inserting order represents the order of the variables.
 * The start counting!!!
 * \code
 * for(int i=0;i<10;++i){
 *   //prints the binary encoding of i
 *   cout << i << "=" << a["x2"] << a["x1"] << a["x0"] << endl;
 *   //increase a
 *   ++a;
 * }
 * \endcode
 * The code above results in:
 * \verbatim
     0=000
     1=001
     2=010
     3=011
     4=100
     5=101
     6=110
     7=111
     8=000
     9=001 \endverbatim
 * There are several ways to handle Assignment. Another one to get the same results would be.
 * \code
 * for(int i=0;i<10;++i){
 *   //prints the binary encoding of i
 *   cout << i << "=" << a["x2"] << a["x1"] << a["x0"] << endl;
 *   //set a
 *   a=i;
 * }
 * \endcode
 * Note that the Assignment must have been initialized. Otherwise it is not clear how a number
 * should be encoded and which variable names should be used.
 */
class Assignment{
public:
  Assignment(void);
  Assignment(const Assignment&);
  ~Assignment(void);
  void clear(void);
  bool find(const std::string&) const;
	Assignment reverse(void) const;
  Assignment operator*(const Assignment&) const;
  Assignment operator+(const Assignment&) const;
  Assignment& operator=(const Assignment&);
  Assignment& operator=(const unsigned long);
  Assignment& operator!(void);
  AssignmentPair operator[](const unsigned long) const;
  bool& operator[](const std::string&);
  bool at(const std::string&) const;
  bool remove(const std::string&);
  unsigned long size(void) const;
private:
  std::map<std::string,bool> assignment;
  std::vector<std::string> stringVector;
};

Assignment& operator++(Assignment&);
Assignment& operator--(Assignment&);

#endif
