/***************************************************************************
                           constants.h  -  description
                             -------------------
    begin                : Thu Dec 8 2005
    copyright            :(C) 2005 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

#ifndef CONSTANTS_H
#define CONSTANTS_H

#include <string>

/**
 * \file constants.h
 * \brief This file is used to store all used constants.
 *
 * This file sets the standard values for many algorithms and contains
 * constants for the whole library.
 * With the help of ConfigReader it is possible to change parameter without
 * compiling.
 * \sa ConfigReader
 * \author Joern Ossowski
 */


//heuristic constants
const short NONE_HEURISTIC=0;
const short FANIN_HEURISTIC=1;
const short WEIGHT_HEURISTIC=2;

//boolean constants
#define AND_OP              0x01
#define OR_OP               0x02
#define DIF_OP              0x03
#define ITE_OP              0x02
#define EXI_OP              0x04
#define COF_OP              0x06
#define CHA_OP              0x08

//artihmetic constants
#define ADD_OP              0x02
#define SUB_OP              0x04
#define MUL_OP              0x06
#define OVE_OP              0x08

//compare constants
#define LES_OP              0x02
#define LEE_OP              0x04
#define EQU_OP              0x06

//abstract constants
#define SUM_OP              0x02
#define PRO_OP              0x04
#define MAX_OP              0x06
#define MIN_OP              0x08

//matrix constants
#define MAV_OP              0x02
#define VMA_OP              0x04
#define MMA_OP              0x06

//move constants
#define MUP_OP              0x02
#define MDO_OP              0x04

//special constants
#define EQI_OP              0x02
#define CON_OP              0x04

//reordering constants
const double MAX_GROWTH_DEFAULT=1.3;
const unsigned long REORDERING_THRESHOLD=1000;
//constants for genetic minimize
const unsigned long POPULATION_SIZE=8;
const float HYBRID_SIFTING_MAX_GROWTH=1.0;
const unsigned long ELITARISM_WINNERS=(POPULATION_SIZE/5>0?POPULATION_SIZE/5:1);
const unsigned long RANDOM_REPLACEMENTS=POPULATION_SIZE/10;
const unsigned long MATING_POOL_SIZE=POPULATION_SIZE*2;
const unsigned long ROULETTE_WHEEL_SELECTION=0;
const unsigned long STOCHASTIC_REMAINDER_SELECTION=1;
const unsigned long DEFAULT_SELECTION=STOCHASTIC_REMAINDER_SELECTION;
const unsigned long ORDERED_CROSSOVER=0;
const unsigned long INVERTED_CROSSOVER=1;
const unsigned long PARTIALLY_MATCHED_CROSSOVER=2;
const unsigned long CYCLE_CROSSOVER=3;
const unsigned long ALTERNATING_CROSSOVER=4;
const unsigned long CROSSOVER_COUNT=POPULATION_SIZE-ELITARISM_WINNERS-RANDOM_REPLACEMENTS;
const unsigned long ORDERED_CROSSOVER_COUNT=CROSSOVER_COUNT/5;
const unsigned long INVERTED_CROSSOVER_COUNT=0;
const unsigned long PARTIALLY_MATCHED_CROSSOVER_COUNT=CROSSOVER_COUNT/3;
const unsigned long ALTERNATING_CROSSOVER_COUNT=CROSSOVER_COUNT/3;
const unsigned long CYCLE_CROSSOVER_COUNT=CROSSOVER_COUNT-ORDERED_CROSSOVER_COUNT-INVERTED_CROSSOVER_COUNT-PARTIALLY_MATCHED_CROSSOVER_COUNT;


//memory management constants
const unsigned long GARBAGE_COLLECTION_DELAY_DEFAULT=1000000;

//computed table constants
const unsigned long INIT_HASHMAP_SIZE=262144;
const unsigned long MAX_HASHMAP_SIZE=1048576;
const double HASHMAP_RESIZE_RATIO=0.8;

//hash unique table constants
const unsigned int INIT_UNIQUE_TABLE_SIZE_DEFAULT=256;
const unsigned int MAX_UNIQUE_TABLE_SIZE_DEFAULT=131072;

//io constants
const short BINARY_FORMAT=0;
const short BDD_FORMAT=1;
const short OUTPUT_DEFAULT=BINARY_FORMAT;

/**
 * \brief Characters that cannot be used in variable or group names.
 *
 * Some BDD variants in this library are provided with a binary output and all
 * with a graph output. To make parsing much easier it is important to restrict the
 * variable and group names.
 */
const std::string badChars=",;=";

//FEVBDD and NADD constants
const double SCALE_THRESHOLD_VALUE=0.0000001;
const double TRANSLATION_THRESHOLD_VALUE=0.0000001;
const double PARAMETER_THRESHOLD_VALUE=0.0000001;

//matrix input constants
const short MTX_INPUT_FORMAT=0;
const short HB_INPUT_FORMAT=1;
const short MATRIX_INPUT_DEFAULT=MTX_INPUT_FORMAT;

#endif
