/***************************************************************************
                          matrixinput.h  -  description
                             -------------------
    begin                : Thu Jun 17 2004
    copyright            : (C) 2004 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MATRIXINPUT_H
#define MATRIXINPUT_H

#include <common/constants.h>
#include <common/stringhelper.h>
#include <common/logsize.h>

#include <cstdlib>
#include <string>
#include <fstream>
#include <cmath>
#include <iostream>
#include <map>

using namespace std;

struct MatrixProgressBar{
  unsigned long index;
  unsigned long size;
  unsigned long lineSize;
};

inline void printMatrixProgressBar(MatrixProgressBar& bar){
  string temp;
  string print;
  unsigned int digits;

  temp=string("")+bar.size;
  digits=temp.size();

  temp=string("")+bar.index;
  while(temp.size()<digits) temp=" "+temp;

  print="element: [";
  print+=temp;
  print+="/";
  print+=string("")+bar.size;
  print+=+"]";

  bar.lineSize=print.length();

  cerr << print;
  cerr.flush();
}

inline void clearMatrixProgressBar(MatrixProgressBar& bar){
  cerr << "\r";
  while(bar.lineSize){
    cerr << " ";
    --bar.lineSize;
  }
  cerr << "\r";
  cerr.flush();
}

template <class T1, class T2>
class MatrixInput{
public:
  static T1 loadMatrix(const string&, const unsigned short=MATRIX_INPUT_DEFAULT, const bool=false);
private:
  ifstream file;

  MatrixInput(void);
  ~MatrixInput(void);
  T1 loadMTXMatrix(const string&,const bool=false);
  string readLine(void);
};

template <class T1, class T2>
T1 MatrixInput<T1,T2>::loadMatrix(const string& filename, const unsigned short format, const bool fastMode){
  MatrixInput temp;
  switch(format){
    case MTX_INPUT_FORMAT: return temp.loadMTXMatrix(filename,fastMode);
    case HB_INPUT_FORMAT:
    default: return T1::ERROR_FUNCTION();
  }
}

template <class T1, class T2>
MatrixInput<T1,T2>::MatrixInput(void){
}

template <class T1, class T2>
MatrixInput<T1,T2>::~MatrixInput(void){
}

template <class T1, class T2>
T1 MatrixInput<T1,T2>::loadMTXMatrix(const string& filename, const bool fastMode){
  file.open(filename.c_str(),ios::in);
  if(!file) return T1::ERROR_FUNCTION();

  char* temp;
  string line;
  unsigned long i,k;
  double n,m;
  line=readLine();
  n=double(atol(popToken(line).c_str()));
  m=double(atol(popToken(line).c_str()));
  k=atol(popToken(line).c_str());

  Assignment x;
  Assignment z;
  for(i=0;(i<logSize(n))||(i<logSize(m));++i){
    if(i<logSize(n)){
      x[string("x")+i]=0;
      T2::addVariable(string("x")+i);
    }
    if(i<logSize(m)){
      z[string("z")+i]=0;
      T2::addVariable(string("z")+i);
    }
  }
  T1 ret=0.0;
  MatrixProgressBar bar;
  bar.size=k;
  bar.index=1;
  printMatrixProgressBar(bar);
  double value;
  map<double,T1> functionMap;
  for(i=0;i<k;++i){
    if(i%100==99){
      bar.index=i+1;
      clearMatrixProgressBar(bar);
      printMatrixProgressBar(bar);
    }
    line=readLine();
    x=atol(popToken(line).c_str())-1;
    z=atol(popToken(line).c_str())-1;
    value=(line.length()?strtod(line.c_str(),&temp):1.0);
    if(fastMode){
      if(functionMap.find(value)!=functionMap.end()) functionMap[value]|=T1(x)*T1(z);
      else functionMap[value]=T1(x+z);
    } else ret+=T1(x+z)*value;
  }
  if(fastMode){
    typedef typename map<double,T1>::iterator functionMapIt;
    functionMapIt it=functionMap.begin();
    while(it!=functionMap.end()){
      ret+=(it->second)*(it->first);
      ++it;
    }
  }
  clearMatrixProgressBar(bar);

  file.close();
  return ret;
}

template <class T1, class T2>
string MatrixInput<T1,T2>::readLine(void){
  string line;
  T1 r;
  while(!file.eof()){
    getline(file,line);
    //is line a comment?
    if(line.find("%")!=string::npos) line=line.substr(0,line.find("%"));
    line=trim(line);

    if(line.length()>0) return line;
  }

  return "";
}

#endif
