/***************************************************************************
                          ptrhashmap.cpp  -  description
                             -------------------
    begin                : Wed Mar 16 2005
    copyright            : (C) 2005 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ptrhashmap.h"
#include <iostream>
#include <string>
#include <cstring>

using namespace std;

//**********************PtrHashMap2*******************
PtrHashMap2::PtrHashMap2(const unsigned long hashValues, const unsigned long maxHashValues, const double resizeR){
  resizes=0;
  maxRatio=0.0;
  resizeRatio=(unsigned short)(100.0*resizeR);
  numberOfHashValues=2;
  //shift for smallest hashmap
  shift=sizeof(unsigned long)*8-1;
  while((numberOfHashValues<<1)<=hashValues){
    numberOfHashValues<<=1;
    --shift;
  }
  maxNumberOfHashValues=maxHashValues;
  counter=0;

  hashMap=new struct PointerTupel2[numberOfHashValues];
  //init hashmap
  memset(hashMap,0,numberOfHashValues*sizeof(PointerTupel2));
}

PtrHashMap2::~PtrHashMap2(void){
  delete[] hashMap;
}

void PtrHashMap2::clear(void){
  double tempRatio=100.0*double(counter)/double(numberOfHashValues);
  if(tempRatio>maxRatio) maxRatio=tempRatio;

  memset(hashMap,0,numberOfHashValues*sizeof(PointerTupel2));

  counter=0;
}

unsigned long PtrHashMap2::count(void) const {
  return counter;
}

bool PtrHashMap2::contains(const void* key1, const void* key2) const {
  return find(key1,key2);
}

void* PtrHashMap2::find(const void* key1, const void* key2) const {
  const struct PointerTupel2* p=hashMap+hashFunction(key1,key2);
  if((p->key1==key1)&&(p->key2==key2)) return p->content;

  return 0;
}

void PtrHashMap2::insert(const void* key1, const void* key2, const void* content){
  struct PointerTupel2* p=hashMap+hashFunction(key1,key2);

  //insert keys
  p->key1=(void*) key1;
  p->key2=(void*) key2;

  //if there is no item increase counter
  if(p->content==0){
    //insert item
    p->content=(void*) content;

    ++counter;
    if((numberOfHashValues<maxNumberOfHashValues)&&(100*counter>numberOfHashValues*resizeRatio)) resize();
  } else {
    //insert item
    p->content=(void*) content;
  }
}

unsigned long PtrHashMap2::hashFunction(const void* key1, const void* key2) const {
  return ((((unsigned long) key1*12582917+(unsigned long) key2)*4256249)>>shift);
}

void PtrHashMap2::resize(void){
  maxRatio=100.0*double(counter)/double(2*numberOfHashValues);

  unsigned long oldSize=numberOfHashValues;
  numberOfHashValues<<=1;
  --shift;

  struct PointerTupel2* p1,* p2;
  struct PointerTupel2* tempHashMap=new PointerTupel2[numberOfHashValues];
  //out of memory
  if(tempHashMap==0){
    ++shift;
    numberOfHashValues=oldSize;
    maxNumberOfHashValues=numberOfHashValues;
    return;
  }
  memset(tempHashMap,0,numberOfHashValues*sizeof(PointerTupel2));
  ++resizes;

  p1=hashMap;
  for(unsigned long i=0;i<oldSize;++i,++p1){
    if(p1->content!=0){
      p2=tempHashMap+hashFunction(p1->key1,p1->key2);

      //copy struct
      (*p2)=(*p1);
    }
  }

  delete[] hashMap;
  hashMap=tempHashMap;
}

void PtrHashMap2::printStat(const string& name) const {
  double tempRatio=100.0*double(counter)/double(numberOfHashValues);
  if(tempRatio>maxRatio) maxRatio=tempRatio;
  
  //do not print debug information for not used hash map
  if(maxRatio<=0.000001) return;
	
  cout << "####################################" << endl;
  cout << "Name             : " << name << endl;	
  cout << "Number of Resizes: " << resizes << endl;	
  cout << "Max Fill Ratio   : " << maxRatio << endl;	
  cout << "Size             : " << numberOfHashValues << endl;	
  cout << "Max Size         : " << maxNumberOfHashValues << endl;	
  cout << "####################################" << endl;
}

//**********************PtrHashMap3*******************
PtrHashMap3::PtrHashMap3(const unsigned long hashValues, const unsigned long maxHashValues, const double resizeR){
  resizes=0;
  maxRatio=0.0;
  resizeRatio=(unsigned short)(100.0*resizeR);
  numberOfHashValues=2;
  //shift for smallest hashmap
  shift=sizeof(unsigned long)*8-1;
  while((numberOfHashValues<<1)<=hashValues){
    numberOfHashValues<<=1;
    --shift;
  }
  maxNumberOfHashValues=maxHashValues;
  counter=0;
	
  hashMap=new struct PointerTupel3[numberOfHashValues];
  //init hashmap
  memset(hashMap,0,numberOfHashValues*sizeof(PointerTupel3));
}

PtrHashMap3::~PtrHashMap3(void){
  delete[] hashMap;
}

void PtrHashMap3::clear(void){
  double tempRatio=100.0*double(counter)/double(numberOfHashValues);
  if(tempRatio>maxRatio) maxRatio=tempRatio;
  
  memset(hashMap,0,numberOfHashValues*sizeof(PointerTupel3));
	
  counter=0;
}

unsigned long PtrHashMap3::count(void) const {
  return counter;
}

bool PtrHashMap3::contains(const void* key1, const void* key2, const void* key3) const {
  return find(key1,key2,key3);
}

bool PtrHashMap3::contains(const void* key1, const void* key2) const {
  return find(key1,key2);
}

void* PtrHashMap3::find(const void* key1, const void* key2, const void* key3) const {
  struct PointerTupel3* p=hashMap+hashFunction(key1,key2,key3);
  if((p->key1==key1)&&(p->key2==key2)&&(p->key3==key3)) return p->content;
	
  return 0;
}

void* PtrHashMap3::find(const void* key1, const void* key2) const {
  struct PointerTupel3* p=hashMap+hashFunction(key1,key2);
  if((p->key1==key1)&&(p->key2==key2)&&(p->key3==0)) return p->content;
	
  return 0;
}

void PtrHashMap3::insert(const void* key1, const void* key2, const void* key3, const void* content){
  struct PointerTupel3* p=hashMap+hashFunction(key1,key2,key3);
	
  //insert keys
  p->key1=(void*) key1;
  p->key2=(void*) key2;
  p->key3=(void*) key3;

  //if there is no item increase counter
  if(p->content==0){
    //insert item
    p->content=(void*) content;

    ++counter;
    if((numberOfHashValues<maxNumberOfHashValues)&&(100*counter>numberOfHashValues*resizeRatio)) resize();
  } else {
    //insert item
    p->content=(void*) content;
  }
}

void PtrHashMap3::insert(const void* key1, const void* key2, const void* content){
  struct PointerTupel3* p=hashMap+hashFunction(key1,key2);
	
  //insert keys
  p->key1=(void*) key1;
  p->key2=(void*) key2;
  p->key3=(void*) 0;
  //if there is no item increase counter
  if(p->content==0){
    //insert item
    p->content=(void*) content;
    
    ++counter;
    if((numberOfHashValues<maxNumberOfHashValues)&&(100*counter>numberOfHashValues*resizeRatio)) resize();
  } else {
    //insert item
    p->content=(void*) content;
  }
}

unsigned long PtrHashMap3::hashFunction(const void* key1, const void* key2, const void* key3) const {
  return (((((unsigned long) key1*12582917+(unsigned long) key2)*4256249+(unsigned long) key3)*741457)>>shift);
}

unsigned long PtrHashMap3::hashFunction(const void* key1, const void* key2) const {
  return (((((unsigned long) key1*12582917+(unsigned long) key2)*4256249)*741457)>>shift);
}

void PtrHashMap3::resize(void){
  maxRatio=100.0*double(counter)/double(2*numberOfHashValues);
	
  unsigned long oldSize=numberOfHashValues;
  numberOfHashValues<<=1;
  --shift;
	
  struct PointerTupel3* p1,* p2;
  struct PointerTupel3* tempHashMap=new PointerTupel3[numberOfHashValues];
  //out of memory
  if(tempHashMap==0){
    ++shift;
    numberOfHashValues=oldSize;
    maxNumberOfHashValues=numberOfHashValues;
    return;
  }
  memset(tempHashMap,0,numberOfHashValues*sizeof(PointerTupel3));
  ++resizes;
  
  p1=hashMap;
  for(unsigned long i=0;i<oldSize;++i,++p1){
    if(p1->content!=0){
      p2=tempHashMap+hashFunction(p1->key1,p1->key2,p1->key3);
      
      //copy struct
      (*p2)=(*p1);
    }
  }
	
  delete[] hashMap;
  hashMap=tempHashMap;
}

void PtrHashMap3::printStat(const string& name) const {
  double tempRatio=100.0*double(counter)/double(numberOfHashValues);
  if(tempRatio>maxRatio) maxRatio=tempRatio;
  
  //do not print debug information for not used hash map
  if(maxRatio<=0.000001) return;
	
  cout << "####################################" << endl;
  cout << "Name             : " << name << endl;	
  cout << "Number of Resizes: " << resizes << endl;	
  cout << "Max Fill Ratio   : " << maxRatio << endl;	
  cout << "Size             : " << numberOfHashValues << endl;	
  cout << "Max Size         : " << maxNumberOfHashValues << endl;	
  cout << "####################################" << endl;
}

//**********************PtrHashMap4*******************
PtrHashMap4::PtrHashMap4(const unsigned long hashValues, const unsigned long maxHashValues, const double resizeR){
  resizes=0;
  maxRatio=0.0;
  resizeRatio=(unsigned short)(100.0*resizeR);
  numberOfHashValues=2;
  //shift for smallest hashmap
  shift=sizeof(unsigned long)*8-1;
  while((numberOfHashValues<<1)<=hashValues){
    numberOfHashValues<<=1;
    --shift;
  }
  maxNumberOfHashValues=maxHashValues;
  counter=0;
	
  hashMap=new struct PointerTupel4[numberOfHashValues];
  //init hashmap
  memset(hashMap,0,numberOfHashValues*sizeof(PointerTupel4));
}

PtrHashMap4::~PtrHashMap4(void){
  delete[] hashMap;
}

void PtrHashMap4::clear(void){
  double tempRatio=100.0*double(counter)/double(numberOfHashValues);
  if(tempRatio>maxRatio) maxRatio=tempRatio;
	
  memset(hashMap,0,numberOfHashValues*sizeof(PointerTupel4));
  
  counter=0;
}

unsigned long PtrHashMap4::count(void) const {
  return counter;
}

bool PtrHashMap4::contains(const void* key1, const void* key2, const void* key3, const void* key4) const {
  return find(key1,key2,key3,key4);
}

void* PtrHashMap4::find(const void* key1, const void* key2, const void* key3, const void* key4) const {
  struct PointerTupel4* p=hashMap+hashFunction(key1,key2,key3,key4);
  if((p->key1==key1)&&(p->key2==key2)&&(p->key3==key3)&&(p->key4==key4)) return p->content;
	
  return 0;
}

void PtrHashMap4::insert(const void* key1, const void* key2, const void* key3, const void* key4, const void* content){
  struct PointerTupel4* p=hashMap+hashFunction(key1,key2,key3,key4);
	
  //insert keys
  p->key1=(void*) key1;
  p->key2=(void*) key2;
  p->key3=(void*) key3;
  p->key4=(void*) key4;

  //if there is no item increase counter
  if(p->content==0){
    //insert item
    p->content=(void*) content;

    ++counter;
    if((numberOfHashValues<maxNumberOfHashValues)&&(100*counter>numberOfHashValues*resizeRatio)) resize();
  } else {
    //insert item
    p->content=(void*) content;
  }
}

unsigned long PtrHashMap4::hashFunction(const void* key1, const void* key2, const void* key3, const void* key4) const {
  return ((((((unsigned long) key1*12582917+(unsigned long) key2)*4256249+(unsigned long) key3)*741457)+(unsigned long) key4)>>shift);
}

void PtrHashMap4::resize(void){
  maxRatio=100.0*double(counter)/double(2*numberOfHashValues);
	
  unsigned long oldSize=numberOfHashValues;
  numberOfHashValues<<=1;
  --shift;
	
  struct PointerTupel4* p1,* p2;
  struct PointerTupel4* tempHashMap=new PointerTupel4[numberOfHashValues];
  //out of memory
  if(tempHashMap==0){
    ++shift;
    numberOfHashValues=oldSize;
    maxNumberOfHashValues=numberOfHashValues;
    return;
  }
  memset(tempHashMap,0,numberOfHashValues*sizeof(PointerTupel4));
  ++resizes;
  
  p1=hashMap;
  for(unsigned long i=0;i<oldSize;++i,++p1){
    if(p1->content!=0){
      p2=tempHashMap+hashFunction(p1->key1,p1->key2,p1->key3,p1->key4);
      
      //copy struct
      (*p2)=(*p1);
    }
  }
	
  delete[] hashMap;
  hashMap=tempHashMap;
}

void PtrHashMap4::printStat(const string& name) const {
  double tempRatio=100.0*double(counter)/double(numberOfHashValues);
  if(tempRatio>maxRatio) maxRatio=tempRatio;
  
  //do not print debug information for not used hash map
  if(maxRatio<=0.000001) return;
	
  cout << "####################################" << endl;
  cout << "Name             : " << name << endl;	
  cout << "Number of Resizes: " << resizes << endl;	
  cout << "Max Fill Ratio   : " << maxRatio << endl;	
  cout << "Size             : " << numberOfHashValues << endl;	
  cout << "Max Size         : " << maxNumberOfHashValues << endl;	
  cout << "####################################" << endl;
}

//**********************PtrHashMap5*******************
PtrHashMap5::PtrHashMap5(const unsigned long hashValues, const unsigned long maxHashValues, const double resizeR){
  resizes=0;
  maxRatio=0.0;
  resizeRatio=(unsigned short)(100.0*resizeR);
  numberOfHashValues=2;
  //shift for smallest hashmap
  shift=sizeof(unsigned long)*8-1;
  while((numberOfHashValues<<1)<=hashValues){
    numberOfHashValues<<=1;
    --shift;
  }
  maxNumberOfHashValues=maxHashValues;
  counter=0;
	
  hashMap=new struct PointerTupel5[numberOfHashValues];
  //init hashmap
  memset(hashMap,0,numberOfHashValues*sizeof(PointerTupel5));
}

PtrHashMap5::~PtrHashMap5(void){
  delete[] hashMap;
}

void PtrHashMap5::clear(void){
  double tempRatio=100.0*double(counter)/double(numberOfHashValues);
  if(tempRatio>maxRatio) maxRatio=tempRatio;
	
  memset(hashMap,0,numberOfHashValues*sizeof(PointerTupel5));
	
  counter=0;
}

unsigned long PtrHashMap5::count(void) const {
  return counter;
}

bool PtrHashMap5::contains(const void* key1, const void* key2, const void* key3, const void* key4, const void* key5) const {
  return find(key1,key2,key3,key4,key5);
}

void* PtrHashMap5::find(const void* key1, const void* key2, const void* key3, const void* key4, const void* key5) const {
  struct PointerTupel5* p=hashMap+hashFunction(key1,key2,key3,key4,key5);
  if((p->key1==key1)&&(p->key2==key2)&&(p->key3==key3)&&(p->key4==key4)&&(p->key5==key5)) return p->content;
	
  return 0;
}

void PtrHashMap5::insert(const void* key1, const void* key2, const void* key3, const void* key4, const void* key5, const void* content){
  struct PointerTupel5* p=hashMap+hashFunction(key1,key2,key3,key4,key5);
	
  //insert keys
  p->key1=(void*) key1;
  p->key2=(void*) key2;
  p->key3=(void*) key3;
  p->key4=(void*) key4;
  p->key5=(void*) key5;

  //if there is no item increase counter
  if(p->content==0){
    //insert item
    p->content=(void*) content;

    ++counter;
    if((numberOfHashValues<maxNumberOfHashValues)&&(100*counter>numberOfHashValues*resizeRatio)) resize();  
  } else {
    //insert item
    p->content=(void*) content;
  }
}

unsigned long PtrHashMap5::hashFunction(const void* key1, const void* key2, const void* key3, const void* key4, const void* key5) const {
  return ((((((((unsigned long) key1*12582917+(unsigned long) key2)*4256249+(unsigned long) key3)*741457)+(unsigned long) key4)*94573)+(unsigned long) key5)>>shift);
}

void PtrHashMap5::resize(void){
  maxRatio=100.0*double(counter)/double(2*numberOfHashValues);
	
  unsigned long oldSize=numberOfHashValues;
  numberOfHashValues<<=1;
  --shift;
	
  struct PointerTupel5* p1,* p2;
  struct PointerTupel5* tempHashMap=new PointerTupel5[numberOfHashValues];
  //out of memory
  if(tempHashMap==0){
    ++shift;
    numberOfHashValues=oldSize;
    maxNumberOfHashValues=numberOfHashValues;
    return;
  }
  memset(tempHashMap,0,numberOfHashValues*sizeof(PointerTupel5));
  ++resizes;
	
  p1=hashMap;
  for(unsigned long i=0;i<oldSize;++i,++p1){
    if(p1->content!=0){
      p2=tempHashMap+hashFunction(p1->key1,p1->key2,p1->key3,p1->key4,p1->key5);
	
      //copy struct
      (*p2)=(*p1);
    }
  }
	
  delete[] hashMap;
  hashMap=tempHashMap;
}

void PtrHashMap5::printStat(const string& name) const {
  double tempRatio=100.0*double(counter)/double(numberOfHashValues);
  if(tempRatio>maxRatio) maxRatio=tempRatio;
  
  //do not print debug information for not used hash map
  if(maxRatio<=0.000001) return;
  
  cout << "####################################" << endl;
  cout << "Name             : " << name << endl;	
  cout << "Number of Resizes: " << resizes << endl;	
  cout << "Max Fill Ratio   : " << maxRatio << endl;	
  cout << "Size             : " << numberOfHashValues << endl;	
  cout << "Max Size         : " << maxNumberOfHashValues << endl;	
  cout << "####################################" << endl;
}
