/***************************************************************************
                          ulongarray.cpp  -  description
                             -------------------
    begin                : Tue Apr 13 2004
    copyright            : (C) 2004 by Wolfgang Lenders, Joern Ossowski
    email                : {lenders,ossowski}@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ulongarray.h"
#include <common/vsizearray.h>

#include <climits>
#include <sstream>

using namespace std;

unsigned long ULongArray::ERROR=ULONG_MAX;

ULongArray::ULongArray(unsigned long n){
  array=new unsigned long[n];
  arraySize=n;
  rh=RandomHelper();
}

ULongArray::ULongArray(const ULongArray& array){
  this->array=new unsigned long[array.arraySize];
  this->arraySize=array.arraySize;
  for(unsigned long i=0;i<array.arraySize;++i) this->array[i]=array.array[i];
  rh=RandomHelper();
}

ULongArray::~ULongArray(void){
  delete[] array;
}

// ****** public *********************************************************
unsigned long& ULongArray::operator[](const unsigned long index){
  if(index>=arraySize) return ERROR;

  return array[index];
}

bool ULongArray::operator==(const ULongArray& array) const{
  if(arraySize!=array.size()) return false;
  for(unsigned long i=0; i<arraySize; ++i) {
    if(this->array[i]!=array.at(i)) return false;
  }
  return true;
}

ULongArray& ULongArray::operator=(const ULongArray& array){
  if(&array!=this){
    if(arraySize!=array.arraySize){
      delete[] this->array;
      this->array=new unsigned long[array.arraySize];
      arraySize=array.arraySize;
    }
    for(unsigned long i=0;i<arraySize;++i) this->array[i]=array.array[i];
  }

  return *this;
}

unsigned long ULongArray::at(const unsigned long index) const{
  if(index>=arraySize) return ERROR;
  return array[index];
}

string ULongArray::print() const{
  ostringstream printedArray;
  printedArray << "(";
  for (unsigned long i=0; i<arraySize; ++i){
    printedArray << array[i];
    if(i+1<arraySize) printedArray << ", ";
  }
  printedArray << ")";
  return printedArray.str();
}

unsigned long ULongArray::find(const unsigned long query) const {
  for(unsigned long i=0; i<arraySize; ++i) if(array[i]==query) return i;
  return ERROR;
}

// distance between two arrays is defined as number of swaps necessary to transform one into the other
unsigned long ULongArray::distance(const ULongArray& secondArray) const{
  unsigned long distance=0;
  unsigned long indexInTempArray;
  ULongArray tempArray(*this);
  //for(unsigned long i=0; i<arraySize; ++i) tempArray[i]=array[i];
  for(unsigned long i=0; i<arraySize; ++i){
    if(tempArray[i]!=secondArray.at(i)){
      indexInTempArray=tempArray.find(secondArray.at(i));
      distance+=indexInTempArray;
      tempArray.shift(indexInTempArray, i);
    }
  }
  return distance;
}

void ULongArray::swap(const unsigned long pos){
  exchange(pos,pos+1);
}

void ULongArray::shift(const unsigned long fromIndex, const unsigned long position){
  unsigned long index=fromIndex;
  if (index==position) return;
  unsigned long temp=array[index];
  if (index<position){ //jump down
    while(index<position) {
      array[index]=array[index+1];
      ++index;
    }
  } else {             //jump up
    while(index>position) {
      array[index]=array[index-1];
      --index;
    }
  }
  array[position]=temp;
}

void ULongArray::exchange(const unsigned long pos1, const unsigned long pos2){
  unsigned long temp=array[pos1];
  array[pos1]=array[pos2];
  array[pos2]=temp;
}

ULongArray ULongArray::getIndicesOrderedByValues() const {
  // returns an array containing the indices of the elements in this ULongArray in ascending order
  ULongArray indices(arraySize);
  VSIZEARRAY(bool,selected,arraySize);
  unsigned long i,highestValueIndex, highestValue;
  for(i=0; i<arraySize; ++i) selected[i]=false;
  for(i=0; i<arraySize; ++i){
    highestValueIndex=0;
    highestValue=0;
    for(unsigned long j=0; j<arraySize; ++j){
      if((array[j]>highestValue) && !selected[j]){
        highestValueIndex=j;
        highestValue=array[j];
      }
    }
    selected[highestValueIndex]=true;
    indices[i]=highestValueIndex;
  }
  return indices;
}

void ULongArray::randomShuffle(const unsigned long level){
  if (arraySize<2) return;
  unsigned long mutationCount=rh.getULong(level*arraySize/10+1);
  unsigned long firstPos, secondPos;
  for(unsigned long i=0; i<mutationCount; ++i){
    // switch two positions at random
    rh.getULongs(firstPos, secondPos, arraySize);
    exchange(firstPos,secondPos);
  }
}

unsigned long ULongArray::size() const{
  return arraySize;
}
