/***************************************************************************
                          zadd.h  -  description
                             -------------------
    begin                : Wed Apr 14 2004
    copyright            : (C) 2004 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ZADD_H
#define ZADD_H

#include <string>

class ZADDNode;
class ZADDGroup;
class ZADDVarOrder;
class PtrHashMap2;
class PtrHashMap3;
class PtrHashMap4;
class PtrHashMap5;

template <class T1, class T2, class T3> class Reordering;

#include <zadd/zaddfunction.h>
#include <zadd/zaddfunctioncontainer.h>
#include <zadd/zaddvarorder.h>
#include <zadd/zaddgroup.h>
#include <common/constants.h>
#include <common/assignment.h>

class ZADD{
friend class ZADDNode;
friend class ZADDFunction;
friend class ZADDGroup;
public:
  ZADD(void);
  ~ZADD(void);

  static bool addVariable(const std::string&);
  static bool clear(void);
  static void reset(void);
  static unsigned long size(void);
  static unsigned long countDeadNodes(void);
  static unsigned long PEAK(void);
  static void printStat(void);

  static ZADDFunction loadFunctionBLIF(const std::string&, const std::string&);
  static ZADDFunctionContainer loadFunctionsBLIF(const std::string&);
  static ZADDFunction loadMatrix(const std::string&, const unsigned short=MATRIX_INPUT_DEFAULT);
  static bool storeFunction(const std::string&, const ZADDFunction&);
  static bool storeFunctions(const std::string&, const ZADDFunctionContainer&);

  static void sifting(const float=MAX_GROWTH_DEFAULT);
  static void variableSifting(const float=MAX_GROWTH_DEFAULT);
  static void groupSifting(const float=MAX_GROWTH_DEFAULT);
  static void windowPermutation2(void);
  static void windowPermutation3(void);
  static void windowPermutation4(void);
  static void geneticMinimize(void);

  static void enableDynamicReordering(void);
  static void disableDynamicReordering(void);
  static void setDynamicReorderingMaxGrowth(float);
  static void setDynamicReorderingTimeout(unsigned int);
  static void setUniqueTableMaxLoadFactor(float load_factor);
  static void reorderingHook(bool forceReordering=false);
  
  static void garbageCollect(void);
  static unsigned long setGarbageCollectionDelay(void);
  static unsigned long setGarbageCollectionDelay(const unsigned long);

  ZADDFunction projection(const std::string&);
  ZADDFunction projection(const unsigned long);
  ZADDFunction negate(const ZADDFunction&);
  ZADDFunction addition(const ZADDFunction&, const ZADDFunction&);
  ZADDFunction sum(const ZADDFunction&, const ZADDFunction&);
  ZADDFunction subtraction(const ZADDFunction&, const ZADDFunction&);
  ZADDFunction multiplication(const ZADDFunction&, const ZADDFunction&);
  ZADDFunction product(const ZADDFunction&, const ZADDFunction&);
  ZADDFunction equivalenceClasses(const ZADDFunction&, const ZADDFunction&, const ZADDFunction&);
  ZADDFunction constrain(const ZADDFunction&, const ZADDFunction&);
  ZADDFunction MM(const ZADDFunction&, const ZADDFunction&, const ZADDFunction&);
  ZADDFunction MV(const ZADDFunction&, const ZADDFunction&, const ZADDFunction&);
  ZADDFunction VM(const ZADDFunction&, const ZADDFunction&, const ZADDFunction&);
  ZADDFunction ITE(const ZADDFunction&, const ZADDFunction&, const ZADDFunction&);
  ZADDFunction NOT(const ZADDFunction&);
  ZADDFunction AND(const ZADDFunction&, const ZADDFunction&);
  ZADDFunction OR(const ZADDFunction&, const ZADDFunction&);
  ZADDFunction cofactor(const ZADDFunction&, const unsigned long, const bool);
  ZADDFunction cofactor(const ZADDFunction&, const ZADDFunction&);
  ZADDFunction change(const ZADDFunction&);
  ZADDFunction change(const ZADDFunction&, const unsigned long);
  ZADDFunction change(const ZADDFunction&, const ZADDFunction&);
  ZADDFunction exists(const ZADDFunction&, const ZADDFunction&);
  ZADDFunction exists(const ZADDFunction&, const unsigned long);
  ZADDFunction forall(const ZADDFunction&, const ZADDFunction&);
  ZADDFunction forall(const ZADDFunction&, const unsigned long);
  ZADDFunction less(const ZADDFunction&, const ZADDFunction&);
  ZADDFunction lessEqual(const ZADDFunction&, const ZADDFunction&);
  ZADDFunction equal(const ZADDFunction&, const ZADDFunction&);
  ZADDFunction maximum(const ZADDFunction&, const ZADDFunction&);
  ZADDFunction max(const ZADDFunction&, const ZADDFunction&);
  double max(const ZADDFunction&);
  ZADDFunction minimum(const ZADDFunction&, const ZADDFunction&);
  ZADDFunction min(const ZADDFunction&, const ZADDFunction&);
  double min(const ZADDFunction&);
  ZADDFunction oneOver(const ZADDFunction&);
  ZADDFunction moveUp(const ZADDFunction&, const unsigned int=1);
  ZADDFunction moveUp(const ZADDFunction&, const ZADDFunction&, const unsigned int=1);
  ZADDFunction moveDown(const ZADDFunction&, const unsigned int=1);
  ZADDFunction moveDown(const ZADDFunction&, const ZADDFunction&, const unsigned int=1);
  ZADDFunction getAssignment(const ZADDFunction&);
  void getAssignment(const ZADDFunction&, char*) const;
  double countAssignments(const ZADDFunction&) const;
  ZADDFunction getPath(const ZADDFunction&);
  void getPath(const ZADDFunction&, char*) const;
protected:
  //basic reordering function
  bool swap(const unsigned long);

  //function for ZADDNode
  void deleteSubTree(ZADDNode*);
  void deleteNode(ZADDNode*);
private:
  static unsigned long deadNodes;
  static ZADDVariableOrdering variableOrdering;
  static PtrHashMap4 ArithmeticComputedTable;
  static PtrHashMap4 ExtremaComputedTable;
  static PtrHashMap3 AbstractComputedTable;
  static PtrHashMap4 CompareComputedTable;
  static PtrHashMap5 MatrixComputedTable;
  static PtrHashMap4 BoolComputedTable;
  static PtrHashMap3 MoveComputedTable;
  static PtrHashMap5 SpecialComputedTable;
  static Reordering<ZADDVarOrder,ZADDGroup,ZADD>* reordering;
  static ZADD* BDD;
  static unsigned long garbageCollectionDelay;
  static unsigned long reorderingThreshold;
  static unsigned long operatorCounter;
  static unsigned long lastReorderSize;
  static bool dynamicReordering;
  static float dynamicReorderingMaxGrowth;
  static unsigned int dynamicReorderingTimeout;
  static ZADDNode* zeroDrain;
  static ZADDNode* oneDrain;
  static ZADDNode* (ZADD::*currentAbstractFunction)(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*);
  static short currentAbstractFunctionID;
  static unsigned long swapcount;

  ZADDNode* calcUnion(ZADDNode*, ZADDNode*);
  ZADDNode* calcSetMinus(ZADDNode*, ZADDNode*);
  ZADDNode* calcAllVars(void);
  ZADDNode* calcLiftedBDD(ZADDNode*, ZADDNode*, ZADDNode*);
  bool equiv(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* abstract(ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* idempotentAbstract(ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* exists(ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* addition(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* addition(ZADDNode*, ZADDNode*);
  ZADDNode* subtraction(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* subtraction(ZADDNode*, ZADDNode*);
  ZADDNode* multiplication(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* multiplication(ZADDNode*, ZADDNode*);
  ZADDNode* constMultiplication(ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* MM(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*, unsigned long);
  ZADDNode* MV(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*, unsigned long);
  ZADDNode* VM(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*, unsigned long);
  ZADDNode* equivalenceClasses(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* constrain(ZADDNode*, ZADDNode*);
  ZADDNode* ITE(ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* AND(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* AND(ZADDNode*, ZADDNode*);
  ZADDNode* OR(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* OR(ZADDNode*, ZADDNode*);
  ZADDNode* diff(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* NOT(ZADDNode*, ZADDNode*);
  ZADDNode* cofactor(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* change(ZADDNode*, ZADDNode*);
  ZADDNode* change(ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* less(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* lessEqual(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* equal(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*);
  ZADDNode* max(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*);
  double max(ZADDNode*, map<ZADDNode*,double>&);
  ZADDNode* min(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*);
  double min(ZADDNode*, map<ZADDNode*,double>&);
  ZADDNode* oneOver(ZADDNode*);
  ZADDNode* moveUp(ZADDNode*);
  ZADDNode* moveUp(ZADDNode*, const unsigned int);
  ZADDNode* moveUp(ZADDNode*, ZADDNode*);
  ZADDNode* moveUp(ZADDNode*, ZADDNode*, const unsigned int);
  ZADDNode* moveDown(ZADDNode*);
  ZADDNode* moveDown(ZADDNode*, const unsigned int);
  ZADDNode* moveDown(ZADDNode*, ZADDNode*);
  ZADDNode* moveDown(ZADDNode*, ZADDNode*, const unsigned int);
  ZADDNode* getAssignment(const ZADDNode*);
  bool getAssignment(const ZADDNode*, Assignment&) const;
  bool getAssignment(const ZADDNode*, char*) const;
  double countAssignments(unsigned long, const ZADDNode*, const ZADDNode*, map<ZADDNode*,double>&) const;
  ZADDNode* getPath(const ZADDNode*);
  bool getPath(const ZADDNode*, char*) const;

  ZADDNode* createConstantFunction(const double);
  inline ZADDFunction createFunction(ZADDNode*, ZADDNode*);
  ZADDNode* findOrAdd(const unsigned long, ZADDNode*, ZADDNode*);
  inline ZADDNode* findOrAdd(ZADDUniqueTable*, ZADDNode*, ZADDNode*);
  ZADDNode* findOrAddDrain(const double);
  inline ZADDNode* findDrain(const double);
};
#endif
