/***************************************************************************
                          zaddnode.cpp  -  description
                             -------------------
    begin                : Wed Apr 14 2004
    copyright            : (C) 2004 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "zaddnode.h"
#include "../common/mempool.h"

#include <climits>

ZADD* ZADDNode::BDD=0;
MemPool<ZADDNode> ZADDNode::memPool=MemPool<ZADDNode>();

//************************ZADDNode***************************
//*************************public*****************************
ZADDNode::ZADDNode(void){
  preNodeCount=0;
  type.children.succ0=type.children.succ1=next=0;
  uniqueTable=0;
}

ZADDNode::~ZADDNode(void){
}

ZADDNode* ZADDNode::getNewNode(void){
	return memPool.alloc();
}

void ZADDNode::freeNode(ZADDNode* node){
	memPool.free(node);
}

unsigned long ZADDNode::PEAK(void){
	return memPool.peak();
}

bool ZADDNode::isDrain(void) const {
  return (uniqueTable->isDrainLevel());
}

ZADDNode* ZADDNode::getSucc(const bool value) const {
  return value?type.children.succ1:type.children.succ0;
}

void ZADDNode::setValue(double value){
  type.value=value;
}

double ZADDNode::getValue(void) const {
  return type.value;
}

void ZADDNode::initNode(ZADDUniqueTable* uTable, ZADDNode* s0, ZADDNode* s1){
  //there is no check if there was an unique table
  //this function should only be used in findOrAdd

  s0->incNodeCount();
  s1->incNodeCount();

  //update node information
  type.children.succ0=s0;
  type.children.succ1=s1;
  uniqueTable=uTable;
  
  preNodeCount=0;
}

void ZADDNode::setNode(ZADDUniqueTable* uTable, ZADDNode* s0, ZADDNode* s1){
  ZADDNode** succ0=&type.children.succ0;
  ZADDNode** succ1=&type.children.succ1;
  
  uniqueTable=uTable;
  ++s0->preNodeCount;
  ++s1->preNodeCount;

  if(--(*succ0)->preNodeCount==0) BDD->deleteNode(*succ0);
  if(--(*succ1)->preNodeCount==0) BDD->deleteNode(*succ1);
  (*succ0)=s0;
  (*succ1)=s1;
  
  //add entry to unique table
  uniqueTable->add(this);
}

void ZADDNode::removeFromUniqueTable(void){
  //remove entry from uniquetable
	uniqueTable->remove(this);
}

ZADDUniqueTable* ZADDNode::getUniqueTable(void) const {
  return uniqueTable;
}

void ZADDNode::garbageCollect(void){
	uniqueTable->incDeadNodeCount();
	++BDD->deadNodes;
  //set correct preNodeCount (zombie node has preNodeCount ULONG_MAX)
  preNodeCount=ULONG_MAX;
}

unsigned long ZADDNode::getLevel(void) const {
  //every node (terminal or inner) has an unique table with proper level information
  return uniqueTable->getLevel();
}

unsigned long ZADDNode::getRealLevel(void) const {
  //every node (terminal or inner) has an unique table with proper level information
  return uniqueTable->getRealLevel();
}

void ZADDNode::deleteNode(void){
  if(--preNodeCount==0){
		uniqueTable->incDeadNodeCount();
		++BDD->deadNodes;
    preNodeCount=ULONG_MAX;
		if(BDD->deadNodes>=BDD->garbageCollectionDelay) ZADD::garbageCollect();
  }
}

bool ZADDNode::operator<(const ZADDNode& node) const {
  return (this->getValue()+0.0000001)<node.getValue();
}
