/***************************************************************************
                          zaddoutput.cpp  -  description
                             -------------------
    begin                : Tue Jul 13 2004
    copyright            : (C) 2004 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "zaddoutput.h"
#include "zaddnode.h"
#include "zaddfunction.h"
#include "zaddgroup.h"
#include "zaddvarorder.h"

bool ZADDOutput::storeFunction(const string& filename, const ZADDFunction& function){
  return ZADDOutput::storeFunctions(filename,ZADDFunctionContainer(function));
}

bool ZADDOutput::storeFunctions(const string& filename, const ZADDFunctionContainer& container){
  ZADDOutput output(filename);
  return output.storeBDD(container);
}

ZADDOutput::ZADDOutput(const string& filename){
  file.open(filename.c_str(),ios::out);
}

ZADDOutput::~ZADDOutput(void){
  if(file) file.close();
}

bool ZADDOutput::storeBDD(const ZADDFunctionContainer& container){
  unsigned long i;

  //file open?
  if(!file) return false;
  //empty container?
  if(container.size()==0) return false;
  //don't know what to with ERROR functions
  for(i=0;i<container.size();++i) if(container.at(i)==ZADDFunction::ERROR_FUNCTION()) return false;

  for(i=0;i<container.size();++i){
    ZADDNode* node=container.at(i).rootNode;
    file << "rootnode {" << endl;
    file << "  id=" << node << endl;
    file << "}" << endl;
    storeBDD(node);
  }

  return true;
}

void ZADDOutput::storeBDD(ZADDNode* node){
  if(visitedSet.find(node)!=visitedSet.end()) return;
  if(!node->isDrain()){
    file << "node {" << endl;
    file << "  id=" << node << endl;
    file << "  level=" << node->getLevel() << endl;
    file << "  succ0=" << node->getSucc0() << endl;
    file << "  succ1=" << node->getSucc1() << endl;
    file << "  label=" << ZADDVarOrder::getVariable(node->getLevel()) << endl;
    file << "  drawtype=circle" << endl;
    file << "}" << endl;
    visitedSet.insert(node);
    storeBDD(node->getSucc0());
    storeBDD(node->getSucc1());
  } else {
    file << "node {" << endl;
    file << "  id=" << node << endl;
    file << "  level=" <<  ZADDVarOrder::size() << endl;
    file << "  label=" << node->getValue() << endl;
    file << "  drawtype=box" << endl;
    file << "}" << endl;
    visitedSet.insert(node);
  }
}
