/***************************************************************************
                          zaddvarorder.cpp  -  description
                             -------------------
    begin                : Wed Apr 14 2004
    copyright            : (C) 2004 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "zadd.h"
#include "zaddnode.h"
#include "zaddgroup.h"
#include "zaddvarorder.h"
#include "../common/varorder.h"
#include "../common/constants.h"

#include <climits>

ZADDVariableOrdering* ZADDVarOrder::varOrder=0;

void ZADDVarOrder::clear(void){
  ZADD::garbageCollect();
  varOrder->clear();
}

void ZADDVarOrder::clearGroups(void){
  varOrder->clearGroups();
}

unsigned long ZADDVarOrder::size(void){
  return varOrder->size();
}

string ZADDVarOrder::getVarOrder(void){
  return varOrder->getVariableOrdering();
}

string ZADDVarOrder::getVariable(const unsigned long index){
  return (*varOrder)[index];
}

bool ZADDVarOrder::insertVariable(const string& label, const unsigned long index){
  return (label.find_first_of(badChars)!=string::npos)?false:varOrder->insertVariable(label,index);
}

bool ZADDVarOrder::findVariable(const string& variable){
	return (varOrder->findVariable(variable));
}

bool ZADDVarOrder::addVariable(const string& label){
  return (label.find_first_of(badChars)!=string::npos)?false:varOrder->appendVariable(label);
}

bool ZADDVarOrder::prependVariable(const string& label){
  return (label.find_first_of(badChars)!=string::npos)?false:varOrder->prependVariable(label);
}

bool ZADDVarOrder::removeVariable(const string& label){
  ZADD::garbageCollect();

  return varOrder->removeVariable(label);
}

unsigned long ZADDVarOrder::getVariable(const string& variable){
  Variable<ZADDUniqueTable>* var=varOrder->getVariable(variable);
  return (var==0)?ULONG_MAX:var->getLevel();
}

bool ZADDVarOrder::isGroupMember(const unsigned long index){
  return (index>=size())?false:varOrder->getVariable(index)->getGroup();
}

unsigned long ZADDVarOrder::groupCount(void){
  return varOrder->groupCount();
}

ZADDGroup ZADDVarOrder::getGroup(const string& name){
  return ZADDGroup(name);
}

ZADDGroup ZADDVarOrder::getGroup(const unsigned long index){
  Group<ZADDUniqueTable>* tempGroupPtr=varOrder->getGroup(index);
  return tempGroupPtr?ZADDVarOrder::getGroup(tempGroupPtr->getName()):ZADDGroup();
}

ZADDGroup ZADDVarOrder::getNextGroup(const ZADDGroup& group){
  return group.group?ZADDGroup(varOrder->getNextGroup(group.group)):ZADDGroup();
}

ZADDGroup ZADDVarOrder::getPrevGroup(const ZADDGroup& group){
  return group.group?ZADDGroup(varOrder->getPrevGroup(group.group)):ZADDGroup();
}
