/***************************************************************************
                          add.cpp  -  description
                             -------------------
    begin                : Wed Apr 14 2004
    copyright            : (C) 2004-2008 by Joern Ossowski
    email                : mail@jossowski.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <add/add.h>
#include <add/addinnernode.h>
#include <add/addterminalnode.h>
#include <common/mempool.h>
#include <common/helper.h>
#include <common/ptrhashmap.h>
#include <common/reordering.h>
#include "vereofy/common/tracing.hpp"


ADDTerminalNode* createZeroDrain(void){
  ADDTerminalNode* node=ADDTerminalNode::getNewNode();
  node->setValue(0.0);
  node->incNodeCount();
  return node;
}
ADDTerminalNode* initAndReturnOneDrain(void){
  Group<ADDUniqueTable>::varOrder=&ADD::variableOrdering;
  ADDTerminalNode* node=ADD::drainUniqueTable.findOrAddValue(1.0);
  node->incNodeCount();
  return node;
}
void deleteSubTree(ADDBaseNode* node){
  if LIKELY(!node->isDrain()){
    ADDInnerNode* innerNode=static_cast<ADDInnerNode*>(node);

    if((--innerNode->getSucc0()->preNodeCount)==0) deleteSubTree(innerNode->getSucc0());
    if((--innerNode->getSucc1()->preNodeCount)==0) deleteSubTree(innerNode->getSucc1());

    //reset node
    innerNode->uniqueTable->remove(innerNode);

    //free mem for node
    ADDInnerNode::freeNode(innerNode);
  } else {
    ADDTerminalNode* terminalNode=static_cast<ADDTerminalNode*>(node);
    //reset node
    ADD::drainUniqueTable.remove(terminalNode);

    //free mem for node
    ADDTerminalNode::freeNode(terminalNode);
  }
}

ADDTerminalNodeUniqueTable ADD::drainUniqueTable;
MemPool<ADDInnerNode> ADD::innerNodeMemPool;
MemPool<ADDTerminalNode> ADD::terminalNodeMemPool;
ADDVariableOrdering ADD::variableOrdering;
ADDTerminalNode* ADD::zeroDrain=createZeroDrain();
ADDTerminalNode* ADD::oneDrain=initAndReturnOneDrain();
unsigned long ADD::deadNodes=0;
unsigned long ADD::garbageCollectionDelay=10000;
bool ADD::dynamicReordering=false;
bool ADD::dynamicReorderingReverse=false;
float ADD::dynamicReorderingMaxGrowth=MAX_GROWTH_DEFAULT;
unsigned int ADD::dynamicReorderingTimeout=0;
unsigned long ADD::lastReorderSize=0;
//TODO: config reader (also init hash unique table)
PtrHashMap3<ADDBaseNode> ADD::BoolComputedTable(1048576,1048576,0.8);
PtrHashMap3<ADDBaseNode> ADD::MoveComputedTable(1048576,1048576,0.8);
PtrHashMap2<ADDBaseNode> ADD::ArithmeticComputedTable(1048576,1048576,0.8);
PtrHashMap2<ADDBaseNode> ADD::AbstractComputedTable(1048576,1048576,0.8);
PtrHashMap2<ADDBaseNode> ADD::ExtremaComputedTable(1048576,1048576,0.8);
PtrHashMap2<ADDBaseNode> ADD::CompareComputedTable(1048576,1048576,0.8);
PtrHashMap3<ADDBaseNode> ADD::MatrixComputedTable(1048576,1048576,0.8);
PtrHashMap3<ADDBaseNode> ADD::SpecialComputedTable(1048576,1048576,0.8);
void ADD::garbageCollect(void){
  TRACE("ADD", "garbageCollect", std::cout << "Garbage Collect, " << deadNodes<< " dead nodes, BDD Size: " << ADD::size() << "\n");

  BoolComputedTable.clear();
  MoveComputedTable.clear();
  ArithmeticComputedTable.clear();
  AbstractComputedTable.clear();
  ExtremaComputedTable.clear();
  CompareComputedTable.clear();
  MatrixComputedTable.clear();
  SpecialComputedTable.clear();

  for(unsigned long i=0;(i<variableOrdering.size()) && (deadNodes>0);++i){
    ADDUniqueTable* uniqueT=variableOrdering.getData(i);
    unsigned long remainingDeadNodes=uniqueT->getDeadNodes();
    if(remainingDeadNodes>0){
      ADDBaseNode* currentTable=uniqueT->getFirst();
      ADDBaseNode* currentNode;
      ADDBaseNode* tempNode;
      while(remainingDeadNodes){
        currentNode=currentTable;
        while(currentNode){
          tempNode=static_cast<ADDInnerNode*>(currentNode)->getNext();
          if(currentNode->preNodeCount==ULONG_MAX){
            deleteSubTree(currentNode);
            if(--remainingDeadNodes==0) break;
          }
          currentNode=tempNode;
        }
        currentTable=uniqueT->next();
      }
      deadNodes-=uniqueT->getDeadNodes();
      uniqueT->resetDeadNodeCount();
    }
  }
  if(deadNodes>0){
    ADDTerminalNodeUniqueTable* uniqueT=&ADD::drainUniqueTable;
    unsigned long remainingDeadNodes=uniqueT->getDeadNodes();
    ADDBaseNode* currentTable=uniqueT->getFirst();
    ADDBaseNode* currentNode;
    ADDBaseNode* tempNode;
    while(remainingDeadNodes){
      currentNode=currentTable;
      while(currentNode){
  tempNode=static_cast<ADDTerminalNode*>(currentNode)->getNext();
  if(currentNode->preNodeCount==ULONG_MAX){
    deleteSubTree(currentNode);
    if(--remainingDeadNodes==0) break;
  }
  currentNode=tempNode;
      }
      currentTable=uniqueT->next();
    }
    deadNodes-=uniqueT->getDeadNodes();
    uniqueT->resetDeadNodeCount();
  }

  // @TODO:
  //  lastReorderSize=size();

  TRACE("ADD", "garbageCollect", std::cout << "After garbage collect, " << deadNodes<< " dead nodes, BDD Size: " << ADD::size() << "\n");
}

// do a full reset of the ADD, delete all nodes and clear the variable ordering
void ADD::reset(void){
  BoolComputedTable.clear();
  MoveComputedTable.clear();
  ArithmeticComputedTable.clear();
  AbstractComputedTable.clear();
  ExtremaComputedTable.clear();
  CompareComputedTable.clear();
  MatrixComputedTable.clear();
  SpecialComputedTable.clear();
  
  for(unsigned long i=0;i<variableOrdering.size();++i){
    ADDUniqueTable* uniqueT=variableOrdering.getData(i);

    ADDBaseNode* currentTable=uniqueT->getFirst();
    ADDBaseNode* currentNode;
    ADDBaseNode* tempNode;
    while(currentTable){
      currentNode=currentTable;
      while(currentNode){
        tempNode=static_cast<ADDInnerNode*>(currentNode)->getNext();
        deleteSubTree(currentNode);
        currentNode=tempNode;
      }
      currentTable=uniqueT->next();
    }
  }
  deadNodes=0;

  variableOrdering.clear();
}


unsigned long ADD::size(void){
  unsigned long ret=0;

  for(unsigned long i=0;i<variableOrdering.size();++i) ret+=variableOrdering.getData(i)->size();
  ret+=drainUniqueTable.size();

  //do not count zero or one drain if they have no predecessors
  if(oneDrain->preNodeCount==1) --ret;
  if(zeroDrain->preNodeCount!=1) ++ret;

  return ret;
}

/**
* \brief This function returns the number of dead nodes
*/
unsigned long ADD::countDeadNodes(void){
  return deadNodes;
}


unsigned long ADD::setGarbageCollectionDelay(const unsigned long newSize){
  if(newSize==garbageCollectionDelay) return garbageCollectionDelay;

  unsigned long oldSize=garbageCollectionDelay;
  garbageCollectionDelay=newSize;
  if(deadNodes>=garbageCollectionDelay) garbageCollect();

  return oldSize;
}

bool ADD::clear(void){
  //garbage collection
  garbageCollect();

  //count number of inner nodes
  unsigned long innerNodes=0;
  for(unsigned long i=0;i<variableOrdering.size();++i) innerNodes+=variableOrdering.getData(i)->size();

  if(innerNodes==0){
    variableOrdering.clear();

    return true;
  } else {
    bool ret=false;
    for(unsigned long i=variableOrdering.size();i>0;--i) ret|=variableOrdering.removeVariable(i-1);

    return ret;
  }
}

bool ADD::swap(const unsigned long index){
  if UNLIKELY(index+1>=variableOrdering.size()) return false;

  ADDBaseNode* w0;
  ADDBaseNode* w1;
  ADDBaseNode* w00;
  ADDBaseNode* w01;
  ADDBaseNode* w10;
  ADDBaseNode* w11;

  ADDUniqueTable* tempTable1=variableOrdering.getData(index);
  ADDUniqueTable* tempTable2=variableOrdering.getData(index+1);
  ADDInnerNode* currentTable=tempTable1->getFirst();
  ADDInnerNode* currentNode;
  ADDInnerNode* tempNode;

  std::list<ADDInnerNode*> to_process;

  // preremove the nodes
  while(currentTable){
    currentNode=currentTable;
    while(currentNode){
      tempNode=currentNode->getNext();

      w0=currentNode->getSucc0();
      w1=currentNode->getSucc1();
      if(w0->getLevel()==index+1  ||
         w1->getLevel()==index+1) {
        tempTable1->remove(currentNode);
        to_process.push_back(currentNode);
      }
      currentNode=tempNode;
    }
    currentTable=tempTable1->next();
  }

  for (std::list<ADDInnerNode*>::iterator it=to_process.begin();
       it!=to_process.end();
       ++it) {
    currentNode=*it;

    w0=currentNode->getSucc0();
    if(w0->getLevel()==index+1){
      w00=static_cast<ADDInnerNode*>(w0)->getSucc0();
      w01=static_cast<ADDInnerNode*>(w0)->getSucc1();
      w1=currentNode->getSucc1();
      if(w1->getLevel()==index+1){
        w10=static_cast<ADDInnerNode*>(w1)->getSucc0();
        w11=static_cast<ADDInnerNode*>(w1)->getSucc1();
      } else w10=w11=w1;
      if(w00==w10) w0=w00;
      else w0=tempTable1->findOrAdd(w00,w10);
      
      if(w01==w11) w1=w11;
      else w1=tempTable1->findOrAdd(w01,w11);
      
      currentNode->setNode(tempTable2,w0,w1);
    } else {
      w1=currentNode->getSucc1();
      if(w1->getLevel()==index+1){
        w00=w01=w0;
        w10=static_cast<ADDInnerNode*>(w1)->getSucc0();
        w11=static_cast<ADDInnerNode*>(w1)->getSucc1();
        if(w00==w10) w0=w00;
        else w0=tempTable1->findOrAdd(w00,w10);
        
        if(w01==w11) w1=w11;
        else w1=tempTable1->findOrAdd(w01,w11);
        
        currentNode->setNode(tempTable2,w0,w1);
      }
    }
  }

  //swap variables
  variableOrdering.swapVariables(index);

  return true;
}


void ADD::enableDynamicReordering() {
  dynamicReordering=true;
}

void ADD::disableDynamicReordering() {
  dynamicReordering=false;
}

void ADD::setDynamicReorderingMaxGrowth(float maxGrowth) {
  dynamicReorderingMaxGrowth=maxGrowth;
}

void ADD::setDynamicReorderingTimeout(unsigned int timeout_ms) {
  dynamicReorderingTimeout=timeout_ms;
}

void ADD::setUniqueTableMaxLoadFactor(float load_factor) {
  ADDUniqueTable::max_load_factor=load_factor;
}


void ADD::reorderingHook(bool force_reordering) {
  ///TODO: methods to change the following two values 
  unsigned long min_reord_bdd_size = 100;
  unsigned long reord_trigger_factor = 2;
  
  unsigned long bdd_size = size();
  if (force_reordering ||
      ((bdd_size > min_reord_bdd_size) && (bdd_size > reord_trigger_factor*lastReorderSize))) {
    garbageCollect();
    bdd_size = size();
    //double check if the real size of the BDD grew or just the number of dead nodes
    if(force_reordering||
       (bdd_size > min_reord_bdd_size && (bdd_size > reord_trigger_factor*lastReorderSize))) {
      ADD::groupSifting(dynamicReorderingMaxGrowth, true);      
      lastReorderSize=size();
    }
  }
}

void ADD::groupSifting(const float maxGrowth, const bool external_garbage_collect){
  if (!external_garbage_collect) garbageCollect();

  unsigned long before=0, after=0;
  TRACE("ADD", "reorder", before=ADD::size();                     \
        std::cout << "Starting reordering: " << before << " nodes";     \
        std::cout.flush());
  
  ReorderHelper<ADD> rH;
  Reordering<ReorderHelper<ADD> > reord(&rH);
  reord.sifting(maxGrowth, dynamicReorderingTimeout);

  TRACE("ADD", "reorder", after=ADD::size();                   \
        std::cout << ", afterwards " << after << " nodes, "         \
        << ((float)before-after)/(float)before*100.0 << "%\n");
  
  TRACE("ADD", "reorder", if (after>before) {throw "BDD is bigger after reordering!";});
}


void ADD::groupExhaustive(ADD::CallBack& call_back) {
  garbageCollect();

  ReorderHelper<ADD> rH;
  Reordering<ReorderHelper<ADD> > reord(&rH);
  reord.groupExhaustive(call_back);
}


