/***************************************************************************
                          addfunction.h  -  description
                             -------------------
    begin                : Thu Apr 15 2004
    copyright            : (C) 2004-2008 by Joern Ossowski
    email                : mail@jossowski.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ADDFUNCTION_H
#define ADDFUNCTION_H

class ADDBaseNode;
class Assignment;

#include <string>
#include <set>
#include <map>

#include <boost/operators.hpp>

class ADDFunction : public boost::operators<ADDFunction>{
public:
  ADDFunction(const ADDFunction&);
  ADDFunction(const Assignment&, const double=1.0);
  ADDFunction(char*, const double=1.0);
  ADDFunction(const std::string&);
  ADDFunction(const double);
  ADDFunction(const ADDBaseNode*);
  ADDFunction(void);
  ~ADDFunction(void);
  static ADDFunction ONE(void);
  static ADDFunction ZERO(void);
  static ADDFunction createConstantFunction(const double);
  static ADDFunction projection(const std::string&);
  static ADDFunction projection(const unsigned long);
  unsigned long size(void) const;
  ADDFunction& operator=(const ADDFunction&);
  ADDFunction& operator+=(const ADDFunction&);
  ADDFunction& operator-=(const ADDFunction&);
  ADDFunction& operator*=(const ADDFunction&);
  ADDFunction& operator|=(const ADDFunction&);
  ADDFunction& operator&=(const ADDFunction&);
  ADDFunction& operator^=(const ADDFunction&);
  ADDFunction& operator++();
  ADDFunction& operator--();
  ADDFunction operator!(void) const;

  bool operator==(const ADDFunction&) const;
  bool operator<(const ADDFunction&) const;
  bool isConstant(void) const;
  double getValue(void) const;
  unsigned long getFirstVariableIndex(void) const;
  ADDFunction cofactor(const unsigned long, const bool) const;
  ADDFunction cofactor(const ADDFunction&) const;
  ADDFunction subSet(const unsigned long, const bool) const;
  ADDFunction subSet(const ADDFunction&) const;
  ADDFunction change(void) const;
  ADDFunction change(const unsigned long) const;
  ADDFunction change(const ADDFunction&) const;
  ADDFunction exists(const ADDFunction&) const;
  ADDFunction exists(const unsigned long) const;
  ADDFunction forall(const ADDFunction&) const;
  ADDFunction forall(const unsigned long) const;
  ADDFunction equivalenceClasses(const ADDFunction&, const ADDFunction&) const;
  ADDFunction constrain(const ADDFunction&) const;
  ADDFunction sum(const ADDFunction&) const;
  ADDFunction product(const ADDFunction&) const;
  ADDFunction maximum(const ADDFunction&) const;
  double max(void) const;
  ADDFunction minimum(const  ADDFunction&) const;
  double min(void) const;
  ADDFunction oneOver(void) const;
  ADDFunction compose(const unsigned long, const ADDFunction&) const;
  ADDFunction moveUp() const;
  ADDFunction moveUp(const ADDFunction&) const;
  ADDFunction moveDown() const;
  ADDFunction moveDown(const ADDFunction&) const;
  void printEssentialVariables(void) const;
  std::set<std::string> getEssentialVariables(void) const;
  operator Assignment() const;
  ADDFunction getAssignment(void) const;
  void getAssignment(char*) const;
  double countAssignments(void) const;
  double countAssignments(const ADDFunction& cubeSet) const;
  ADDFunction getRandomAssignment(const ADDFunction& cube_set) const;
  void printPaths(void) const;
  double countPaths(void) const;
  ADDFunction getPath(void) const;
  void getPath(char*) const;

  struct less : public std::binary_function<ADDFunction, ADDFunction, bool>  {
    bool operator()(const ADDFunction& x, const ADDFunction& y) const {
      std::less<ADDBaseNode*> l;
      return l(x.rootNode, y.rootNode);
    }
  };

  std::set<ADDBaseNode*> getNodes() const;
  ADDBaseNode* rootNode;
};

inline ADDFunction operator+(const double value, const ADDFunction& function){return ADDFunction(value)+function;}
inline ADDFunction operator-(const double value, const ADDFunction& function){return ADDFunction(value)-function;}
inline ADDFunction operator*(const double value, const ADDFunction& function){return ADDFunction(value)*function;}
ADDFunction operator-(const ADDFunction&);
inline ADDFunction EQUIV(const ADDFunction& function1, const ADDFunction& function2){return !(function1^function2);}
inline ADDFunction IMPLIES(const ADDFunction& function1, const ADDFunction& function2){return !function1|function2;}
ADDFunction ITE(const ADDFunction&, const ADDFunction&, const ADDFunction&);
ADDFunction LESS(const ADDFunction&, const ADDFunction&);
ADDFunction LESS_EQUAL(const ADDFunction&, const ADDFunction&);
ADDFunction EQUAL(const ADDFunction&, const ADDFunction&);
inline ADDFunction GREATER(const ADDFunction& function1, const ADDFunction& function2){return LESS(function2,function1);}
inline ADDFunction GREATER_EQUAL(const ADDFunction& function1, const ADDFunction& function2){return LESS_EQUAL(function2,function1);}
ADDFunction MAXIMUM(const ADDFunction&, const ADDFunction&);
ADDFunction MINIMUM(const ADDFunction&, const ADDFunction&);

ADDFunction MM(const ADDFunction&, const ADDFunction&, const ADDFunction&);
ADDFunction MV(const ADDFunction&, const ADDFunction&, const ADDFunction&);
ADDFunction VM(const ADDFunction&, const ADDFunction&, const ADDFunction&);

#endif
