/***************************************************************************
                          addfunction.h  -  description
                             -------------------
    begin                : Thu Apr 15 2004
    copyright            : (C) 2004 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ADDFUNCTION_H
#define ADDFUNCTION_H

class ADD;
class ADDNode;
class Assignment;
class VarSet;

#include <string>
#include <set>
#include <map>

//#include <boost/operators.hpp>

using namespace std;

class ADDFunction{ // : boost::operators<ADDFunction>{
friend class ADD;
friend class ADDOutput;
public:
  ADDFunction(const ADDFunction&);
  ADDFunction(const Assignment&, const double=1.0);
  ADDFunction(char*, const double=1.0);
  ADDFunction(const VarSet& varSet);
  ADDFunction(const string&);
  ADDFunction(const double);
  ADDFunction(void);
  ~ADDFunction(void);
  static ADDFunction ERROR(void);
  static ADDFunction ONE(void);
  static ADDFunction ZERO(void);
  static ADDFunction createConstantFunction(const double);
  static ADDFunction createStabilityFunction(const VarSet&, const VarSet&);
  static ADDFunction projection(const string&);
  static ADDFunction projection(const unsigned long);
  unsigned long size(void) const;
  ADDFunction& operator=(const ADDFunction&);
  ADDFunction& operator+=(const ADDFunction&);
  ADDFunction& operator-=(const ADDFunction&);
  ADDFunction& operator*=(const ADDFunction&);
  ADDFunction& operator|=(const ADDFunction&);
  ADDFunction& operator&=(const ADDFunction&);
  ADDFunction& operator^=(const ADDFunction&);
  ADDFunction& operator++();
  ADDFunction& operator--();
  ADDFunction operator!(void) const;
  bool operator==(const ADDFunction&) const;
  bool operator<(const ADDFunction&) const;
  bool isConstant(void) const;
  double getValue(void) const;
  string getFirstVariable(void) const;
  unsigned long getFirstVariableIndex(void) const;
  ADDFunction cofactor(const string&, const bool) const;
  ADDFunction cofactor(const unsigned long, const bool) const;
  ADDFunction cofactor(const ADDFunction&) const;
  ADDFunction subSet(const string&, const bool) const;
  ADDFunction subSet(const unsigned long, const bool) const;
  ADDFunction subSet(const ADDFunction&) const;
  ADDFunction change(void) const;
  ADDFunction change(const string&) const;
  ADDFunction change(const unsigned long) const;
  ADDFunction change(const ADDFunction&) const;
  ADDFunction exists(const ADDFunction&) const;
  ADDFunction exists(const string&) const;
  ADDFunction exists(const unsigned long) const;
  ADDFunction forall(const ADDFunction&) const;
  ADDFunction forall(const string&) const;
  ADDFunction forall(const unsigned long) const;
  ADDFunction equivalenceClasses(const ADDFunction&, const ADDFunction&) const;
  ADDFunction constrain(const ADDFunction&) const;
  ADDFunction sum(const ADDFunction&) const;
  ADDFunction product(const ADDFunction&) const;
  ADDFunction maximum(const ADDFunction&) const;
  double max(void) const;
  ADDFunction minimum(const  ADDFunction&) const;
  double min(void) const;
  ADDFunction oneOver(void) const;
  ADDFunction compose(const string&, const ADDFunction&) const;
  ADDFunction compose(const unsigned long, const ADDFunction&) const;
  ADDFunction moveUp(const unsigned int=1) const;
  ADDFunction moveUp(const ADDFunction&, const unsigned int=1) const;
  ADDFunction moveDown(const unsigned int=1) const;
  ADDFunction moveDown(const ADDFunction&, const unsigned int=1) const;
  void printEssentialVariables(void) const;
  set<string> getEssentialVariables(void) const;
  operator Assignment() const;
  ADDFunction getAssignment(void) const;
  void getAssignment(char*) const;
  double countAssignments(void) const;
  void printPaths(void) const;
  double countPaths(void) const;
  ADDFunction getPath(void) const;
  void getPath(char*) const;
private:
  static ADD* BDD;
  ADDNode* rootNode;

  ADDFunction(const ADDNode* rootNode);
  void printPaths(const unsigned long, const ADDNode*, char*) const;
  double countPaths(const ADDNode*, map<ADDNode*,double>*) const;
};

ADDFunction operator+(const double, const ADDFunction&);
ADDFunction operator-(const double, const ADDFunction&);
ADDFunction operator*(const double, const ADDFunction&);
ADDFunction operator-(const ADDFunction&);
ADDFunction EQUIV(const ADDFunction&, const ADDFunction&);
ADDFunction IMPLIES(const ADDFunction&, const ADDFunction&);
ADDFunction ITE(const ADDFunction&, const ADDFunction&, const ADDFunction&);
ADDFunction LESS(const ADDFunction&, const ADDFunction&);
ADDFunction LESS_EQUAL(const ADDFunction&, const ADDFunction&);
ADDFunction EQUAL(const ADDFunction&, const ADDFunction&);
ADDFunction GREATER(const ADDFunction&, const ADDFunction&);
ADDFunction GREATER_EQUAL(const ADDFunction&, const ADDFunction&);
ADDFunction MAXIMUM(const ADDFunction&, const ADDFunction&);
ADDFunction MINIMUM(const ADDFunction&, const ADDFunction&);

ADDFunction MM(const ADDFunction&, const ADDFunction&, const ADDFunction&);
ADDFunction MV(const ADDFunction&, const ADDFunction&, const ADDFunction&);
ADDFunction VM(const ADDFunction&, const ADDFunction&, const ADDFunction&);

/*
ADDFunction MM(const ADDFunction&, const ADDFunction&, const unsigned long, const short=2);
ADDFunction MV(const ADDFunction&, const ADDFunction&, const unsigned long);

ADDFunction JACOBI(const ADDFunction&, const ADDFunction&, const unsigned long, const unsigned long, const double=0.0);
ADDFunction BF(const ADDFunction&, const unsigned long, const unsigned long);
ADDFunction BF(const ADDFunction&, const unsigned long);
ADDFunction FLOYD(const ADDFunction&, const unsigned long);

void printMatrix(const ADDFunction&, const unsigned long, const unsigned long);
void printMatrix2PNM(const ADDFunction&, const unsigned long, const unsigned long);
void printLeftMatrix(const ADDFunction&, const unsigned long, const unsigned long);
void printRightMatrix(const ADDFunction&, const unsigned long, const unsigned long);
*/

#endif
