/***************************************************************************
                          addinnernode.h  -  description
                             -------------------
    begin                : Sat Mar 29 2008
    copyright            : (C) 2008 by Joern Ossowski
    email                : mail@jossowski.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ADDINNERNODE_H
#define ADDINNERNODE_H

#include <add/addbasenode.h>
#include <add/addterminalnode.h>
#include <add/add.h>
#include <add/addvarorder.h>
#include <common/helper.h>
#include <common/mempool.h>

template <class T> class MemPool;

class ADDInnerNode : public ADDBaseNode {
public:
  ADDInnerNode(void): succ0(0), succ1(0) {}
  static ADDInnerNode* getNewNode(void){return ADD::innerNodeMemPool.alloc();}
  static void freeNode(ADDInnerNode* node){ADD::innerNodeMemPool.free(node);}
  static unsigned long PEAK(void){return ADD::innerNodeMemPool.peak();}
  ADDInnerNode* getNext(void) const {return static_cast<ADDInnerNode*>(next);}
  void setNext(const ADDInnerNode* next){this->next=static_cast<ADDBaseNode*>(const_cast<ADDInnerNode*>(next));}
  ADDBaseNode* getSucc0(void) const {return succ0;}
  ADDBaseNode* getSucc1(void) const {return succ1;}
  ADDBaseNode* getSucc(const bool succX) const {return succX?succ1:succ0;}
  void initNode(ADDUniqueTable* uTable, ADDBaseNode* s0, ADDBaseNode* s1){
    s0->incNodeCount();
    s1->incNodeCount();

    //update node information
    succ0=s0;
    succ1=s1;
    uniqueTable=uTable;

    preNodeCount=0;
  }
  void setNode(ADDUniqueTable* uTable, ADDBaseNode* s0, ADDBaseNode* s1){
    ++s0->preNodeCount;
    ++s1->preNodeCount;

    if UNLIKELY(--succ0->preNodeCount==0){
      if UNLIKELY(succ0->isDrain()){
	      ADD::drainUniqueTable.remove(static_cast<ADDTerminalNode*>(succ0));
	      ADDTerminalNode::freeNode(static_cast<ADDTerminalNode*>(succ0));
      } else {
	      --static_cast<ADDInnerNode*>(succ0)->getSucc0()->preNodeCount;
	      --static_cast<ADDInnerNode*>(succ0)->getSucc1()->preNodeCount;
	      succ0->uniqueTable->remove(static_cast<ADDInnerNode*>(succ0));
	      ADDInnerNode::freeNode(static_cast<ADDInnerNode*>(succ0));
      }
    }
    if UNLIKELY(--succ1->preNodeCount==0){
      if UNLIKELY(succ1->isDrain()){
	      ADD::drainUniqueTable.remove(static_cast<ADDTerminalNode*>(succ1));
	      ADDTerminalNode::freeNode(static_cast<ADDTerminalNode*>(succ1));
      } else {
	      --static_cast<ADDInnerNode*>(succ1)->getSucc0()->preNodeCount;
	      --static_cast<ADDInnerNode*>(succ1)->getSucc1()->preNodeCount;
	      succ1->uniqueTable->remove(static_cast<ADDInnerNode*>(succ1));
	      ADDInnerNode::freeNode(static_cast<ADDInnerNode*>(succ1));
      }
    }
    succ0=s0;
    succ1=s1;
    uniqueTable=uTable;
    uniqueTable->add(this);
  }
private:
  ADDBaseNode* succ0;
  ADDBaseNode* succ1;
};

#endif
