/***************************************************************************
                          addnode.cpp  -  description
                             -------------------
    begin                : Wed Apr 14 2004
    copyright            : (C) 2004 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <add/addnode.h>
#include <common/mempool.h>

#include <climits>

ADDNode* ADDNode::zeroDrain=0;
ADDNode* ADDNode::oneDrain=0;
ADD* ADDNode::BDD=0;
MemPool<ADDNode> ADDNode::memPool=MemPool<ADDNode>();

//************************ADDNode***************************
//*************************public*****************************
ADDNode::ADDNode(void){
  preNodeCount=0;
  type.children.succ0=type.children.succ1=next=0;
  uniqueTable=0;
}

ADDNode::~ADDNode(void){
}

ADDNode* ADDNode::getNewNode(void){
  return memPool.alloc();
}

void ADDNode::freeNode(ADDNode* node){
  memPool.free(node);
}

unsigned long ADDNode::PEAK(void){
  return memPool.peak();
}

ADDNode* ADDNode::getSucc(const bool value) const {
  return value?type.children.succ1:type.children.succ0;
}

void ADDNode::setValue(double value){
  type.value=value;
}

double ADDNode::getValue(void) const {
  return type.value;
}

void ADDNode::initNode(ADDUniqueTable* uTable, ADDNode* s0, ADDNode* s1){
  //there is no check if there was an unique table
  //this function should only be used in findOrAdd

  s0->incNodeCount();
  s1->incNodeCount();

  //update node information
  type.children.succ0=s0;
  type.children.succ1=s1;
  uniqueTable=uTable;
  
  preNodeCount=0;
}

void ADDNode::setNode(ADDUniqueTable* uTable, ADDNode* s0, ADDNode* s1){
  ADDNode** succ0=&type.children.succ0;
  ADDNode** succ1=&type.children.succ1;
  
  uniqueTable=uTable;
  ++s0->preNodeCount;
  ++s1->preNodeCount;

  if(--(*succ0)->preNodeCount==0) BDD->deleteNode(*succ0);
  if(--(*succ1)->preNodeCount==0) BDD->deleteNode(*succ1);
  (*succ0)=s0;
  (*succ1)=s1;
  
  //add entry to unique table
  uniqueTable->add(this);
}

void ADDNode::removeFromUniqueTable(void){
  //remove entry from uniquetable
  uniqueTable->remove(this);
}

ADDUniqueTable* ADDNode::getUniqueTable(void) const {
  return uniqueTable;
}

void ADDNode::garbageCollect(void){
  uniqueTable->incDeadNodeCount();
  ++BDD->deadNodes;
  //set correct preNodeCount (zombie node has preNodeCount ULONG_MAX)
  preNodeCount=ULONG_MAX;
}

unsigned long ADDNode::getLevel(void) const {
  //every node (terminal or inner) has an unique table with proper level information
  return uniqueTable->getLevel();
}

unsigned long ADDNode::getRealLevel(void) const {
  //every node (terminal or inner) has an unique table with proper level information
  return uniqueTable->getRealLevel();
}

void ADDNode::deleteNode(void){
  if(--preNodeCount==0){
    uniqueTable->incDeadNodeCount();
    ++BDD->deadNodes;
    preNodeCount=ULONG_MAX;
    if(BDD->deadNodes>=BDD->garbageCollectionDelay) ADD::garbageCollect();
  }
}

bool ADDNode::operator<(const ADDNode& node) const {
  return (this->getValue()+0.0000001)<node.getValue();
}
