/***************************************************************************
                          addnode.h  -  description
                             -------------------
    begin                : Wed Apr 14 2004
    copyright            : (C) 2004 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ADDNODE_H
#define ADDNODE_H

#include <add/add.h>
#include <add/addvarorder.h>
#include <common/helper.h>

template <class T> class MemPool;

class ADDNode;
struct ADDNodeIntern{
  ADDNode* succ0;
  ADDNode* succ1;
};

class ADDNode{
friend class ADD;
public:
  ADDNode(void);
  ~ADDNode(void);
  static ADDNode* zeroDrain;
  static ADDNode* oneDrain;
  static ADDNode* getNewNode(void);
  static void freeNode(ADDNode*);
  static unsigned long PEAK(void);
  inline bool isDrain(void) const {return (uniqueTable->isDrainLevel());}

  ADDNode* getSucc0(void) const {return type.children.succ0;};
  ADDNode* getSucc1(void) const {return type.children.succ1;};
  ADDNode* getSucc(const bool) const;
  inline ADDNode* getNext(void) const {return next;};
  inline void setNext(const ADDNode* next){this->next=(ADDNode*) next;}
  void setValue(double);
  double getValue(void) const;
  void initNode(ADDUniqueTable*, ADDNode*, ADDNode*);
  void setNode(ADDUniqueTable*, ADDNode*, ADDNode*);
  inline void setDrainTable(const ADDUniqueTable* uTable){uniqueTable=(ADDUniqueTable*) uTable;preNodeCount=0;}
  void removeFromUniqueTable(void);
  ADDUniqueTable* getUniqueTable(void) const;
  void incNodeCount(void){
    if UNLIKELY(preNodeCount==ULONG_MAX){
      uniqueTable->decDeadNodeCount();
      --BDD->deadNodes;
      preNodeCount=1;
    } else ++preNodeCount;
  }
  void garbageCollect(void);
  unsigned long getLevel(void) const;
  unsigned long getRealLevel(void) const;
  void deleteNode(void);
  bool operator<(const ADDNode&) const;
private:
  static ADD* BDD;
  static MemPool<ADDNode> memPool;
  unsigned long preNodeCount;
  union{
    ADDNodeIntern children;
    double value;
  } type;
  ADDNode* next;
  ADDUniqueTable* uniqueTable;
};

#endif
