/***************************************************************************
                          addoutput.cpp  -  description
                             -------------------
    begin                : Tue Jul 13 2004
    copyright            : (C) 2004-2008 by Joern Ossowski
    email                : mail@jossowski.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <add/addoutput.h>
#include <add/addbasenode.h>
#include <add/addinnernode.h>
#include <add/addterminalnode.h>
#include <add/addfunction.h>
#include <add/addvarorder.h>

using namespace std;

bool ADDOutput::storeFunction(const string& filename, const ADDFunction& function){
  return ADDOutput::storeFunctions(filename,ADDFunctionContainer(function));
}

bool ADDOutput::storeFunctions(const string& filename, const ADDFunctionContainer& container){
  ADDOutput output(filename);
  return output.storeBDD(container);
}

ADDOutput::ADDOutput(const string& filename){
  file.open(filename.c_str(),ios::out);
}

ADDOutput::~ADDOutput(void){
  if(file) file.close();
}

bool ADDOutput::storeBDD(const ADDFunctionContainer& container){
  unsigned long i;

  //file open?
  if(!file) return false;
  //empty container?
  if(container.size()==0) return false;

  for(i=0;i<container.size();++i){
    ADDBaseNode* node=container[i].rootNode;
    file << "rootnode {" << endl;
    file << "  id=" << node << endl;
    file << "}" << endl;
    storeBDD(node);
  }

  return true;
}

void ADDOutput::storeBDD(ADDBaseNode* node){
  if(visitedSet.find(node)!=visitedSet.end()) return;
  if(!node->isDrain()){
    file << "node {" << endl;
    file << "  id=" << node << endl;
    file << "  level=" << node->getLevel() << endl;
    file << "  succ0=" << static_cast<ADDInnerNode*>(node)->getSucc0() << endl;
    file << "  succ1=" << static_cast<ADDInnerNode*>(node)->getSucc1() << endl;
    file << "  label=" << ADDVarOrder::getVariableName(node->getLevel()) << endl;
    file << "  drawtype=circle" << endl;
    file << "}" << endl;
    visitedSet.insert(node);
    storeBDD(static_cast<ADDInnerNode*>(node)->getSucc0());
    storeBDD(static_cast<ADDInnerNode*>(node)->getSucc1());
  } else {
    file << "node {" << endl;
    file << "  id=" << node << endl;
    file << "  level=" <<  ADDVarOrder::size() << endl;
    file << "  label=" << (static_cast<ADDTerminalNode*>(node)->getValue()) << endl;
    file << "  drawtype=box" << endl;
    file << "}" << endl;
    visitedSet.insert(node);
  }
}
