/***************************************************************************
                          functioncontainer.h  -  description
                             -------------------
    begin                : Fri Nov 25 2005
    copyright            : (C) 2005-2008 by Joern Ossowski
    email                : mail@jossowski.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FUNCTION_CONTAINER_H
#define FUNCTION_CONTAINER_H

#include <vector>


template <class T>
class FunctionContainer{
public:
  FunctionContainer(void);
  FunctionContainer(const T&);
  FunctionContainer(const FunctionContainer&);
  ~FunctionContainer(void);
  FunctionContainer& operator=(const FunctionContainer&);
  FunctionContainer& operator=(const T&);

  bool operator==(const FunctionContainer&) const;
  bool operator==(const T&) const;
  bool operator!=(const FunctionContainer&) const;
  bool operator!=(const T&) const;
  T& operator[](const unsigned short);
  const T& operator[](const unsigned short) const;
  const T& at(const unsigned short) const;

  void push(const T&);
  void push(const FunctionContainer&);
  bool pop(void);
  bool remove(unsigned long);
  unsigned long size(void) const;
  void clear(void);
private:
  std::vector<T> functions;
};

template <class T>
FunctionContainer<T>::FunctionContainer(void){}

template <class T>
FunctionContainer<T>::FunctionContainer(const T& function){functions.push_back(function);}

template <class T>
FunctionContainer<T>::FunctionContainer(const FunctionContainer& container){
  for(unsigned int i=0;i<container.functions.size();++i) functions.push_back(container.functions[i]);
}

template <class T>
FunctionContainer<T>::~FunctionContainer(void){}

template <class T>
FunctionContainer<T>& FunctionContainer<T>::operator=(const FunctionContainer& container){
  if(&container!=this) functions=container.functions;

  return *this;
}

template <class T>
FunctionContainer<T>& FunctionContainer<T>::operator=(const T& function){
  functions.clear();
  functions.push_back(function);

  return *this;
}

template <class T>
bool FunctionContainer<T>::operator==(const FunctionContainer& container) const {
  if(functions.size()!=container.functions.size()) return false;
  for(unsigned int i=0;i<functions.size();++i) if(functions[i]!=container.functions[i]) return false;

  return true;
}

template <class T>
bool FunctionContainer<T>::operator==(const T& function) const {
  return *this==FunctionContainer<T>(function);
}

template <class T>
bool FunctionContainer<T>::operator!=(const FunctionContainer& container) const {
  return !((*this)==container);
}

template <class T>
bool FunctionContainer<T>::operator!=(const T& function) const {
  return *this!=FunctionContainer<T>(function);
}

template <class T>
T& FunctionContainer<T>::operator[](const unsigned short index){
  return functions[index];
}

template <class T>
const T& FunctionContainer<T>::operator[](const unsigned short index) const {
  return functions[index];
}

template <class T>
const T& FunctionContainer<T>::at(const unsigned short index) const {
  return functions[index];
}


template <class T>
void FunctionContainer<T>::push(const T& function){
  functions.push_back(function);
}

template <class T>
void FunctionContainer<T>::push(const FunctionContainer& container){
  for(unsigned short i=0;i<container.functions.size();++i) functions.push_back(container.functions[i]);
}

template <class T>
bool FunctionContainer<T>::pop(void){
  if(!functions.size()) return false;

  functions.pop_back();

  return true;
}

template <class T>
bool FunctionContainer<T>::remove(const unsigned long i){
  if(i<functions.size()){
    functions.erase(functions.begin()+i);
    return true;
  } else return false;
}

template <class T>
unsigned long FunctionContainer<T>::size(void) const {
  return functions.size();
}

template <class T>
void FunctionContainer<T>::clear(void){
  functions.clear();
}

#endif

