/***************************************************************************
                          hashmap.h  -  description
                             -------------------
    begin                : Thu Oct 24 2002
    copyright            : (C) 2002 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef HASHMAP_H
#define HASHMAP_H

#include <iostream>
#include <string>

using namespace std;

template <class T1,class T2>
struct PointerPair{
  T1* container;
  T2* content;
};

//**********************HashMap*******************
template <class T1,class T2>
class HashMap{
public:
  HashMap(const unsigned long, const unsigned long, const double);
  ~HashMap(void);
  void clear(void);
  unsigned long count(void) const;
  inline bool contains(const T1&) const;
  inline T2* find(const T1&) const;
  inline bool insert(const T1&, const T2*);
	void printStat(const string&) const;
private:
	unsigned short resizes;
	mutable double maxRatio;
	unsigned short resizeRatio;
  unsigned long counter;
  unsigned long numberOfHashValues;
  unsigned long maxNumberOfHashValues;
  unsigned short shift;
  struct PointerPair<T1,T2>* hashMap;
	
  inline unsigned long hashFunction(const unsigned long) const;
  void resize(void);
};

template <class T1, class T2>
HashMap<T1,T2>::HashMap(const unsigned long hashValues, const unsigned long maxHashValues, const double resizeR){
	resizes=0;
	maxRatio=0.0;
	resizeRatio=(unsigned short)(100.0*resizeR);
  numberOfHashValues=2;
  //shift for smallest hashmap
  shift=sizeof(unsigned long)*8-1;
  while((numberOfHashValues<<1)<=hashValues){
    numberOfHashValues<<=1;
    --shift;
  }
  maxNumberOfHashValues=maxHashValues;
  counter=0;
	
  hashMap=new struct PointerPair<T1,T2>[numberOfHashValues];
  //init hashmap
  memset(hashMap,0,numberOfHashValues*sizeof(struct PointerPair<T1,T2>));
}

template <class T1, class T2>
HashMap<T1,T2>::~HashMap(void){
	clear();
  delete[] hashMap;
}

template <class T1, class T2>
void HashMap<T1,T2>::clear(void){
	double tempRatio=100.0*double(counter)/double(numberOfHashValues);
	if(tempRatio>maxRatio) maxRatio=tempRatio;
	
  struct PointerPair<T1,T2>* elementPointer=hashMap;
  struct PointerPair<T1,T2>* endPointer=hashMap+numberOfHashValues;
	
	while(elementPointer!=endPointer){
		if(elementPointer->content){
			delete elementPointer->content;
			elementPointer->content=0;
			delete elementPointer->container;
			elementPointer->container=0;
		}
		++elementPointer;
	}
	
  counter=0;
}

template <class T1, class T2>
unsigned long HashMap<T1,T2>::count(void) const {
  return counter;
}

template <class T1, class T2>
bool HashMap<T1,T2>::contains(const T1& container) const {
  return find(container);
}

template <class T1, class T2>
T2* HashMap<T1,T2>::find(const T1& container) const {
  struct PointerPair<T1,T2>* p=hashMap+hashFunction(container.key());
	if(p->content){
	  if(*(p->container)==container) return p->content;
	}
	
  //search failed
  return 0;
}

template <class T1, class T2>
bool HashMap<T1,T2>::insert(const T1& container, const T2* content){
  struct PointerPair<T1,T2>* p=hashMap+hashFunction(container.key());

	if(p->content==0){
		p->container=new T1(container);
		p->content=(T2*) content;
		++counter;
	} else {
		(*p->container)=container;
		delete p->content;
		p->content=(T2*) content;
	}

	return true;
}

template <class T1, class T2>
inline unsigned long HashMap<T1,T2>::hashFunction(const unsigned long key) const {
  return ((key*4256249)>>shift);
}

template <class T1, class T2>
void HashMap<T1,T2>::printStat(const string& name) const {
	double tempRatio=100.0*double(counter)/double(numberOfHashValues);
	if(tempRatio>maxRatio) maxRatio=tempRatio;
	
	//do not print debug information for not used hash map
	if(maxRatio<=0.000001) return;
	
  cout << "####################################" << endl;
  cout << "Name             : " << name << endl;	
  cout << "Number of Resizes: " << resizes << endl;	
  cout << "Max Fill Ratio   : " << maxRatio << endl;	
  cout << "Size             : " << numberOfHashValues << endl;	
  cout << "Max Size         : " << maxNumberOfHashValues << endl;	
  cout << "####################################" << endl;
}

#endif
