#ifndef HELPER_H
#define HELPER_H
//#define OLD_HASH_FUNCTION

#if __GNUC__ >= 3
  #define LIKELY(X) (__builtin_expect((X),true))
  #define UNLIKELY(X) (__builtin_expect((X),false))
  #define ABS(X) (__builtin_fabs((X)))
#else
  #define LIKELY(X) (X)
  #define UNLIKELY(X) (X)
  #define ABS(X) ((X<0.0?-X:X))
#endif

#ifdef OLD_HASH_FUNCTION
  namespace Common{
    template <class T>
    inline unsigned long hashFunction(T key){
      return ((unsigned long) (key))*12582917;
    }
  
    template <class T>
    inline unsigned long hashFunction(T key1, T key2){
      return (hashFunction<T>(key1)+((unsigned long)(key2)))*4256249;
    }
  
    template <class T>
    inline unsigned long hashFunction(T key1, T key2, T key3){
      return (hashFunction<T>(key1,key2)+((unsigned long)(key3)))*741457;
    }
  
    template <class T>
    inline unsigned long hashFunction(T key1, T key2, T key3, T key4){
      return (hashFunction<T>(key1,key2,key3)+((unsigned long)(key4)))*94573;
    }
  
    template <class T>
    inline unsigned long hashFunction(T key1, T key2, T key3, T key4, T key5){
      return (hashFunction<T>(key1,key2,key3,key4)+((unsigned long)(key5)));
    }
  }

#else

  namespace Common{
    template <std::size_t = sizeof(std::size_t)>
    struct hash_fct{
      static std::size_t hash(const char* first, std::size_t length){
        std::size_t result=0;
        for(;length>0;--length) result=(result*131)+ *first++;
        return result;
      }
    };
  
    template <>
    struct hash_fct<4>{
      static std::size_t hash(const char* first, std::size_t length){
        std::size_t result=static_cast<std::size_t>(2166136261UL);
        for(;length>0;--length){
          result^=static_cast<std::size_t>(*first++);
          result*=static_cast<std::size_t>(16777619UL);
        }
        return result;
      }
    };
  
    template <>
    struct hash_fct<8>{
      static std::size_t hash(const char* first, std::size_t length){
        std::size_t result=static_cast<std::size_t>(14695981039346656037ULL);
        for(;length>0;--length){
          result^=static_cast<std::size_t>(*first++);
          result*=static_cast<std::size_t>(1099511628211ULL);
        }
        return result;
      }
    };
  
    template <typename T> struct hash;
  
    template <>
    struct hash<unsigned long>{
      std::size_t operator()(unsigned long lval) const {
        std::size_t result=0;
        result=hash_fct<>::hash(reinterpret_cast<const char*>(&lval),sizeof(lval));
        return result;
      }
    };
  
    template <typename T>
    inline unsigned long hashFunction(T key){
      hash<unsigned long> h;
      return h((unsigned long) (key));
    }
  
    template <typename T>
    inline unsigned long hashFunction(T key1, T key2){
      hash<unsigned long> h;
      return hashFunction<T>(key1)^(h((unsigned long)(key2))*12582917);
    }
  
    template <typename T>
    inline unsigned long hashFunction(T key1, T key2, T key3){
      hash<unsigned long> h;
      return hashFunction<T>(key1,key2)^(h((unsigned long)(key3))*12582917);
    }
  
    template <typename T>
    inline unsigned long hashFunction(T key1, T key2, T key3, T key4){
      hash<unsigned long> h;
      return hashFunction<T>(key1,key2,key3)^(h((unsigned long)(key4))*12582917);
    }
  
    template <typename T>
    inline unsigned long hashFunction(T key1, T key2, T key3, T key4, T key5){
      hash<unsigned long> h;
      return hashFunction<T>(key1,key2,key3,key4)^(h((unsigned long)(key5))*12582917);
    }
  }
#endif

#endif
