/***************************************************************************
                          mempool.h  -  description
                             -------------------
    begin                : Tue Dec 09 2003
    copyright            : (C) 2003-2008 by Joern Ossowski
    email                : mail@jossowski.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MEMPOOL_H
#define MEMPOOL_H

//**********************MemPool*******************
template <class T>
class MemPool{
public:
  MemPool(void);
  ~MemPool(void);

  inline T* alloc(void);
  inline void free(T*);
  inline unsigned long peak(void) const {return numberOfObjects<=peakNumberOfObjects?peakNumberOfObjects:numberOfObjects;}
  inline unsigned long size(void) const {return numberOfObjects;}
private:
  struct Chunk{
    enum{size=100000};
    T mem[size];
    Chunk* next;
  };
  Chunk* chunks;
  T* head;
  unsigned long peakNumberOfObjects;
  unsigned long numberOfObjects;

  inline void grow(void);
};

template <class T>
MemPool<T>::MemPool(void){
  head=0;
  chunks=0;
  peakNumberOfObjects=0;
  numberOfObjects=0;
}

template <class T>
MemPool<T>::~MemPool(void){
  Chunk* p;
  Chunk* n=chunks;
  while(n){
    p=n;
    n=n->next;
    delete p;
  }
}

template <class T>
T* MemPool<T>::alloc(void){
  //get more memory if needed
  if(!head) grow();
  T* p=head;
  head=p->getNext();
  p->setNext(0);
  ++numberOfObjects;

  return p;
}

template <class T>
void MemPool<T>::free(T* n){
  T* p=n;
  p->setNext(head);
  head=p;
  if(numberOfObjects>peakNumberOfObjects) peakNumberOfObjects=numberOfObjects;
  --numberOfObjects;
}

template <class T>
void MemPool<T>::grow(void){
  //get new memory
  Chunk* n=new Chunk;
  //set pointer from old chunk to new
  n->next=chunks;
  //set new chunk
  chunks=n;

  T* p=n->mem;
  //link all new objects
  for(unsigned long i=0;i<Chunk::size-1;++i,++p) p->setNext(&p[1]);
  //last object has no successor
  p->setNext(0);

  //set pointer to first object
  head=n->mem;
}
#endif
