/***************************************************************************
                          varset.h  -  description
                             -------------------
    begin                : Tue Apr 28 2004
    copyright            : (C) 2004 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/**
 * \file varset.h
 * \author Joern Ossowski
 */

#ifndef VARSET_H
#define VARSET_H

#include <set>
#include <vector>
#include <string>


/**
 * \brief VarSets represents sets of variables.
 *
 * VarSets are used to store a set of variables with an ordering.
 * VarSet are a combination of a set and a vector. So that searching for a
 * variable can be performed in O(log n). The drawback is that inserting is also in
 * O(log n) and removing is even in O(n).
 * \sa insert(const std::string&), remove(const std::string&)
 */
class VarSet{
public:
  VarSet(void);
  VarSet(const VarSet&);
  ~VarSet(void);
  void clear(void);
  bool find(const std::string&) const;
  bool insert(const std::string&);
  bool remove(const std::string&);
	VarSet reverse(void) const;
  VarSet& operator=(const VarSet&);
  VarSet& operator<<(const std::string&);
  VarSet& operator>>(const std::string&);
  std::string operator[](const unsigned long) const;
  unsigned long size(void) const;
private:
  std::set<std::string> varSet;
  std::vector<std::string> stringVector;
};

#endif
