/***************************************************************************
                          zadd.cpp  -  description
                             -------------------
    begin                : Wed Apr 14 2004
    copyright            : (C) 2004-2008 by Joern Ossowski
    email                : mail@jossowski.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <zadd/zadd.h>
#include <zadd/zaddinnernode.h>
#include <zadd/zaddterminalnode.h>
#include <common/mempool.h>
#include <common/helper.h>
#include <common/ptrhashmap.h>
#include <common/reordering.h>
#include "vereofy/common/tracing.hpp"


ZADDTerminalNode* createZeroDrain(void){
  ZADDTerminalNode* node=ZADDTerminalNode::getNewNode();
  node->setValue(0.0);
  node->incNodeCount();
  return node;
}
ZADDTerminalNode* initAndReturnOneDrain(void){
  Group<ZADDUniqueTable>::varOrder=&ZADD::variableOrdering;
  ZADDTerminalNode* node=ZADD::drainUniqueTable.findOrAddValue(1.0);
  node->incNodeCount();
  return node;
}
void deleteSubTree(ZADDBaseNode* node){
  if LIKELY(!node->isDrain()){
    ZADDInnerNode* innerNode=static_cast<ZADDInnerNode*>(node);

    if((--innerNode->getSucc0()->preNodeCount)==0) deleteSubTree(innerNode->getSucc0());
    if((--innerNode->getSucc1()->preNodeCount)==0) deleteSubTree(innerNode->getSucc1());

    //reset node
    innerNode->uniqueTable->remove(innerNode);

    //free mem for node
    ZADDInnerNode::freeNode(innerNode);
  } else {
    ZADDTerminalNode* terminalNode=static_cast<ZADDTerminalNode*>(node);
    //reset node
    ZADD::drainUniqueTable.remove(terminalNode);

    //free mem for node
    ZADDTerminalNode::freeNode(terminalNode);
  }
}

ZADDTerminalNodeUniqueTable ZADD::drainUniqueTable;
MemPool<ZADDInnerNode> ZADD::innerNodeMemPool;
MemPool<ZADDTerminalNode> ZADD::terminalNodeMemPool;
ZADDVariableOrdering ZADD::variableOrdering;
ZADDTerminalNode* ZADD::zeroDrain=createZeroDrain();
ZADDTerminalNode* ZADD::oneDrain=initAndReturnOneDrain();
unsigned long ZADD::deadNodes=0;
unsigned long ZADD::garbageCollectionDelay=10000;
bool ZADD::dynamicReordering=false;
float ZADD::dynamicReorderingMaxGrowth=MAX_GROWTH_DEFAULT;
unsigned long ZADD::lastReorderSize=0;
//TODO: config reader (also init hash unique table)
PtrHashMap4<ZADDBaseNode> ZADD::BoolComputedTable(256,1048576,0.8);
PtrHashMap3<ZADDBaseNode> ZADD::MoveComputedTable(1048576,1048576,0.8);
PtrHashMap4<ZADDBaseNode> ZADD::ArithmeticComputedTable(256,1048576,0.8);
PtrHashMap3<ZADDBaseNode> ZADD::AbstractComputedTable(1048576,1048576,0.8);
PtrHashMap4<ZADDBaseNode> ZADD::ExtremaComputedTable(1048576,1048576,0.8);
PtrHashMap4<ZADDBaseNode> ZADD::CompareComputedTable(1048576,1048576,0.8);
PtrHashMap5<ZADDBaseNode> ZADD::MatrixComputedTable(1048576,1048576,0.8);
void ZADD::garbageCollect(void){
  BoolComputedTable.clear();
  MoveComputedTable.clear();
  ArithmeticComputedTable.clear();
  AbstractComputedTable.clear();
  ExtremaComputedTable.clear();
  CompareComputedTable.clear();
  MatrixComputedTable.clear();

  for(unsigned long i=0;(i<variableOrdering.size()) && (deadNodes>0);++i){
    ZADDUniqueTable* uniqueT=variableOrdering.getData(i);
    unsigned long remainingDeadNodes=uniqueT->getDeadNodes();
    if(remainingDeadNodes>0){
      ZADDBaseNode* currentTable=uniqueT->getFirst();
      ZADDBaseNode* currentNode;
      ZADDBaseNode* tempNode;
      while(remainingDeadNodes){
        currentNode=currentTable;
        while(currentNode){
          tempNode=static_cast<ZADDInnerNode*>(currentNode)->getNext();
          if(currentNode->preNodeCount==ULONG_MAX){
            deleteSubTree(currentNode);
            if(--remainingDeadNodes==0) break;
          }
          currentNode=tempNode;
        }
        currentTable=uniqueT->next();
      }
      deadNodes-=uniqueT->getDeadNodes();
      uniqueT->resetDeadNodeCount();
    }
  }
  if(deadNodes>0){
    ZADDTerminalNodeUniqueTable* uniqueT=&ZADD::drainUniqueTable;
    unsigned long remainingDeadNodes=uniqueT->getDeadNodes();
    ZADDBaseNode* currentTable=uniqueT->getFirst();
    ZADDBaseNode* currentNode;
    ZADDBaseNode* tempNode;
    while(remainingDeadNodes){
      currentNode=currentTable;
      while(currentNode){
  tempNode=static_cast<ZADDTerminalNode*>(currentNode)->getNext();
  if(currentNode->preNodeCount==ULONG_MAX){
    deleteSubTree(currentNode);
    if(--remainingDeadNodes==0) break;
  }
  currentNode=tempNode;
      }
      currentTable=uniqueT->next();
    }
    deadNodes-=uniqueT->getDeadNodes();
    uniqueT->resetDeadNodeCount();
  }
}

unsigned long ZADD::size(void){
  unsigned long ret=0;

  for(unsigned long i=0;i<variableOrdering.size();++i) ret+=variableOrdering.getData(i)->size();
  ret+=drainUniqueTable.size();

  //do not count zero or one drain if they have no predecessors
  if(oneDrain->preNodeCount==1) --ret;
  if(zeroDrain->preNodeCount!=1) ++ret;

  return ret;
}

/**
* \brief This function returns the number of dead nodes
*/
unsigned long ZADD::countDeadNodes(void){
  return deadNodes;
}


unsigned long ZADD::setGarbageCollectionDelay(const unsigned long newSize){
  if(newSize==garbageCollectionDelay) return garbageCollectionDelay;

  unsigned long oldSize=garbageCollectionDelay;
  garbageCollectionDelay=newSize;
  if(deadNodes>=garbageCollectionDelay) garbageCollect();

  return oldSize;
}

bool ZADD::clear(void){
  //garbage collection
  garbageCollect();

  //count number of inner nodes
  unsigned long innerNodes=0;
  for(unsigned long i=0;i<variableOrdering.size();++i) innerNodes+=variableOrdering.getData(i)->size();

  if(innerNodes==0){
    variableOrdering.clear();

    return true;
  } else {
    bool ret=false;
    for(unsigned long i=variableOrdering.size();i>0;--i) ret|=variableOrdering.removeVariable(i-1);

    return ret;
  }
}

bool ZADD::swap(const unsigned long index){
  if UNLIKELY(index+1>=variableOrdering.size()) return false;

  ZADDBaseNode* w0;
  ZADDBaseNode* w1;
  ZADDBaseNode* w00;
  ZADDBaseNode* w01;
  ZADDBaseNode* w10;
  ZADDBaseNode* w11;

  ZADDUniqueTable* tempTable1=variableOrdering.getData(index);
  ZADDUniqueTable* tempTable2=variableOrdering.getData(index+1);
  ZADDInnerNode* currentTable=tempTable1->getFirst();
  ZADDInnerNode* currentNode;
  ZADDInnerNode* tempNode;
  while(currentTable){
    currentNode=currentTable;
    while(currentNode){
      tempNode=currentNode->getNext();

      w0=currentNode->getSucc0();
      if(w0->getLevel()==index+1){
  w00=static_cast<ZADDInnerNode*>(w0)->getSucc0();
  w01=static_cast<ZADDInnerNode*>(w0)->getSucc1();
  w1=currentNode->getSucc1();
  if(w1->getLevel()==index+1){
    w10=static_cast<ZADDInnerNode*>(w1)->getSucc0();
    w11=static_cast<ZADDInnerNode*>(w1)->getSucc1();
  } else w10=w11=w1;
  if(w00==w10) w0=w00;
  else w0=tempTable1->findOrAdd(w00,w10);

  if(w01==w11) w1=w11;
  else w1=tempTable1->findOrAdd(w01,w11);

  tempTable1->remove(currentNode);
  currentNode->setNode(tempTable2,w0,w1);
      } else {
  w1=currentNode->getSucc1();
  if(w1->getLevel()==index+1){
    w00=w01=w0;
    w10=static_cast<ZADDInnerNode*>(w1)->getSucc0();
    w11=static_cast<ZADDInnerNode*>(w1)->getSucc1();
    if(w00==w10) w0=w00;
    else w0=tempTable1->findOrAdd(w00,w10);

    if(w01==w11) w1=w11;
    else w1=tempTable1->findOrAdd(w01,w11);

    tempTable1->remove(currentNode);
    currentNode->setNode(tempTable2,w0,w1);
  }
      }
      currentNode=tempNode;
    }
    currentTable=tempTable1->next();
  }

  //swap variables
  variableOrdering.swapVariables(index);

  return true;
}


void ZADD::enableDynamicReordering() {
  dynamicReordering=true;
}

void ZADD::disableDynamicReordering() {
  dynamicReordering=false;
}

void ZADD::setDynamicReorderingMaxGrowth(float maxGrowth) {
  dynamicReorderingMaxGrowth=maxGrowth;
}

void ZADD::setUniqueTableMaxLoadFactor(float load_factor) {
  ZADDUniqueTable::max_load_factor=load_factor;
}

void ZADD::dynamicReorderHook() {
  if(dynamicReordering){
    ///TODO: methods to change the following two values 
    unsigned long min_reord_bdd_size = 100;
    unsigned long reord_trigger_factor = 2;
    
    unsigned long bdd_size = size();
    if ((bdd_size > min_reord_bdd_size) && (bdd_size > reord_trigger_factor*lastReorderSize)) {
      garbageCollect();
      bdd_size = size();
      //double check if the real size of the BDD grew or just the number of dead nodes
      if(bdd_size > min_reord_bdd_size && bdd_size > reord_trigger_factor*lastReorderSize) {
        ZADD::groupSifting(dynamicReorderingMaxGrowth, true);      
        lastReorderSize=size();
      }
    }
  }
}

void ZADD::groupSifting(const float maxGrowth, const bool external_garbage_collect){
  if (!external_garbage_collect) garbageCollect();
  
  unsigned long before=0, after=0;
  TRACE("ZADD", "reorder", before=ZADD::size();                     \
        cout << "Starting reordering: " << before << " nodes";     \
        cout.flush());
  
  ReorderHelper<ZADD> rH;
  Reordering<ReorderHelper<ZADD> > reord(&rH);
  reord.sifting(maxGrowth);
  
  TRACE("ZADD", "reorder", after=ZADD::size();                   \
        cout << ", afterwards " << after << " nodes, "         \
        << ((float)before-after)/(float)before*100.0 << "%\n");
  
  TRACE("ZADD", "reorder", if (after>before) {throw "BDD is bigger after reordering!";});
}
