/***************************************************************************
                          zadd.cpp  -  description
                             -------------------
    begin                : Wed Apr 14 2004
    copyright            : (C) 2004 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <zadd/zadd.h>
#include <zadd/zaddnode.h>
#include <zadd/zaddfunction.h>
#include <zadd/zaddgroup.h>
#include <zadd/zaddvarorder.h>
#include <zadd/zaddoutput.h>
#include <common/varorder.h>
#include <common/ptrhashmap.h>
#include <common/reordering.h>
#include <common/blifinput.h>
#include <common/matrixinput.h>
#include <common/configreader.h>

#include <stack>

using namespace std;

ConfigReader ZADDConfigReader("");

//init number of dead nodes
unsigned long ZADD::deadNodes=0;
//init garbage-collection delay
unsigned long ZADD::garbageCollectionDelay=ZADDConfigReader.getULong("garbage collection delay",GARBAGE_COLLECTION_DELAY_DEFAULT);
//init variableorder
ZADDVariableOrdering ZADD::variableOrdering=ZADDVariableOrdering();
//init hashmaps
PtrHashMap4 ZADD::ArithmeticComputedTable=PtrHashMap4(ZADDConfigReader.getULong("init arithmetic hash size",INIT_HASHMAP_SIZE),ZADDConfigReader.getULong("max arithmetic hash size",MAX_HASHMAP_SIZE),ZADDConfigReader.getDouble("arithmetic resize ratio",HASHMAP_RESIZE_RATIO));
PtrHashMap4 ZADD::ExtremaComputedTable=PtrHashMap4(ZADDConfigReader.getULong("init extrema hash size",INIT_HASHMAP_SIZE),ZADDConfigReader.getULong("max extrema hash size",MAX_HASHMAP_SIZE),ZADDConfigReader.getDouble("extrema resize ratio",HASHMAP_RESIZE_RATIO));
PtrHashMap3 ZADD::AbstractComputedTable=PtrHashMap3(ZADDConfigReader.getULong("init abstract hash size",INIT_HASHMAP_SIZE),ZADDConfigReader.getULong("max abstract hash size",MAX_HASHMAP_SIZE),ZADDConfigReader.getDouble("abstract resize ratio",HASHMAP_RESIZE_RATIO));
PtrHashMap4 ZADD::CompareComputedTable=PtrHashMap4(ZADDConfigReader.getULong("init compare hash size",INIT_HASHMAP_SIZE),ZADDConfigReader.getULong("max compare hash size",MAX_HASHMAP_SIZE),ZADDConfigReader.getDouble("compare resize ratio",HASHMAP_RESIZE_RATIO));
PtrHashMap5 ZADD::MatrixComputedTable=PtrHashMap5(ZADDConfigReader.getULong("init matrix hash size",INIT_HASHMAP_SIZE),ZADDConfigReader.getULong("max matrix hash size",MAX_HASHMAP_SIZE),ZADDConfigReader.getDouble("matrix resize ratio",HASHMAP_RESIZE_RATIO));
PtrHashMap4 ZADD::BoolComputedTable=PtrHashMap4(ZADDConfigReader.getULong("init bool hash size",INIT_HASHMAP_SIZE),ZADDConfigReader.getULong("max bool hash size",MAX_HASHMAP_SIZE),ZADDConfigReader.getDouble("bool resize ratio",HASHMAP_RESIZE_RATIO));
PtrHashMap3 ZADD::MoveComputedTable=PtrHashMap3(ZADDConfigReader.getULong("init move hash size",INIT_HASHMAP_SIZE),ZADDConfigReader.getULong("max move hash size",MAX_HASHMAP_SIZE),ZADDConfigReader.getDouble("move resize ratio",HASHMAP_RESIZE_RATIO));
PtrHashMap5 ZADD::SpecialComputedTable=PtrHashMap5(ZADDConfigReader.getULong("init equiv hash size",INIT_HASHMAP_SIZE),ZADDConfigReader.getULong("max equiv hash size",MAX_HASHMAP_SIZE),ZADDConfigReader.getDouble("equiv resize ratio",HASHMAP_RESIZE_RATIO));
//init reordering
Reordering<ZADDVarOrder,ZADDGroup,ZADD>* ZADD::reordering=0;
unsigned long ZADD::reorderingThreshold=ZADDConfigReader.getULong("reordering threshold",REORDERING_THRESHOLD);
unsigned long ZADD::operatorCounter=0;
bool ZADD::dynamicReordering=false;
//init zadd
ZADD* ZADD::BDD=0;
typedef ZADDNode* (ZADD::*functionPtr)(ZADDNode*, ZADDNode*, ZADDNode*, ZADDNode*);
functionPtr ZADD::currentAbstractFunction=0;
short ZADD::currentAbstractFunctionID=0;
unsigned long ZADD::swapcount=0;
//init drains
ZADDNode* ZADD::zeroDrain=0;
ZADDNode* ZADD::oneDrain=0;
//create ZADD
ZADD zadd;

inline ZADDNode* calcPtr(const ZADDNode* node, unsigned long op){return (ZADDNode*) ((unsigned long) node+op);}
inline void swapPointer(ZADDNode** u, ZADDNode** uVS, ZADDNode** v, ZADDNode** vVS){
  //swap nodes for more computed table hits
  if((*u)>(*v)){
    ZADDNode* temp=*u;
    *u=*v;
    *v=temp;
    temp=*uVS;
    *uVS=*vVS;
    *vVS=temp;
  }
}
inline void swapPointer(ZADDNode** u, ZADDNode** v){
  //swap nodes for more computed table hits
  if((*u)>(*v)){
    ZADDNode* temp=*u;
    *u=*v;
    *v=temp;
  }
}

//**********************************public**********************************
ZADD::ZADD(void){
  if(!ZADD::BDD){
    //init hash unique table sizes
    ZADDUniqueTable::setInitHashSize(ZADDConfigReader.getULong("init unique table hash size",INIT_UNIQUE_TABLE_SIZE_DEFAULT));
    ZADDUniqueTable::setMaxHashSize(ZADDConfigReader.getULong("max unique table hash size",MAX_UNIQUE_TABLE_SIZE_DEFAULT));
    //create reordering object
    reordering=new Reordering<ZADDVarOrder,ZADDGroup,ZADD>(&ZADD::swap,this);

    //create drains and save it (garbage-collection)
    //zero drain cannot be created with findOrAddDrain, because a value close to zero returns the zeroDrain and thus
    //no node would be created
    zeroDrain=ZADDNode::getNewNode();
    zeroDrain->setValue(0.0);
    zeroDrain->setDrainTable(variableOrdering.getDrainData());
    variableOrdering.getDrainData()->addValue(zeroDrain);
    zeroDrain->incNodeCount();
    oneDrain=findOrAddDrain(1.0);
    oneDrain->incNodeCount();

    //init first ZADD
    ZADD::BDD=this;

    //give objects the information which ZADD to use
    ZADDNode::BDD=this;
    ZADDFunction::BDD=this;
    ZADDGroup::BDD=this;
    ZADDGroup::varOrder=&variableOrdering;
    ZADDVarOrder::varOrder=&variableOrdering;
  }
}

ZADD::~ZADD(void){
  if(ZADD::BDD==this){
    //garbageCollect();

    delete reordering;
  }
}

bool ZADD::addVariable(const string& variable){
  return ZADDVarOrder::addVariable(variable);
}

bool ZADD::clear(void){
  //garbage collection
  garbageCollect();

  //count number of inner nodes
  unsigned long innerNodes=0;
  for(unsigned long i=0;i<variableOrdering.size();++i) innerNodes+=variableOrdering.getData(i)->size();

  if(innerNodes==0){
    variableOrdering.clear();

    return true;
  } else {
    bool ret=false;
    for(unsigned long i=variableOrdering.size();i>0;--i) ret|=variableOrdering.removeVariable(i-1);
    variableOrdering.clearGroups();

    return ret;
  }
}

/**
* \brief This function returns the number of nodes (including dead nodes).
*
* The returned number of nodes includes the number of dead nodes. To get the real number of
* nodes proceed a garbage collection before calling this function.
*/
unsigned long ZADD::size(void){
  unsigned long ret=0;

  for(unsigned long i=0;i<variableOrdering.size();++i) ret+=variableOrdering.getData(i)->size();
  ret+=variableOrdering.getDrainData()->size();

  //do not count zero or one drain if they have no predecessors
  if(zeroDrain->preNodeCount==1) --ret;
  if(oneDrain->preNodeCount==1) --ret;

  return ret;
}

/**
* \brief This function returns the peak number of nodes.
*
* The returned number of peak nodes does contain the number of drains.
*/
unsigned long ZADD::PEAK(void){
  return ZADDNode::PEAK();
}

void ZADD::printStat(void){
  ArithmeticComputedTable.printStat("Arithmetic Table");
  AbstractComputedTable.printStat("Abstract Table");
  CompareComputedTable.printStat("Compare Table");
  MatrixComputedTable.printStat("Matrix Table");
  BoolComputedTable.printStat("Bool Table");
  MoveComputedTable.printStat("Move Table");
  SpecialComputedTable.printStat("Special Table");
}

ZADDFunction ZADD::loadFunctionBLIF(const string& filename, const string& outputname){
  return BLIFInput<ZADD,ZADDFunction,ZADDFunctionContainer>::getOutput(filename,outputname);
}

ZADDFunctionContainer ZADD::loadFunctionsBLIF(const string& filename){
  return BLIFInput<ZADD,ZADDFunction,ZADDFunctionContainer>::getAllOutputs(filename);
}

ZADDFunction ZADD::loadMatrix(const string& filename, const unsigned short format){
  return MatrixInput<ZADDFunction,ZADDVarOrder>::loadMatrix(filename,format);
}

bool ZADD::storeFunction(const string& filename, const ZADDFunction& function){
  return ZADDOutput::storeFunction(filename,function);
}

bool ZADD::storeFunctions(const string& filename, const ZADDFunctionContainer& container){
  return ZADDOutput::storeFunctions(filename,container);
}

void ZADD::sifting(const float maxGrowth){
  ZADD::variableSifting(maxGrowth);
  ZADD::groupSifting(maxGrowth);
}

void ZADD::variableSifting(const float maxGrowth){
  reordering->variableSifting(maxGrowth);
}

void ZADD::groupSifting(const float maxGrowth){
  reordering->groupSifting(maxGrowth);
}

void ZADD::windowPermutation2(void){
  reordering->windowPermutation2();
}

void ZADD::windowPermutation3(void){
  reordering->windowPermutation3();
}

void ZADD::windowPermutation4(void){
  reordering->windowPermutation4();
}

void ZADD::geneticMinimize(void){
  reordering->geneticMinimize();
}

void ZADD::enableDynamicReordering(void){
  dynamicReordering=true;
}

void ZADD::disableDynamicReordering(void){
  dynamicReordering=false;
}

void ZADD::garbageCollect(void){
  CompareComputedTable.clear();
  ArithmeticComputedTable.clear();
  AbstractComputedTable.clear();
  ExtremaComputedTable.clear();
  MatrixComputedTable.clear();
  BoolComputedTable.clear();
  MoveComputedTable.clear();
  SpecialComputedTable.clear();

  ZADDUniqueTable* uniqueT;
  for(unsigned long i=0;(i<=variableOrdering.size()) && (deadNodes>0);++i){
    if(i==variableOrdering.size()) uniqueT=variableOrdering.getDrainData();
    else uniqueT=variableOrdering.getData(i);
    if(uniqueT->getDeadNodes()>0){
      unsigned long remainingDeadNodes=uniqueT->getDeadNodes();
      ZADDNode* currentTable=(ZADDNode*) uniqueT->getFirst();
      ZADDNode* currentNode;
      ZADDNode* tempNode;
      while(remainingDeadNodes){
        currentNode=currentTable;
        while(currentNode){
          tempNode=currentNode->getNext();
          if(currentNode->preNodeCount==ULONG_MAX){
            BDD->deleteSubTree(currentNode);
            --remainingDeadNodes;
          }
          currentNode=tempNode;
        }
        currentTable=(ZADDNode*) uniqueT->next();
      }
      deadNodes-=uniqueT->getDeadNodes();
      uniqueT->resetDeadNodeCount();
    }
  }
  deadNodes=0;
}

unsigned long ZADD::setGarbageCollectionDelay(void){
  return garbageCollectionDelay;
}

unsigned long ZADD::setGarbageCollectionDelay(const unsigned long newSize){
  if(newSize==garbageCollectionDelay) return garbageCollectionDelay;

  unsigned long oldSize=garbageCollectionDelay;
  garbageCollectionDelay=newSize;
  if(deadNodes>=garbageCollectionDelay) garbageCollect();

  return oldSize;
}

ZADDFunction ZADD::projection(const string& variable){
  //append label if variableOrdering has no such variable
  unsigned long index=variableOrdering[variable];
  if(index==ULONG_MAX){
    if(variable.find_first_of(badChars)!=string::npos) return ZADDFunction::ERROR();
    if(!variableOrdering.appendVariable(variable)) return ZADDFunction::ERROR();
    index=variableOrdering.size()-1;
  }

  ZADDNode* node=findOrAdd(variableOrdering.getData(index),oneDrain,zeroDrain);
  return createFunction(node,node);
}

ZADDFunction ZADD::projection(const unsigned long index){
  if(index>=variableOrdering.size()) return ZADDFunction::ERROR();

  ZADDNode* node=findOrAdd(variableOrdering.getData(index),oneDrain,zeroDrain);
  return createFunction(node,node);
}

ZADDFunction ZADD::negate(const ZADDFunction& function){
  return ((!function.rootNode)||(!function.varSet))?ZADDFunction::ERROR():createFunction(subtraction(zeroDrain,oneDrain,function.rootNode,function.varSet),function.varSet);
}

ZADDFunction ZADD::addition(const ZADDFunction& function1, const ZADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode)||(!function1.varSet)||(!function2.varSet))?ZADDFunction::ERROR():createFunction(addition(function1.rootNode,function1.varSet,function2.rootNode,function2.varSet),calcUnion(function1.varSet,function2.varSet));
}

ZADDFunction ZADD::sum(const ZADDFunction& function, const ZADDFunction& varSet){
  if((!function.rootNode)||(!function.varSet)||(!varSet.rootNode)||(!varSet.varSet)) return ZADDFunction::ERROR();

  currentAbstractFunction=&ZADD::addition;
  currentAbstractFunctionID=SUM_OP;

  return createFunction(abstract(function.rootNode,function.varSet,varSet.rootNode),calcSetMinus(function.varSet,varSet.rootNode));
}

ZADDFunction ZADD::subtraction(const ZADDFunction& function1, const ZADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode)||(!function1.varSet)||(!function2.varSet))?ZADDFunction::ERROR():createFunction(subtraction(function1.rootNode,function1.varSet,function2.rootNode,function2.varSet),calcUnion(function1.varSet,function2.varSet));
}

ZADDFunction ZADD::multiplication(const ZADDFunction& function1, const ZADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode)||(!function1.varSet)||(!function2.varSet))?ZADDFunction::ERROR():createFunction(multiplication(function1.rootNode,function1.varSet,function2.rootNode,function2.varSet),calcUnion(function1.varSet,function2.varSet));
}

ZADDFunction ZADD::product(const ZADDFunction& function, const ZADDFunction& varSet){
  if((!function.rootNode)||(!function.varSet)||(!varSet.rootNode)||(!varSet.varSet)) return ZADDFunction::ERROR();

  currentAbstractFunction=&ZADD::multiplication;
  currentAbstractFunctionID=PRO_OP;

  return createFunction(abstract(function.rootNode,function.varSet,varSet.rootNode),calcSetMinus(function.varSet,varSet.rootNode));
}

ZADDFunction ZADD::MM(const ZADDFunction& A, const ZADDFunction& B, const ZADDFunction& varSet){
  if((!A.rootNode)||(!B.rootNode)||(!varSet.rootNode)) return ZADDFunction::ERROR();

  unsigned long m=varSet.isConstant()?0:varSet.size()-2;

  return createFunction(MM(A.rootNode,A.varSet,B.rootNode,B.varSet,varSet.rootNode,m),calcUnion(A.varSet,B.varSet));
}

ZADDFunction ZADD::MV(const ZADDFunction& A, const ZADDFunction& b, const ZADDFunction& varSet){
  if((!A.rootNode)||(!b.rootNode)||(!varSet.rootNode)) return ZADDFunction::ERROR();

  unsigned long m=varSet.isConstant()?0:varSet.size()-2;

  return createFunction(MV(A.rootNode,A.varSet,b.rootNode,b.varSet,varSet.rootNode,m),varSet.rootNode);
}

ZADDFunction ZADD::VM(const ZADDFunction& b, const ZADDFunction& A, const ZADDFunction& varSet){
  if((!A.rootNode)||(!b.rootNode)||(!varSet.rootNode)) return ZADDFunction::ERROR();

  unsigned long m=varSet.isConstant()?0:varSet.size()-2;

  return createFunction(VM(b.rootNode,b.varSet,A.rootNode,A.varSet,varSet.rootNode,m),varSet.rootNode);
}

ZADDFunction ZADD::equivalenceClasses(const ZADDFunction& u, const ZADDFunction& e, const ZADDFunction& c){
  return ((!u.rootNode)||(!e.rootNode)||(!c.rootNode))?ZADDFunction::ERROR():createFunction(equivalenceClasses(u.rootNode,u.varSet,e.rootNode,e.varSet,c.rootNode),calcSetMinus(u.varSet,c.rootNode));
}

ZADDFunction ZADD::constrain(const ZADDFunction& f, const ZADDFunction& g){
  return ((!f.rootNode)||(!g.rootNode))?ZADDFunction::ERROR():createFunction(constrain(f.rootNode,g.rootNode),zeroDrain);
}

ZADDFunction ZADD::ITE(const ZADDFunction& i, const ZADDFunction& t, const ZADDFunction& e){
  return ZADDFunction::ERROR();//return ((!i.rootNode)||(!t.rootNode)||(!e.rootNode))?ZADDFunction::ERROR():createFunction(ITE(i.rootNode,t.rootNode,e.rootNode));
}

ZADDFunction ZADD::NOT(const ZADDFunction& function){
  return ((!function.rootNode)||(!function.varSet))?ZADDFunction::ERROR():createFunction(NOT(function.rootNode,function.varSet),function.varSet);
}

ZADDFunction ZADD::AND(const ZADDFunction& function1, const ZADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode)||(!function1.varSet)||(!function2.varSet))?ZADDFunction::ERROR():createFunction(AND(function1.rootNode,function1.varSet,function2.rootNode,function2.varSet),calcUnion(function1.varSet,function2.varSet));
}

ZADDFunction ZADD::OR(const ZADDFunction& function1, const ZADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode)||(!function1.varSet)||(!function2.varSet))?ZADDFunction::ERROR():createFunction(OR(function1.rootNode,function1.varSet,function2.rootNode,function2.varSet),calcUnion(function1.varSet,function2.varSet));
}

ZADDFunction ZADD::cofactor(const ZADDFunction& function, const unsigned long index, const bool value){
  if ((index>=variableOrdering.size())||(!function.rootNode)||(!function.varSet)) return function;

  //create temporary function
  ZADDFunction tempFunction=projection(index);
  if(!value) tempFunction=!tempFunction;

  return cofactor(function,tempFunction);
}

ZADDFunction ZADD::cofactor(const ZADDFunction& function, const ZADDFunction& assignment){
  return ((!function.rootNode)||(!function.varSet)||(!assignment.rootNode)||(!assignment.varSet))?ZADDFunction::ERROR():createFunction(cofactor(function.rootNode,function.varSet,assignment.rootNode,assignment.varSet),calcSetMinus(function.varSet,assignment.varSet));
}

ZADDFunction ZADD::change(const ZADDFunction& function){
  return (!function.rootNode)?ZADDFunction::ERROR():createFunction(change(function.rootNode,function.varSet),function.varSet);
}

ZADDFunction ZADD::change(const ZADDFunction& function, const unsigned long index){
  if((index>=variableOrdering.size())||(!function.rootNode)) return function;

  //create temporary node
  ZADDFunction tempProj=ZADDFunction::projection(index);
  ZADDFunction tempFunction=createFunction(change(function.rootNode,function.varSet,tempProj.rootNode),function.varSet);

  return tempFunction;
}

ZADDFunction ZADD::change(const ZADDFunction& function, const ZADDFunction& cubeSet){
  return ((!function.rootNode)||(!cubeSet.rootNode))?ZADDFunction::ERROR():createFunction(change(function.rootNode,function.varSet,cubeSet.rootNode),function.varSet);
}

ZADDFunction ZADD::exists(const ZADDFunction& function, const ZADDFunction& varSet){
  if((!function.rootNode)||(!varSet.rootNode)||(!function.varSet)) return ZADDFunction::ERROR();
  return createFunction(exists(function.rootNode,function.varSet,varSet.rootNode),calcSetMinus(function.varSet,varSet.rootNode));
}

ZADDFunction ZADD::exists(const ZADDFunction& function, const unsigned long index){
  return ((index>=variableOrdering.size())||(!function.rootNode))?function:(cofactor(function,index,0)|cofactor(function,index,1));
}

ZADDFunction ZADD::forall(const ZADDFunction& function, const ZADDFunction& varSet){
  return !exists(!function,varSet);
}

ZADDFunction ZADD::forall(const ZADDFunction& function, const unsigned long index){
  return ((index>=variableOrdering.size())||(!function.rootNode))?function:(cofactor(function,index,0)&cofactor(function,index,1));
}

ZADDFunction ZADD::less(const ZADDFunction& function1, const ZADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode))?ZADDFunction::ERROR():createFunction(less(function1.rootNode,function1.varSet,function2.rootNode,function2.varSet),calcUnion(function1.varSet,function2.varSet));
}

ZADDFunction ZADD::lessEqual(const ZADDFunction& function1, const ZADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode))?ZADDFunction::ERROR():createFunction(lessEqual(function1.rootNode,function1.varSet,function2.rootNode,function2.varSet),calcUnion(function1.varSet,function2.varSet));
}

ZADDFunction ZADD::equal(const ZADDFunction& function1, const ZADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode))?ZADDFunction::ERROR():createFunction(equal(function1.rootNode,function1.varSet,function2.rootNode,function2.varSet),calcUnion(function1.varSet,function2.varSet));
}

ZADDFunction ZADD::maximum(const ZADDFunction& function, const ZADDFunction& varSet){
  if((!function.rootNode)||(!varSet.rootNode)) return ZADDFunction::ERROR();

  currentAbstractFunction=&ZADD::max;
  currentAbstractFunctionID=MAX_OP;

  return createFunction(idempotentAbstract(function.rootNode,function.varSet,varSet.rootNode),calcSetMinus(function.varSet,varSet.rootNode));
}

ZADDFunction ZADD::max(const ZADDFunction& function1, const ZADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode))?ZADDFunction::ERROR():createFunction(max(function1.rootNode,function1.varSet,function2.rootNode,function2.varSet),calcUnion(function1.varSet,function2.varSet));
}

double ZADD::max(const ZADDFunction& function){
  map<ZADDNode*,double> CT;

  return (!function.rootNode)?0.0:max(function.rootNode,CT);
}

ZADDFunction ZADD::minimum(const ZADDFunction& function, const ZADDFunction& varSet){
  if((!function.rootNode)||(!varSet.rootNode)) return ZADDFunction::ERROR();

  currentAbstractFunction=&ZADD::min;
  currentAbstractFunctionID=MIN_OP;

  return createFunction(idempotentAbstract(function.rootNode,function.varSet,varSet.rootNode),calcSetMinus(function.varSet,varSet.rootNode));
}

ZADDFunction ZADD::min(const ZADDFunction& function1, const ZADDFunction& function2){
  return ((!function1.rootNode)||(!function2.rootNode))?ZADDFunction::ERROR():createFunction(min(function1.rootNode,function1.varSet,function2.rootNode,function2.varSet),calcUnion(function1.varSet,function2.varSet));
}

double ZADD::min(const ZADDFunction& function){
  map<ZADDNode*,double> CT;

  return (!function.rootNode)?0.0:min(function.rootNode,CT);
}

ZADDFunction ZADD::oneOver(const ZADDFunction& function){
  return (!function.rootNode)?ZADDFunction::ERROR():createFunction(oneOver(function.rootNode),function.varSet);
}

ZADDFunction ZADD::moveUp(const ZADDFunction& function, const unsigned int skip){
  if(!function.rootNode) return ZADDFunction::ERROR();
  if(function.isConstant()) return function;
  if(skip==0) return function;
  if(function.rootNode->getLevel()<skip) return function;

  if(skip==1) return createFunction(moveUp(function.rootNode),moveUp(function.varSet));
  else return createFunction(moveUp(function.rootNode,skip),moveUp(function.varSet,skip));
}

ZADDFunction ZADD::moveUp(const ZADDFunction& function, const ZADDFunction& set, const unsigned int skip){
  if((!function.rootNode)||(!set.rootNode)) return ZADDFunction::ERROR();
  if(function.isConstant()) return function;
  if(skip==0) return function;

  if(skip==1) return createFunction(moveUp(function.rootNode,set.rootNode),moveUp(function.varSet,set.rootNode));
  else return createFunction(moveUp(function.rootNode,set.rootNode,skip),moveUp(function.varSet,set.rootNode,skip));
}

ZADDFunction ZADD::moveDown(const ZADDFunction& function, const unsigned int skip){
  if(!function.rootNode) return ZADDFunction::ERROR();
  if(function.isConstant()) return function;
  if(skip==0) return function;

  if(skip==1) return createFunction(moveDown(function.rootNode),moveDown(function.varSet));
  else return createFunction(moveDown(function.rootNode,skip),moveDown(function.varSet,skip));
}

ZADDFunction ZADD::moveDown(const ZADDFunction& function, const ZADDFunction& set, const unsigned int skip){
  if((!function.rootNode)||(!set.rootNode)) return ZADDFunction::ERROR();
  if(function.isConstant()) return function;
  if(skip==0) return function;

  if(skip==1) return createFunction(moveDown(function.rootNode,set.rootNode),moveDown(function.varSet,set.rootNode));
  else return createFunction(moveDown(function.rootNode,set.rootNode,skip),moveDown(function.varSet,set.rootNode,skip));
}

ZADDFunction ZADD::getAssignment(const ZADDFunction& function){
  if(!function.rootNode) return ZADDFunction::ERROR();

  return createFunction(getAssignment(function.rootNode),calcAllVars());
}

void ZADD::getAssignment(const ZADDFunction& function, char* buffer) const {
  //initially, every variable gets the value false
  for(unsigned long i=0;i<variableOrdering.size();++i) buffer[i]='0';

  if(!function.rootNode) return;

  getAssignment(function.rootNode,buffer);
}

double ZADD::countAssignments(const ZADDFunction& function) const {
  if(!function.rootNode) return 0.0;

  map<ZADDNode*,double> CT;
  return countAssignments(0,function.rootNode,function.varSet,CT);
}

ZADDFunction ZADD::getPath(const ZADDFunction& function){
  if(!function.rootNode) return function;
  return createFunction(getPath(function.rootNode),0);
}

void ZADD::getPath(const ZADDFunction& function, char* buffer) const {
  //initially, every variable is a don't care symbol
  for(unsigned long i=0;i<variableOrdering.size();++i) buffer[i]='-';

  if(!function.rootNode) return;

  getPath(function.rootNode,buffer);
}

//********************************protected**********************************
bool ZADD::swap(const unsigned long index){
  if(index+1>=variableOrdering.size()) return false;

  ZADDNode* w00;
  ZADDNode* w01;
  ZADDNode* w10;
  ZADDNode* w11;
  ZADDNode* temp0;
  ZADDNode* temp1;
  ZADDNode* firstListElement=0;

  ZADDUniqueTable* tempTable1=variableOrdering.getData(index);
  ZADDUniqueTable* tempTable2=variableOrdering.getData(index+1);
  ZADDNode* currentTable=(ZADDNode*) tempTable1->getFirst();
  ZADDNode* currentNode;
  ZADDNode* tempNode;
  while(currentTable){
    currentNode=currentTable;
    while(currentNode){
      tempNode=currentNode->getNext();

      if((currentNode->getSucc0()->getLevel()==index+1)||(currentNode->getSucc1()->getLevel()==index+1)){
        currentNode->removeFromUniqueTable();
        currentNode->setNext(firstListElement);
        firstListElement=currentNode;
      }
      currentNode=tempNode;
    }
    currentTable=(ZADDNode*) tempTable1->next();
  }

  currentNode=firstListElement;
  ZADDNode* w0;
  ZADDNode* w1;
  while(currentNode){
    tempNode=currentNode->getNext();
    w0=currentNode->getSucc0();
    w1=currentNode->getSucc1();

    if(w0->getLevel()==index+1){
      w00=w0->getSucc0();
      w01=w0->getSucc1();
    } else {
      w00=w01=w0;
    }

    if(w1->getLevel()==index+1){
      w10=w1->getSucc0();
      w11=w1->getSucc1();
    } else {
      w10=w11=w1;
    }

    if(w00==w10) temp0=w00;
    else temp0=findOrAdd(tempTable1,w10,w00);

    if(w01==w11) temp1=w11;
    else temp1=findOrAdd(tempTable1,w11,w01);

    currentNode->setNode(tempTable2,temp0,temp1);
    currentNode=tempNode;
  }

  //swap variables
  variableOrdering.swapVariables(index);

  ++swapcount;

  return true;
}

void ZADD::deleteSubTree(ZADDNode* node){
  if(!node->isDrain()){
    if(--node->type.children.succ0->preNodeCount==0) deleteSubTree(node->type.children.succ0);
    if(--node->type.children.succ1->preNodeCount==0) deleteSubTree(node->type.children.succ1);
  }

  //reset node
  node->removeFromUniqueTable();

  //free mem for node
  ZADDNode::freeNode(node);
}

void ZADD::deleteNode(ZADDNode* node){
  --node->type.children.succ0->preNodeCount;
  --node->type.children.succ1->preNodeCount;
  node->removeFromUniqueTable();
  ZADDNode::freeNode(node);
}

//**********************************private**********************************
ZADDNode* ZADD::calcUnion(ZADDNode* node1, ZADDNode* node2){
  if(node1==node2) return node1;
  if(node1->isDrain()) return node2;
  if(node2->isDrain()) return node1;

  ZADDNode* node=(ZADDNode*) BoolComputedTable.find(calcPtr(node1,AND_OP),node1,node2,node2);
  if(node) return node;

  ZADDUniqueTable* uniqueTable;
  ZADDNode* n1;
  ZADDNode* n2;
  if(node1->getRealLevel()<=node2->getRealLevel()){
    uniqueTable=node1->getUniqueTable();
    if(node2->getUniqueTable()==uniqueTable){
      n1=node1->getSucc1();
      n2=node2->getSucc1();
    } else {
      n1=node1->getSucc1();
      n2=node2;
    }
  } else {
    uniqueTable=node2->getUniqueTable();
    n1=node1;
    n2=node2->getSucc1();
  }

  node=findOrAdd(uniqueTable,calcUnion(n1,n2),zeroDrain);

  //insert result into computed table
  BoolComputedTable.insert(calcPtr(node1,AND_OP),node1,node2,node2,node);

  return node;
}

ZADDNode* ZADD::calcSetMinus(ZADDNode* node1, ZADDNode* node2){
  if(node1==node2) return oneDrain;
  if(node1->isDrain()) return oneDrain;
  if(node2->isDrain()) return node1;

  ZADDNode* temp=(ZADDNode*) BoolComputedTable.find(calcPtr(node1,EXI_OP),node1,node2,0);
  if(temp) return temp;

  while(node1->getRealLevel()>node2->getRealLevel()) node2=node2->getSucc1();

  if(node1->getRealLevel()==node2->getRealLevel()) temp=calcSetMinus(node1->getSucc1(),node2->getSucc1());
  else temp=findOrAdd(node1->getUniqueTable(),calcSetMinus(node1->getSucc1(),node2),zeroDrain);

  BoolComputedTable.insert(calcPtr(node1,EXI_OP),node1,node2,0,temp);

  return temp;
}

ZADDNode* ZADD::calcAllVars(void){
  ZADDNode* tempVar=oneDrain;
  for(unsigned long i=variableOrdering.size();i>0;--i) tempVar=findOrAdd(i-1,tempVar,zeroDrain);

  return tempVar;
}

ZADDNode* ZADD::calcLiftedBDD(ZADDNode* function, ZADDNode* functionVS, ZADDNode* varSet){
  if(varSet==oneDrain) return function;
  if(function==zeroDrain) return zeroDrain;

  ZADDNode* temp=(ZADDNode*) BoolComputedTable.find(calcPtr(function,OR_OP),functionVS,zeroDrain,varSet);
  if(temp) return temp;

  if(varSet->getRealLevel()<=functionVS->getRealLevel()){
    if(varSet->getUniqueTable()==functionVS->getUniqueTable()){
      if(function->getUniqueTable()==functionVS->getUniqueTable()){
        ZADDNode* w0;
        ZADDNode* w1;

        w0=calcLiftedBDD(function->getSucc0(),functionVS->getSucc1(),varSet->getSucc1());
        w1=calcLiftedBDD(function->getSucc1(),functionVS->getSucc1(),varSet->getSucc1());

        temp=(w1==zeroDrain)?w0:findOrAdd(function->getUniqueTable(),w1,w0);
      } else {
        temp=calcLiftedBDD(function,functionVS->getSucc1(),varSet->getSucc1());
      }
    } else {
      temp=calcLiftedBDD(function,functionVS,varSet->getSucc1());
      temp=findOrAdd(varSet->getUniqueTable(),temp,temp);
    }
  } else {
    if(function->getUniqueTable()==functionVS->getUniqueTable()){
      ZADDNode* w0;
      ZADDNode* w1;

      w0=calcLiftedBDD(function->getSucc0(),functionVS->getSucc1(),varSet);
      w1=calcLiftedBDD(function->getSucc1(),functionVS->getSucc1(),varSet);

      temp=(w1==zeroDrain)?w0:findOrAdd(function->getUniqueTable(),w1,w0);
    } else {
      temp=calcLiftedBDD(function,functionVS->getSucc1(),varSet);
    }
  }

  BoolComputedTable.insert(calcPtr(function,OR_OP),functionVS,zeroDrain,varSet,temp);

  return temp;
}

bool ZADD::equiv(ZADDNode* u, ZADDNode* uVS, ZADDNode* v, ZADDNode* vVS){
  if(u==0) return (v==0);
  if(v==0) return (u==0);

  if((uVS->isDrain())&&(vVS->isDrain())) return (u==v);

  if(uVS->getRealLevel()<vVS->getRealLevel()){
    if(u->getUniqueTable()!=uVS->getUniqueTable()) return false; //zero suppressed vs don't care
    if(u->getSucc0()!=u->getSucc1()) return false;
    return equiv(u->getSucc0(),uVS->getSucc1(),v,vVS);
  }

  if(vVS->getRealLevel()<uVS->getRealLevel()){
    if(v->getUniqueTable()!=vVS->getUniqueTable()) return false; //zero suppressed vs don't care
    if(v->getSucc0()!=v->getSucc1()) return false;
    return equiv(u,uVS,v->getSucc0(),vVS->getSucc1());
  }

  if(u->getUniqueTable()!=v->getUniqueTable()) return false;

  if(u->getUniqueTable()==uVS->getUniqueTable()){
    if(!equiv(u->getSucc0(),uVS->getSucc1(),v->getSucc0(),vVS->getSucc1())) return false;
    return equiv(u->getSucc1(),uVS->getSucc1(),v->getSucc1(),vVS->getSucc1());
  } else {
    return equiv(u,uVS->getSucc1(),v,vVS->getSucc1());
  }
}

ZADDNode* ZADD::abstract(ZADDNode* node, ZADDNode* nodeVarSet, ZADDNode* varSet){
  if(varSet->isDrain()) return node;
	
  ZADDNode* temp=(ZADDNode*) AbstractComputedTable.find(calcPtr(node,currentAbstractFunctionID),nodeVarSet,varSet);
  if(temp) return temp;
	
  ZADDNode* w0;
  ZADDNode* w1;
  ZADDNode* n0;
  ZADDNode* n1;
  if(node->getUniqueTable()==nodeVarSet->getUniqueTable()){
    n0=node->getSucc0();
    n1=node->getSucc1();
  } else {
    n0=node;
    n1=zeroDrain;
  }
	
  if(varSet->getUniqueTable()==nodeVarSet->getUniqueTable()){
    w0=abstract(n0,nodeVarSet->getSucc1(),varSet->getSucc1());
    w1=abstract(n1,nodeVarSet->getSucc1(),varSet->getSucc1());
		
    ZADDNode* tempVarSet=calcSetMinus(nodeVarSet,varSet);
		
    temp=(this->*currentAbstractFunction)(w0,tempVarSet,w1,tempVarSet);
		
    //prevent from garbage collection
    temp->incNodeCount();
    if(w0->preNodeCount==0) w0->garbageCollect();
    if(w1->preNodeCount==0) w1->garbageCollect();
    //reverse protection
    --temp->preNodeCount;
		
    if(tempVarSet->preNodeCount==0) tempVarSet->garbageCollect();
		
    AbstractComputedTable.insert(calcPtr(node,currentAbstractFunctionID),nodeVarSet,varSet,temp);
		
    return temp;
  } else {
    if(varSet->getRealLevel()<nodeVarSet->getRealLevel()){
      w0=abstract(node,nodeVarSet,varSet->getSucc1());
      temp=(this->*currentAbstractFunction)(w0,nodeVarSet,w0,nodeVarSet);
      //prevent from garbage collection
      temp->incNodeCount();
      if(w0->preNodeCount==0) w0->garbageCollect();
      //reverse protection
      --temp->preNodeCount;
			AbstractComputedTable.insert(calcPtr(node,currentAbstractFunctionID),nodeVarSet,varSet,temp);
			
			return temp;
		} else {
      w0=abstract(n0,nodeVarSet->getSucc1(),varSet);
      w1=abstract(n1,nodeVarSet->getSucc1(),varSet);
			
      temp=(w1==zeroDrain)?w0:findOrAdd(node->getUniqueTable(),w1,w0);
			AbstractComputedTable.insert(calcPtr(node,currentAbstractFunctionID),nodeVarSet,varSet,temp);
			
			return temp;
		}
  }
}

ZADDNode* ZADD::idempotentAbstract(ZADDNode* node, ZADDNode* nodeVarSet, ZADDNode* varSet){
  if(node->isDrain()) return node;
	
  unsigned long nodeVarSetLevel=nodeVarSet->getRealLevel();
  while(varSet->getRealLevel()<nodeVarSetLevel) varSet=varSet->getSucc1();
	
  if(varSet->isDrain()) return node;
	
  ZADDNode* temp=(ZADDNode*) AbstractComputedTable.find(calcPtr(node,currentAbstractFunctionID),nodeVarSet,varSet);
  if(temp) return temp;
	
  ZADDNode* w0;
  ZADDNode* w1;
  ZADDNode* n0;
  ZADDNode* n1;
  if(node->getUniqueTable()==nodeVarSet->getUniqueTable()){
    n0=node->getSucc0();
    n1=node->getSucc1();
  } else {
    n0=node;
    n1=zeroDrain;
  }
	
  if(varSet->getUniqueTable()==nodeVarSet->getUniqueTable()){
    w0=idempotentAbstract(n0,nodeVarSet->getSucc1(),varSet->getSucc1());
    w1=idempotentAbstract(n1,nodeVarSet->getSucc1(),varSet->getSucc1());
		
    ZADDNode* tempVarSet=calcSetMinus(nodeVarSet,varSet);
		
    temp=(this->*currentAbstractFunction)(w0,tempVarSet,w1,tempVarSet);
		
    //prevent from garbage collection
    temp->incNodeCount();
    if(w0->preNodeCount==0) w0->garbageCollect();
    if(w1->preNodeCount==0) w1->garbageCollect();
    //reverse protection
    --temp->preNodeCount;
		
    if(tempVarSet->preNodeCount==0) tempVarSet->garbageCollect();
		
    AbstractComputedTable.insert(calcPtr(node,currentAbstractFunctionID),nodeVarSet,varSet,temp);
		
    return temp;
  } else {
    w0=idempotentAbstract(n0,nodeVarSet->getSucc1(),varSet);
    w1=idempotentAbstract(n1,nodeVarSet->getSucc1(),varSet);
		
    temp=(w1==zeroDrain)?w0:findOrAdd(node->getUniqueTable(),w1,w0);
    AbstractComputedTable.insert(calcPtr(node,currentAbstractFunctionID),nodeVarSet,varSet,temp);
		
    return temp;
  }
}

ZADDNode* ZADD::exists(ZADDNode* node, ZADDNode* nodeVarSet, ZADDNode* varSet){
  if(node->isDrain()) return node;

  unsigned long nodeVarSetLevel=nodeVarSet->getRealLevel();
  while(varSet->getRealLevel()<nodeVarSetLevel) varSet=varSet->getSucc1();

  if(varSet->isDrain()) return node;

  ZADDNode* temp=(ZADDNode*) BoolComputedTable.find(calcPtr(node,EXI_OP),nodeVarSet,varSet,0);
  if(temp) return temp;

  ZADDNode* w0;
  ZADDNode* w1;
  ZADDNode* n0;
  ZADDNode* n1;
  if(node->getUniqueTable()==nodeVarSet->getUniqueTable()){
    n0=node->getSucc0();
    n1=node->getSucc1();
  } else {
    n0=node;
    n1=zeroDrain;
  }

  if(varSet->getUniqueTable()==nodeVarSet->getUniqueTable()){
    w0=exists(n0,nodeVarSet->getSucc1(),varSet->getSucc1());
    w1=exists(n1,nodeVarSet->getSucc1(),varSet->getSucc1());

    ZADDNode* tempVarSet=calcSetMinus(nodeVarSet,varSet);

    temp=OR(w0,tempVarSet,w1,tempVarSet);

    //prevent from garbage collection
    temp->incNodeCount();
    if(w0->preNodeCount==0) w0->garbageCollect();
    if(w1->preNodeCount==0) w1->garbageCollect();
    //reverse protection
    --temp->preNodeCount;

    if(tempVarSet->preNodeCount==0) tempVarSet->garbageCollect();

    BoolComputedTable.insert(calcPtr(node,EXI_OP),nodeVarSet,varSet,0,temp);

    return temp;
  } else {
    w0=exists(n0,nodeVarSet->getSucc1(),varSet);
    w1=exists(n1,nodeVarSet->getSucc1(),varSet);

    temp=(w1==zeroDrain)?w0:findOrAdd(node->getUniqueTable(),w1,w0);
    BoolComputedTable.insert(calcPtr(node,EXI_OP),nodeVarSet,varSet,0,temp);

    return temp;
  }
}

ZADDNode* ZADD::addition(ZADDNode* v1, ZADDNode* v1VS, ZADDNode* v2, ZADDNode* v2VS){
  if(v1VS==v2VS) return addition(v1,v2);
  if(v1==zeroDrain) return calcLiftedBDD(v2,v2VS,v1VS);
  if(v2==zeroDrain) return calcLiftedBDD(v1,v1VS,v2VS);

  swapPointer(&v1,&v1VS,&v2,&v2VS);

  //try to get result from computed table
  ZADDNode* node=(ZADDNode*) ArithmeticComputedTable.find(calcPtr(v1,ADD_OP),v1VS,v2,v2VS);
  if(node) return node;

  ZADDNode* w0;
  ZADDNode* w1;
  ZADDNode* v10;
  ZADDNode* v11;
  ZADDNode* v20;
  ZADDNode* v21;
  ZADDNode* newV1VS;
  ZADDNode* newV2VS;

  ZADDUniqueTable* minTable;

  if(v1VS->getRealLevel()<=v2VS->getRealLevel()){
    minTable=v1VS->getUniqueTable();
    if(v1->getUniqueTable()==minTable){
      v10=v1->getSucc0();
      v11=v1->getSucc1();
    } else {
      v10=v1;
      v11=zeroDrain;
    }
    newV1VS=v1VS->getSucc1();
    if(v2VS->getUniqueTable()==minTable){
      if(v2->getUniqueTable()==minTable){
        v20=v2->getSucc0();
        v21=v2->getSucc1();
      } else {
        v20=v2;
        v21=zeroDrain;
      }
      newV2VS=v2VS->getSucc1();
    } else {
      newV2VS=v2VS;
      v20=v21=v2;
    }
  } else {
    minTable=v2VS->getUniqueTable();
    v10=v11=v1;
    if(v2->getUniqueTable()==minTable){
      v20=v2->getSucc0();
      v21=v2->getSucc1();
    } else {
      v20=v2;
      v21=zeroDrain;
    }
    newV1VS=v1VS;
    newV2VS=v2VS->getSucc1();
  }

  w0=addition(v10,newV1VS,v20,newV2VS);
  w1=addition(v11,newV1VS,v21,newV2VS);

  node=(w1==zeroDrain)?w0:findOrAdd(minTable,w1,w0);

  //insert result into computed table
  ArithmeticComputedTable.insert(calcPtr(v1,ADD_OP),v1VS,v2,v2VS,node);

  return node;
}

ZADDNode* ZADD::addition(ZADDNode* n1, ZADDNode* n2){
  if(n1==zeroDrain) return n2;
  if(n2==zeroDrain) return n1;
  if((n1->isDrain())&&(n2->isDrain())) return findOrAddDrain(n1->getValue()+n2->getValue());

  swapPointer(&n1,&n2);

  //try to get result from computed table
  ZADDNode* node=(ZADDNode*) ArithmeticComputedTable.find(calcPtr(n1,ADD_OP),0,n2,0);
  if(node) return node;

  ZADDNode* w0;
  ZADDNode* w1;
  ZADDUniqueTable* uTable;

  if(n1->getRealLevel()<n2->getRealLevel()){
    uTable=n1->getUniqueTable();
    w0=addition(n1->getSucc0(),n2);
    w1=n1->getSucc1();
  } else if(n1->getRealLevel()>n2->getRealLevel()){
    uTable=n2->getUniqueTable();
    w0=addition(n1,n2->getSucc0());
    w1=n2->getSucc1();
  } else {
    uTable=n1->getUniqueTable();
    w0=addition(n1->getSucc0(),n2->getSucc0());
    w1=addition(n1->getSucc1(),n2->getSucc1());
  }
  node=(w1==zeroDrain)?w0:findOrAdd(uTable,w1,w0);
  //insert result into computed table
  ArithmeticComputedTable.insert(calcPtr(n1,ADD_OP),0,n2,0,node);

  return node;
}

ZADDNode* ZADD::subtraction(ZADDNode* v1, ZADDNode* v1VS, ZADDNode* v2, ZADDNode* v2VS){
  if((v1==v2)&&(v1VS==v2VS)) return zeroDrain;

  //case 2: (*,*)
  if((v1VS->isDrain())&&(v2VS->isDrain())) return findOrAddDrain(v1->getValue()-v2->getValue());

  //try to get result from computed table
  ZADDNode* node=(ZADDNode*) ArithmeticComputedTable.find(calcPtr(v1,SUB_OP),v1VS,v2,v2VS);
  if(node) return node;

  ZADDNode* w0;
  ZADDNode* w1;
  ZADDNode* v10;
  ZADDNode* v11;
  ZADDNode* v20;
  ZADDNode* v21;
  ZADDNode* newV1VS;
  ZADDNode* newV2VS;

  ZADDUniqueTable* minTable;

  if(v1VS->getRealLevel()<=v2VS->getRealLevel()){
    minTable=v1VS->getUniqueTable();
    if(v1->getUniqueTable()==minTable){
      v10=v1->getSucc0();
      v11=v1->getSucc1();
    } else {
      v10=v1;
      v11=zeroDrain;
    }
    newV1VS=v1VS->getSucc1();
    if(v2VS->getUniqueTable()==minTable){
      if(v2->getUniqueTable()==minTable){
        v20=v2->getSucc0();
        v21=v2->getSucc1();
      } else {
        v20=v2;
        v21=zeroDrain;
      }
      newV2VS=v2VS->getSucc1();
    } else {
      newV2VS=v2VS;
      v20=v21=v2;
    }
  } else {
    minTable=v2VS->getUniqueTable();
    v10=v11=v1;
    if(v2->getUniqueTable()==minTable){
      v20=v2->getSucc0();
      v21=v2->getSucc1();
    } else {
      v20=v2;
      v21=zeroDrain;
    }
    newV1VS=v1VS;
    newV2VS=v2VS->getSucc1();
  }

  w0=subtraction(v10,newV1VS,v20,newV2VS);
  w1=subtraction(v11,newV1VS,v21,newV2VS);

  node=(w1==zeroDrain)?w0:findOrAdd(minTable,w1,w0);

  //insert result into computed table
  ArithmeticComputedTable.insert(calcPtr(v1,SUB_OP),v1VS,v2,v2VS,node);

  return node;
}

ZADDNode* ZADD::subtraction(ZADDNode* n1, ZADDNode* n2){
  if(n1==n2) return zeroDrain;
  if((n1->isDrain())&&(n2->isDrain())) return findOrAddDrain(n1->getValue()-n2->getValue());

  //try to get result from computed table
  ZADDNode* node=(ZADDNode*) ArithmeticComputedTable.find(calcPtr(n1,SUB_OP),0,n2,0);
  if(node) return node;

  ZADDNode* w0;
  ZADDNode* w1;
  ZADDUniqueTable* uTable;

  if(n1->getRealLevel()<n2->getRealLevel()){
    uTable=n1->getUniqueTable();
    w0=subtraction(n1->getSucc0(),n2);
    w1=n1->getSucc1();
  } else if(n1->getRealLevel()>n2->getRealLevel()){
    uTable=n2->getUniqueTable();
    w0=subtraction(n1,n2->getSucc0());
    w1=n2->getSucc1();
  } else {
    uTable=n1->getUniqueTable();
    w0=subtraction(n1->getSucc0(),n2->getSucc0());
    w1=subtraction(n1->getSucc1(),n2->getSucc1());
  }
  node=(w1==zeroDrain)?w0:findOrAdd(uTable,w1,w0);
  //insert result into computed table
  ArithmeticComputedTable.insert(calcPtr(n1,SUB_OP),0,n2,0,node);

  return node;
}

ZADDNode* ZADD::multiplication(ZADDNode* v1, ZADDNode* v1VS, ZADDNode* v2, ZADDNode* v2VS){
  ZADDUniqueTable* minTable;
	if(v1->getRealLevel()<=v2->getRealLevel()) minTable=v1->getUniqueTable();
	else minTable=v2->getUniqueTable();
	
	while(v1VS->getRealLevel()<minTable->getRealLevel()) v1VS=v1VS->getSucc1();
	while(v2VS->getRealLevel()<minTable->getRealLevel()) v2VS=v2VS->getSucc1();
	  
	if(v1VS->isDrain()){
    if(v2VS->isDrain()) return findOrAddDrain(v1->getValue()*v2->getValue());
    else return constMultiplication(v1,v2,v2VS);
  }
	
	if(v1VS==v2VS) return multiplication(v1,v2);
  if((v1==zeroDrain)||(v2==zeroDrain)) return zeroDrain;
  if(v2VS->isDrain()) return constMultiplication(v2,v1,v1VS);

  swapPointer(&v1,&v1VS,&v2,&v2VS);

  //try to get result from computed table
  ZADDNode* node=(ZADDNode*) ArithmeticComputedTable.find(calcPtr(v1,MUL_OP),v1VS,v2,v2VS);
  if(node) return node;

  ZADDNode* w0;
  ZADDNode* w1;
  ZADDNode* newV1VS;
  ZADDNode* newV2VS;

  if(v1VS->getRealLevel()<=v2VS->getRealLevel()){
    minTable=v1VS->getUniqueTable();
    newV1VS=v1VS->getSucc1();
    if(v1->getUniqueTable()==minTable){
      if(v2VS->getUniqueTable()==minTable){
        newV2VS=v2VS->getSucc1();
        if(v2->getUniqueTable()==minTable){
          w0=multiplication(v1->getSucc0(),newV1VS,v2->getSucc0(),newV2VS);
          w1=multiplication(v1->getSucc1(),newV1VS,v2->getSucc1(),newV2VS);

          node=(w1==zeroDrain)?w0:findOrAdd(minTable,w1,w0);
        } else {
          node=multiplication(v1->getSucc0(),newV1VS,v2,newV2VS);
        }
      } else {
        newV2VS=v2VS;
        w0=multiplication(v1->getSucc0(),newV1VS,v2,newV2VS);
        w1=multiplication(v1->getSucc1(),newV1VS,v2,newV2VS);

        node=(w1==zeroDrain)?w0:findOrAdd(minTable,w1,w0);
      }
    } else {
      if(v2VS->getUniqueTable()==minTable){
        if(v2->getUniqueTable()==minTable) node=multiplication(v1,newV1VS,v2->getSucc0(),v2VS->getSucc1());
        else node=multiplication(v1,newV1VS,v2,v2VS->getSucc1());
      } else {
        node=multiplication(v1,newV1VS,v2,v2VS);
      }
    }
  } else {
    newV2VS=v2VS->getSucc1();
    if(v2->getUniqueTable()==v2VS->getUniqueTable()){
      w0=multiplication(v1,v1VS,v2->getSucc0(),newV2VS);
      w1=multiplication(v1,v1VS,v2->getSucc1(),newV2VS);

      node=(w1==zeroDrain)?w0:findOrAdd(v2VS->getUniqueTable(),w1,w0);
    } else {
      node=multiplication(v1,v1VS,v2,newV2VS);
    }
  }

  //insert result into computed table
  ArithmeticComputedTable.insert(calcPtr(v1,MUL_OP),v1VS,v2,v2VS,node);

  return node;
}

ZADDNode* ZADD::multiplication(ZADDNode* v1, ZADDNode* v2){
  if((v1==zeroDrain)||(v2==zeroDrain)) return zeroDrain;
  if((v1->isDrain())&&(v2->isDrain())) return findOrAddDrain(v1->getValue()*v2->getValue());

  swapPointer(&v1,&v2);

  //try to get result from computed table
  ZADDNode* node=(ZADDNode*) ArithmeticComputedTable.find(calcPtr(v1,MUL_OP),0,v2,0);
  if(node) return node;

  ZADDNode* w0;
  ZADDNode* w1;

  if(v1->getRealLevel()<v2->getRealLevel()){
    node=multiplication(v1->getSucc0(),v2);
  } else if(v1->getRealLevel()>v2->getRealLevel()){
    node=multiplication(v1,v2->getSucc0());
  } else {
    w0=multiplication(v1->getSucc0(),v2->getSucc0());
    w1=multiplication(v1->getSucc1(),v2->getSucc1());

    node=(w1==zeroDrain)?w0:findOrAdd(v1->getUniqueTable(),w1,w0);
  }

  //insert result into computed table
  ArithmeticComputedTable.insert(calcPtr(v1,MUL_OP),0,v2,0,node);

  return node;
}

ZADDNode* ZADD::constMultiplication(ZADDNode* v1, ZADDNode* v2, ZADDNode* v2VS){
  if(v2==zeroDrain) return zeroDrain;
  if(v2VS->isDrain()) return findOrAddDrain(v1->getValue()*v2->getValue());

  //try to get result from computed table
  ZADDNode* node=(ZADDNode*) ArithmeticComputedTable.find(calcPtr(v1,MUL_OP),oneDrain,v2,v2VS);
  if(node) return node;

  ZADDNode* w0;
  ZADDNode* w1;
  ZADDNode* v20;
  ZADDNode* v21;
  ZADDNode* newV2VS;

  ZADDUniqueTable* minTable;

  minTable=v2VS->getUniqueTable();
  if(v2->getUniqueTable()==minTable){
    v20=v2->getSucc0();
    v21=v2->getSucc1();
  } else {
    v20=v2;
    v21=zeroDrain;
  }
  newV2VS=v2VS->getSucc1();

  w0=constMultiplication(v1,v20,newV2VS);
  w1=constMultiplication(v1,v21,newV2VS);

  node=(w1==zeroDrain)?w0:findOrAdd(minTable,w1,w0);

  //insert result into computed table
  ArithmeticComputedTable.insert(calcPtr(v1,MUL_OP),oneDrain,v2,v2VS,node);

  return node;
}

ZADDNode* ZADD::MM(ZADDNode* A, ZADDNode* AVS, ZADDNode* B, ZADDNode* BVS, ZADDNode* varSet, unsigned long m){
  if((A==zeroDrain) || (B==zeroDrain)) return zeroDrain;
  if((AVS->isDrain() && BVS->isDrain()) || (m==0)){
    ZADDNode* tempDrain=findOrAddDrain(double(1<<m));
    ZADDNode* tempNode=multiplication(multiplication(A,B),tempDrain);

    //prevent from garbage collection
    tempNode->incNodeCount();
    if(tempDrain->preNodeCount==0) tempDrain->garbageCollect();
    //reverse protection
    --tempNode->preNodeCount;

    return tempNode;
  }

  ZADDNode* tempNode=(ZADDNode*) MatrixComputedTable.find(calcPtr(A,MMA_OP),AVS,B,BVS,varSet);
  if(tempNode) return tempNode;

  ZADDUniqueTable* uniqueTable=varSet->getUniqueTable();
  ZADDNode* A0;
  ZADDNode* A1;
  ZADDNode* A00, *A01;
  ZADDNode* A10, *A11;
  ZADDNode* newAVS;

  if(uniqueTable==AVS->getUniqueTable()){
    if(A->getUniqueTable()==AVS->getUniqueTable()){
      A0=A->getSucc0();
      A1=A->getSucc1();
    } else {
      A1=zeroDrain;
      A0=A;
    }
    newAVS=AVS->getSucc1();
  } else {
    A0=A1=A;
    newAVS=AVS;
  }
  if(newAVS->getLevel()==uniqueTable->getLevel()+1){
    if(A0->getLevel()==newAVS->getLevel()){
      A00=A0->getSucc0();
      A01=A0->getSucc1();
    } else {
      A00=A0;
      A01=zeroDrain;
    }
    if(A1->getLevel()==newAVS->getLevel()){
      A10=A1->getSucc0();
      A11=A1->getSucc1();
    } else {
      A10=A1;
      A11=zeroDrain;
    }
    newAVS=newAVS->getSucc1();
  } else {
    A00=A01=A0;
    A11=A10=A1;
  }

  ZADDNode* B0;
  ZADDNode* B1;
  ZADDNode* B00, *B01;
  ZADDNode* B10, *B11;
  ZADDNode* newBVS;

  if(uniqueTable==BVS->getUniqueTable()){
    if(B->getUniqueTable()==BVS->getUniqueTable()){
      B0=B->getSucc0();
      B1=B->getSucc1();
    } else {
      B1=zeroDrain;
      B0=B;
    }
    newBVS=BVS->getSucc1();
  } else {
    B0=B1=B;
    newBVS=BVS;
  }
  if(newBVS->getLevel()==uniqueTable->getLevel()+1){
    if(B0->getLevel()==newBVS->getLevel()){
      B00=B0->getSucc0();
      B01=B0->getSucc1();
    } else {
      B00=B0;
      B01=zeroDrain;
    }
    if(B1->getLevel()==newBVS->getLevel()){
      B10=B1->getSucc0();
      B11=B1->getSucc1();
    } else {
      B10=B1;
      B11=zeroDrain;
    }
    newBVS=newBVS->getSucc1();
  } else {
    B00=B01=B0;
    B11=B10=B1;
  }

  ZADDNode* newVarSet=varSet->getSucc1();

  ZADDNode* w0000=MM(A00,newAVS,B00,newBVS,newVarSet,m-1);
  ZADDNode* w0110=MM(A01,newAVS,B10,newBVS,newVarSet,m-1);
  ZADDNode* w0001=MM(A00,newAVS,B01,newBVS,newVarSet,m-1);
  ZADDNode* w0111=MM(A01,newAVS,B11,newBVS,newVarSet,m-1);
  ZADDNode* w1000=MM(A10,newAVS,B00,newBVS,newVarSet,m-1);
  ZADDNode* w1110=MM(A11,newAVS,B10,newBVS,newVarSet,m-1);
  ZADDNode* w1001=MM(A10,newAVS,B01,newBVS,newVarSet,m-1);
  ZADDNode* w1111=MM(A11,newAVS,B11,newBVS,newVarSet,m-1);

  ZADDNode* c00=addition(w0000,w0110);
  ZADDNode* c01=addition(w0001,w0111);
  ZADDNode* c10=addition(w1000,w1110);
  ZADDNode* c11=addition(w1001,w1111);

  ZADDNode* temp0=(c01==zeroDrain)?c00:findOrAdd(uniqueTable->getLevel()+1,c01,c00);
  ZADDNode* temp1=(c11==zeroDrain)?c10:findOrAdd(uniqueTable->getLevel()+1,c11,c10);
  tempNode=(temp1==zeroDrain)?temp0:findOrAdd(uniqueTable,temp1,temp0);

  //prevent from garbage collection
  tempNode->incNodeCount();
  if(w0000->preNodeCount==0) w0000->garbageCollect();
  if(w0110->preNodeCount==0) w0110->garbageCollect();
  if(w0001->preNodeCount==0) w0001->garbageCollect();
  if(w0111->preNodeCount==0) w0111->garbageCollect();
  if(w1000->preNodeCount==0) w1000->garbageCollect();
  if(w1110->preNodeCount==0) w1110->garbageCollect();
  if(w1001->preNodeCount==0) w1001->garbageCollect();
  if(w1111->preNodeCount==0) w1111->garbageCollect();

  //reverse protection
  --tempNode->preNodeCount;

  MatrixComputedTable.insert(calcPtr(A,MMA_OP),AVS,B,BVS,varSet,tempNode);

  return tempNode;
}

ZADDNode* ZADD::MV(ZADDNode* A, ZADDNode* AVS, ZADDNode* b, ZADDNode* bVS, ZADDNode* varSet, unsigned long m){
  if((A==zeroDrain) || (b==zeroDrain)) return zeroDrain;
  if((AVS->isDrain() && bVS->isDrain()) || (m==0)){
    ZADDNode* tempDrain=findOrAddDrain(double(1<<m));
    ZADDNode* tempNode=multiplication(multiplication(A,b),tempDrain);

    //prevent from garbage collection
    tempNode->incNodeCount();
    if(tempDrain->preNodeCount==0) tempDrain->garbageCollect();
    //reverse protection
    --tempNode->preNodeCount;

    return tempNode;
  }

  ZADDNode* tempNode=(ZADDNode*) MatrixComputedTable.find(calcPtr(A,MAV_OP),AVS,b,bVS,varSet);
  if(tempNode) return tempNode;

  ZADDUniqueTable* uniqueTable=varSet->getUniqueTable();
  ZADDNode* A0;
  ZADDNode* A1;
  ZADDNode* A00, *A01;
  ZADDNode* A10, *A11;
  ZADDNode* newAVS;

  if(uniqueTable==AVS->getUniqueTable()){
    if(A->getUniqueTable()==AVS->getUniqueTable()){
      A0=A->getSucc0();
      A1=A->getSucc1();
    } else {
      A1=zeroDrain;
      A0=A;
    }
    newAVS=AVS->getSucc1();
  } else {
    A0=A1=A;
    newAVS=AVS;
  }
  if(newAVS->getLevel()==uniqueTable->getLevel()+1){
    if(A0->getLevel()==newAVS->getLevel()){
      A00=A0->getSucc0();
      A01=A0->getSucc1();
    } else {
      A00=A0;
      A01=zeroDrain;
    }
    if(A1->getLevel()==newAVS->getLevel()){
      A10=A1->getSucc0();
      A11=A1->getSucc1();
    } else {
      A10=A1;
      A11=zeroDrain;
    }
    newAVS=newAVS->getSucc1();
  } else {
    A00=A01=A0;
    A11=A10=A1;
  }

  ZADDNode* b0, *b1;
  ZADDNode* newBVS;
  if(bVS->getLevel()==uniqueTable->getLevel()){
    if(b->getLevel()==bVS->getLevel()){
      b0=b->getSucc0();
      b1=b->getSucc1();
    } else {
      b0=b;
      b1=zeroDrain;
    }
    newBVS=bVS->getSucc1();
  } else {
    b0=b1=b;
    newBVS=bVS;
  }

  ZADDNode* newVarSet=varSet->getSucc1();

  ZADDNode* w00=MV(A00,newAVS,b0,newBVS,newVarSet,m-1);
  ZADDNode* w01=MV(A01,newAVS,b1,newBVS,newVarSet,m-1);
  ZADDNode* w10=MV(A10,newAVS,b0,newBVS,newVarSet,m-1);
  ZADDNode* w11=MV(A11,newAVS,b1,newBVS,newVarSet,m-1);

  ZADDNode* c0=addition(w00,w01);
  ZADDNode* c1=addition(w10,w11);
  tempNode=((c0->getUniqueTable()==uniqueTable)||(c1->getUniqueTable()==uniqueTable)||(c1==zeroDrain))?c0:findOrAdd(uniqueTable,c1,c0);
  tempNode->incNodeCount();
  if(w00->preNodeCount==0) w00->garbageCollect();
  if(w01->preNodeCount==0) w01->garbageCollect();
  if(w10->preNodeCount==0) w10->garbageCollect();
  if(w11->preNodeCount==0) w11->garbageCollect();
  --tempNode->preNodeCount;
  MatrixComputedTable.insert(calcPtr(A,MAV_OP),AVS,b,bVS,varSet,tempNode);

  return tempNode;
}

ZADDNode* ZADD::VM(ZADDNode* b, ZADDNode* bVS, ZADDNode* A, ZADDNode* AVS, ZADDNode* varSet, unsigned long m){
  if((A==zeroDrain) || (b==zeroDrain)) return zeroDrain;
  if((AVS->isDrain() && bVS->isDrain()) || (m==0)){
    ZADDNode* tempDrain=findOrAddDrain(double(1<<m));
    ZADDNode* tempNode=multiplication(multiplication(A,b),tempDrain);

    //prevent from garbage collection
    tempNode->incNodeCount();
    if(tempDrain->preNodeCount==0) tempDrain->garbageCollect();
    //reverse protection
    --tempNode->preNodeCount;

    return tempNode;
  }

  ZADDNode* tempNode=(ZADDNode*) MatrixComputedTable.find(calcPtr(b,VMA_OP),bVS,A,AVS,varSet);
  if(tempNode) return tempNode;

  ZADDUniqueTable* uniqueTable=varSet->getUniqueTable();
  ZADDNode* A0;
  ZADDNode* A1;
  ZADDNode* A00, *A01;
  ZADDNode* A10, *A11;
  ZADDNode* newAVS;

  if(uniqueTable==AVS->getUniqueTable()){
    if(A->getUniqueTable()==AVS->getUniqueTable()){
      A0=A->getSucc0();
      A1=A->getSucc1();
    } else {
      A1=zeroDrain;
      A0=A;
    }
    newAVS=AVS->getSucc1();
  } else {
    A0=A1=A;
    newAVS=AVS;
  }
  if(newAVS->getLevel()==uniqueTable->getLevel()+1){
    if(A0->getLevel()==newAVS->getLevel()){
      A00=A0->getSucc0();
      A01=A0->getSucc1();
    } else {
      A00=A0;
      A01=zeroDrain;
    }
    if(A1->getLevel()==newAVS->getLevel()){
      A10=A1->getSucc0();
      A11=A1->getSucc1();
    } else {
      A10=A1;
      A11=zeroDrain;
    }
    newAVS=newAVS->getSucc1();
  } else {
    A00=A01=A0;
    A11=A10=A1;
  }

  ZADDNode* b0, *b1;
  ZADDNode* newBVS;
  if(bVS->getLevel()==uniqueTable->getLevel()){
    if(b->getLevel()==bVS->getLevel()){
      b0=b->getSucc0();
      b1=b->getSucc1();
    } else {
      b0=b;
      b1=zeroDrain;
    }
    newBVS=bVS->getSucc1();
  } else {
    b0=b1=b;
    newBVS=bVS;
  }

  ZADDNode* newVarSet=varSet->getSucc1();

  ZADDNode* w00=VM(b0,newBVS,A00,newAVS,newVarSet,m-1);
  ZADDNode* w01=VM(b0,newBVS,A01,newAVS,newVarSet,m-1);
  ZADDNode* w10=VM(b1,newBVS,A10,newAVS,newVarSet,m-1);
  ZADDNode* w11=VM(b1,newBVS,A11,newAVS,newVarSet,m-1);

  ZADDNode* c0=addition(w00,w10);
  ZADDNode* c1=addition(w01,w11);

  tempNode=((c0->getUniqueTable()==uniqueTable)||(c1->getUniqueTable()==uniqueTable)||(c1==zeroDrain))?c0:findOrAdd(uniqueTable,c1,c0);
  tempNode->incNodeCount();
  if(w00->preNodeCount==0) w00->garbageCollect();
  if(w01->preNodeCount==0) w01->garbageCollect();
  if(w10->preNodeCount==0) w10->garbageCollect();
  if(w11->preNodeCount==0) w11->garbageCollect();
  --tempNode->preNodeCount;
  MatrixComputedTable.insert(calcPtr(b,VMA_OP),bVS,A,AVS,varSet,tempNode);

  return tempNode;
}

ZADDNode* ZADD::equivalenceClasses(ZADDNode* u, ZADDNode* uVarSet, ZADDNode* equiv, ZADDNode* equivVarSet, ZADDNode* cubeSet){
  if((uVarSet->isDrain())&&(equivVarSet->isDrain())){
    if(u==equiv) return oneDrain;
    else return zeroDrain;
  }

  ZADDUniqueTable* uniqueTable=uVarSet->getUniqueTable();
  unsigned long index=uniqueTable->getRealLevel();
  if(equivVarSet->getRealLevel()<index){
    if(equiv->getUniqueTable()!=equivVarSet->getUniqueTable()) return zeroDrain;
    if(equiv->getSucc0()!=equiv->getSucc1()) return zeroDrain;
  }

  while(cubeSet->getRealLevel()<index){
    cubeSet=cubeSet->getSucc1();
  }

  ZADDNode* w0;
  ZADDNode* w1;

  ZADDNode* result=(ZADDNode*) SpecialComputedTable.find(calcPtr(u,EQI_OP),uVarSet,equiv,equivVarSet,cubeSet);
  if(result) return result;

  if(uniqueTable==cubeSet->getUniqueTable()){
    ZADDNode* equiv0;
    ZADDNode* equiv1;
    ZADDNode* u0;
    ZADDNode* u1;
    ZADDNode* newEquivVarSet;

    if(u->getUniqueTable()==uniqueTable){
      u0=u->getSucc0();
      u1=u->getSucc1();
    } else {
      u0=u;
      u1=zeroDrain;
    }

    if(equivVarSet->getUniqueTable()==uniqueTable){
      newEquivVarSet=equivVarSet->getSucc1();
      if(equiv->getUniqueTable()==equivVarSet->getUniqueTable()){
        equiv0=equiv->getSucc0();
        equiv1=equiv->getSucc1();
      } else {
        equiv0=equiv;
        equiv1=zeroDrain;
      }
    } else {
      newEquivVarSet=equivVarSet;
      equiv0=equiv1=equiv;
    }

    w0=equivalenceClasses(u0,uVarSet->getSucc1(),equiv0,newEquivVarSet,cubeSet->getSucc1());
    w1=equivalenceClasses(u1,uVarSet->getSucc1(),equiv1,newEquivVarSet,cubeSet->getSucc1());

    result=AND(w0,w1);

    //protect from garbage collection
    result->incNodeCount();
    if(w0->preNodeCount==0) w0->garbageCollect();
    if(w1->preNodeCount==0) w1->garbageCollect();
    //revert protection
    --result->preNodeCount;
  } else {
    ZADDNode* u0;
    ZADDNode* u1;

    if(u->getUniqueTable()==uniqueTable){
      u0=u->getSucc0();
      u1=u->getSucc1();
    } else {
      u0=u;
      u1=zeroDrain;
    }

    w0=equivalenceClasses(u0,uVarSet->getSucc1(),equiv,equivVarSet,cubeSet);
    w1=equivalenceClasses(u1,uVarSet->getSucc1(),equiv,equivVarSet,cubeSet);
    result=(w1==zeroDrain)?w0:findOrAdd(uniqueTable,w1,w0);
  }

  SpecialComputedTable.insert(calcPtr(u,EQI_OP),uVarSet,equiv,equivVarSet,cubeSet,result);

  return result;
}

ZADDNode* ZADD::constrain(ZADDNode* f, ZADDNode* g){
  //if g is a drain return constrain(f,0)=0 or constrain(f,1)=f
  if(g->isDrain()) return (g==zeroDrain)?zeroDrain:f;

  //if f is a drain and g is not, return constrain(1,g)=1 or constrain(0,g)=0
  if(f->isDrain()) return f;

  //if v=w constrain(v,w)=1
  if(f==g) return oneDrain;

  ZADDNode* result=(ZADDNode*) SpecialComputedTable.find(calcPtr(f,CON_OP),g,0,0,0);
  if(result) return result;

  ZADDUniqueTable* minTable;

  ZADDNode* f0, *f1;
  ZADDNode* g0, *g1;

  if(g->getRealLevel()<=f->getRealLevel()){
    minTable=g->getUniqueTable();
    g0=g->getSucc0();
    g1=g->getSucc1();
    if(f->getUniqueTable()==minTable){
      f0=f->getSucc0();
      f1=f->getSucc1();
    } else f0=f1=f;
    if(g0==zeroDrain){
      result=constrain(f1,g1);
      SpecialComputedTable.insert(calcPtr(f,CON_OP),g,0,0,0,result);

      return result;
    } else if(g1==zeroDrain){
      result=constrain(f0,g0);
      SpecialComputedTable.insert(calcPtr(f,CON_OP),g,0,0,0,result);

      return result;
    }
  } else {
    minTable=f->getUniqueTable();
    g0=g1=g;
    f0=f->getSucc0();
    f1=f->getSucc1();
  }

  ZADDNode* w0, *w1;

  w0=constrain(f0,g0);
  w1=constrain(f1,g1);

  result=(w0==w1)?w0:findOrAdd(minTable,w1,w0);

  SpecialComputedTable.insert(calcPtr(f,CON_OP),g,0,0,0,result);

  return result;

}

ZADDNode* ZADD::ITE(ZADDNode* u, ZADDNode* v1, ZADDNode* v2){
  //case 1: (h,*,*)
  if(v1==v2) return v1;

  unsigned long index=v1->getRealLevel()<v2->getRealLevel()?v1->getRealLevel():v2->getRealLevel();

  if(u->getRealLevel()<index){
    if((u->getSucc0()==zeroDrain) && (u->getSucc1()==oneDrain)) return findOrAdd(u->getUniqueTable(),v1,v2);
    if((u->getSucc1()==zeroDrain) && (u->getSucc0()==oneDrain)) return findOrAdd(u->getUniqueTable(),v2,v1);
  }

  //case 2: (*,f,g)
  if(u->isDrain()) return (u==zeroDrain)?v2:v1;

  //case 3: (h,f,0)
  if(v2==zeroDrain) return 0;//AND(u,v1);

  //case 4: (h,1,g)
  if(v1==oneDrain) return 0;//OR(u,v2);

  //try to get result from computed table
  ZADDNode* tempZADDNode=0;//(ZADDNode*) BoolComputedTable.find(calcPtr(u,ITE_OP),v1,v2);
  if(tempZADDNode) return tempZADDNode;

  ZADDNode* w0;
  ZADDNode* w1;
  ZADDNode* u0, *u1;
  ZADDNode* v10, *v11;
  ZADDNode* v20, *v21;
  ZADDNode* node;

  ZADDUniqueTable* minTable=0;

  if(u->getRealLevel()<=index){
    minTable=u->getUniqueTable();
    u0=u->getSucc0();
    u1=u->getSucc1();
  } else u0=u1=u;
  if(v1->getRealLevel()==index){
    minTable=v1->getUniqueTable();
    v10=v1->getSucc0();
    v11=v1->getSucc1();
  } else v10=v11=v1;
  if(v2->getRealLevel()==index) {
    minTable=v2->getUniqueTable();
    v20=v2->getSucc0();
    v21=v2->getSucc1();
  } else v20=v21=v2;

  w0=ITE(u0,v10,v20);
  w1=ITE(u1,v11,v21);

  node=(w0==w1)?w0:findOrAdd(minTable,w1,w0);

  //insert result into computed table
  //BoolComputedTable.insert(calcPtr(u,ITE_OP),v1,v2,node);

  return node;
}

ZADDNode* ZADD::AND(ZADDNode* v1, ZADDNode* v1VS, ZADDNode* v2, ZADDNode* v2VS){
  ZADDUniqueTable* minTable;
	if(v1->getRealLevel()<=v2->getRealLevel()) minTable=v1->getUniqueTable();
	else minTable=v2->getUniqueTable();
	
	while(v1VS->getRealLevel()<minTable->getRealLevel()) v1VS=v1VS->getSucc1();
	while(v2VS->getRealLevel()<minTable->getRealLevel()) v2VS=v2VS->getSucc1();

  if(v1VS==v2VS) return AND(v1,v2);
  if(v1==v2) return v1;
  if((v1==zeroDrain)||(v2==zeroDrain)) return zeroDrain;
	
  swapPointer(&v1,&v1VS,&v2,&v2VS);

  //try to get result from computed table
  ZADDNode* node=(ZADDNode*) BoolComputedTable.find(calcPtr(v1,AND_OP),v1VS,v2,v2VS);
  if(node) return node;

  ZADDNode* w0;
  ZADDNode* w1;
  ZADDNode* newV1VS;
  ZADDNode* newV2VS;

  if(v1VS->getRealLevel()<=v2VS->getRealLevel()){
    minTable=v1VS->getUniqueTable();
    newV1VS=v1VS->getSucc1();
    if(v1->getUniqueTable()==minTable){
      if(v2VS->getUniqueTable()==minTable){
        newV2VS=v2VS->getSucc1();
        if(v2->getUniqueTable()==minTable){
          w0=AND(v1->getSucc0(),newV1VS,v2->getSucc0(),newV2VS);
          w1=AND(v1->getSucc1(),newV1VS,v2->getSucc1(),newV2VS);

          node=(w1==zeroDrain)?w0:findOrAdd(minTable,w1,w0);
        } else {
          node=AND(v1->getSucc0(),newV1VS,v2,newV2VS);
        }
      } else {
        w0=AND(v1->getSucc0(),newV1VS,v2,v2VS);
        w1=AND(v1->getSucc1(),newV1VS,v2,v2VS);

        node=(w1==zeroDrain)?w0:findOrAdd(minTable,w1,w0);
      }
    } else {
      if(v2VS->getUniqueTable()==minTable){
        if(v2->getUniqueTable()==minTable) node=AND(v1,newV1VS,v2->getSucc0(),v2VS->getSucc1());
        else node=AND(v1,newV1VS,v2,v2VS->getSucc1());
      } else {
        node=AND(v1,newV1VS,v2,v2VS);
      }
    }
  } else {
    minTable=v2VS->getUniqueTable();
    newV2VS=v2VS->getSucc1();
    if(v2->getUniqueTable()==minTable){
      w0=AND(v1,v1VS,v2->getSucc0(),newV2VS);
      w1=AND(v1,v1VS,v2->getSucc1(),newV2VS);

      node=(w1==zeroDrain)?w0:findOrAdd(minTable,w1,w0);
    } else {
      node=AND(v1,v1VS,v2,newV2VS);
    }
  }

  //insert result into computed table
  BoolComputedTable.insert(calcPtr(v1,AND_OP),v1VS,v2,v2VS,node);

  return node;
}

ZADDNode* ZADD::AND(ZADDNode* v1, ZADDNode* v2){
  if(v1==v2) return v1;
  if((v1==zeroDrain)||(v2==zeroDrain)) return zeroDrain;
  if((v1->isDrain())&&(v2->isDrain())) return oneDrain;

  swapPointer(&v1,&v2);

  //try to get result from computed table
  ZADDNode* node=(ZADDNode*) BoolComputedTable.find(calcPtr(v1,AND_OP),0,v2,0);
  if(node) return node;

  ZADDNode* w0;
  ZADDNode* w1;

  if(v1->getRealLevel()<v2->getRealLevel()){
    node=AND(v1->getSucc0(),v2);
  } else if(v1->getRealLevel()>v2->getRealLevel()){
    node=AND(v1,v2->getSucc0());
  } else {
    w0=AND(v1->getSucc0(),v2->getSucc0());
    w1=AND(v1->getSucc1(),v2->getSucc1());

    node=(w1==zeroDrain)?w0:findOrAdd(v1->getUniqueTable(),w1,w0);
  }

  //insert result into computed table
  BoolComputedTable.insert(calcPtr(v1,AND_OP),0,v2,0,node);

  return node;
}

ZADDNode* ZADD::OR(ZADDNode* v1, ZADDNode* v1VS, ZADDNode* v2, ZADDNode* v2VS){
  if(v1VS==v2VS) return OR(v1,v2);
  if(v1==zeroDrain) return calcLiftedBDD(v2,v2VS,v1VS);
  if(v2==zeroDrain) return calcLiftedBDD(v1,v1VS,v2VS);

  swapPointer(&v1,&v1VS,&v2,&v2VS);

  //try to get result from computed table
  ZADDNode* node=(ZADDNode*) BoolComputedTable.find(calcPtr(v1,OR_OP),v1VS,v2,v2VS);
  if(node) return node;

  ZADDNode* w0;
  ZADDNode* w1;
  ZADDNode* v10;
  ZADDNode* v11;
  ZADDNode* v20;
  ZADDNode* v21;
  ZADDNode* newV1VS;
  ZADDNode* newV2VS;

  ZADDUniqueTable* minTable;

  if(v1VS->getRealLevel()<=v2VS->getRealLevel()){
    minTable=v1VS->getUniqueTable();
    if(v1->getUniqueTable()==minTable){
      v10=v1->getSucc0();
      v11=v1->getSucc1();
    } else {
      v10=v1;
      v11=zeroDrain;
    }
    newV1VS=v1VS->getSucc1();
    if(v2VS->getUniqueTable()==minTable){
      newV2VS=v2VS->getSucc1();
      if(v2->getUniqueTable()==minTable){
        v20=v2->getSucc0();
        v21=v2->getSucc1();
      } else {
        v20=v2;
        v21=zeroDrain;
      }
    } else {
      newV2VS=v2VS;
      v20=v21=v2;
    }
  } else {
    minTable=v2VS->getUniqueTable();
    newV1VS=v1VS;
    newV2VS=v2VS->getSucc1();
    v10=v11=v1;
    if(v2->getUniqueTable()==minTable){
      v20=v2->getSucc0();
      v21=v2->getSucc1();
    } else {
      v20=v2;
      v21=zeroDrain;
    }
  }

  w0=OR(v10,newV1VS,v20,newV2VS);
  w1=OR(v11,newV1VS,v21,newV2VS);

  node=(w1==zeroDrain)?w0:findOrAdd(minTable,w1,w0);

  //insert result into computed table
  BoolComputedTable.insert(calcPtr(v1,OR_OP),v1VS,v2,v2VS,node);

  return node;
}

ZADDNode* ZADD::OR(ZADDNode* n1, ZADDNode* n2){
  if(n1==zeroDrain) return n2;
  if(n2==zeroDrain) return n1;
  if(n1==n2) return n1;
  if((n1->isDrain())&&(n2->isDrain())) return oneDrain;

  //try to get result from computed table
  ZADDNode* node=(ZADDNode*) BoolComputedTable.find(calcPtr(n1,OR_OP),0,n2,0);
  if(node) return node;

  ZADDNode* w0;
  ZADDNode* w1;
  ZADDUniqueTable* uTable;

  if(n1->getRealLevel()<n2->getRealLevel()){
    uTable=n1->getUniqueTable();
    w0=OR(n1->getSucc0(),n2);
    w1=n1->getSucc1();
  } else if(n1->getRealLevel()>n2->getRealLevel()){
    uTable=n2->getUniqueTable();
    w0=OR(n1,n2->getSucc0());
    w1=n2->getSucc1();
  } else {
    uTable=n1->getUniqueTable();
    if((n1->getSucc0()==n1->getSucc1())&&(n2->getSucc0()==n2->getSucc1())){
      w0=OR(n1->getSucc0(),n2->getSucc0());
      w1=w0;
    } else {
      w0=OR(n1->getSucc0(),n2->getSucc0());
      w1=OR(n1->getSucc1(),n2->getSucc1());
    }
  }
  node=(w1==zeroDrain)?w0:findOrAdd(uTable,w1,w0);
  //insert result into computed table
  BoolComputedTable.insert(calcPtr(n1,OR_OP),0,n2,0,node);


  return node;
}

ZADDNode* ZADD::NOT(ZADDNode* v, ZADDNode* vVS){
  if(v==zeroDrain) return calcLiftedBDD(oneDrain,oneDrain,vVS);
  if(vVS->isDrain()){
    if(v==zeroDrain) return oneDrain;
    else return zeroDrain;
  }

  //try to get result from computed table
  ZADDNode* node=(ZADDNode*) BoolComputedTable.find(calcPtr(oneDrain,DIF_OP),oneDrain,v,vVS);
  if(node) return node;

  ZADDNode* w0;
  ZADDNode* w1;
  ZADDNode* newVVS;

  newVVS=vVS->getSucc1();
  if(v->getUniqueTable()==vVS->getUniqueTable()){
    w0=NOT(v->getSucc0(),newVVS);
    w1=NOT(v->getSucc1(),newVVS);

    node=(w1==zeroDrain)?w0:findOrAdd(vVS->getUniqueTable(),w1,w0);
  } else {
    w0=NOT(v,newVVS);
    w1=calcLiftedBDD(oneDrain,oneDrain,newVVS);

    node=findOrAdd(vVS->getUniqueTable(),w1,w0);
  }

  //insert result into computed table
  BoolComputedTable.insert(calcPtr(oneDrain,DIF_OP),oneDrain,v,vVS,node);

  return node;
}

ZADDNode* ZADD::cofactor(ZADDNode* node, ZADDNode* nodeVarSet, ZADDNode* assignment, ZADDNode* assignmentVarSet){
  while(assignmentVarSet->getRealLevel()<nodeVarSet->getRealLevel()){
    if(assignment->getRealLevel()==assignmentVarSet->getRealLevel()){
      assignment=(assignment->getSucc0()==zeroDrain)?assignment->getSucc1():assignment->getSucc0();
    }
    assignmentVarSet=assignmentVarSet->getSucc1();
  }

  if(assignmentVarSet->isDrain()) return node;

  //check for computed table
  ZADDNode* tempNode=(ZADDNode*) BoolComputedTable.find(calcPtr(node,COF_OP),nodeVarSet,assignment,assignmentVarSet);
  if(tempNode) return tempNode;

  if(nodeVarSet->getUniqueTable()==assignmentVarSet->getUniqueTable()){
    bool value;
    ZADDNode* nextAssignment;
    if(assignment->getUniqueTable()==assignmentVarSet->getUniqueTable()){
      value=(assignment->getSucc0()==zeroDrain)?true:false;
      nextAssignment=assignment->getSucc(value);
    } else {
      value=false; //zero suppressed
      nextAssignment=assignment;
    }

    if(nodeVarSet->getUniqueTable()==node->getUniqueTable()){
      tempNode=cofactor(node->getSucc(value),nodeVarSet->getSucc1(),nextAssignment,assignmentVarSet->getSucc1());
    } else {
      //zero suppressed
      if(value) tempNode=zeroDrain;
      else tempNode=cofactor(node,nodeVarSet->getSucc1(),nextAssignment,assignmentVarSet->getSucc1());
    }
  } else {
    if(nodeVarSet->getUniqueTable()==node->getUniqueTable()){
      ZADDNode* w0=cofactor(node->getSucc0(),nodeVarSet->getSucc1(),assignment,assignmentVarSet);
      ZADDNode* w1=cofactor(node->getSucc1(),nodeVarSet->getSucc1(),assignment,assignmentVarSet);

      tempNode=(w1==zeroDrain)?w0:findOrAdd(node->getUniqueTable(),w1,w0);
    } else {
      tempNode=cofactor(node,nodeVarSet->getSucc1(),assignment,assignmentVarSet);
    }
  }

  //insert result into computed table
  BoolComputedTable.insert(calcPtr(node,COF_OP),nodeVarSet,assignment,assignmentVarSet,tempNode);

  return tempNode;
}

ZADDNode* ZADD::change(ZADDNode* node, ZADDNode* nodeCubeSet){
  if(nodeCubeSet->isDrain()) return node;

  //check computed table
  ZADDNode* tempNode=(ZADDNode*) BoolComputedTable.find(calcPtr(node,CHA_OP),nodeCubeSet,0,0);
  if(tempNode) return tempNode;

  //do it the hard way
  ZADDNode* w0;
  ZADDNode* w1;

  if(node->getRealLevel()==nodeCubeSet->getRealLevel()){
    w0=change(node->getSucc0(),nodeCubeSet->getSucc1());
    w1=change(node->getSucc1(),nodeCubeSet->getSucc1());
  } else {
    w0=change(node,nodeCubeSet->getSucc1());
    w1=zeroDrain;
  }

  tempNode=(w0==zeroDrain)?w1:findOrAdd(node->getUniqueTable(),w0,w1);

  //insert result into computed table
  BoolComputedTable.insert(calcPtr(node,CHA_OP),nodeCubeSet,0,0,tempNode);

  return tempNode;
}

ZADDNode* ZADD::change(ZADDNode* node, ZADDNode* nodeCubeSet, ZADDNode* cubeSet){
  if(nodeCubeSet->isDrain()) return node;

  while(cubeSet->getRealLevel()<nodeCubeSet->getRealLevel()) cubeSet=cubeSet->getSucc1();
  if(cubeSet->isDrain()) return node;

  //check computed table
  ZADDNode* tempNode=(ZADDNode*) BoolComputedTable.find(calcPtr(node,CHA_OP),nodeCubeSet,cubeSet,0);
  if(tempNode) return tempNode;

  ZADDNode* node0;
  ZADDNode* node1;

  if(node->getRealLevel()==nodeCubeSet->getRealLevel()){
    node0=node->getSucc0();
    node1=node->getSucc1();
  } else {
    node0=node;
    node1=zeroDrain;
  }

  //do it the hard way
  if(cubeSet->getRealLevel()==nodeCubeSet->getRealLevel()){
    ZADDNode* w0=change(node0,nodeCubeSet->getSucc1(),cubeSet->getSucc1());
    ZADDNode* w1=change(node1,nodeCubeSet->getSucc1(),cubeSet->getSucc1());

    tempNode=(w0==zeroDrain)?w1:findOrAdd(nodeCubeSet->getUniqueTable(),w0,w1);
  } else {
    ZADDNode* w0=change(node0,nodeCubeSet->getSucc1(),cubeSet);
    ZADDNode* w1=change(node1,nodeCubeSet->getSucc1(),cubeSet);

    tempNode=(w1==zeroDrain)?w0:findOrAdd(nodeCubeSet->getUniqueTable(),w1,w0);
  }

  //insert result into computed table
  BoolComputedTable.insert(calcPtr(node,CHA_OP),nodeCubeSet,cubeSet,0,tempNode);

  return tempNode;
}

ZADDNode* ZADD::less(ZADDNode* v1, ZADDNode* v1VS, ZADDNode* v2, ZADDNode* v2VS){
  if((v1VS->isDrain())&&(v2VS->isDrain())) return (v1->getValue()<v2->getValue())?oneDrain:zeroDrain;

  //try to get result from computed table
  ZADDNode* node=(ZADDNode*) CompareComputedTable.find(calcPtr(v1,LES_OP),v1VS,v2,v2VS);
  if(node) return node;

  ZADDNode* w0;
  ZADDNode* w1;
  ZADDNode* v10;
  ZADDNode* v11;
  ZADDNode* v20;
  ZADDNode* v21;
  ZADDNode* newV1VS;
  ZADDNode* newV2VS;

  ZADDUniqueTable* minTable;

  if(v1VS->getRealLevel()<=v2VS->getRealLevel()){
    minTable=v1VS->getUniqueTable();
    if(v1->getUniqueTable()==minTable){
      v10=v1->getSucc0();
      v11=v1->getSucc1();
    } else {
      v10=v1;
      v11=zeroDrain;
    }
    newV1VS=v1VS->getSucc1();
    if(v2VS->getUniqueTable()==minTable){
      if(v2->getUniqueTable()==minTable){
        v20=v2->getSucc0();
        v21=v2->getSucc1();
      } else {
        v20=v2;
        v21=zeroDrain;
      }
      newV2VS=v2VS->getSucc1();
    } else {
      newV2VS=v2VS;
      v20=v21=v2;
    }
  } else {
    minTable=v2VS->getUniqueTable();
    v10=v11=v1;
    if(v2->getUniqueTable()==minTable){
      v20=v2->getSucc0();
      v21=v2->getSucc1();
    } else {
      v20=v2;
      v21=zeroDrain;
    }
    newV1VS=v1VS;
    newV2VS=v2VS->getSucc1();
  }

  w0=less(v10,newV1VS,v20,newV2VS);
  w1=less(v11,newV1VS,v21,newV2VS);

  node=(w1==zeroDrain)?w0:findOrAdd(minTable,w1,w0);

  //insert result into computed table
  CompareComputedTable.insert(calcPtr(v1,LES_OP),v1VS,v2,v2VS,node);

  return node;
}

ZADDNode* ZADD::lessEqual(ZADDNode* v1, ZADDNode* v1VS, ZADDNode* v2, ZADDNode* v2VS){
  if((v1VS->isDrain())&&(v2VS->isDrain())) return (v1->getValue()<=v2->getValue())?oneDrain:zeroDrain;

  //try to get result from computed table
  ZADDNode* node=(ZADDNode*) CompareComputedTable.find(calcPtr(v1,LEE_OP),v1VS,v2,v2VS);
  if(node) return node;

  ZADDNode* w0;
  ZADDNode* w1;
  ZADDNode* v10;
  ZADDNode* v11;
  ZADDNode* v20;
  ZADDNode* v21;
  ZADDNode* newV1VS;
  ZADDNode* newV2VS;

  ZADDUniqueTable* minTable;

  if(v1VS->getRealLevel()<=v2VS->getRealLevel()){
    minTable=v1VS->getUniqueTable();
    if(v1->getUniqueTable()==minTable){
      v10=v1->getSucc0();
      v11=v1->getSucc1();
    } else {
      v10=v1;
      v11=zeroDrain;
    }
    newV1VS=v1VS->getSucc1();
    if(v2VS->getUniqueTable()==minTable){
      if(v2->getUniqueTable()==minTable){
        v20=v2->getSucc0();
        v21=v2->getSucc1();
      } else {
        v20=v2;
        v21=zeroDrain;
      }
      newV2VS=v2VS->getSucc1();
    } else {
      newV2VS=v2VS;
      v20=v21=v2;
    }
  } else {
    minTable=v2VS->getUniqueTable();
    v10=v11=v1;
    if(v2->getUniqueTable()==minTable){
      v20=v2->getSucc0();
      v21=v2->getSucc1();
    } else {
      v20=v2;
      v21=zeroDrain;
    }
    newV1VS=v1VS;
    newV2VS=v2VS->getSucc1();
  }

  w0=lessEqual(v10,newV1VS,v20,newV2VS);
  w1=lessEqual(v11,newV1VS,v21,newV2VS);

  node=(w1==zeroDrain)?w0:findOrAdd(minTable,w1,w0);

  //insert result into computed table
  CompareComputedTable.insert(calcPtr(v1,LEE_OP),v1VS,v2,v2VS,node);

  return node;
}

ZADDNode* ZADD::equal(ZADDNode* v1, ZADDNode* v1VS, ZADDNode* v2, ZADDNode* v2VS){
  if((v1VS->isDrain())&&(v2VS->isDrain())) return (v1->getValue()==v2->getValue())?oneDrain:zeroDrain;

  //try to get result from computed table
  ZADDNode* node=(ZADDNode*) CompareComputedTable.find(calcPtr(v1,EQU_OP),v1VS,v2,v2VS);
  if(node) return node;

  ZADDNode* w0;
  ZADDNode* w1;
  ZADDNode* v10;
  ZADDNode* v11;
  ZADDNode* v20;
  ZADDNode* v21;
  ZADDNode* newV1VS;
  ZADDNode* newV2VS;

  ZADDUniqueTable* minTable;

  if(v1VS->getRealLevel()<=v2VS->getRealLevel()){
    minTable=v1VS->getUniqueTable();
    if(v1->getUniqueTable()==minTable){
      v10=v1->getSucc0();
      v11=v1->getSucc1();
    } else {
      v10=v1;
      v11=zeroDrain;
    }
    newV1VS=v1VS->getSucc1();
    if(v2VS->getUniqueTable()==minTable){
      if(v2->getUniqueTable()==minTable){
        v20=v2->getSucc0();
        v21=v2->getSucc1();
      } else {
        v20=v2;
        v21=zeroDrain;
      }
      newV2VS=v2VS->getSucc1();
    } else {
      newV2VS=v2VS;
      v20=v21=v2;
    }
  } else {
    minTable=v2VS->getUniqueTable();
    v10=v11=v1;
    if(v2->getUniqueTable()==minTable){
      v20=v2->getSucc0();
      v21=v2->getSucc1();
    } else {
      v20=v2;
      v21=zeroDrain;
    }
    newV1VS=v1VS;
    newV2VS=v2VS->getSucc1();
  }

  w0=equal(v10,newV1VS,v20,newV2VS);
  w1=equal(v11,newV1VS,v21,newV2VS);

  node=(w1==zeroDrain)?w0:findOrAdd(minTable,w1,w0);

  //insert result into computed table
  CompareComputedTable.insert(calcPtr(v1,EQU_OP),v1VS,v2,v2VS,node);

  return node;
}

ZADDNode* ZADD::max(ZADDNode* v1, ZADDNode* v1VS, ZADDNode* v2,  ZADDNode* v2VS){
  if(v1VS==v2VS){
		if(v1==v2) return v1;
		if(v1VS->isDrain()) return findOrAddDrain(v1->getValue()>v2->getValue()?v1->getValue():v2->getValue());
	}

  swapPointer(&v1,&v1VS,&v2,&v2VS);
  //try to get result from computed table
  ZADDNode* node=(ZADDNode*) ExtremaComputedTable.find(calcPtr(v1,MAX_OP),v1VS,v2,v2VS);
  if(node) return node;
	
  ZADDNode* w0;
  ZADDNode* w1;
  ZADDNode* v10;
  ZADDNode* v11;
  ZADDNode* v20;
  ZADDNode* v21;
  ZADDNode* newV1VS;
  ZADDNode* newV2VS;
	
  ZADDUniqueTable* minTable;
	
  if(v1VS->getRealLevel()<=v2VS->getRealLevel()){
    minTable=v1VS->getUniqueTable();
    if(v1->getUniqueTable()==minTable){
      v10=v1->getSucc0();
      v11=v1->getSucc1();
    } else {
      v10=v1;
      v11=zeroDrain;
    }
    newV1VS=v1VS->getSucc1();
    if(v2VS->getUniqueTable()==minTable){
      newV2VS=v2VS->getSucc1();
      if(v2->getUniqueTable()==minTable){
        v20=v2->getSucc0();
        v21=v2->getSucc1();
      } else {
        v20=v2;
        v21=zeroDrain;
      }
    } else {
      newV2VS=v2VS;
      v20=v21=v2;
    }
  } else {
    minTable=v2VS->getUniqueTable();
    newV1VS=v1VS;
    newV2VS=v2VS->getSucc1();
    v10=v11=v1;
    if(v2->getUniqueTable()==minTable){
      v20=v2->getSucc0();
      v21=v2->getSucc1();
    } else {
      v20=v2;
      v21=zeroDrain;
    }
  }
	
  w0=max(v10,newV1VS,v20,newV2VS);
  w1=max(v11,newV1VS,v21,newV2VS);
	
  node=(w1==zeroDrain)?w0:findOrAdd(minTable,w1,w0);
  //insert result into computed table
  ExtremaComputedTable.insert(calcPtr(v1,MAX_OP),v1VS,v2,v2VS,node);
	
  return node;
}

double ZADD::max(ZADDNode* node, map<ZADDNode*,double>& CT){
  if(node->isDrain()) return node->getValue();
  
  if(CT.find(node)!=CT.end()) return CT[node];
	
  double w0=max(node->getSucc0(),CT);
  double w1=max(node->getSucc1(),CT);
	
  double m=(w0>=w1)?w0:w1;
  CT[node]=m;
  
  return m;
}

ZADDNode* ZADD::min(ZADDNode* v1, ZADDNode* v1VS, ZADDNode* v2,  ZADDNode* v2VS){
  if(v1VS==v2VS){
		if(v1==v2) return v1;
		if(v1VS->isDrain()) return findOrAddDrain(v1->getValue()<v2->getValue()?v1->getValue():v2->getValue());
	}

  swapPointer(&v1,&v1VS,&v2,&v2VS);
  //try to get result from computed table
  ZADDNode* node=(ZADDNode*) ExtremaComputedTable.find(calcPtr(v1,MIN_OP),v1VS,v2,v2VS);
  if(node) return node;
	
  ZADDNode* w0;
  ZADDNode* w1;
  ZADDNode* v10;
  ZADDNode* v11;
  ZADDNode* v20;
  ZADDNode* v21;
  ZADDNode* newV1VS;
  ZADDNode* newV2VS;
	
  ZADDUniqueTable* minTable;
	
  if(v1VS->getRealLevel()<=v2VS->getRealLevel()){
    minTable=v1VS->getUniqueTable();
    if(v1->getUniqueTable()==minTable){
      v10=v1->getSucc0();
      v11=v1->getSucc1();
    } else {
      v10=v1;
      v11=zeroDrain;
    }
    newV1VS=v1VS->getSucc1();
    if(v2VS->getUniqueTable()==minTable){
      newV2VS=v2VS->getSucc1();
      if(v2->getUniqueTable()==minTable){
        v20=v2->getSucc0();
        v21=v2->getSucc1();
      } else {
        v20=v2;
        v21=zeroDrain;
      }
    } else {
      newV2VS=v2VS;
      v20=v21=v2;
    }
  } else {
    minTable=v2VS->getUniqueTable();
    newV1VS=v1VS;
    newV2VS=v2VS->getSucc1();
    v10=v11=v1;
    if(v2->getUniqueTable()==minTable){
      v20=v2->getSucc0();
      v21=v2->getSucc1();
    } else {
      v20=v2;
      v21=zeroDrain;
    }
  }
	
  w0=min(v10,newV1VS,v20,newV2VS);
  w1=min(v11,newV1VS,v21,newV2VS);
	
  node=(w1==zeroDrain)?w0:findOrAdd(minTable,w1,w0);
  //insert result into computed table
  ExtremaComputedTable.insert(calcPtr(v1,MIN_OP),v1VS,v2,v2VS,node);

	return node;
}

double ZADD::min(ZADDNode* node, map<ZADDNode*,double>& CT){
  if(node->isDrain()) return node->getValue();

  if(CT.find(node)!=CT.end()) return CT[node];

  double w0=min(node->getSucc0(),CT);
  double w1=min(node->getSucc1(),CT);

  double m=(w0<=w1)?w0:w1;
  CT[node]=m;

  return m;
}

ZADDNode* ZADD::oneOver(ZADDNode* node){
  if(node->isDrain()) return (node==zeroDrain)?node:findOrAddDrain(1.0/node->getValue());

  ZADDNode* tempZADDNode=(ZADDNode*) ArithmeticComputedTable.find(calcPtr(node,OVE_OP),0,0,0);
  if(tempZADDNode) return tempZADDNode;

  ZADDNode* w0=oneOver(node->getSucc0());
  ZADDNode* w1=oneOver(node->getSucc1());

  tempZADDNode=findOrAdd(node->getUniqueTable(),w1,w0);
  ArithmeticComputedTable.insert(calcPtr(node,OVE_OP),0,0,0,tempZADDNode);

  return tempZADDNode;
}

ZADDNode* ZADD::moveUp(ZADDNode* u){
  if(u->isDrain()) return u;

  ZADDNode* tempNode=(ZADDNode*) MoveComputedTable.find(calcPtr(u,MUP_OP),0,(ZADDNode*) 1);
  if(tempNode) return tempNode;

  tempNode=findOrAdd(u->getLevel()-1,moveUp(u->getSucc1()),moveUp(u->getSucc0()));
  MoveComputedTable.insert(calcPtr(u,MUP_OP),0,(ZADDNode*) 1,tempNode);

  return tempNode;
}

ZADDNode* ZADD::moveUp(ZADDNode* u, const unsigned int skip){
  if(u->isDrain()) return u;

  ZADDNode* tempNode=(ZADDNode*) MoveComputedTable.find(calcPtr(u,MUP_OP),0,(ZADDNode*) skip);
  if(tempNode) return tempNode;

  tempNode=findOrAdd(u->getLevel()-skip,moveUp(u->getSucc1(),skip),moveUp(u->getSucc0(),skip));
  MoveComputedTable.insert(calcPtr(u,MUP_OP),0,(ZADDNode*) skip,tempNode);

  return tempNode;
}

ZADDNode* ZADD::moveUp(ZADDNode* u, ZADDNode* v){
  if(u->isDrain()) return u;

  unsigned long uLevel=u->getLevel();
  while(v->getLevel()<uLevel) v=v->getSucc1();

  if((uLevel<1)&&(uLevel==v->getLevel())) return u;
  if(v->isDrain()) return u;

  ZADDNode* tempNode=(ZADDNode*) MoveComputedTable.find(calcPtr(u,MUP_OP),v,(ZADDNode*) 1);
  if(tempNode) return tempNode;

  ZADDNode* w0;
  ZADDNode* w1;
  if(uLevel==v->getLevel()){
    w0=moveUp(u->getSucc0(),v->getSucc1());
    w1=moveUp(u->getSucc1(),v->getSucc1());

    tempNode=findOrAdd(uLevel-1,w1,w0);
  } else {
    w0=moveUp(u->getSucc0(),v);
    w1=moveUp(u->getSucc1(),v);

    //check result
    if((w0->getLevel()<=uLevel)||(w1->getLevel()<=uLevel)){
      if(w0->preNodeCount==0) w0->garbageCollect();
      if(w1->preNodeCount==0) w1->garbageCollect();
      tempNode=u;
    } else tempNode=findOrAdd(uLevel,w1,w0);
  }

  MoveComputedTable.insert(calcPtr(u,MUP_OP),v,(ZADDNode*) 1,tempNode);

  return tempNode;
}

ZADDNode* ZADD::moveUp(ZADDNode* u, ZADDNode* v, const unsigned int skip){
  if(u->isDrain()) return u;

  unsigned long uLevel=u->getLevel();
  while(v->getLevel()<uLevel) v=v->getSucc1();

  if((uLevel<skip)&&(uLevel==v->getLevel())) return u;
  if(v->isDrain()) return u;

  ZADDNode* tempNode=(ZADDNode*) MoveComputedTable.find(calcPtr(u,MUP_OP),v,(ZADDNode*) skip);
  if(tempNode) return tempNode;

  ZADDNode* w0;
  ZADDNode* w1;
  if(uLevel==v->getLevel()){
    w0=moveUp(u->getSucc0(),v->getSucc1(),skip);
    w1=moveUp(u->getSucc1(),v->getSucc1(),skip);

    tempNode=findOrAdd(uLevel-skip,w1,w0);
  } else {
    w0=moveUp(u->getSucc0(),v,skip);
    w1=moveUp(u->getSucc1(),v,skip);

    //check result
    if((w0->getLevel()<=uLevel)||(w1->getLevel()<=uLevel)){
      if(w0->preNodeCount==0) w0->garbageCollect();
      if(w1->preNodeCount==0) w1->garbageCollect();
      tempNode=u;
    } else tempNode=findOrAdd(uLevel,w1,w0);
  }

  MoveComputedTable.insert(calcPtr(u,MUP_OP),v,(ZADDNode*) skip,tempNode);

  return tempNode;
}

ZADDNode* ZADD::moveDown(ZADDNode* u){
  if(u->isDrain()) return u;

  ZADDNode* tempNode=(ZADDNode*) MoveComputedTable.find(calcPtr(u,MDO_OP),0,(ZADDNode*) 1);
  if(tempNode) return tempNode;

  ZADDNode* w0=moveDown(u->getSucc0());
  ZADDNode* w1=moveDown(u->getSucc1());

  tempNode=(u->getLevel()+1>=variableOrdering.size())?zeroDrain:findOrAdd(u->getLevel()+1,w1,w0);
  MoveComputedTable.insert(calcPtr(u,MDO_OP),0,(ZADDNode*) 1,tempNode);

  return tempNode;
}

ZADDNode* ZADD::moveDown(ZADDNode* u, const unsigned int skip){
  if(u->isDrain()) return u;

  ZADDNode* tempNode=(ZADDNode*) MoveComputedTable.find(calcPtr(u,MDO_OP),0,(ZADDNode*) skip);
  if(tempNode) return tempNode;

  ZADDNode* w0=moveDown(u->getSucc0(),skip);
  ZADDNode* w1=moveDown(u->getSucc1(),skip);

  tempNode=(u->getLevel()+skip>=variableOrdering.size())?zeroDrain:findOrAdd(u->getLevel()+skip,w1,w0);
  MoveComputedTable.insert(calcPtr(u,MDO_OP),0,(ZADDNode*) skip,tempNode);

  return tempNode;
}

ZADDNode* ZADD::moveDown(ZADDNode* u, ZADDNode* v){
  if(u->isDrain()) return u;

  unsigned long uLevel=u->getLevel();
  while(v->getLevel()<uLevel) v=v->getSucc1();

  if(v->isDrain()) return u;

  ZADDNode* tempNode=(ZADDNode*) MoveComputedTable.find(calcPtr(u,MDO_OP),v,(ZADDNode*) 1);
  if(tempNode) return tempNode;

  ZADDNode* w0;
  ZADDNode* w1;
  if(uLevel==v->getLevel()){
    w0=moveDown(u->getSucc0(),v->getSucc1());
    w1=moveDown(u->getSucc1(),v->getSucc1());
    if((uLevel+1>=w0->getLevel())||(uLevel+1>=w1->getLevel())) tempNode=findOrAdd(uLevel,w1,w0);
    else tempNode=findOrAdd(uLevel+1,w1,w0);
  } else {
    w0=moveDown(u->getSucc0(),v);
    w1=moveDown(u->getSucc1(),v);
    tempNode=findOrAdd(u->getLevel(),w1,w0);
  }

  MoveComputedTable.insert(calcPtr(u,MDO_OP),v,(ZADDNode*) 1,tempNode);

  return tempNode;
}

ZADDNode* ZADD::moveDown(ZADDNode* u, ZADDNode* v, const unsigned int skip){
  if(u->isDrain()) return u;

  unsigned long uLevel=u->getLevel();
  while(v->getLevel()<uLevel) v=v->getSucc1();

  if(v->isDrain()) return u;

  ZADDNode* tempNode=(ZADDNode*) MoveComputedTable.find(calcPtr(u,MDO_OP),v,(ZADDNode*) skip);
  if(tempNode) return tempNode;

  ZADDNode* w0;
  ZADDNode* w1;
  if(uLevel==v->getLevel()){
    w0=moveDown(u->getSucc0(),v->getSucc1(),skip);
    w1=moveDown(u->getSucc1(),v->getSucc1(),skip);
    if((uLevel+skip>=w0->getLevel())||(uLevel+skip>=w1->getLevel())) tempNode=findOrAdd(uLevel,w1,w0);
    else tempNode=findOrAdd(uLevel+skip,w1,w0);
  } else {
    w0=moveDown(u->getSucc0(),v,skip);
    w1=moveDown(u->getSucc1(),v,skip);
    tempNode=findOrAdd(uLevel,w1,w0);
  }

  MoveComputedTable.insert(calcPtr(u,MDO_OP),v,(ZADDNode*) skip,tempNode);

  return tempNode;
}

ZADDNode* ZADD::getAssignment(const ZADDNode* node){
  if(node->isDrain()) return (node==zeroDrain)?zeroDrain:oneDrain;

  ZADDNode* tempNode;
  if(node->getSucc0()!=zeroDrain){
    tempNode=getAssignment(node->getSucc0());
  } else {
    tempNode=findOrAdd(node->getUniqueTable(),getAssignment(node->getSucc1()),zeroDrain);
  }

  return tempNode;
}

bool ZADD::getAssignment(const ZADDNode* node, Assignment& assignment) const {
  if(node->isDrain()) return (node!=zeroDrain);

  //all variables are set to zero before this function
  if(!getAssignment(node->getSucc0(),assignment)){
    getAssignment(node->getSucc1(),assignment);
    assignment[variableOrdering[node->getLevel()]]=true;
  }

  return true;
}

bool ZADD::getAssignment(const ZADDNode* node, char* buffer) const {
  if(node->isDrain()) return (node!=zeroDrain);

  if(!getAssignment(node->getSucc0(),buffer)){
    getAssignment(node->getSucc1(),buffer);
    buffer[node->getLevel()]='1';
  }

  return true;
}

double ZADD::countAssignments(unsigned long lastLevel, const ZADDNode* node, const ZADDNode* varSet, map<ZADDNode*,double>& cache) const {
  if(node==zeroDrain) return 0.0;

  double diff;

  unsigned long jumps=0;
  while(varSet->getRealLevel()<node->getRealLevel()){
    varSet=varSet->getSucc1();
    ++jumps;
  }

  if(varSet->isDrain()) diff=(double) (1<<(variableOrdering.size()-lastLevel-jumps));
  else diff=(double) (1<<(varSet->getLevel()-lastLevel-jumps));

  double ret=0.0;
  if(!varSet->isDrain()){
    if(cache.find((ZADDNode*) node)!=cache.end()) ret=cache[(ZADDNode*) node];
    else {
      if(node->getSucc0()==node->getSucc1()){
        ret=2.0*countAssignments(varSet->getLevel()+1,node->getSucc0(),varSet->getSucc1(),cache);
      } else {
        ret=countAssignments(varSet->getLevel()+1,node->getSucc0(),varSet->getSucc1(),cache);
        ret+=countAssignments(varSet->getLevel()+1,node->getSucc1(),varSet->getSucc1(),cache);
      }
      cache[(ZADDNode*) node]=ret;
    }
  } else {
    if(node!=zeroDrain) ret=1.0;
    else ret=0.0;
  }

  return diff*ret;
}

ZADDNode* ZADD::getPath(const ZADDNode* node){
  if(node->isDrain()) return (node==zeroDrain)?zeroDrain:oneDrain;

  ZADDNode* tempNode;

  if(node->getSucc0()!=zeroDrain){
    tempNode=findOrAdd(node->getUniqueTable(),zeroDrain,getPath(node->getSucc0()));
  } else {
    tempNode=findOrAdd(node->getUniqueTable(),getPath(node->getSucc1()),zeroDrain);
  }

  return tempNode;
}

bool ZADD::getPath(const ZADDNode* node, char* buffer) const {
  if(node->isDrain()) return (node!=zeroDrain);

  if(getPath(node->getSucc0(),buffer)){
    buffer[node->getLevel()]='0';
  } else {
    getPath(node->getSucc1(),buffer);
    buffer[node->getLevel()]='1';
  }

  return true;
}

ZADDNode* ZADD::createConstantFunction(const double value){
  return findOrAddDrain(value);
}

ZADDFunction ZADD::createFunction(ZADDNode* node, ZADDNode* varSet){
  if(node==zeroDrain){
    if(varSet->preNodeCount==0) varSet->garbageCollect();
    varSet=oneDrain;
  }

  //dynamic reordering
  if(dynamicReordering){
    if(++operatorCounter>=reorderingThreshold){
      //protect node from garbage collection
      node->incNodeCount();
      varSet->incNodeCount();
      //TODO: different reordering methods
      //variableSifting(1.0);
      ZADD::garbageCollect();
      ZADD::variableSifting();

      //reverse increasing
      --node->preNodeCount;
      --varSet->preNodeCount;

      operatorCounter=0;
    }
  }

  return ZADDFunction(node,varSet);
}

ZADDNode* ZADD::findOrAdd(const unsigned long index, ZADDNode* succ1, ZADDNode* succ0){
  return findOrAdd(variableOrdering.getData(index),succ1,succ0);
}

ZADDNode* ZADD::findOrAdd(ZADDUniqueTable* uTable, ZADDNode* succ1, ZADDNode* succ0){
  //this functions checks if there is an isomorphic node
  //in this case the isomorphic node will be returned otherwise a new node will be created (and returned)

  return uTable->findOrAdd(succ0,succ1);
}

ZADDNode* ZADD::findOrAddDrain(const double value){
  ZADDUniqueTable* uniqueTable=variableOrdering.getDrainData();
  if(value<0.0){
    if(value>-0.0000001) return zeroDrain;
  } else if(value<0.0000001) return zeroDrain;

  ZADDNode node;
  node.setValue(value);

  return uniqueTable->findOrAddValue(&node);
}

ZADDNode* ZADD::findDrain(const double value){
  ZADDUniqueTable* uniqueTable=variableOrdering.getDrainData();
  ZADDNode node;
  node.setValue(value);

  return uniqueTable->getValue(&node);
}
