/***************************************************************************
                          zaddbasenode.h  -  description
                             -------------------
    begin                : Sat Mar 29 2008
    copyright            : (C) 2008 by Joern Ossowski
    email                : mail@jossowski.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ZADDBASENODE_H
#define ZADDBASENODE_H

#include <common/helper.h>
#include <zadd/zaddvarorder.h>
#include <zadd/zadd.h>

#include <climits>

class ZADDBaseNode{
public:
  ZADDBaseNode(void): uniqueTable(0), next(0), preNodeCount(0) {}
  bool isDrain(void) const {return (uniqueTable==0);}
  ZADDUniqueTable* getUniqueTable(void) const {return uniqueTable;}
  void setUniqueTable(const ZADDUniqueTable* uTable){uniqueTable=const_cast<ZADDUniqueTable*>(uTable);}
  void incNodeCount(void){
    if UNLIKELY(preNodeCount==ULONG_MAX){
      if UNLIKELY(uniqueTable==0) ZADD::drainUniqueTable.decDeadNodeCount();
      else uniqueTable->decDeadNodeCount();
      --ZADD::deadNodes;
      preNodeCount=1;
    } else ++preNodeCount;
  }
  void deleteNode(void){
    if UNLIKELY(--preNodeCount==0){
      if UNLIKELY(isDrain()) ZADD::drainUniqueTable.incDeadNodeCount();
      else uniqueTable->incDeadNodeCount();
      ++ZADD::deadNodes;
      preNodeCount=ULONG_MAX;
      if UNLIKELY(ZADD::deadNodes>=ZADD::garbageCollectionDelay) ZADD::garbageCollect();
    }
  }
  void garbageCollect(void){
    if UNLIKELY(isDrain()) ZADD::drainUniqueTable.incDeadNodeCount();
    else uniqueTable->incDeadNodeCount();
    ++ZADD::deadNodes;
    preNodeCount=ULONG_MAX;
  }
  unsigned long getLevel(void) const {return LIKELY(uniqueTable!=0)?uniqueTable->getLevel():ZADD::drainUniqueTable.getLevel();}
  unsigned long getRealLevel(void) const {return LIKELY(uniqueTable!=0)?uniqueTable->getRealLevel():ZADD::drainUniqueTable.getRealLevel();}

  ZADDUniqueTable* uniqueTable;
  ZADDBaseNode* next;
  unsigned long preNodeCount;
};

#endif
