/***************************************************************************
                          zaddfunction.h  -  description
                             -------------------
    begin                : Thu Apr 15 2004
    copyright            : (C) 2004-2008 by Joern Ossowski
    email                : mail@jossowski.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ZADDFUNCTION_H
#define ZADDFUNCTION_H

class ZADDBaseNode;
class Assignment;

#include <string>
#include <set>
#include <map>

#include <boost/operators.hpp>

class ZADDFunction : public boost::operators<ZADDFunction>{
public:
  ZADDFunction(const ZADDFunction&);
  ZADDFunction(const Assignment&, const double=1.0);
  ZADDFunction(char*, const double=1.0);
  ZADDFunction(const std::string&);
  ZADDFunction(const double);
  ZADDFunction(const ZADDBaseNode*,const ZADDBaseNode*);
  ZADDFunction(void);
  ~ZADDFunction(void);
  static ZADDFunction ONE(void);
  static ZADDFunction ZERO(void);
  static ZADDFunction createConstantFunction(const double);
  static ZADDFunction projection(const std::string&);
  static ZADDFunction projection(const unsigned long);
  unsigned long size(void) const;
  ZADDFunction& operator=(const ZADDFunction&);
  ZADDFunction& operator+=(const ZADDFunction&);
  ZADDFunction& operator-=(const ZADDFunction&);
  ZADDFunction& operator*=(const ZADDFunction&);
  ZADDFunction& operator|=(const ZADDFunction&);
  ZADDFunction& operator&=(const ZADDFunction&);
  ZADDFunction& operator^=(const ZADDFunction&);
  ZADDFunction& operator++();
  ZADDFunction& operator--();
  ZADDFunction operator!(void) const;
  bool operator==(const ZADDFunction&) const;
  bool operator<(const ZADDFunction&) const;
  bool isConstant(void) const;
  double getValue(void) const;
  unsigned long getFirstVariableIndex(void) const;
  ZADDFunction cofactor(const unsigned long, const bool) const;
  ZADDFunction cofactor(const ZADDFunction&) const;
  ZADDFunction subSet(const unsigned long, const bool) const;
  ZADDFunction subSet(const ZADDFunction&) const;
  ZADDFunction change(void) const;
  ZADDFunction change(const unsigned long) const;
  ZADDFunction change(const ZADDFunction&) const;
  ZADDFunction exists(const ZADDFunction&) const;
  ZADDFunction exists(const unsigned long) const;
  ZADDFunction forall(const ZADDFunction&) const;
  ZADDFunction forall(const unsigned long) const;
  ZADDFunction equivalenceClasses(const ZADDFunction&, const ZADDFunction&) const;
  ZADDFunction constrain(const ZADDFunction&) const;
  ZADDFunction sum(const ZADDFunction&) const;
  ZADDFunction product(const ZADDFunction&) const;
  ZADDFunction maximum(const ZADDFunction&) const;
  double max(void) const;
  ZADDFunction minimum(const  ZADDFunction&) const;
  double min(void) const;
  ZADDFunction oneOver(void) const;
  ZADDFunction compose(const unsigned long, const ZADDFunction&) const;
  ZADDFunction moveUp() const;
  ZADDFunction moveUp(const ZADDFunction&) const;
  ZADDFunction moveDown() const;
  ZADDFunction moveDown(const ZADDFunction&) const;
  void printEssentialVariables(void) const;
  std::set<std::string> getEssentialVariables(void) const;
  operator Assignment() const;
  ZADDFunction getAssignment(void) const;
  void getAssignment(char*) const;
  double countAssignments(void) const;
  void printPaths(void) const;
  double countPaths(void) const;
  ZADDFunction getPath(void) const;
  void getPath(char*) const;

  std::set<ZADDBaseNode*> getNodes() const;

  struct less : public std::binary_function<ZADDFunction, ZADDFunction, bool>  {
    bool operator()(const ZADDFunction& x, const ZADDFunction& y) const {
      std::less<ZADDBaseNode*> l;

      return 
        (x.rootNode==y.rootNode) 
        ? l(x.varSet,y.varSet) 
        : l(x.rootNode,y.rootNode);
    }
  };


  ZADDBaseNode* rootNode;
  ZADDBaseNode* varSet;
};

inline ZADDFunction operator+(const double value, const ZADDFunction& function){return ZADDFunction(value)+function;}
inline ZADDFunction operator-(const double value, const ZADDFunction& function){return ZADDFunction(value)-function;}
inline ZADDFunction operator*(const double value, const ZADDFunction& function){return ZADDFunction(value)*function;}
ZADDFunction operator-(const ZADDFunction&);
inline ZADDFunction EQUIV(const ZADDFunction& function1, const ZADDFunction& function2){return !(function1^function2);}
inline ZADDFunction IMPLIES(const ZADDFunction& function1, const ZADDFunction& function2){return !function1|function2;}
ZADDFunction ITE(const ZADDFunction&, const ZADDFunction&, const ZADDFunction&);
ZADDFunction LESS(const ZADDFunction&, const ZADDFunction&);
ZADDFunction LESS_EQUAL(const ZADDFunction&, const ZADDFunction&);
ZADDFunction EQUAL(const ZADDFunction&, const ZADDFunction&);
inline ZADDFunction GREATER(const ZADDFunction& function1, const ZADDFunction& function2){return LESS(function2,function1);}
inline ZADDFunction GREATER_EQUAL(const ZADDFunction& function1, const ZADDFunction& function2){return LESS_EQUAL(function2,function1);}
ZADDFunction MAXIMUM(const ZADDFunction&, const ZADDFunction&);
ZADDFunction MINIMUM(const ZADDFunction&, const ZADDFunction&);

ZADDFunction MM(const ZADDFunction&, const ZADDFunction&, const ZADDFunction&);
ZADDFunction MV(const ZADDFunction&, const ZADDFunction&, const ZADDFunction&);
ZADDFunction VM(const ZADDFunction&, const ZADDFunction&, const ZADDFunction&);

#endif
