/***************************************************************************
                          zaddgroup.cpp  -  description
                             -------------------
    begin                : Wed Apr 14 2004
    copyright            : (C) 2004 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <zadd/zadd.h>
#include <zadd/zaddnode.h>
#include <zadd/zaddgroup.h>
#include <zadd/zaddvarorder.h>
#include <common/varorder.h>
#include <common/constants.h>
#include <common/reordering.h>

#include <climits>

ZADDVariableOrdering* ZADDGroup::varOrder=0;
ZADD* ZADDGroup::BDD=0;

ZADDGroup::ZADDGroup(void){
  group=new Group<ZADDUniqueTable>(*varOrder);
  group->incRefCount();
}

ZADDGroup::ZADDGroup(const string& groupName){
  group=varOrder->getGroup(groupName);
  if(!group){
    group=new Group<ZADDUniqueTable>(*varOrder);
    if(groupName.find_first_of(badChars)==string::npos) group->setName(groupName);
  }
  group->incRefCount();
}

ZADDGroup::ZADDGroup(const ZADDGroup& newGroup){
  group=newGroup.group;
  group->incRefCount();
}

ZADDGroup::~ZADDGroup(void){
  group->decRefCount();
  if(!group->getFirst()) varOrder->removeGroup(group->getName());
}

string ZADDGroup::getName(void) const {
  return group->getName();
}

bool ZADDGroup::setName(const string& name){
  return (name.find_first_of(badChars)==string::npos)?group->setName(name):false;
}

string ZADDGroup::getGroupOrder(void){
  return group->getVariableOrdering();
}

bool ZADDGroup::groupVariable(const string& variable){
  Variable<ZADDUniqueTable>* tempVariable=varOrder->getVariable(variable);

  //check if variable exists
  if(!tempVariable) return false;
  //check if variable has a group
  if(tempVariable->getGroup()) return (tempVariable->getGroup()==group);

  //move variable to the edge of the group and group it
  if(group->getFirst()){
    unsigned long index=tempVariable->getLevel();
    //minimize swaps for variable
    if(index+1<group->getFirst()->getLevel()){
      unsigned long tempDelay=ZADD::setGarbageCollectionDelay(0);
      BDD->reordering->shift(index,group->getFirst()->getLevel()-1);
      ZADD::setGarbageCollectionDelay(tempDelay);
    } else if(index>group->getLast()->getLevel()+1){
      unsigned long tempDelay=ZADD::setGarbageCollectionDelay(0);
      BDD->reordering->shift(index,group->getLast()->getLevel()+1);
      ZADD::setGarbageCollectionDelay(tempDelay);
    }
  }
  group->group(variable);

  return true;
}

void ZADDGroup::join(ZADDGroup& group){
  //join two groups
  if(this->group!=group.group){
    string tempVariable;
    //while group has elements
    while(group.size()){
      //get first variable
      tempVariable=group[0];
      //ungroup variable
      group.ungroupVariable(tempVariable);
      //group it to this group
      groupVariable(tempVariable);
    }
  }
}

bool ZADDGroup::ungroupVariable(const string& variable){
  Variable<ZADDUniqueTable>* tempVariable=varOrder->getVariable(variable);

  //check if variable exists
  if(!tempVariable) return false;
  //check if variable is inside this group
  if(tempVariable->getGroup()!=group) return false;
  //if variable is on the edge of this group ungroup can to the work
  if((tempVariable==group->getFirst()) || (tempVariable==group->getLast())) return group->ungroup(variable);

  //prepare reordering
  unsigned long tempDelay=ZADD::setGarbageCollectionDelay(0);
  unsigned long index=tempVariable->getLevel();
  //swap variable to the edge of the group and ungroup it
  if((index-group->getFirst()->getLevel())<(group->getLast()->getLevel()-index)) BDD->reordering->shift(index,group->getFirst()->getLevel());
  else BDD->reordering->shift(index,group->getLast()->getLevel());
  group->ungroup(variable);
  ZADD::setGarbageCollectionDelay(tempDelay);

  return true;
}

void ZADDGroup::ungroup(void){
  group->ungroup();
}

bool ZADDGroup::addVariable(const string& variable){
  if(variable.find_first_of(badChars)!=string::npos) return false;
  else if(!varOrder->getVariable(variable)) return group->addVariable(variable);
  else return groupVariable(variable);
}

bool ZADDGroup::removeVariable(const string& variable){
  return group->removeVariable(variable);
}

unsigned long ZADDGroup::size(void) const {
  return (!group->getFirst())?0:(group->getLast()->getLevel()-group->getFirst()->getLevel()+1);
}

unsigned long ZADDGroup::getIndexOfFirstElement(void) const {
  return (!group->getFirst())?ULONG_MAX:group->getFirst()->getLevel();
}

string ZADDGroup::operator[](unsigned long index) const {
  return (index>=size())?"[ERROR]":varOrder->getVariable(group->getFirst()->getLevel()+index)->getName();
}

ZADDGroup& ZADDGroup::operator<<(const string& variable){
  addVariable(variable);

  return *this;
}

ZADDGroup& ZADDGroup::operator<<(ZADDGroup& group){
  join(group);

  return *this;
}

ZADDGroup& ZADDGroup::operator>>(const string& variable){
  ungroupVariable(variable);

  return *this;
}

ZADDGroup& ZADDGroup::operator=(const ZADDGroup& newGroup){
  if(group!=newGroup.group){
    group->decRefCount();
    group=newGroup.group;
    group->incRefCount();
  }

  return *this;
}

bool ZADDGroup::operator==(const ZADDGroup& group) const {
  return (group.group==this->group);
}

bool ZADDGroup::operator!=(const ZADDGroup& group) const {
  return (group.group!=this->group);
}

ZADDGroup::ZADDGroup(Group<ZADDUniqueTable>* newGroup){
  group=newGroup?newGroup:new Group<ZADDUniqueTable>(*varOrder);
  group->incRefCount();
}
