/***************************************************************************
                          zaddinnernode.h  -  description
                             -------------------
    begin                : Sat Mar 29 2008
    copyright            : (C) 2008 by Joern Ossowski
    email                : mail@jossowski.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ZADDINNERNODE_H
#define ZADDINNERNODE_H

#include <zadd/zaddbasenode.h>
#include <zadd/zaddterminalnode.h>
#include <zadd/zadd.h>
#include <zadd/zaddvarorder.h>
#include <common/helper.h>
#include <common/mempool.h>

template <class T> class MemPool;

class ZADDInnerNode : public ZADDBaseNode {
public:
  ZADDInnerNode(void): succ0(0), succ1(0) {}
  static ZADDInnerNode* getNewNode(void){return ZADD::innerNodeMemPool.alloc();}
  static void freeNode(ZADDInnerNode* node){ZADD::innerNodeMemPool.free(node);}
  static unsigned long PEAK(void){return ZADD::innerNodeMemPool.peak();}
  ZADDInnerNode* getNext(void) const {return static_cast<ZADDInnerNode*>(next);}
  void setNext(const ZADDInnerNode* next){this->next=static_cast<ZADDBaseNode*>(const_cast<ZADDInnerNode*>(next));}
  ZADDBaseNode* getSucc0(void) const {return succ0;}
  ZADDBaseNode* getSucc1(void) const {return succ1;}
  ZADDBaseNode* getSucc(const bool succX) const {return succX?succ1:succ0;}
  void initNode(ZADDUniqueTable* uTable, ZADDBaseNode* s0, ZADDBaseNode* s1){
    s0->incNodeCount();
    s1->incNodeCount();

    //update node information
    succ0=s0;
    succ1=s1;
    uniqueTable=uTable;

    preNodeCount=0;
  }
  void setNode(ZADDUniqueTable* uTable, ZADDBaseNode* s0, ZADDBaseNode* s1){
    ++s0->preNodeCount;
    ++s1->preNodeCount;

    if UNLIKELY(--succ0->preNodeCount==0){
      if UNLIKELY(succ0->isDrain()){
	      ZADD::drainUniqueTable.remove(static_cast<ZADDTerminalNode*>(succ0));
	      ZADDTerminalNode::freeNode(static_cast<ZADDTerminalNode*>(succ0));
      } else {
	      --static_cast<ZADDInnerNode*>(succ0)->getSucc0()->preNodeCount;
	      --static_cast<ZADDInnerNode*>(succ0)->getSucc1()->preNodeCount;
	      succ0->uniqueTable->remove(static_cast<ZADDInnerNode*>(succ0));
	      ZADDInnerNode::freeNode(static_cast<ZADDInnerNode*>(succ0));
      }
    }
    if UNLIKELY(--succ1->preNodeCount==0){
      if UNLIKELY(succ1->isDrain()){
	      ZADD::drainUniqueTable.remove(static_cast<ZADDTerminalNode*>(succ1));
	      ZADDTerminalNode::freeNode(static_cast<ZADDTerminalNode*>(succ1));
      } else {
	      --static_cast<ZADDInnerNode*>(succ1)->getSucc0()->preNodeCount;
	      --static_cast<ZADDInnerNode*>(succ1)->getSucc1()->preNodeCount;
	      succ1->uniqueTable->remove(static_cast<ZADDInnerNode*>(succ1));
	      ZADDInnerNode::freeNode(static_cast<ZADDInnerNode*>(succ1));
      }
    }
    succ0=s0;
    succ1=s1;
    uniqueTable=uTable;
    uniqueTable->add(this);
  }
private:
  ZADDBaseNode* succ0;
  ZADDBaseNode* succ1;
};

#endif
