/***************************************************************************
                          zaddnode.h  -  description
                             -------------------
    begin                : Wed Apr 14 2004
    copyright            : (C) 2004 by Joern Ossowski
    email                : ossowski@cs.uni-bonn.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ZADDNODE_H
#define ZADDNODE_H

#include <zadd/zadd.h>
#include <zadd/zaddvarorder.h>

template <class T> class MemPool;

class ZADDNode;
struct ZADDNodeIntern{
  ZADDNode* succ0;
  ZADDNode* succ1;
};

class ZADDNode{
friend class ZADD;
public:
  ZADDNode(void);
  ~ZADDNode(void);
	static ZADDNode* getNewNode(void);
	static void freeNode(ZADDNode*);
	static unsigned long PEAK(void);
  bool isDrain(void) const;
  ZADDNode* getSucc0(void) const {
		return type.children.succ0;		
	};
  ZADDNode* getSucc1(void) const {
		return type.children.succ1;
	};
  ZADDNode* getSucc(const bool) const;
  ZADDNode* getNext(void) const {
		return next;
	};
  void setNext(const ZADDNode* next){
		this->next=(ZADDNode*) next;
	}
  void setValue(double);
  double getValue(void) const;
  void initNode(ZADDUniqueTable*, ZADDNode*, ZADDNode*);
  void setNode(ZADDUniqueTable*, ZADDNode*, ZADDNode*);
  inline void setDrainTable(const ZADDUniqueTable* uTable){uniqueTable=(ZADDUniqueTable*) uTable;preNodeCount=0;}
  void removeFromUniqueTable(void);
  ZADDUniqueTable* getUniqueTable(void) const;
  void incNodeCount(void){
		if(++preNodeCount==0){
			uniqueTable->decDeadNodeCount();
			--BDD->deadNodes;
			//set correct preNodeCount (zombie node has preNodeCount ULONG_MAX)
			preNodeCount=1;
		}
	}
	void garbageCollect(void);
  unsigned long getLevel(void) const;
  unsigned long getRealLevel(void) const;
  void deleteNode(void);
  bool operator<(const ZADDNode&) const;
private:
  static ZADD* BDD;
	static MemPool<ZADDNode> memPool;
  unsigned long preNodeCount;
  union{
    ZADDNodeIntern children;
	double value;
  } type;
  ZADDNode* next;
  ZADDUniqueTable* uniqueTable;
};

#endif
