/***************************************************************************
                          zaddoutput.cpp  -  description
                             -------------------
    begin                : Tue Jul 13 2004
    copyright            : (C) 2004-2008 by Joern Ossowski
    email                : mail@jossowski.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <zadd/zaddoutput.h>
#include <zadd/zaddbasenode.h>
#include <zadd/zaddinnernode.h>
#include <zadd/zaddterminalnode.h>
#include <zadd/zaddfunction.h>
#include <zadd/zaddvarorder.h>

bool ZADDOutput::storeFunction(const std::string& filename, const ZADDFunction& function){
  return ZADDOutput::storeFunctions(filename,ZADDFunctionContainer(function));
}

bool ZADDOutput::storeFunctions(const std::string& filename, const ZADDFunctionContainer& container){
  ZADDOutput output(filename);
  return output.storeBDD(container);
}

ZADDOutput::ZADDOutput(const std::string& filename){
  file.open(filename.c_str(),ios::out);
}

ZADDOutput::~ZADDOutput(void){
  if(file) file.close();
}

bool ZADDOutput::storeBDD(const ZADDFunctionContainer& container){
  unsigned long i;

  //file open?
  if(!file) return false;
  //empty container?
  if(container.size()==0) return false;

  for(i=0;i<container.size();++i){
    ZADDBaseNode* node=container[i].rootNode;
    file << "rootnode {" << endl;
    file << "  id=" << node << endl;
    file << "}" << endl;
    storeBDD(node);
  }

  return true;
}

void ZADDOutput::storeBDD(ZADDBaseNode* node){
  if(visitedSet.find(node)!=visitedSet.end()) return;
  if(!node->isDrain()){
    file << "node {" << endl;
    file << "  id=" << node << endl;
    file << "  level=" << node->getLevel() << endl;
    file << "  succ0=" << static_cast<ZADDInnerNode*>(node)->getSucc0() << endl;
    file << "  succ1=" << static_cast<ZADDInnerNode*>(node)->getSucc1() << endl;
    file << "  label=" << ZADDVarOrder::getVariableName(node->getLevel()) << endl;
    file << "  drawtype=circle" << endl;
    file << "}" << endl;
    visitedSet.insert(node);
    storeBDD(static_cast<ZADDInnerNode*>(node)->getSucc0());
    storeBDD(static_cast<ZADDInnerNode*>(node)->getSucc1());
  } else {
    file << "node {" << endl;
    file << "  id=" << node << endl;
    file << "  level=" <<  ZADDVarOrder::size() << endl;
    file << "  label=" << (static_cast<ZADDTerminalNode*>(node)->getValue()) << endl;
    file << "  drawtype=box" << endl;
    file << "}" << endl;
    visitedSet.insert(node);
  }
}
