#ifndef AUTOMATON_HPP
#define AUTOMATON_HPP

#include "APSet.hpp"
#include "APElement.hpp"

#include "EdgeContainerExplicit_APElement.hpp"

#include "DA.hpp"
#include "DRA.hpp"
#include "DPA.hpp"
#include "DA_State.hpp"


#include <boost/shared_ptr.hpp>

namespace ltl2dstar {

class Automaton {
public:
  typedef boost::shared_ptr<Automaton> ptr;

  typedef DRA<APElement, EdgeContainerExplicit_APElement> DRA_t;
  typedef DRA_t::shared_ptr DRA_ptr;
  
  typedef DPA<APElement, EdgeContainerExplicit_APElement> DPA_t;
  typedef DPA_t::shared_ptr DPA_ptr;

  enum automaton_t {
    DETERMINISTIC_RABIN_AUTOMATON, 
    DETERMINISTIC_STREETT_AUTOMATON, 
    DETERMINISTIC_PARITY_AUTOMATON};

  Automaton(DRA_ptr dra_) : automaton_type(DETERMINISTIC_RABIN_AUTOMATON), dra(dra_) {
    if (dra->isStreett()) {automaton_type=DETERMINISTIC_STREETT_AUTOMATON;}
  }

  Automaton(DPA_ptr dpa_) : automaton_type(DETERMINISTIC_PARITY_AUTOMATON), dpa(dpa_) {}


  automaton_t getAutomatonType() {return automaton_type;}

  DRA_ptr getDRA() {return dra;}
  DRA_ptr getDSA() {return dra;}
  DPA_ptr getDPA() {return dpa;}

  void print_explicit_v2(std::ostream& out) {
    switch (automaton_type) {
    case DETERMINISTIC_RABIN_AUTOMATON:
    case DETERMINISTIC_STREETT_AUTOMATON:
      dra->print_explicit_v2(out);
      return;
    case DETERMINISTIC_PARITY_AUTOMATON:
      dpa->print_explicit_v2(out);
      return;
    }
    throw "unimplemented";
  }

  void setStartState(unsigned int index) {
    switch (automaton_type) {
    case DETERMINISTIC_RABIN_AUTOMATON:
    case DETERMINISTIC_STREETT_AUTOMATON:
      dra->setStartState(dra->get(index));
      return;
    case DETERMINISTIC_PARITY_AUTOMATON:
      dpa->setStartState(dpa->get(index));
      return;
    }
    throw "unimplemented";
  }

private:
  automaton_t automaton_type;
  DRA_ptr dra;
  DPA_ptr dpa;

};

}
#endif
