/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "Option.hpp"
#include <boost/lexical_cast.hpp>

namespace vereofy {

  Option::Option(const std::string& name_, 
                 const std::string& description_,
                 const std::string& default_value_,
                 bool can_be_shared_) :
    name(name_), 
    description(description_),
    default_value(default_value_), 
    can_be_shared(can_be_shared_) {}

  const std::string Option::getName() const {
    return name;
  }

  const std::string Option::getDescription() const {
    return description;
  }

  const std::string Option::getDefaultValue() const {
    return default_value;
  }

  const std::string Option::getValue() const {
    return value;
  }

  std::string& Option::getValueRef() {
    return value;
  }
  
  bool Option::canBeShared() const {
    return can_be_shared;
  }

  bool Option::canBeSharedWith(const Option& other) const {
    if (!canBeShared() || !other.canBeShared()) {
      return false;
    }

    if (getDefaultValue()!=other.getDefaultValue()) {
      return false;
    }

    return true;
  }
  
  void Option::setValue(const std::string& new_value) {
    value=new_value;
  }

  bool Option::hasEmptyValue() const {
    return value=="";
  }

  void Option::setToDefault() {
    if (hasEmptyValue()) {
      setValue(getDefaultValue());
    }
  }

  unsigned int Option::getUintValue() const {
    return boost::lexical_cast<unsigned int>(value);
  }

  int Option::getIntValue() const {
    return boost::lexical_cast<int>(value);
  }

  bool Option::getBoolValue() const {
    if (value=="true" || value=="1") {
      return true;
    } else if (value=="false" || value=="0") {
      return false;
    }

    throw value+" is not a valid boolean value in option "+name;
  }
}
