/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef OPTION_HPP
#define OPTION_HPP

#include <string>

namespace vereofy {

class Option {
public:
  Option(const std::string& name, 
         const std::string& description,
         const std::string& default_value="",
         bool can_be_shared=false);

  const std::string getName() const;
  const std::string getDescription() const;
  const std::string getDefaultValue() const;
  const std::string getValue() const;
  std::string& getValueRef();

  bool canBeShared() const;
  bool canBeSharedWith(const Option& other) const;

  void setValue(const std::string& new_value);
  bool hasEmptyValue() const;

  void setToDefault();

  unsigned int getUintValue() const;
  int getIntValue() const;
  bool getBoolValue() const;

private:
  std::string name;
  std::string description;
  std::string value;
  std::string default_value;
  bool can_be_shared;
};


}


#endif
