/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/




#include "OptionContainer.hpp"

namespace vereofy {

  void OptionContainer::addOption(const Option& option) {
    if (!options.insert(make_pair(option.getName(), option)).second) {
      throw "There is already an option named "+option.getName();
    }
  }
  
  const Option& OptionContainer::getOption(const std::string& name) const {
    option_map_t::const_iterator it=options.find(name);
    
    if (it==options.end()) {
      throw "There is no option named "+name;
    }
    
    return it->second;
  }

  void OptionContainer::setToDefaults() {
    for (option_map_t::iterator it=options.begin();
         it!=options.end();
         ++it) {
      it->second.setToDefault();
    }
  }
  
  OptionContainer::iterator OptionContainer::begin() {
    return options.begin();
  }

  OptionContainer::const_iterator OptionContainer::begin() const {
    return options.begin();
  }

  OptionContainer::iterator OptionContainer::end() {
    return options.end();
  }
  
  OptionContainer::const_iterator OptionContainer::end() const {
    return options.end();
  }

  void OptionContainer::mergeOptions(const OptionContainer& other) {
    for (const_iterator it=other.begin();
         it!=other.end();
         ++it) {
      const Option& cur=it->second;
      if (options.count(cur.getName())>0) {
        if (cur.canBeSharedWith(getOption(cur.getName()))) {
          // ok, nothing to do
        } else {
          throw "Clashing options "+cur.getName();
        }
      } else {
        this->addOption(cur);
      }
    }
  }
}
