#ifndef PERMUTATION_BY_SWAP_HPP
#define PERMUTATION_BY_SWAP_HPP

class PermutationBySwap {
public:
  PermutationBySwap(unsigned int n) :
  elements(n) {
    for (unsigned int i=0;i<n;i++) {
      elements[i].value = i;
      elements[i].direction = (i==0) ? ZERO : NEGATIVE;
    }
  }

  unsigned long long countIterations() const {
    unsigned long long result = 1;

    for (unsigned int i=2;i<=elements.size();i++) {
      result*=i;
    }

    return result;
  }

  bool hasNext() {
    for (unsigned int i=0;i<elements.size();i++) {
      if (elements[i].direction != ZERO) {
        return true;
      }
    }
    return false;
  }

  void print(std::ostream& out, bool verbose) {
    for (unsigned int i = 0; i < elements.size(); i++) {
      if (i>0) out << " ";
      out << elements[i].value;
    }
    out << "\n";
  }

  struct PermutationPrinter {
    PermutationPrinter(PermutationBySwap& perm_) : perm(perm_) {}

    void operator()(unsigned int a, unsigned int b) const {
      perm.print(std::cout, false);
    }

    PermutationBySwap& perm;
  };

  struct SwapPrinter {
    SwapPrinter(std::ostream& out_) : out(out_) {}

    void operator()(unsigned int a, unsigned int b) const {
      out << a << " <-> " << b << "\n";
    }

    std::ostream& out;
  };


  template <class SwapOperator>
  void next(SwapOperator& swap_op) {
    // find the largest element with non-zero direction then swap it in that direction
    
    unsigned int index = 0;
    unsigned int largest = 0;
    for (unsigned int i = 0; i < elements.size(); i++) {
      if (elements[i].direction != ZERO && largest < elements[i].value) {
        largest = elements[i].value;
        index = i;
      }
    }
    
    int direction = elements[index].direction == NEGATIVE ? -1 : 1;
    unsigned int next_index = index + direction;
    
    // swap elements
    element_t tmp = elements.at(index);
    elements.at(index) = elements.at(next_index);
    elements.at(next_index) = tmp;

    swap_op(index, next_index);
	
    // If the new index is at the end of the range
    // or if the next element in the same direction is larger, the direction of the chosen element is set to ZERO:
    if (next_index == 0 || next_index == elements.size() - 1 || elements[next_index + direction].value > largest) {
      elements.at(next_index).direction = ZERO;
    }
    
    // All elements that are larger than the swapped element have their directions set to POSITIVE or NEGATIVE,
    // depending on their position in the first or second half of hte permutation:
    for (unsigned int i = 0; i < elements.size(); i++) {
      if (elements[i].value > largest) {
        elements[i].direction = (i < elements.size()/2) ? POSITIVE : NEGATIVE;
      }
    }
  }

private:
  enum direction_t {POSITIVE, NEGATIVE, ZERO};

  struct element_t {
    unsigned int value;
    direction_t direction;
  };

  std::vector<element_t> elements;
};

#endif
