/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef BDD_CACHED_FUNCTION_HPP
#define BDD_CACHED_FUNCTION_HPP

#include "vereofy/common/errors.hpp"

namespace vereofy {

  class BDDCachedFunction {
  public:
    BDDCachedFunction() : valid(false), function_value(BDDFunction::ZERO()) {}

    BDDCachedFunction(BDDFunction function_value_, 
                      const BDDFunctionContainer& dependencies_) :
      valid(true), function_value(function_value_), dependencies(dependencies_) {
    }
    
    bool haveDependenciesChanged(const BDDFunctionContainer& current_dependencies) const {
      CHECK_SUCCESS(dependencies.size()==current_dependencies.size());

      return dependencies!=current_dependencies;
    }

    bool isValid() const {return valid;}

    BDDFunction getValue() const {
      CHECK_SUCCESS(valid);
      return function_value;
    }
  private:
    bool valid;
    BDDFunction function_value;
    BDDFunctionContainer dependencies;

  };

}

#endif
