/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef BDD_VAR_HELPERS_HPP
#define BDD_VAR_HELPERS_HPP

// don't include individually, include bdd_helpers.hpp instead

namespace vereofy {

  inline bool bdd_var_exists(const std::string& name) {
#ifdef JINC_NEW
    // VARIABLE_INDEX_MAX is the error function
    return BDDVarOrder::getVariable(name)!=VARIABLE_INDEX_MAX;
#else
    return BDDVarOrder::findVariable(name);
#endif
  }


  /**
   * Returns the name of the variable of the projection function
   */
  inline std::string bdd_var_name(const BDDFunction& proj) {
    BDD_CHECK(3, BDDChecks::isProjection(proj), "", proj);

    VarManager::ReorderLock reorder_lock; // inhibit reordering within this scope
#ifdef JINC_NEW
    return BDDVarOrder::getVariableName(proj.getFirstVariableIndex());
#else
    return BDDVarOrder::getVariable(proj.getFirstVariableIndex());
#endif  
  }

  /**
   * Returns the name of the variable of the given BDD level
   */
  inline std::string bdd_var_name(unsigned int level) {

    VarManager::ReorderLock reorder_lock; // inhibit reordering within this scope
#ifdef JINC_NEW
    return BDDVarOrder::getVariableName(level);
#else
    return BDDVarOrder::getVariable(level);
#endif  
  }


  /**
   * Returns the number of variables in the BDD
   */
  inline unsigned int bdd_var_count() {
    return BDDVarOrder::size();
  }

}

#endif
