/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef CHECKS_HPP
#define CHECKS_HPP

#include <stdexcept>
#include <iostream>
#include <string>
#include <boost/lexical_cast.hpp>

extern unsigned int check_level;

#ifdef NO_CHECKS
#define ERROR_CHECK(expression, error_info, error_action)
#else

#define ERROR_CHECK(expression, error_info, error_action)               \
  do {                                                                  \
    if (expression) {                                                   \
    } else {                                                            \
      error_action;                                                     \
      std::string e_info(error_info);                                   \
      throw std::logic_error("ERROR_CHECK failed for "+std::string(#expression)+" at "+__FILE__+", line "+boost::lexical_cast<std::string>(__LINE__)+(e_info!="" ? (": "+e_info) : "")); \
    }                                                                   \
  } while (0)

#endif

#define ERROR_CHECK_ASSERT(expression) \
  ERROR_CHECK(expression, "", )

#define CHECK_AND_ACT(cost, expression, error_info, error_action)    \
  do {                                                               \
    if (cost<=check_level) {                                         \
      ERROR_CHECK(expression, error_info, error_action);             \
    }                                                                \
  } while (0)

#define CHECK(cost, expression, error_info)     \
  CHECK_AND_ACT(cost, expression, error_info, );


#define BDD_CHECK(cost, expression, error_info, bdd_function)           \
  CHECK_AND_ACT(cost, expression, error_info,                           \
                std::cout << "BDDFunction " << #bdd_function << "\n";   \
                (bdd_function).printPaths());


#define BDD_CHECK2(cost, expression, error_info, bdd_function1, bdd_function2) \
  CHECK_AND_ACT(cost, expression, error_info,                           \
                std::cout << "BDDFunction " << #bdd_function1 << "\n";  \
                (bdd_function1).printPaths();                          \
                std::cout << "\nBDDFunction " << #bdd_function2 << "\n"; \
                (bdd_function2).printPaths());                          \



#endif
