/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef DISJOINT_SETS_HPP
#define DISJOINT_SETS_HPP

#include <boost/pending/disjoint_sets.hpp>
#include <boost/shared_ptr.hpp>
#include <map>

namespace vereofy {

  template <typename T>
  class DisjointSets {
  public:
    DisjointSets() :
      partition(ranks, parents) {}

    void make_set(T element) {
      partition.make_set(element);
    }

    void union_set(T element1, T element2) {
      if (parents.count(element1)==0) {make_set(element1);}
      if (parents.count(element2)==0) {make_set(element2);}

      partition.union_set(element1, element2);
    }

    T find_set(T element) {
      if (parents.count(element)==0) {make_set(element);}

      return partition.find_set(element);
    }

    typename std::map<T,T>::const_iterator parent_map_begin() {
      return parents.begin();
    }

    typename std::map<T,T>::const_iterator parent_map_end() {
      return parents.end();
    }

  private:
    typedef std::map<T, unsigned int> rank_map;
    typedef std::map<T, T> parent_map;

    typedef boost::associative_property_map< rank_map > rank_property_map;
    typedef boost::associative_property_map< parent_map > parent_property_map;

    typedef boost::disjoint_sets<rank_property_map, parent_property_map> disjoint_sets_t;
    
    rank_map ranks;
    parent_map parents;

    disjoint_sets_t partition;
  };
}

#endif
