/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "file_ops.hpp"


#include <boost/algorithm/string.hpp>
#include <fstream>
#include <iostream>

using namespace vereofy;
using namespace std;


bool FileOps::file_exists(const std::string& filename) {
  std::ifstream check_file(filename.c_str());
  if (check_file) {
    return true;
  }
  return false;
}

#if (__WIN32__ || _WIN32)
ReturnValue<std::string> FileOps::get_dirname(const std::string& pathname_) {
  // modified 'dirname' behavior, as appropriate for win

  std::string pathname=pathname_;

  if (pathname == "//" || pathname == "\\\\") return pathname;  // (1)

  string drive_name="";
  if (pathname.length()>=2 && pathname.at(1)==':') {
    drive_name=pathname.substr(0,2);
    pathname=pathname.substr(2);
  }

  if (pathname.find_first_not_of("/\\")==std::string::npos) {  // (2)
    // pathname consists only of / or \ characters
    return drive_name+std::string("\\");
  }
  
  // remove trailing / or \ (3)
  std::string path=trim_right_copy_if(pathname, boost::is_any_of("/\\"));

  if (path.find_first_of("/\\")==std::string::npos) {
    // there are no more remaining /, -> return "."
    return drive_name+std::string("."); // (4)
  }

  // remove trailing non-/ and non-\ characters (5)
  path=path.substr(0, path.find_last_of("/\\"));
  if (path=="//" || path=="\\") return drive_name+"\\"; // (6)
  
  // trim trailing / (7)
  trim_right_if(path, boost::is_any_of("/\\"));
  
  // if empty -> "\" (8)
  if (path.length()==0) {
    path="\\";
  }

  return drive_name+path;
}



ReturnValue<std::string> FileOps::get_basename(const std::string& pathname_) {
  // modified 'basename' behavior, as appropriate for windows

  std::string pathname=pathname_;

  if (pathname.length()>=2 && pathname.at(1)==':') {
    // skip driver letter
    pathname=pathname.substr(2);
  }

  if (pathname =="") return std::string("");  // (1)
  if (pathname.find_first_not_of("/\\")==std::string::npos) {  // (2,3)
    // pathname consists only of / or \ characters
    return std::string("\\");
  }

  // remove trailing / or \ (4)
  std::string path=trim_right_copy_if(pathname, boost::is_any_of("/\\"));
  
  std::size_t last_slash_pos=path.find_last_of("/\\");
  if (last_slash_pos!=std::string::npos) {
    // remove everything to the left of last / or \ (5)
    if (last_slash_pos+1 == std::string::npos) {
      path="";
    } else {
      path=path.substr(last_slash_pos+1);  // from character after / or \\ to end of string
    }
  }

  return path;
}

#else 
// POSIX
ReturnValue<std::string> FileOps::get_dirname(const std::string& pathname) {
  // according to 'dirname' behavior (The Open Group Base Specifications Issue 6)

  if (pathname == "//") return pathname;  // (1)
  if (pathname.find_first_not_of('/')==std::string::npos) {  // (2)
    // pathname consists only of / characters
    return std::string("/");
  }

  // remove trailing / (3)
  std::string path=trim_right_copy_if(pathname, boost::is_any_of("/"));
  if (path.find_first_of('/')==std::string::npos) {
    // there are no more remaining /, -> return "."
    return std::string("."); // (4)
  }

  // remove trailing non-/ characters (5)
  path=path.substr(0, path.find_last_of('/'));
  if (path=="//") return path; // (6)
  
  // trim trailing / (7)
  trim_right_if(path, boost::is_any_of("/"));
  
  // if empty -> "/" (8)
  if (path.length()==0) {
    path="/";
  }

  return path;
}

ReturnValue<std::string> FileOps::get_basename(const std::string& pathname) {
  // according to 'basename' behavior (The Open Group Base Specifications Issue 6)

  if (pathname =="") return std::string("");  // (1)
  if (pathname.find_first_not_of('/')==std::string::npos) {  // (2,3)
    // pathname consists only of / characters
    return std::string("/");
  }

  // remove trailing / (4)
  std::string path=trim_right_copy_if(pathname, boost::is_any_of("/"));
  
  std::size_t last_slash_pos=path.find_last_of('/');
  if (last_slash_pos!=std::string::npos) {
    // remove everything to the left of / (5)
    if (last_slash_pos+1 == std::string::npos) {
      path="";
    } else {
      path=path.substr(last_slash_pos+1);  // from character after / to end of string
    }
  }

  return path;
}



#endif

