/***************************************************************************
 *   Copyright (C) 2008-2009 by Tobias Blechmann                           *
 *   blechman@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef FUNCTIONS_H
#define FUNCTIONS_H

#include <iostream>
#include <list>
#include <string>
#include <vector>
#include "vereofy/setup.hpp"

namespace vereofy {

  std::pair<std::string, std::string> splitAt(const std::string& text, char splitting_symbol);

  std::string setOfStrToStr(const std::set<std::string>&);
  std::list<std::string> stringToList(std::string);
  std::vector<std::string> stringToVector(std::string);
  
  inline void debugOut(const int minLevel, const std::string& message, const void* pointer=0) {  
    if (debugLevel >= minLevel) {
      for (int i=0; i<minLevel; ++i) std::cout << ".";
      if (pointer) std::cout << pointer << " ";
      std::cout << message << std::endl;
    }
  }
  
  inline void debugBDD(const BDDFunction& func, const std::string& text="") {
    std::cout << "DEBUGGING: " << text << std::endl;
    func.printEssentialVariables();
    func.printPaths();
    std::cout << "****************************************" << std::endl;
  }

  inline void debugBDD(const BDDFunctionContainer& cont, const std::string& text="") {
    std::cout << "DEBUGGING Container: " << text << std::endl;
    for (unsigned int i=0; i<cont.size(); ++i) {
      cont.at(i).printEssentialVariables();
      cont.at(i).printPaths();
    }
    std::cout << "****************************************" << std::endl;
  }
  
  
}
#endif
