/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef GRAPH_HELPERS_HPP
#define GRAPH_HELPERS_HPP

#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/depth_first_search.hpp>
#include "vereofy/common/functor_helpers.hpp"

using namespace boost;

namespace vereofy {

  template <class EdgeMap, class Tag>
  struct edge_recorder
    : public base_visitor<edge_recorder<EdgeMap, Tag> >
  {
    typedef Tag event_filter;
    
    edge_recorder(EdgeMap& edge_map_) : edge_map(edge_map_) { }
    template <class Edge, class Graph>
    void operator()(Edge e, const Graph& g) {
      edge_map.insert(e);
    }
    EdgeMap& edge_map;
  };

  template <class EdgeMap, class Tag>
  edge_recorder<EdgeMap, Tag>
  record_edges(EdgeMap& edge_map, Tag) {
    return edge_recorder<EdgeMap, Tag> (edge_map);
  }



  template <class VertexSequence, class Tag, class FilterPredicate>
  struct vertex_recorder
    : public base_visitor<vertex_recorder<VertexSequence, Tag, FilterPredicate> >
  {
    typedef Tag event_filter;
    
    vertex_recorder(VertexSequence& vertex_sequence_,
                    FilterPredicate filter_) :
      vertex_sequence(vertex_sequence_),
      filter(filter_) { }

    template <class Vertex, class Graph>
    void operator()(Vertex v, const Graph& g) {
      if (filter(v)) {
        // push back
        vertex_sequence.insert(vertex_sequence.end(), v);
      }
    }

    VertexSequence& vertex_sequence;
    FilterPredicate filter;
  };

  template <class VertexSequence, class Tag>
  vertex_recorder<VertexSequence, Tag, unary_true<typename VertexSequence::value_type> >
  record_vertices(VertexSequence& vertex_sequence, Tag) {
    typedef unary_true<typename VertexSequence::value_type> unary_true_;

    return vertex_recorder<VertexSequence, Tag, unary_true_> (vertex_sequence, unary_true_());
  }

  template <class VertexSequence, class Tag, class FilterPredicate>
  vertex_recorder<VertexSequence, Tag, FilterPredicate>
  record_vertices(VertexSequence& vertex_sequence, Tag, FilterPredicate filter) {
    return vertex_recorder<VertexSequence, Tag, FilterPredicate> (vertex_sequence, filter);
  }





  template <class DirectedGraphT>
  void dfs_remove_back_edges(DirectedGraphT& graph) {
    typedef typename graph_traits< DirectedGraphT >::vertex_descriptor vertex_t;
    typedef typename graph_traits< DirectedGraphT >::vertex_iterator vertex_iter_t;
    typedef typename graph_traits< DirectedGraphT >::out_edge_iterator out_edge_iter_t;
    typedef typename graph_traits< DirectedGraphT >::edge_descriptor edge_descriptor_t;

    std::map<vertex_t, default_color_type> color_map;

    typedef std::set<edge_descriptor_t> edge_set;
    edge_set back_edges;
    
    depth_first_search(graph,
                       boost::make_dfs_visitor(record_edges(back_edges, boost::on_back_edge())),
                       make_assoc_property_map(color_map));

    for (typename edge_set::iterator it=back_edges.begin();
         it!=back_edges.end();
         ++it) {
      remove_edge(*it, graph);
    }    
  }

}

#endif
