/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef MATH_HELPERS_HPP
#define MATH_HELPERS_HPP

#include <boost/limits.hpp>
#include <cmath>

namespace vereofy {
  /** 
   * Returns the number of bits needed to encode n_distinct_values
   */
  inline unsigned int logSize(unsigned int n_distinct_values) {
    if (n_distinct_values<=1) {return 0;}

    /* Test for GCC > 3.4 */
#if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4)) && !defined(LOG2_FLOAT)
    // __builtin_clz returns the number of consecutive 0 bits, starting from the most significant bit
    // We first calculate the number of bits in an unsigned int and substract from that
    std::numeric_limits<unsigned char> uchar_info;
    return (sizeof(unsigned int) * uchar_info.digits) - __builtin_clz(n_distinct_values - 1);
#else
    return (unsigned int)ceil(log10((double)n_distinct_values)/log10(2.0));
#endif
  }
}


#endif
