/***************************************************************************
 *   Copyright (C) 2008-2009 by Tobias Blechmann                           *
 *   reo@blechmann.de                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
@author Tobias Blechmann
*/

/*! \file setup.h
  This is the main configuration file.
  You can switch between using ADDs oder ZADDs as a symbolic representation for switching functions.
	Also this file defines how verbose the program will be.
*/

#ifndef SETUP_H
#define SETUP_H

// determine which kind of BDDs should be used for representing switching functions
#ifdef JINC_ZADD
  #define USE_ZADD
#else
  #define USE_ADD
#endif

#ifdef USE_ADD
#include "add/add.h"
#include "add/addfunction.h"
#include "add/addoutput.h"
#include "add/addwriter.h"
#include "add/addvarorder.h"

#ifndef JINC_NEW
#include "add/addgroup.h"
#endif
#endif

#ifdef USE_ZADD
#include "zadd/zadd.h"
#include "zadd/zaddfunction.h"
#include "zadd/zaddoutput.h"
#include "zadd/zaddvarorder.h"

#ifndef JINC_NEW
#include "zadd/zaddgroup.h"
#endif
#endif

#include "common/assignment.h"
#include "common/stringhelper.h"

#ifdef USE_ADD
typedef ADD BDD;
typedef ADDFunction BDDFunction;
typedef ADDFunctionContainer BDDFunctionContainer;
typedef ADDVarOrder BDDVarOrder;
typedef ADDGroup BDDGroup;
typedef ADDWriter BDDWriter;
#endif

#ifdef USE_ZADD
typedef ZADD BDD;
typedef ZADDFunction BDDFunction;
typedef ZADDFunctionContainer BDDFunctionContainer;
typedef ZADDVarOrder BDDVarOrder;
typedef ZADDGroup BDDGroup;
#endif

extern int debugLevel;         ///< produces some output usefull for debugging   
extern bool tracingEnabled;


#define MIX_POSITIONS 1        ///< Positions for Variables should be mixed


#endif
