/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SHARED_PTR_MAP_HPP
#define SHARED_PTR_MAP_HPP 

#include <set>
#include <map>

template <class T>
struct ptr_less {
   bool operator()(const T* first, const T* second) const {
     return (*first) < (*second);
  }
};

template <class T>
struct shared_ptr_less {
   bool operator()(const T& first, const T& second) const {
    return first.get() < second.get();
  }
};

template <class T>
struct shared_ptr_id_less {
   bool operator()(const T& first, const T& second) const {
     return first->getID() < second->getID();
  }
};

template <class T>
struct ptr_id_less {
   bool operator()(const T& first, const T& second) const {
     return first->getID() < second->getID();
  }
};

// to use, use the type shared_ptr_set<Key,Value>::type 
template <typename Key>
class shared_ptr_set {
  public:
  typedef std::set<Key, shared_ptr_less<Key> > type;
};


// to use, use the type shared_ptr_map<Key,Value>::type 
template <typename Key, typename Value>
class shared_ptr_map {
  public:
  typedef std::map<Key, Value, shared_ptr_less<Key> > type;
};


// to use, use the type shared_ptr_multimap<Key,Value>::type 
template <typename Key, typename Value>
class shared_ptr_multimap {
  public:
  typedef std::multimap<Key, Value, shared_ptr_less<Key> > type;
};



// to use, use the type shared_ptr_id_set<Key,Value>::type 
template <typename Key>
class shared_ptr_id_set {
  public:
  typedef std::set<Key, shared_ptr_id_less<Key> > type;
};


// to use, use the type shared_ptr_id_map<Key,Value>::type 
template <typename Key, typename Value>
class shared_ptr_id_map {
  public:
  typedef std::map<Key, Value, shared_ptr_id_less<Key> > type;
};


// to use, use the type shared_ptr_id_multimap<Key,Value>::type 
template <typename Key, typename Value>
class shared_ptr_id_multimap {
  public:
  typedef std::multimap<Key, Value, shared_ptr_id_less<Key> > type;
};

#endif
