/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SINGLETON_SET_HPP
#define SINGLETON_SET_HPP 

#include <boost/iterator/iterator_facade.hpp>

template <typename T>
class SingletonSet {
public:

  SingletonSet(T& value_) : value(value_) {}

  T& getValue() const {return value;}

  template <class Value>
  class Iterator : public boost::iterator_facade<
    Iterator<Value>,
    Value,
    boost::forward_traversal_tag> {
    
  public:
    Iterator() : is_end(true) {}
    
    explicit Iterator(const SingletonSet& s_set_, bool end=false) :
      s_set(s_set_), is_end(end) {}
    
  private:
    friend class boost::iterator_core_access;
    
    void increment() {is_end=true;}
    
    bool equal(Iterator const& other) const {
      return this->is_end==other.is_end && &this->s_set==&other.s_set;
    }
  
    Value& dereference() const {
      if (is_end) {
	throw "Derefencing end iterator!";
      }
      return s_set.getValue();
    }
    
    const SingletonSet& s_set;
    bool is_end;
  };

  typedef Iterator<T> iterator;
  typedef Iterator<T const> const_iterator;
  typedef T& value_type;

  iterator begin() {
    return iterator(*this, false);
  }

  const_iterator begin() const {
    return const_iterator(*this, false);
  }

  iterator end() {
    return iterator(*this, true);
  }

  const_iterator end() const {
    return const_iterator(*this, true);
  }


  T& value;
};

#endif

