/***************************************************************************
 *   Copyright (C) 2008-2010 by Tobias Blechmann                           *
 *   blechman@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
@author Tobias Blechmann
*/

#include "stopwatch.hpp"
#include "functions.hpp"
#include <iostream>
#include <iomanip>
#include <boost/lexical_cast.hpp>

using namespace std;

vereofy::StopWatch::StopWatch() {
  startTime=0;
  elapsedTime=0;
  startCPUTimeSec=0;
  startCPUTimeUSec=0;
  CPUTimeSec=0;
  CPUTimeUSec=0;
}

vereofy::StopWatch::~StopWatch() {
}

void vereofy::StopWatch::start() {
  startClock=clock();
  startTime=time(NULL);

#if !defined(_WIN32)
  getrusage(RUSAGE_SELF, &usage);
  startCPUTimeSec = usage.ru_utime.tv_sec;
  startCPUTimeUSec = usage.ru_utime.tv_usec;
#endif
}

void vereofy::StopWatch::stop() {
#if !defined(_WIN32)
  getrusage(RUSAGE_SELF, &usage);
#endif

  clock_t endClock=clock();
  time_t endTime=time(NULL);
  
  double time_diff=difftime(endTime, startTime);
  if (time_diff<10) {
    // use more precise clock
    elapsedTime=((double)(endClock-startClock))/CLOCKS_PER_SEC;
  } else {
    elapsedTime=time_diff;
  }

#if !defined(_WIN32)
  CPUTimeSec = usage.ru_utime.tv_sec - startCPUTimeSec;
  if (usage.ru_utime.tv_usec >= startCPUTimeUSec) 
    CPUTimeUSec = usage.ru_utime.tv_usec - startCPUTimeSec;
  else {
    --CPUTimeSec;
    CPUTimeUSec = (usage.ru_utime.tv_usec + 1000000) - startCPUTimeUSec;
  }
#endif
}

void vereofy::StopWatch::print(const string& text) {
  cout << "*****************stopWatch*****************" << endl;
  cout << text << endl;
  cout << "Time: " << elapsedTime << "sec" << endl;
  cout << "CPU time: " << getCPUTime() << "sec (user)" << endl;
  cout << "*******************************************" << endl;
}

string vereofy::StopWatch::getCPUTime() const {
#if defined(_WIN32) 
  return "unknown";
#else
  string USecString = boost::lexical_cast<string>(CPUTimeUSec);
  while (USecString.length() < 6) USecString = "0" + USecString; 
  return boost::lexical_cast<string>(CPUTimeSec) + "." + USecString;
#endif
}

string vereofy::StopWatch::getElapsedTimeString() const {
  stringstream ss;
  ss << setprecision(3) << fixed << getElapsedTime();
  return ss.str();
}

double vereofy::StopWatch::getElapsedTime() const {
  return elapsedTime;
}

string vereofy::StopWatch::toString() const {
  string result=getElapsedTimeString() + " sec (elapsed)";
#if !defined(_WIN32) 
  result+=", "+getCPUTime()+" sec (CPUTime)";
#endif
  return result;
}
