/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef TRACING_HPP
#define TRACING_HPP

#include "vereofy/setup.hpp"
#include <boost/shared_ptr.hpp>
#include <iostream>
#include <string>
#include <map>

#define TRACE_GET_FULL_NAME(scope, name) scope "::" name 
#define TRACE(scope, name, action)					\
  do {									\
    if (tracingEnabled && Tracing::isTraced(scope, TRACE_GET_FULL_NAME(scope, name))) { \
      action;								\
      Tracing::incTraceCount(TRACE_GET_FULL_NAME(scope,name));		\
    }									\
  } while (0)

class Tracing {
public:
  
  static void activateTracePoint(const std::string& trace_point) {
    getInstance().trace_points[trace_point]=0;
  }
  
  static bool isTraced(const std::string& scope, const std::string& full_name) {
    if (getInstance().trace_points.count(scope)>0) {
      return true;
    }
    
    return getInstance().trace_points.count(full_name)>0;
  }

  static unsigned int getTraceCount(const std::string& full_name) {
    std::map<std::string, unsigned int>::iterator it=getInstance().trace_points.find(full_name);
    if (it != getInstance().trace_points.end()) {
      return it->second;
    }
    return 0;
  }

  static void incTraceCount(const std::string& full_name) {
    getInstance().trace_points[full_name]++;
  }
  
private:
  explicit Tracing() {}
  
  static Tracing& getInstance() {
    if (!instance) {
      std::cerr << "instantiate\n";
      instance=Tracing::ptr(new Tracing());
    }
    return *instance;
  }
  
  typedef boost::shared_ptr<Tracing> ptr;
  static Tracing::ptr instance;
  
  std::map<std::string, unsigned int> trace_points;
};


#endif
