/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef WEAK_PTR_HELPER_H
#define WEAK_PTR_HELPER_H

#include <boost/shared_ptr.hpp>

namespace vereofy {

  /* 
   * Get a shared_ptr for the given weak_ptr, with check to ensure that
   * the resulting shared_ptr can be used for access.
   * Throws exception if the resulting shared_ptr is empty
   */
  template <class Y>
  inline boost::shared_ptr<Y> access_weak_ptr(const boost::weak_ptr<Y>& y) {
    if (y.expired()) {
      throw "access_weak_ptr: reference gone!";
    }
    return y.lock();
  }

  /*
   * Get a shared_ptr from a weak_ptr. If the weak_ptr is expired, will return
   * an empty shared_ptr.
   */
  template <class Y>
  inline boost::shared_ptr<Y> get_shared_ptr_from_weak_ptr(const boost::weak_ptr<Y>& y) {
    return y.lock();
  }

}

#endif
