
#ifndef BDD_G4G_TRANSFORMATION_HPP
#define BDD_G4G_TRANSFORMATION_HPP

#include "VarManagerG4G.hpp"

#include "vereofy/setup.hpp"
#include "vereofy/common/errors.hpp"
#include "vereofy/common/bdd_helpers.hpp"
#include "vereofy/modelcheck/checkers/ltl_check/NBA_I.hpp"
#include "BDD_NBA_Unary.hpp"
#include "vereofy/common/stopwatch.hpp"
#include "vereofy/common/bdd_cached_function.hpp"
#include <boost/lexical_cast.hpp>
#include <boost/shared_ptr.hpp>
#include <string>
#include <iostream>

#include "BDD_RabinAcceptance.hpp"
#include "BDD_ParityAcceptance.hpp"

#include "BDD_G4G_NRA.hpp"

namespace vereofy {namespace ltl_check {

class BDD_G4G_Transformation : public BDD_G4G_NRA {
public:
  typedef boost::shared_ptr<BDD_G4G_Transformation> ptr;

  BDD_G4G_Transformation(APSet_cp apset_,
            VarManagerG4G& varmanager_,
            std::ostream* dot_stream_=(std::ostream*)(NULL)) :
    BDD_G4G_NRA(apset_, varmanager_),
    dot_stream(dot_stream_) {

    if (dot_stream) {
      (*dot_stream) << "digraph { \n";
    }
  }

  virtual ~BDD_G4G_Transformation();

  virtual void slim() {
    nba.reset();
    state_pattern = BDDFunction::ZERO();
    state_pattern_primed = BDDFunction::ZERO();

    vars->free_above(max_set);
  }
  
  void print_stats(std::ostream& out, bool count_sccs, const std::string &prefix) {
    out << std::endl;
    out << prefix << "NBA-number of states: " << nba->getStateCount() << std::endl << std::endl;
    BDD_G4G_NRA::print_stats(out, count_sccs, prefix);
  }

  void set_complete_state_pattern(const bool& complete_state_pattern_) {
    complete_state_pattern = complete_state_pattern_;
  }

  void set_twice_state_pattern(const bool& twice_state_pattern_) {
    twice_state_pattern = twice_state_pattern_;
  }

  void set_duplicate_sets(const bool& duplicate_sets_) {
    duplicate_sets = duplicate_sets_;
  }

  void from_nba(const BDD_NBA_Unary::ptr nba, 
                StateVars_G4G::ptr vars,
                const unsigned int max_set);

  virtual std::string print_single_state(const BDDFunction& q, bool with_acceptance=false);

protected:
  BDD_NBA_Unary::ptr nba;
  StateVars_G4G::ptr vars;

  bool complete_state_pattern;
  bool twice_state_pattern;
  bool duplicate_sets;

  BDDFunction state_pattern;
  BDDFunction state_pattern_primed;

  unsigned int max_set;

  std::ostream* dot_stream;

  void print_states(BDDFunction Q);
  BDDFunction empty_set(const BDDFunctionContainer& container); 
  BDDFunction create_state_pattern(const bool& primed);
  BDDFunction ind_E(unsigned int min);
  BDDFunction ind_F(unsigned int min);
  BDDFunction get_odd_parity_set(unsigned int i);
  BDDFunction get_even_parity_set(unsigned int i);
};

}}

#endif
