#ifndef BDD_G4G_UNION_HPP
#define BDD_G4G_UNION_HPP

#include "BDD_G4G_NRA.hpp"

namespace vereofy { namespace ltl_check {

class BDD_G4G_Union : public BDD_G4G_NRA {
public:
  typedef boost::shared_ptr<BDD_G4G_Union> ptr;

  ~BDD_G4G_Union() {
    // registered state vars will be removed in
    // ~BDD_G4G_NRA
  }
  
  BDD_G4G_Union(BDD_G4G_NRA::ptr left, BDD_G4G_NRA::ptr right) :
    BDD_G4G_NRA(left->getAPSet(), left->getVarManager())
  {
    CHECK_SUCCESS(left->getAPSet() == right->getAPSet());
    CHECK_SUCCESS(&left->getVarManager() == &right->getVarManager());

    // slim the left and right automaton
    left->slim();
    right->slim();

    // register the state vars
    registerStateVars_G4G(left->getRegisteredStateVars_G4G());
    registerStateVars_G4G(right->getRegisteredStateVars_G4G());
    
    initial = left->getInitialStates() & right->getInitialStates();
    delta = left->getDelta() & right->getDelta();
    
    csStateVariables = 
      left->get_state_vars(false) &  
      right->get_state_vars(false);
    
    CHECK_SUCCESS(left->get_transition_vars() == right->get_transition_vars());
    csTransitionVariables = left->get_transition_vars();
    
    state_variables = left->get_state_var_container(false);
    state_variables.push(right->get_state_var_container(false));
    
    for (unsigned int i=0; i<left->countRabinPairs(); i++) {
      acceptance_rabin.addPair(left->getRabinPair(i));
    }
    for (unsigned int i=0; i<right->countRabinPairs(); i++) {
      acceptance_rabin.addPair(right->getRabinPair(i));
    }
  }

};

}}

#endif
