#include "BDD_NBA_Unary.hpp"
#include "Reachability.hpp"
//#include "common/varset.h"
#include "vereofy/common/stopwatch.hpp"


namespace vereofy { namespace ltl_check {

unsigned int BDD_NBA_Unary::nba_i_newState() {
  ++states;
      
  BDDFunction state_var = varmanager.create_nba_state_variable_in(*vars);

  state_variables.push(state_var);

  to_implications.push(BDDFunction::ZERO());

  state_vars &= state_var;
  all_states |= state_var;

  initial &= !state_var;

  accepting_container.push(BDDFunction::ZERO());

  return vars->size()-1;
}



void BDD_NBA_Unary::nba_i_addEdge(unsigned int from_state,
                                  const APMonom& m,
                                  unsigned int to_state) {
  if (m.isFalse()) {return;}

  BDDFunction from = state_variables.at(from_state);
  TRACE_BDD_FUNCTION("nba_i_addEdge", from, "From: " << from_state);

  BDDFunction m_bdd=monom2bdd(m);
  TRACE_BDD_FUNCTION("nba_i_addEdge", m_bdd, "Monom: " << m);

  TRACE_BDD_FUNCTION("nba_i_addEdge", to_implications[to_state], 
                     "To: " << to_state << "(before)");
  to_implications[to_state] |= from & m_bdd;
  TRACE_BDD_FUNCTION("nba_i_addEdge", to_implications[to_state], 
                     "To: " << to_state << "(after)");

  if (dot_stream) {
    (*dot_stream) << from_state << " -> " << to_state << " [label=\"" << m << "\"]\n";
  }
}

APSet_cp BDD_NBA_Unary::nba_i_getAPSet() {
  return apset;
}

void BDD_NBA_Unary::nba_i_setFinal(unsigned int state, bool final) {
  if(final) {
      accepting_container[state] = BDDFunction::ONE();
  } else {
      accepting_container[state] = BDDFunction::ZERO();
  }
  //TRACE_BDD_FUNCTION("nba_i_setFinal", accepting, "before");
  //if (final) {
  //  accepting&=state2bdd(state, false);
  //} else {
  //  accepting&=!state2bdd(state, false);
  //}
  //TRACE_BDD_FUNCTION("nba_i_setFinal", accepting, "after");
}

void BDD_NBA_Unary::nba_i_setStartState(unsigned int state) {
  initial=state2bdd(state, false);
  TRACE_BDD_FUNCTION("nba_i_setStartState", initial, "");
  initial_state_index = state;
}

void BDD_NBA_Unary::nba_i_finished() {
  
  delta = BDDFunction::ONE();
  for (unsigned int i=0;i<to_implications.size();i++) {

    BDDFunction to = state_variables.at(i).moveDown();
    TRACE_BDD_FUNCTION("nba_i_finished", to, "NBA state "<< i);


    BDDFunction to_implication = to_implications[i];
    TRACE_BDD_FUNCTION("nba_i_finished", to_implication, "NBA state "<< i);

    BDDFunction rule = IMPLIES(to, to_implication);
    TRACE_BDD_FUNCTION("nba_i_finished", rule, "");

    TRACE_BDD_FUNCTION("nba_i_finished", delta, "(before)");
    delta &= rule;
    TRACE_BDD_FUNCTION("nba_i_finished", delta, "(after)");
  }

  //accepting_container.clear();
  //for(unsigned int i = 0; i < states; i++) {
  //  bool is_accepting = bdd_is_element_of_set(state2bdd(i, false), accepting);
  //  if(is_accepting) {
  //      accepting_container.push(BDDFunction::ONE());
  //  } else {
  //      accepting_container.push(BDDFunction::ZERO());
  //  }
  //}


  if (dot_stream) {
    for (unsigned int i=0;i<states;i++) {
      bool is_accepting = (accepting_container.at(i) == BDDFunction::ONE());
      bool is_initial = bdd_is_element_of_set(state2bdd(i, false),
                                              initial);
      
      (*dot_stream) 
        << i 
        << " [label=\"" 
        << i
        << (is_initial ? "*" : "")
        << "\"";
      if (is_accepting) {
        (*dot_stream) << ", shape=box"; 
      } else {
        (*dot_stream) << ", shape=ellipse"; 
      }
      (*dot_stream) << "]\n";
    }
    
    (*dot_stream) << "}\n";

    dot_stream->flush();
    dot_stream = NULL;
  }
}


BDDFunction BDD_NBA_Unary::calcReachable() {
  return Reachability::reachable(getInitialStates(),
                                 delta,
                                 getStateVars(),
                                 getAPSetVars());
}


void BDD_NBA_Unary::toDot(std::ostream& out) {
  VarManagerG4G::ReorderLock reorder_lock; // inhibit reordering for the rest of this scope
  
  out << "digraph G {\n";
  BDDFunction reachable = calcReachable();
  TRACE_BDD_FUNCTION("toDot", reachable, "");
  
  BDDFunction reachableCopy = reachable;
  while (reachableCopy != BDDFunction::ZERO()) {
    BDDFunction from=popAssignment(reachableCopy, getStateVars());
    out << "  \"" << assignmentToBin(from, getStateVars()) << "\" [";
    
    bool is_initial= (from == getInitialStates());
    bool is_accepting = false; // TODO

    out << "shape=box, color=";
    if (is_initial && is_accepting) {
      out << "violet";
    } else if (is_initial) {
      out << "blue";
    } else if (is_accepting) {
      out << "red";
    } else {
      out << "black";
    }

    // add label
    out << ", label=\"";
    out << assignmentToBin(from, getStateVars());
    out << "\"]\n";
    
    out << std::endl;

    BDDFunction deltaFrom = (delta & from).exists(getStateVars());
    while (deltaFrom != BDDFunction::ZERO()) {
      BDDFunction to_pair = popAssignment(deltaFrom, 
                                          getStateVars().moveDown() &
                                          getAPSetVars());
      
      out << " \"" << assignmentToBin(from, getStateVars()) << "\" -> \""
          << assignmentToBin(to_pair, getStateVars().moveDown())
          << "\" [label=\"" << assignmentToBin(to_pair, getAPSetVars())
          << "\"];\n";
    }
  }

  out << "}\n";
}


}}
