
#include "LTL2DSTAR.hpp"
#include "ext/ltl2dstar/parsers/DAParser.hpp"
#include "vereofy/common/TempFile.hpp"
#include "vereofy/common/RunProgram.hpp"
#include "vereofy/common/stopwatch.hpp"

using namespace vereofy;
using namespace ltl2dstar;

LTL2DSTAR::DA_ptr LTL2DSTAR::ltl2da(ltl_check::LTLFormula_ptr ltl, 
                                    const std::string &path,
                                    const std::vector<std::string> &params) {
  //  // Create canonical APSet (with 'p0', 'p1', ... as AP)
  //  ltl_check::LTLFormula_ptr ltl_canonical=ltl->copy();
  //  APSet_cp canonical_apset(ltl->getAPSet()->createCanonical());
  //  ltl_canonical->switchAPSet(canonical_apset);
  
  NamedTempFile infile(true);
  NamedTempFile outfile(true);
  
  std::ostream& o=infile.getOStream();
  o << ltl->toStringPrefix() << std::endl;
  o.flush();
  
  std::vector<std::string> arguments;
  for (unsigned int i=0;i<params.size();i++) {
    arguments.push_back(params.at(i));
  }
  arguments.push_back(infile.getFileName());
  arguments.push_back(outfile.getFileName());

  StopWatch stopwatch_ltl2dstar;
  stopwatch_ltl2dstar.start();
  
  const char *program_path;
  if (path!="") {
    program_path=path.c_str();
  } else {
    program_path="./ltl2dstar";
  }

  std::cerr << "Running ltl2dstar... "<< std::flush;
  
  RunProgram ltl2dstar(program_path,
                       arguments,
                       true,
                       0,
                       0,
                       0);
  
  int rv=ltl2dstar.waitForTermination();
  stopwatch_ltl2dstar.stop();
  if (rv==0) {
    std::cerr << "(" <<
      stopwatch_ltl2dstar.getElapsedTimeString() << "s)\n";

    StopWatch stopwatch_parsing;
    stopwatch_parsing.start();
    std::cerr << "Parsing DA... " << std::flush;
    std::istream& in = outfile.getIStream();
    Automaton::ptr automaton = DAParser::parse(in);
    stopwatch_parsing.stop();    
    std::cerr << " (" << stopwatch_parsing.getElapsedTimeString() <<
      "s)\n";
    return automaton;
  } else {
    // There was an error, return null ptr
    return Automaton::ptr();
  }
}


