#ifndef RABIN_GAME_HPP
#define RABIN_GAME_HPP

#include "vereofy/setup.hpp"
#include "vereofy/common/bdd_helpers.hpp"
#include "vereofy/common/BitSet.hpp"
#include "DotStateWriter.hpp"
#include <iostream>

#include "BDD_RabinAcceptance.hpp"
#include "BDD_StreettAcceptance.hpp"

namespace vereofy {

class RabinGame {
public:

  RabinGame(BDDFunction initial_vertices_,
            BDDFunction V_0_, 
            BDDFunction V_1_,
            BDDFunction csVertices_,
            BDDFunction E_)  : 
    initial_vertices(initial_vertices_),
    V_0(V_0_),
    V_1(V_1_),
    E(E_),
    csVertices(csVertices_) {
    TRACE_BDD_FUNCTION("RabinGame", initial_vertices, "");
    TRACE_BDD_FUNCTION("RabinGame", V_0, "");
    TRACE_BDD_FUNCTION("RabinGame", V_1, "");
    TRACE_BDD_FUNCTION("RabinGame", E, "");
    TRACE_BDD_FUNCTION("RabinGame", csVertices, "");

    CHECK(1, check_game_structure(), "");
  }

  void addRabinPairs(const std::vector<BDD_RabinPair>& p) {
    for (unsigned int i=0;i<p.size();i++) {
      pairs.addPair(p.at(i));
    }
  }

  void addRabinPair(BDD_RabinPair rg) {
    TRACE_BDD_FUNCTION("RabinGame", rg.R, pairs.countPairs());
    TRACE_BDD_FUNCTION("RabinGame", rg.G, pairs.countPairs());
    pairs.addPair(rg);
  }

  BDDFunction calculateW();

  BDDFunction calculateReachable() const {
    return calculateReachable(initial_vertices);
  }
  BDDFunction calculateReachable(BDDFunction from) const;

  BDDFunction getE() const {return E;}
  BDDFunction getCsVertices() const {return csVertices;}
  BDDFunction getRestrictedE(BDDFunction state_restriction) const {
    return E & state_restriction & state_restriction.moveDown();
  }


  RabinGame accordingToV0Strategy(BDDFunction strategy) const;
  RabinGame restrictedToVertices(BDDFunction V) const;

  void toDot(std::ostream& out, 
             BDDFunction winning = BDDFunction::ZERO(),
             DotStateWriter *state_writer = (DotStateWriter*)NULL ) const;

  void print_stats(std::ostream& out) const;

  BDDFunction find_strategy() const;
  bool check_strategy(BDDFunction strategy) const;
  

  bool check_game_structure() const;
private:
  BDDFunction initial_vertices;
  BDDFunction V_0, V_1, E;
  BDDFunction csVertices;
  BDD_RabinAcceptance pairs;  

  BDDFunction cpred(BDDFunction Q);
  BDDFunction post(BDDFunction Q) const;
  BDDFunction pre(BDDFunction Q) const;
  BDDFunction Rabin(const BitSet &rabin_pairs,
                    BDDFunction seqnr,
                    BDDFunction right);
};
}

#endif
