#ifndef SCC_FINDER_HPP
#define SCC_FINDER_HPP


class SCCFinder {
public:
  SCCFinder(BDDFunction E_, 
            BDDFunction csStateVars_) :
    E(E_),
    csStateVars(csStateVars_) {
  }
  
  const BDDFunction& getCsStateVars() const {
    return csStateVars;
  }


  template <class ReportFct>
  bool Lockstep(BDDFunction P, ReportFct& report) {
    BDDFunction F, B;
    BDDFunction Ffront, Bfront, C, Converged;

    if (P == BDDFunction::ZERO()) {
      return true;
    }

    // pick assignment = node v
    BDDFunction scratch = P;
    BDDFunction v = popAssignment(scratch, csStateVars);
    F = Ffront = v;
    B = Bfront = v;

    while (Ffront != BDDFunction::ZERO() &&
           Bfront != BDDFunction::ZERO()) {
      Ffront = (post(Ffront) & P) & !F;
      Bfront = (pre(Bfront) & P) & !B;
      F |= Ffront;
      B |= Bfront;
    }
    
    if (Ffront == BDDFunction::ZERO()) {
      Converged = F;
    } else {
      Converged = B;
    }
    
    while ((Ffront & B) != BDDFunction::ZERO() ||
           (Bfront & F) != BDDFunction::ZERO()) {
      Ffront = (post(Ffront) & P) & !F;
      Bfront = (pre(Bfront) & P) & !B;

      F |= Ffront;
      B |= Bfront;
    }

    C = F & B;
    
    bool go_on = report(C);
    if (!go_on) {return false;}
    
    go_on = Lockstep(Converged & !C, report);
    if (!go_on) {return false;}
    go_on = Lockstep(P & !Converged, report);
    return go_on;
  }


  BDDFunction post(BDDFunction Q) const {
    return ((E & Q).exists(csStateVars)).moveUp();
  }

  BDDFunction pre(BDDFunction Q) const {
    return ((E & Q.moveDown()).exists(csStateVars.moveDown()));
  }

private:
  BDDFunction E;
  BDDFunction csStateVars;
};

#endif
