#ifndef VARMANAGER_G4G_HPP
#define VARMANAGER_G4G_HPP

#ifdef VARMANAGER_H
#error Please include VarManagerG4G.hpp before other #includes!
#endif
// define VARMANAGER_H so that vereofy/system/symb_automaton/varmanager.hpp
// will not be loaded

#define VARMANAGER_H

#include "vereofy/setup.hpp"
#include "vereofy/common/errors.hpp"
#include "APElement.hpp"
#include "APTransformation.hpp"
#include <list>
#include <boost/shared_ptr.hpp>
#include <iostream>

namespace vereofy {
  // fwd
  class Symb_FlatDataDomain; 
}


#include "StateVars_NBA.hpp"
#include "StateVars_G4G.hpp"
#include "StateVars_DA.hpp"


class VarManagerG4G {
public:

  VarManagerG4G() : 
    have_ap_variables(false),
    game_variable(BDDFunction::ZERO())
  {}
  
  // create ap variables. If there are already some, check that size is
  // is the same and don't create new ones
  void create_ap_variables(unsigned int size);


  BDDFunction create_nba_state_variable_in(StateVars_NBA& nba_variables);


  StateVars_DA::ptr create_da_state_variables(unsigned int state_count);

  BDDFunctionContainer create_action_variables(BDDFunctionContainer state_vars_prime,
                                               bool action_vars_first);

  BDDFunction create_game_variable();
  void free_game_variable();
  BDDFunction getGameVariable() const {
    if (game_variable == BDDFunction::ZERO()) {
      const_cast<VarManagerG4G&>(*this).create_game_variable();
    }
    return game_variable;
  }

  const BDDFunctionContainer& getAPVariables() const {
    return ap_variables;
  }

  StateVars_G4G::ptr createG4GVariablesFor(const StateVars_NBA& nba_variables);

  BDDFunction apelement_2_bdd(ltl2dstar::APElement element) const;
  BDDFunction apelement_2_bdd(ltl2dstar::APElement element,
                              APTransformation& transformation) const;


  void print_varorder(std::ostream& out) const;
  void print_vars(std::ostream& out, const BDDFunctionContainer& vars, bool compact=true) const;

  struct ReorderLock {
    ReorderLock();
    ~ReorderLock();
  };

  // returns the number of primed copies of normal state variables
  unsigned int primed_count() const {return 1;}

  void enableDynamicReordering();
  void disableDynamicReordering();
  void setDynamicReorderingMaxGrowth(float maxGrowth);
  void setDynamicReorderingTimeout(unsigned int timeout_ms);
  
  void triggerReordering();

  bool free(BDDFunction& proj);

protected:
  BDDFunctionContainer ap_variables;
  bool have_ap_variables;
  BDDFunction game_variable;

  static bool dynamic_reorder_enabled;
  static unsigned int reorder_lock;
  
  static void setReorderingState();

  static void acquireReorderingLock();
  static void releaseReorderingLock();
  

  BDDFunction createVariable(const std::string& name, 
                             unsigned int position);
  BDDFunction createVariablePair(const std::string& name, 
                                 unsigned int position);

  BDDFunctionContainer createVarsInGroup(const std::vector<std::string>& names,
                                         unsigned int position);


  unsigned int after_var(BDDFunction projection) const;
  unsigned int after_prime_var(const BDDFunction projection) const;
  unsigned int after_last_var(const BDDFunctionContainer& vars) const;
  unsigned int after_last_prime_var(const BDDFunctionContainer& vars) const;
  unsigned int at_start() const;
  unsigned int at_end() const;

  typedef boost::shared_ptr< BDDGroup > BDDGroup_ptr;
  typedef std::list< BDDGroup_ptr > BDDGroupList;
  
  BDDGroupList group_list;
};


// add typedef so that VarManager from Vereofy will not be used!
typedef VarManagerG4G VarManager;



#endif
