
#include "VarManagerAM.hpp"

#include <stdio.h>
#include "vereofy/setup.hpp"

#include "vereofy/common/tracing.hpp"
#include "vereofy/common/checks.hpp"
#include "vereofy/common/stopwatch.hpp"
#include "vereofy/common/bdd_helpers.hpp"
#include "vereofy/common/TempFile.hpp"
#include "vereofy/common/RunProgram.hpp"
#include "vereofy/modelcheck/specifications/ltl/LTLInfixParser.hpp"
#include "vereofy/modelcheck/specifications/ltl/LTLPrefixParser.hpp"
#include "Reachability.hpp"
/*
#include "vereofy/modelcheck/checkers/ltl_check/LTL2NBA.hpp"
#include "BDD_NBA_Unary.hpp"
#include "DRA_BDD.hpp"
#include "LTL2DSTAR.hpp"
*/

#include <boost/program_options.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/algorithm/string/trim_all.hpp>
#include <boost/algorithm/string/split.hpp>

namespace po=boost::program_options;
using namespace vereofy::ltl_check;
using namespace vereofy;
using namespace std;
using namespace vereofy;

struct DSA {
    BDDFunction initial;
    BDDFunction delta;
    vector<pair<BDDFunction,BDDFunction> > acc;
};

class AM2BDD {
private:
  VarManagerAM varmanager;
  std::ofstream* dot_stream;
  map<BDDFunction,unsigned long long int,BDDFunction::less> reachables_index; 

  //DRA_BDD::ptr bdd_dra;

  APSet_cp ap_set;
  
  //std::string da_type;

  map<std::string,BDDFunction> symbol_table;
  BDDFunctionContainer statevars;
  map<std::string,BDDFunction> statevar_table;
  std::string param_lbtt;
  std::string param_ltl_formula;
  std::string param_ltl_file;
  std::string param_dump_bdd;
  std::string param_averest;
  std::string param_dot_file;
  bool param_ltl_is_prefix_format;
  bool param_rabin;
  bool param_stats;
  bool param_direct;

  //std::vector<std::string> ltl2dstar_options;

  bool dynamic_reordering;
  float dynamic_reordering_max_growth;
  unsigned int dynamic_reordering_timeout;

  DSA dsa;

public:
  
  AM2BDD() : 
    dot_stream(new std::ofstream("nba.dot")) {}

  ~AM2BDD() {
    delete dot_stream;
  }

  LTLFormula_ptr parse_formula_infix(const std::string& formula) {
    return LTLInfixParser::parse(formula);
  }
  
  LTLFormula_ptr parse_formula_prefix(const std::string& formula) {
    return LTLPrefixParser::parse(formula);
  }
  
  BDDFunction getStateVars(bool primed=false) const {
      if(primed) {
        return container2cs(statevars).moveDown();
      } else {
        return container2cs(statevars);
      }
  }
  
  BDDFunction calcReachable() const {
      return Reachability::reachable(dsa.initial, dsa.delta, getStateVars(), 
              container2cs(varmanager.getAPVariables()));
  }
  
  
  int usage(const std::string& message="") {
    if (message!="") {
      std::cout << "Error: " << message << "\n";    
    }
    std::cout << "\n\n am2bdd formula\n";
    return 2;
  }

  std::string ltl_to_quartz(const std::string& formula) {
    std::string result;
    LTLFormula_ptr ltl_formula;
    if (param_ltl_is_prefix_format) {
      ltl_formula = parse_formula_prefix(formula);
    } else {
      ltl_formula = parse_formula_infix(formula);
    }
    if(param_rabin) {
        ltl_formula = ltl_formula->negate();
    }
    ap_set = ltl_formula->getAPSet();
    if(ap_set->size() > 0) {
        result = "spec v1(bool ";
        for(int i = 0; i < ap_set->size() - 1; i++) {
          result += ap_set->getAP(i) + ",";
        } 
        result += ap_set->getAP(ap_set->size() - 1) + ")";
    } else {
        result = "spec v1()";
    }
    result += "{\n\tAssert1: A " + ltl_formula->toStringInfix(true) + ";\n}";
    return result;
  }

  void toDot(std::ostream& out) const {
    VarManagerAM::ReorderLock reorder_lock; // inhibit reordering for the rest of this scope
    
    out << "digraph G {\n";
    BDDFunction reachable = calcReachable();
    TRACE_BDD_FUNCTION("toDot", reachable, "");
    
    BDDFunction reachableCopy = reachable;
    BDDFunction csStates = getStateVars();
    BDDFunction csAP = container2cs(varmanager.getAPVariables());
    while (reachableCopy != BDDFunction::ZERO()) {
      BDDFunction from=popAssignment(reachableCopy, csStates);
      out << "  \"" << assignmentToBin(from, csStates) << "\" [";
      
      out << "shape=box";
      
      // add label
      out << ", label=\"";
      out << assignmentToBin(from, csStates);
      out << " | ";
      
      for (unsigned int i=0;i<dsa.acc.size();i++) {
        if ((from & dsa.acc.at(i).first) != BDDFunction::ZERO()) {
          out << "-" << i;
        } else if ((from & dsa.acc.at(i).second) != BDDFunction::ZERO()) {
          out << "+" << i;
        }
      }
      out << "\"]\n";
      
      out << std::endl;
      
      BDDFunction delta_from = (dsa.delta & from).exists(csStates);
      while (delta_from != BDDFunction::ZERO()) {
        BDDFunction toPair = popAssignment(delta_from, 
                                           csAP & 
                                           csStates.moveDown());
        
        out << " \"" << assignmentToBin(from, csStates) << "\" -> \""
          << assignmentToBin(toPair, csStates.moveDown())
            << "\" [label = \"" <<assignmentToBin(toPair, csAP) <<  "\"];\n";
      }
    }
    
    out << "}\n";
  }

  std::string get_substring_until_space(const std::string& str, const unsigned int pos = 0) {
    size_t pos_space = str.find(" ", pos);
    if(pos_space == std::string::npos) {
        return str.substr(pos);
    }
    return str.substr(pos, pos_space - pos);
  }

  BDDFunction switch_fct_to_bdd(const vector<std::string>& prefix_splitted, unsigned int& cur_index) {
    BDDFunction result;
    std::string op = prefix_splitted[cur_index];
    if(op == "0") {
        return BDDFunction::ZERO();
    }
    if(op == "1") {
        return BDDFunction::ONE();
    }
    if(op == "!") {
        cur_index++;
        return !switch_fct_to_bdd(prefix_splitted, cur_index);
    }
    if(op == "&" || op == "|") {
        cur_index++;
        BDDFunction lhs = switch_fct_to_bdd(prefix_splitted, cur_index);
        cur_index++;
        BDDFunction rhs = switch_fct_to_bdd(prefix_splitted, cur_index);
        if(op == "&") {
            return lhs & rhs;
        }
        return lhs | rhs;
    }
    if(op == "ite") {
        cur_index++;
        BDDFunction guard = switch_fct_to_bdd(prefix_splitted, cur_index);
        cur_index++;
        BDDFunction true_op = switch_fct_to_bdd(prefix_splitted, cur_index);
        cur_index++;
        BDDFunction false_op = switch_fct_to_bdd(prefix_splitted, cur_index);
        return (guard & true_op) | (!guard & false_op);
    }
    //Symbol must be an input symbol or an state var
    std::string identifier = prefix_splitted[cur_index];
    std::map<std::string,BDDFunction>::iterator it = symbol_table.find(identifier);
    if(it != symbol_table.end()) {
        return it->second;
    }
    it = statevar_table.find(identifier);
    if(it != statevar_table.end()) {
        return it->second;
    }
    throw std::string("Unknown operator: " + identifier);
  }

  void qrz_det_to_bdd(const std::string& det_file_name) {
    enum qrz_section_enum {COUNT, ALPHABET, STATE, ACC};
    std::ifstream det_ifs(det_file_name.c_str());
    std::string line;
    unsigned int street_pairs_count;
    qrz_section_enum qrz_section = COUNT;
    streampos state_def_pos;
    bool got_all_id_of_statevars = false;
    dsa.initial = BDDFunction::ONE();
    dsa.acc.clear();
    dsa.delta = BDDFunction::ONE();
    unsigned int line_count = 0;
    BDDFunction acc_lhs;

    if(det_ifs.is_open()) {
        
        while(getline(det_ifs, line)) {
            boost::algorithm::trim_all(line);
            if(line == "# input variables state variables streett/rabin pairs") {
                qrz_section = COUNT;
            } else {
            if(line == "#input variables #") {
                qrz_section = ALPHABET;
            } else {
            if(line == "# state variable definition #") {
                qrz_section = STATE;
                state_def_pos = det_ifs.tellg();
            } else {
            if(line == "# Streett Pairs") {
                qrz_section = ACC;
            } else {
            if(line[0] != '#') {
                switch(qrz_section) {
                    case COUNT: {
                                    unsigned int ap_size = boost::lexical_cast<unsigned int>(
                                        get_substring_until_space(line));
                                    varmanager.create_ap_variables(ap_size);
                                    size_t statevar_pos = line.find(" ");
                                    size_t acc_pair_pos = line.find(" ", statevar_pos + 1);
                                    street_pairs_count = boost::lexical_cast<unsigned int>(
                                        get_substring_until_space(line,acc_pair_pos + 1)); 
                                    break;
                                }
                    case ALPHABET: {
                                    symbol_table.insert(make_pair(line, varmanager.getAPVariables().at(symbol_table.size())));
                                    break;
                                   }
                    case STATE: {
                                    if(got_all_id_of_statevars) {
                                        std::vector<std::string> prefix_splitted;
                                        std::string statevar_id = get_substring_until_space(line);
                                        BDDFunction primed_var = 
                                            statevar_table[
                                                statevar_id].moveDown();
                                        line = line.substr(line.find_first_of(" ") + 1);                                                 
                                        boost::algorithm::split(prefix_splitted, 
                                                                line,
                                                                boost::is_any_of(" "), 
                                                                boost::algorithm::token_compress_on);
                                        //Creating initial
                                        if(prefix_splitted[0] == "0") {
                                            dsa.initial &= !statevar_table[statevar_id];
                                        } else {
                                            dsa.initial &= statevar_table[statevar_id];
                                        }
                                        prefix_splitted.erase(prefix_splitted.begin());
                                        //Creating delta
                                        unsigned int temp = 0;
                                        BDDFunction guard = switch_fct_to_bdd(prefix_splitted,temp);
                                        TRACE_BDD_FUNCTION("DA", guard, statevar_id + " " + line);
                                        dsa.delta &= (primed_var & guard) |
                                                    (!primed_var & !guard);
                                    } else {
                                        std::string statevar_id = get_substring_until_space(line);
                                        //cout << "Found following state var: " << statevar_id << std::endl;
                                        statevar_table.insert(make_pair(
                                                    statevar_id,
                                                    varmanager.create_dsa_state_variable(statevars)));
                                    }
                                    break;
                                }
                    case ACC: {
                                if(!got_all_id_of_statevars) {
                                    got_all_id_of_statevars = true;
                                    qrz_section = STATE;
                                    det_ifs.seekg(state_def_pos);
                                } else {
                                    if(line_count % 2 == 0) {
                                        std::vector<std::string> prefix_splitted;
                                        boost::algorithm::split(prefix_splitted,
                                                                line,
                                                                boost::is_any_of(" "),
                                                                boost::algorithm::token_compress_on);
                                        unsigned int temp = 0;
                                        acc_lhs =  switch_fct_to_bdd(prefix_splitted, temp);
                                    } else {
                                        std::vector<std::string> prefix_splitted;
                                        boost::algorithm::split(prefix_splitted,
                                                                line,
                                                                boost::is_any_of(" "),
                                                                boost::algorithm::token_compress_on);
                                        unsigned int temp = 0;
                                        BDDFunction acc_rhs = switch_fct_to_bdd(prefix_splitted, temp);
                                        if(param_rabin) {
                                            dsa.acc.push_back(make_pair(acc_rhs, acc_lhs));
                                        } else {
                                            dsa.acc.push_back(make_pair(acc_lhs, acc_rhs));
                                        }
                                    }
                                    line_count++;
                                }
                                break;
                              }
                    default:
                        std::cout << "Putt kaputt" << std::endl;
                }
            }
            }
            } 
            }
            }
        }
    }
    det_ifs.close();
  }

  void dump_bdd() {
    VarManagerAM::ReorderLock lock;

    std::ofstream dump_stream(param_dump_bdd.c_str());
    if(param_rabin) {
        dump_stream << "BDD-DRA Pairs: " << dsa.acc.size() << " ";
    } else {
        dump_stream << "BDD-DSA Pairs: " << dsa.acc.size() << " ";
    }
    dump_stream << "Vars: " << bdd_var_count() << " ";      
    dump_stream << "AP: ";
    std::cout << "APset.size() = " << ap_set->size() << std::endl;
    for (unsigned int i=0;i<ap_set->size();i++) {
      if (i>0) dump_stream << " ";
      dump_stream << ap_set->getAP(i);
    }
    dump_stream << "\n";
        
    BDDFunctionContainer to_dump;
    to_dump.push(dsa.delta);
    to_dump.push(dsa.initial);
    for (unsigned int i=0;i<dsa.acc.size();i++) {
      to_dump.push(dsa.acc[i].first);
      to_dump.push(dsa.acc[i].second);
    }
    
    BDDWriter::storeFunctions(dump_stream, to_dump);

  }
  
  
  //LTL2DSTAR::DA_ptr ltl_to_da(const std::string& formula) {
  //  LTLFormula_ptr ltl_formula;
  //  if (param_ltl_is_prefix_format) {
  //    ltl_formula = parse_formula_prefix(formula);
  //  } else {
  //    ltl_formula = parse_formula_infix(formula);
  //  }
  //  
  //  unsigned int ap_size = ltl_formula->getAPSet()->size();
  //  
  //  ap_set = ltl_formula->getAPSet();

  //  // create AP variables
  //  varmanager.create_ap_variables(ap_size);
  //      

  //  LTL2DSTAR::DA_ptr da = LTL2DSTAR::ltl2da(ltl_formula,
  //                                           "",
  //                                           ltl2dstar_options);
  //                                           
  //  return da;
  //}

  std::string ap2string(const BDDFunction& ap) {
    const BDDFunctionContainer& ap_container = varmanager.getAPVariables();
    if(ap_container.size() == 0) {
        return "t";
    } else {
        std::string result("");
        for(unsigned int i = 0; i < ap_container.size() - 1; i++) {
            result += "& ";
        }
        for(unsigned int i = 0; i < ap_container.size(); i++) {
            if(bdd_is_bit_set(ap, ap_container.at(i))) {
                result +=  ap_set->getAP(i)+ " ";
            } else {
                result += "!" + ap_set->getAP(i) + " ";
            }
        }
        return result;
    }
  }

  unsigned long long int state2int (const BDDFunction& q) {
          return reachables_index[q];
  }

  void create_reachable_index() {
    BDDFunction reachable_copy = calcReachable();
    unsigned long long int index = 0;
    while(reachable_copy != BDDFunction::ZERO()) {
        BDDFunction state = popAssignment(reachable_copy, getStateVars(false));
        reachables_index[state] = index;
        index++;
    }
  }


  //Coding of the states
  //every state has an index i
  //Assume the G4G NRA has n Rabin pairs
  //The uncopied state has number i * n
  //state i of copy j has number i * n + j
  void to_lbtt(std::ostream& out) {
    VarManagerAM::ReorderLock reorder_lock; // inhibit reordering for the rest of this scope
    create_reachable_index();
    unsigned int reachable_nba_states;
    reachable_nba_states = (dsa.acc.size() + 1) * (reachables_index.size());

    out << reachable_nba_states << " " << "1s" << std::endl;

    BDDFunction csTransitionVariables = container2cs(varmanager.getAPVariables());
    BDDFunction reachable_copy = calcReachable();
    while(reachable_copy != BDDFunction::ZERO()) {
        BDDFunction from=popAssignment(reachable_copy, getStateVars(false));
        unsigned int from_index;
        from_index = (state2int(from) * (dsa.acc.size() + 1));
        
        out << from_index << " ";
        if(bdd_is_element_of_set(from, dsa.initial)) {
            out << "1 -1";
        } else {
            out << "0 -1";
        }
        out << std::endl;

        //calculate the successors states
        BDDFunction deltaFrom = (dsa.delta & from).exists(getStateVars(false));
        while (deltaFrom != BDDFunction::ZERO()) {
            BDDFunction to_pair = popAssignment(deltaFrom, 
                                              getStateVars(true) & 
                                              csTransitionVariables);
          
            BDDFunction to_state = to_pair.exists(csTransitionVariables).moveUp();
            unsigned long long int state_index;
            state_index = state2int(to_state) * (dsa.acc.size() + 1);
            out << state_index << " ";
            BDDFunction ap = to_pair.exists(getStateVars(true));
            out << ap2string(ap) << std::endl;
            for(unsigned int i = 1; i <= dsa.acc.size(); i++) {
                out << (state_index + i) << " " << ap2string(ap) << std::endl;
            }
        }
        out << "-1" << std::endl;

        //calculate for rabin pairs 
        for(unsigned int i = 0; i < dsa.acc.size(); i++) {
            pair<BDDFunction,BDDFunction> rabin_pair = dsa.acc[i];
            out << (state2int(from) * (dsa.acc.size() + 1) + i + 1) << " ";
            if(bdd_is_element_of_set(from, rabin_pair.second)) {
                out << "0 0 -1";
            } else {
                out << "0 -1";
            }
            out << std::endl;
        

            if(!bdd_is_element_of_set(from, rabin_pair.first)) {
                deltaFrom = (dsa.delta & from).exists(getStateVars(false));
                while (deltaFrom != BDDFunction::ZERO()) {
                    BDDFunction to_pair = popAssignment(deltaFrom, 
                                                      getStateVars(true) & 
                                                      csTransitionVariables);
                  
                    BDDFunction to_state = to_pair.exists(csTransitionVariables).moveUp();
                    unsigned long long int state_index = state2int(to_state) * (dsa.acc.size() + 1) + i + 1;
                    out << state_index << " ";
                    BDDFunction ap = to_pair.exists(getStateVars(true));
                    out << ap2string(ap) << std::endl;
                }
            }
            out << "-1" << std::endl;
        }
    }
}
  
  
  
  bool apply_options(int argc, char** argv) {
    po::options_description opt_general("General options");
    opt_general.add_options()
      ("help", "produce a help message")
      ("ltl", 
       po::value<string>(&param_ltl_formula),
       "LTL formula to generate")
      ("ltl-file",
       po::value<string>(&param_ltl_file)->default_value(""),
       "Specifies the file from which the formula should be read")
      ("show-stats",
       po::value<bool>(&param_stats)->default_value(true),
       "Write statistics about the automaton to std::cout")
      ("prefix-format",
       po::value<bool>(&param_ltl_is_prefix_format)->default_value(true),
       "LTL formula is in prefix format (like input for ltl2dstar)")
      ("dump-bdd",
       po::value<string>(&param_dump_bdd),
       "Dump the BDD to the specified file")
      ("averest",
       po::value<string>(&param_averest)->default_value("./Quartz2DetStreett"),
       "Path to the Averest/Quartz2DetStreett binary")
      ("rabin",
       po::value<bool>(&param_rabin)->default_value(false),
       "Create automaton with Rabin accepting condition instead of Streett")
      ("to-dot",
       po::value<string>(&param_dot_file),
       "The dot file, to which the DRA/DSA is written")
      ("to-lbtt",
       po::value<std::string>(&param_lbtt)->default_value(""),
       "The file to which the NBA for LBTT should be written (- stands for writing to stdout)")
      ("direct",
       po::value<bool>(&param_direct)->default_value(false),
       "Enables direct simulation in Quartz2DetStreett")
      ;
    
    po::options_description opt_bdd("BDD options");
    opt_bdd.add_options()
      ("dynamic-reorder", 
       po::value<bool>(&dynamic_reordering)->default_value(0),
       "Dynamic reordering of the variables")
      ("dynamic-reorder-max-growth",
       po::value<float>(&dynamic_reordering_max_growth)->default_value(MAX_GROWTH_DEFAULT),
       "The maximum intermediate growth of the BDD size during dynamic reordering (1.0 = no growth, 2.0 = may double)")
      ("dynamic-reorder-timeout",
       po::value<unsigned int>(&dynamic_reordering_timeout)->default_value(0),
       "A timeout (in milliseconds) for unsuccessful sifting for each variable during reordering. 0 = no timeout");

    po::options_description opt_debug("Debug options");
    opt_debug.add_options()
      ("debug",
       po::value<int>(&debugLevel),  // global variable
       "Debug Level (0=no info, 5=max info)")  
      ("trace,T", 
       po::value<vector<string> >(),
       "trace points for debugging")
      ("check-level",
       po::value<unsigned int>(&check_level),  // global variable
       "level of internal sanity checks (0=off)");
    
    po::options_description opt_visible("Options");
    opt_visible.add(opt_general).add(opt_bdd).add(opt_debug);
    
    po::options_description opt_all("All Options");
    opt_all.add(opt_visible);
   
    po::positional_options_description p;
    p.add("ltl-file", 1);
    p.add("to-lbtt", 2);

    po::variables_map vm;
    std::vector<std::string> cmdline_arguments;
    for (int i=1; i<argc; i++) {
      cmdline_arguments.push_back(argv[i]);
    }
    po::store(po::command_line_parser(cmdline_arguments).options(opt_all).positional(p).run(), vm);
    po::notify(vm);    
    
    if (vm.count("help")) {
      cout << opt_visible;
      return false;
    }
    
    if (vm.count("trace")) {
      std::vector<std::string> trace_points=vm["trace"].as< vector<string> >();
      
      tracingEnabled=true;
      for (unsigned int i=0;i<trace_points.size();i++) {
        Tracing::activateTracePoint(trace_points.at(i));
        cerr << "Activate trace point : " << trace_points.at(i) << endl;
      }
    }

    if (dynamic_reordering) {
      varmanager.enableDynamicReordering();
    } else {
      varmanager.disableDynamicReordering();
    }
    varmanager.setDynamicReorderingMaxGrowth(dynamic_reordering_max_growth);
    varmanager.setDynamicReorderingTimeout(dynamic_reordering_timeout);

    //if (vm.count("option")) {
    //  ltl2dstar_options=vm["option"].as< vector<string> >();
    //}


    return true;
  }


  std::string read_ltl_formula(const std::string& file) {
    std::string ltl_formula;
    if (file=="-") {
        std::getline(std::cin, ltl_formula);
    } else {
        std::ifstream ltl_input(file.c_str());
        std::getline(ltl_input, ltl_formula);
    }

    return ltl_formula;
  }

  //void dra_to_bdd(ltl2dstar::Automaton::DRA_ptr dra) {
  //  StopWatch stopwatch_bdd;
  //  stopwatch_bdd.start();
  //  
  //  bdd_dra = DRA_BDD::dra_to_bdd(varmanager,
  //                                dra,
  //                                dra->getAPSet_cp());
  //  
  //  stopwatch_bdd.stop();
  //}


  int run(int argc, char **argv) {
    if (!apply_options(argc, argv)) {
      return 2;
    }

    if (param_ltl_file!="") {
        param_ltl_formula = read_ltl_formula(param_ltl_file);
    }
    if (param_ltl_formula=="") {
        std::cerr << "No LTL formula. Abort...\n";
        return 1;
    }

    if(param_lbtt != "-") {
        std::cout << "----------------------------------------------------\n";
        std::cout << "LTL-Input: " << param_ltl_formula << std::endl;
    }


    StopWatch stopwatch_total;
    stopwatch_total.start();

    NamedTempFile qrz_file(true);
    std::string qrz_file_name = qrz_file.getFileName();

    std::ofstream qrz_stream((qrz_file_name + ".qrz").c_str());
    //cout << "Generating " << qrz_file_name << ".qrz" << endl;
    qrz_stream << ltl_to_quartz(param_ltl_formula) << std::endl;

    vector<string> arguments;
    if(param_direct) {
        arguments.push_back("-minimize");
        arguments.push_back("Direct");
    }
    arguments.push_back(qrz_file_name + ".qrz");
    RunProgram Quartz2DetStreet(param_averest,
                                arguments,
                                true);
    int rv = Quartz2DetStreet.waitForTermination();
    if(rv==0) {
        qrz_det_to_bdd(qrz_file_name + ".det");
        TRACE_BDD_FUNCTION("DA", dsa.initial, "initial");
        TRACE_BDD_FUNCTION("DA", dsa.delta, "delta");
        for(int i = 0; i < dsa.acc.size(); i++) {
            TRACE_BDD_FUNCTION("DA", dsa.acc[i].first, "Acc" + boost::lexical_cast<string>(i) + ": lhs");
            TRACE_BDD_FUNCTION("DA", dsa.acc[i].second, "Acc" + boost::lexical_cast<string>(i) + ": rhs");
        } 
        if(param_dump_bdd != "") {
            dump_bdd();
        }
        remove((qrz_file_name + ".qrz").c_str());
        remove((qrz_file_name + ".det").c_str());
    } else {
        throw "Quartz2DetStreet terminated unnormally.";
    }

    if(param_dot_file != "") {
        std::ofstream dot_stream(param_dot_file.c_str());
        toDot(dot_stream);
    }

    if(param_lbtt!="") { 
        if(param_lbtt=="-") { 
            to_lbtt(std::cout); 
        } else { 
            std::ofstream lbtt_stream(param_lbtt.c_str()); 
            to_lbtt(lbtt_stream); 
            lbtt_stream.close(); 
        } 
    }
                                

    //LTL2DSTAR::DA_ptr da = ltl_to_da(param_ltl_formula);

    //switch (da->getAutomatonType()) {
    //case Automaton::DETERMINISTIC_RABIN_AUTOMATON:
    //  dra_to_bdd(da->getDRA());
    //  da_type="Rabin";
    //  break;
    //case Automaton::DETERMINISTIC_STREETT_AUTOMATON:
    //  //      da_to_bdd(da->getDSA());
    //  da_type="Streett";
    //  break;
    //case Automaton::DETERMINISTIC_PARITY_AUTOMATON:
    //  //      da_to_bdd(da->getDPA());
    //  da_type="parity";
    //  break;
    //default:
    //  THROW_UNIMPLEMENTED();
    //}

    if (dynamic_reordering) {
    //  print_stats("PRE-REORDER ");
      
    //  // do reorder at the end
      std::cout << "\n\nDo one (last) iteration of dynamic reordering..." << std::flush;
      varmanager.triggerReordering();
      std::cout << "\nAfter reordering:\n";
    }
    if(param_stats) {
        print_stats();
    }
    //
    stopwatch_total.stop();
    std::cout << "Total-time: " << stopwatch_total.getElapsedTimeString() << "s\n";

    //if (param_dump_bdd != "") {
    //  dump_dra();
    //}

    return 0;
  }

  void print_stats(std::string prefix ="") {
    if(param_rabin) {
        std::cout << prefix << "Deterministic Rabin automaton:\n";
    } else {
        std::cout << prefix << "Deterministic Streett automaton:\n";
    }

    std::cout << prefix << "States    : " << calcReachable().countAssignments(getStateVars(false)) << "\n";
    std::cout << "\n";
    std::cout << prefix << "BDD encoding:\n";
    std::cout << prefix << "Variables : " << bdd_var_count() << "\n";
    std::cout << prefix << "Delta     : " << dsa.delta.size() << " nodes\n";
    std::cout << prefix << "Initial   : " << dsa.initial.size() << " nodes\n"; 
  }    

};



  
int main(int argc, char **argv) {
  AM2BDD am2bdd;

  try {
    return am2bdd.run(argc, argv);
  } catch (std::string& s) {
    std::cerr << "Exception!\n" << s << "\n";
    return -2;
  } catch (const char * s) {
    std::cerr << "Exception!\n" << s << "\n";
    return -2;
  }
}


