/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/



#include "ModelChecker.hpp"

using namespace std;
using namespace vereofy;

// static map
ModelChecker::checker_map ModelChecker::checkers;


OptionContainer ModelChecker::getOptions() const {
  OptionContainer options;
  return options;
}



// static functions

void ModelChecker::registerChecker(specification_type type, ModelChecker::ptr checker) {
  bool success=checkers.insert(checker_map::value_type(type, checker)).second;
  if (!success) {
    throw "There is already a checker registered for this type!";
  }
}

ModelChecker::ptr ModelChecker::getChecker(specification_type type) {
  checker_map::iterator it=checkers.find(type);
  if (it==checkers.end()) {
    return ModelChecker::ptr();
  }
  return (*it).second->clone();
}

OptionContainer ModelChecker::getAllOptions() {
  OptionContainer options;

  // General options
  
  Option opt_precompute_reachable("precompute-reachable", "Precompute the reachable states before model checking",
                                  "0");
  options.addOption(opt_precompute_reachable);

  for (checker_map::iterator it=checkers.begin();
       it!=checkers.end();
       ++it) {
    options.mergeOptions(it->second->getOptions());
  }

  return options;
}
