/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef MODELCHECKER_HPP
#define MODELCHECKER_HPP

#include <boost/shared_ptr.hpp>
#include <map>

#include "vereofy/common/OptionContainer.hpp"
#include "vereofy/system/automaton.hpp"
#include "vereofy/modelcheck/specifications/Specification.hpp"

#include "ModelCheckingResult.hpp"

namespace vereofy{

class ModelChecker {
public:
  typedef boost::shared_ptr<ModelChecker> ptr;

  virtual ~ModelChecker(){};
  virtual ModelCheckingResult::ptr analyze(Automaton::ptr, Specification::ptr, const OptionContainer&) = 0;
  virtual ModelChecker::ptr clone() = 0;
  virtual OptionContainer getOptions() const;

  static void registerChecker(specification_type, ModelChecker::ptr checker);
  static ModelChecker::ptr getChecker(specification_type);
  static OptionContainer getAllOptions();

private:
  typedef std::map<specification_type, ModelChecker::ptr> checker_map;
  static checker_map checkers;
}; 
 typedef ModelChecker::ptr ModelChecker_ptr;
}
	
#endif
