/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "ModelCheckingResult.hpp"
#include "vereofy/system/symb_automaton/symb_component_instance.hpp"

using namespace vereofy;

void ModelCheckingResult::print(std::ostream& out) {
  print_pass_fail(out);
  if (additional_info!="") {
    out << additional_info << std::endl;
  }
  
  if (countTraces()>0) {
    for (unsigned int i=0;i<countTraces();i++) {
      ModelCheckingResultTrace& trace=*getTrace(i);
      trace.print(out, *system);
    }
  }
}

void ModelCheckingResult::print_pass_fail(std::ostream& out) {
  out << "Result: " << (check_passed ? "PASSED" : "FAIL") << "\n" << std::endl ;
}

void ModelCheckingResult::print_xml(std::ostream& out) {
  xml_tag_simple(out, "modelchecking_result", xml_parameter("status", (check_passed ? "PASSED" : "FAIL")));
  out << "\n";

  xml_tag_open(out, "traces");
  out << "\n";
  for (unsigned int i=0;i<countTraces();i++) {
    ModelCheckingResultTrace& trace=*getTrace(i);
    trace.print_xml(out, *system);
    out << "\n";
  }
  xml_tag_close(out, "traces");
  out << "\n";
}


bool ModelCheckingResult::checkResult(boost::shared_ptr<SystemContext> system_context,
                                      boost::shared_ptr<SymbComponentInstance> system,
                                      const OptionContainer& options) const {
  THROW_UNIMPLEMENTED();
}
