/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef MODELCHECKING_RESULT_HPP
#define MODELCHECKING_RESULT_HPP

#include "vereofy/setup.hpp"
#include "vereofy/modelcheck/checkers/ModelCheckingResultTrace.hpp"

#include <boost/shared_ptr.hpp>
#include <iostream>
#include <string>

namespace vereofy{

// fwd
class SymbComponentInstance;
class SystemContext;
class OptionContainer;

class ModelCheckingResult {
public:
  typedef boost::shared_ptr<ModelCheckingResult> ptr;

  ModelCheckingResult(boost::shared_ptr<SymbComponentInstance> system_) :
    system(system_), check_passed(false), validSatSet(false) {}

  virtual ~ModelCheckingResult() {}

  virtual void print(std::ostream& out);
  virtual void print_pass_fail(std::ostream& out);
  virtual void print_xml(std::ostream& out);  
    
  virtual BDDFunction getSatSet(void) {return satSet;};


  virtual bool canCheckResult() const {return false;}
  virtual bool checkResult(boost::shared_ptr<SystemContext> system_context,
                           boost::shared_ptr<SymbComponentInstance> system,
                           const OptionContainer& options) const;

  void setCheckPassed(bool v) {check_passed=v;};
  void setSatSet(const BDDFunction& sat) {satSet=sat; validSatSet=true;};

  bool checkPassed() {return check_passed;}
  bool hasSatSet() {return validSatSet;}
  
  void setAdditionalInfo(const std::string& additional_info_) {
    additional_info=additional_info_;
  }

  unsigned int addTrace(ModelCheckingResultTrace::ptr trace) {
    unsigned int index=traces.size();
    traces.push_back(trace);
    return index;
  }

  unsigned int countTraces() const {return traces.size();}
  ModelCheckingResultTrace::ptr getTrace(unsigned int index) {return traces.at(index);}
  
  boost::shared_ptr<SymbComponentInstance> getSystem() {return system;}
private:
  boost::shared_ptr<SymbComponentInstance> system;

  bool check_passed; 
  BDDFunction satSet;
  bool validSatSet;
  std::string additional_info;

  typedef std::vector<ModelCheckingResultTrace::ptr> ModelCheckingResultTraces;
  ModelCheckingResultTraces traces;
  
}; 
 typedef ModelCheckingResult::ptr ModelCheckingResult_ptr;
}

#endif
