/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "ModelCheckingResultTrace.hpp"
#include "vereofy/system/symb_automaton/symb_trace_lasso.hpp"
#include "vereofy/system/symb_automaton/symb_component_instance.hpp"

using namespace vereofy;

ModelCheckingResultTrace::ModelCheckingResultTrace(trace_type_t trace_type_,
                                                   const std::string& text_,
                                                   SymbTraceLasso::ptr trace_,
                                                   int parent_) :
  trace_type(trace_type_),
  text(text_),
  trace(trace_),
  parent(parent_) {}
  
bool ModelCheckingResultTrace::hasParent() const {
  return parent>=0;
}

unsigned int ModelCheckingResultTrace::getParent() const {
  CHECK_SUCCESS(hasParent());
  return (unsigned int)parent;
}

const std::string& ModelCheckingResultTrace::getText() const {
  return text;
}

SymbTraceLasso::ptr ModelCheckingResultTrace::getTrace() {
  return trace;
}


const ModelCheckingResultTrace::trace_type_t ModelCheckingResultTrace::getTraceType() const {
  return trace_type;
}

std::string ModelCheckingResultTrace::getTraceTypeString() const {
  switch (getTraceType()) {
  case ModelCheckingResultTrace::TRACE_WITNESS:
    return "Witness";
  case ModelCheckingResultTrace::TRACE_COUNTEREXAMPLE:
    return "Counterexample";
  case ModelCheckingResultTrace::TRACE_EXAMPLE:
    return "Example";
  case ModelCheckingResultTrace::TRACE_UNKNOWN:
    return "Trace";
  case ModelCheckingResultTrace::TRACE_NONE:
  case ModelCheckingResultTrace::TRACE_NOT_EVALUATED:
    return "";
  }
  THROW_UNIMPLEMENTED();
}

void ModelCheckingResultTrace::print(std::ostream& out, SymbComponentInstance& system) {
  std::string trace_type_text=getTraceTypeString();

  if (getTraceType()==ModelCheckingResultTrace::TRACE_NONE ||
      getTraceType()==ModelCheckingResultTrace::TRACE_NOT_EVALUATED) {
    if (getText()!="") {
      out << "No trace for " << getText();
    }
    
    if (getTraceType()==ModelCheckingResultTrace::TRACE_NOT_EVALUATED) {
      out << " (not evaluated)";
    }
    out << "\n";
    return;
  }

  out << trace_type_text;
      
  if (getText()!="") {
    out << " for " << getText();
  }
  out << ":\n";
  
  if (getTrace()) {
    getTrace()->print(system, out);
  } else {
    out << "There is none in this context.";
  }
  
  out << "\n";
}


void ModelCheckingResultTrace::print_xml(std::ostream& out, SymbComponentInstance& system) {
  if (getTraceType()==ModelCheckingResultTrace::TRACE_NONE ||
      getTraceType()==ModelCheckingResultTrace::TRACE_NOT_EVALUATED ||
      !getTrace())  {
    return;
  }

  std::string trace_type=getTraceTypeString();

  xml_tag_open(out, "trace",
               xml_parameter("type", trace_type)+
               xml_parameter("for", getText()));

  getTrace()->print_xml(system, out);
  xml_tag_close(out, "trace");
}
