/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef MODELCHECKING_RESULT_TRACE_HPP
#define MODELCHECKING_RESULT_TRACE_HPP

#include <string>
#include <iostream>
#include <boost/shared_ptr.hpp>

#include "vereofy/system/symb_automaton/symb_trace_lasso.hpp"

namespace vereofy {

class ModelCheckingResultTrace {
public:
  typedef boost::shared_ptr<ModelCheckingResultTrace> ptr;

  enum trace_type_t {TRACE_WITNESS,
                     TRACE_COUNTEREXAMPLE,
                     TRACE_EXAMPLE,
                     TRACE_NONE,
                     TRACE_NOT_EVALUATED,
                     TRACE_UNKNOWN};

  ModelCheckingResultTrace(trace_type_t trace_type_,
                           const std::string& text_,
                           SymbTraceLasso::ptr trace_,
                           int parent_=-1);

  bool hasParent() const;
  unsigned int getParent() const;
  const std::string& getText() const;
  SymbTraceLasso::ptr getTrace();
  const trace_type_t getTraceType() const;
  std::string getTraceTypeString() const;

  void print(std::ostream& out, SymbComponentInstance& system);
  void print_xml(std::ostream& out, SymbComponentInstance& system);
private:
  trace_type_t trace_type;
  std::string text;
  SymbTraceLasso::ptr trace;
  int parent;
};


}


#endif
