/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "ModelCheckingResultTrace_for_java.hpp"
#include "ModelCheckingResultTrace.hpp"

#include "vereofy/common/weak_ptr_helper.hpp"
#include "vereofy/system/symb_automaton/symb_component_instance.hpp"

using namespace vereofy;

bool ModelCheckingResultTrace_for_java::hasParent() const {
  return access_weak_ptr(result_trace)->hasParent();
}

unsigned int ModelCheckingResultTrace_for_java::getParent() const {
  return access_weak_ptr(result_trace)->getParent();
}

std::string ModelCheckingResultTrace_for_java::getText() const {
  return access_weak_ptr(result_trace)->getText();
}

SymbTrace_for_java ModelCheckingResultTrace_for_java::getTracePrefix() {
  SymbTrace_for_java st_j;
  st_j.context=context;
  st_j.trace=access_weak_ptr(result_trace)->getTrace()->prefix;

  return st_j;
}

SymbTrace_for_java ModelCheckingResultTrace_for_java::getTraceCycle() {
  SymbTrace_for_java st_j;
  st_j.context=context;
  st_j.trace=access_weak_ptr(result_trace)->getTrace()->cycle;

  return st_j;
}

std::string ModelCheckingResultTrace_for_java::getTraceType() const {
  switch (access_weak_ptr(result_trace)->getTraceType()) {
  case ModelCheckingResultTrace::TRACE_WITNESS: return "TRACE_WITNESS";
  case ModelCheckingResultTrace::TRACE_COUNTEREXAMPLE: return "TRACE_COUNTEREXAMPLE";
  case ModelCheckingResultTrace::TRACE_EXAMPLE: return "TRACE_EXAMPLE";
  case ModelCheckingResultTrace::TRACE_NONE: return "TRACE_NONE";
  case ModelCheckingResultTrace::TRACE_NOT_EVALUATED: return "TRACE_NOT_EVALUATED";
  case ModelCheckingResultTrace::TRACE_UNKNOWN: return "TRACE_UNKNOWN";
  }

  return "TRACE_UNKOWN";
}

std::string ModelCheckingResultTrace_for_java::toString() const {
  std::stringstream out;
  SymbComponentInstance::ptr system_automaton=access_weak_ptr(context)->getSystemAutomaton();
  if (!system_automaton) {
    throw "Can not access system automaton";
  }

  access_weak_ptr(result_trace)->print(out, *system_automaton);
  return out.str();
}

