/***************************************************************************
 *   Copyright (C) 2008-2009 by Tobias Blechmann                           *
 *   blechman@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 @author Tobias Blechmann
 */
#ifndef BISIMULATION_HPP
#define BISIMULATION_HPP

#include <iostream>
#include <deque> 

#include <boost/shared_ptr.hpp>
#include <boost/program_options.hpp>

#include "vereofy/setup.hpp"
#include "vereofy/common/tracing.hpp"
#include "vereofy/common/checks.hpp"

#include "vereofy/system/symb_automaton/symb_automaton.hpp"

#include "vereofy/system/symb_automaton/symb_component_instance.hpp"

#include "vereofy/modelcheck/checkers/ModelCheckingResult.hpp"

namespace vereofy {
  class Bisimulation {
  public:
    Bisimulation(SymbComponentInstance::ptr, SymbComponentInstance::ptr);
    ~Bisimulation();
    ModelCheckingResult::ptr compute();
  private:
    SymbComponentInstance::ptr symbCompA;
    SymbComponentInstance::ptr symbCompB;    
    SymbAutomaton::ptr automaton;
    int totalNrOfSplitters;
    int nrOfSplitter;
    int nrOfPartitions;
    int firstNonStableBlock;
    BDDFunction allVars;
    BDDFunction delta;
    BDDFunction setOfReachable;
    BDDFunction csPorts;
    BDDFunction csConstraints;
    BDDFunction csOriginalX;
    BDDFunction csCopyX;
    BDDFunction csCopyCopyX;
    BDDFunction csNotPorts;
    BDDFunction csNotOriginalX;
    BDDFunction csSplitterVars;
    deque<BDDFunction> splitterContainer;    
    BDDFunctionContainer blockContainer;


    void symb_ce_patch(const ComponentInterface& interface_a, const ComponentInterface& interface_b);

    void calculateSplitter(BDDFunction, BDDFunction);
    void markBlockAsStable(int);
    void insertStableBlock(BDDFunction);
  };
}

#endif

  
/*  

class Bisimulation {
public:
  Bisimulation(Automata*);
  ~Bisimulation();
  void calculate();
private:
  int totalNrOfSplitters;

  void calculateSplitter(BDDFunction, BDDFunction);
  BDDFunctionContainer indexContainer;
  deque<BDDFunction> splitterContainer;
  BDDFunctionContainer blockContainer;
  void insertStableBlock(BDDFunction);
  void markBlockAsStable(int);

  Automata* automata;
  int nrOfSplitter;
  int nrOfPartitions;
  int firstNonStableBlock;
  BDDFunction allVars;
  BDDFunction delta;
  BDDFunction setOfReachable;
  BDDFunction csPorts;
  BDDFunction csConstraints;
  BDDFunction csOriginalX;
  BDDFunction csCopyX;
  BDDFunction csCopyCopyX;
  BDDFunction csNotPorts;
  BDDFunction csNotOriginalX;
  BDDFunction csSplitterVars;
  static const BDDFunction ONE;
  static const BDDFunction ZERO;
};

*/
