/***************************************************************************
 *   Copyright (C) 2011-2011 by Sascha Klueppelholz                        *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "ASLModelCheckingResult.hpp"
#include "vereofy/modelcheck/specifications/Specification.hpp"
#include "vereofy/modelcheck/specifications/SpecParser.hpp"
#include "vereofy/modelcheck/checkers/ModelChecker.hpp"

#include "vereofy/system/symb_automaton/symb_component_instance.hpp"

using namespace vereofy;

bool ASLModelCheckingResult::canCheckResult() const {
  return hasStrategy();
}

bool ASLModelCheckingResult::checkResult(boost::shared_ptr<SystemContext> system_context,
                                         boost::shared_ptr<SymbComponentInstance> system,
                                         const OptionContainer& options) const {
  Specification::ptr strategy_spec=SpecParser::parse_global("ASL<<"+getStrategyObjective()+">>",
                                                            system_context,
                                                            system->info->getComponentInterface());
  
  ModelChecker::ptr strategy_checker=ModelChecker::getChecker(strategy_spec->getType());
  if (!strategy_checker) throw "No ModelChecker for this specification defined!";
  
  // MODIFIES SYSTEM AUTOMATON!
  system->product(*(getStrategy()));
  TRACE_SYMB_COMPONENT_DOT("ASLStrategyAutomaton","product",*system,true);
  
  std::cout << "Checking strategy for objective: " << strategy_spec->getSpecification() << std::endl;
  
  ModelCheckingResult::ptr strategy_result=strategy_checker->analyze(system, strategy_spec, options);
  if (strategy_result->checkPassed()) {
    std::cout << "Strategy is winning!" << std::endl;
  }
  
  return strategy_result->checkPassed();
}
