/***************************************************************************
 *   Copyright (C) 2011-2011 by Sascha Klueppelholz                        *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef ASL_MODELCHECKING_RESULT_HPP
#define ASL_MODELCHECKING_RESULT_HPP

/**
 @author Sascha Klueppelholz
 */

#include "vereofy/modelcheck/checkers/ModelCheckingResult.hpp"
#include <string>

namespace vereofy{

class SymbAutomaton;

class ASLModelCheckingResult : public ModelCheckingResult {
public:
  typedef boost::shared_ptr<ASLModelCheckingResult> ptr;

  ASLModelCheckingResult(boost::shared_ptr<SymbComponentInstance> system_) :
  ModelCheckingResult(system_), haveStrategy(false) {}
  
  virtual void print(std::ostream& out) { ModelCheckingResult::print(out); if (hasStrategy()) out << "Have strategy for: " << strategyObjective << std::endl; }

  virtual bool canCheckResult() const;
  virtual bool checkResult(boost::shared_ptr<SystemContext> system_context,
                           boost::shared_ptr<SymbComponentInstance> system,
                           const OptionContainer& options) const;


  void setStrategy(const boost::shared_ptr<SymbAutomaton> s) {strategy = s; haveStrategy = true;}
  boost::shared_ptr<SymbAutomaton> getStrategy() const { CHECK_SUCCESS(haveStrategy); return strategy; }
  bool hasStrategy() const {return haveStrategy;}
  
  std::string getStrategyObjective() const {return strategyObjective;}
  void setStrategyObjective(const std::string& ob) { strategyObjective = ob; }
  
private:
  boost::shared_ptr<SymbAutomaton> strategy;
  bool haveStrategy;
  std::string strategyObjective;
}; 
 typedef ASLModelCheckingResult::ptr ASLModelCheckingResult_ptr;
}

#endif
