/***************************************************************************
 *   Copyright (C) 2008-2011 by Sascha Klueppelholz                        *
 *   blechman@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ASL_STRATEGY_AUTOMATON_H
#define ASL_STRATEGY_AUTOMATON_H

/**
 @author Sascha Klueppelholz
 */

#include "vereofy/system/symb_automaton/symb_automaton.hpp"

namespace vereofy {
  
  class ASLStrategyAutomaton : public SymbAutomaton {
  public:
    typedef boost::shared_ptr<ASLStrategyAutomaton> ptr;

    ASLStrategyAutomaton(SymbAutomaton::ptr _system, BDDFunction strategy) : 
      SymbAutomaton(*_system){

      delta=strategy;
      
      TRACE_SYMB_AUTOMATON_DOT("ASLStrategyAutomaton","strategy",*this);

    }
    
    ASLStrategyAutomaton(SymbAutomaton::ptr _system, SymbAutomatonLanguageAcceptor::ptr Z, BDDFunction strategy) : 
      SymbAutomaton(*_system) {
      initialStates         &= Z->getInitialStates();
      csOriginalX           &= Z->getOriginalStateVars();
      csCopyY               = csOriginalX.moveDown();
      csCopyCopyZ           = csCopyY.moveDown();
      haltSystem            &= Z->getHaltSystem();
      
      // @TODO: for additional ports (or lables), e.g. for A_stopp port, we need to adopt the ports of Z as well
      
      delta=strategy;
      
      TRACE_SYMB_AUTOMATON_DOT("ASLStrategyAutomaton","strategy",*this);
      
    }
    
  };
}

#endif
