/***************************************************************************
 *   Copyright (C) 2008-2009 by Sascha Klueppelholz                        *
 *   klueppel@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "ASL_ModelChecker.hpp"

#include "vereofy/system/symb_automaton/symb_automaton.hpp"
#include "vereofy/modelcheck/checkers/branching_time_check/mcreo.hpp"

/**
 @author Sascha Klueppelholz
 */

using namespace std;
using namespace vereofy;

ASL_ModelChecker::ASL_ModelChecker(){
}

ModelCheckingResult::ptr ASL_ModelChecker::analyze(Automaton::ptr automaton, Specification::ptr spec,
                                                   const OptionContainer& options) {
  SymbAutomaton::ptr ca=boost::static_pointer_cast<SymbAutomaton>(automaton);
  if (!ca) throw "Not a symbolic automaton!";

  if (spec->getType()!=ASL_SPEC) {
    throw "Specification is not ASL!";
  }

  mcreo checker(options);
  checker.findOrAddRequirement(spec->getSpecification());
  cout << "checking: " << spec->getSpecification() << endl;

  return checker.analyse(ca);
}

OptionContainer ASL_ModelChecker::getOptions() const {
  OptionContainer options;

  //  Option opt_fair("asl-fairness",
  //                  "Use Fairness in ASL Model Checking",
  //                  "false");
  //  options.addOption(opt_fair);

  Option opt_witness_level("btsl-witness-level",
                           "Set the printing level for witness output",
                           "0");
  options.addOption(opt_witness_level);
    
  Option opt_keep_sat("keep-sat",
                      "Keep the Satisfaction sets",
                      "0");
  options.addOption(opt_keep_sat);
  
  Option opt_compute_asl_strategy("compute-asl-strategy",
                      "Compute ASL strategy",
                      "0");
  options.addOption(opt_compute_asl_strategy);
  
  return options;
}
