/***************************************************************************
 *   Copyright (C) 2008-2009 by Sascha Klueppelholz                        *
 *   klueppel@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef SYMB_AUTOMATON_LANGUAGE_ACCEPTOR_HPP
#define SYMB_AUTOMATON_LANGUAGE_ACCEPTOR_HPP

#include <string>
#include <boost/shared_ptr.hpp>
#include "vereofy/system/symb_automaton/symb_automaton.hpp"

/**
 @author Sascha Klueppelholz
 */

namespace vereofy{
  
  class SymbAutomatonLanguageAcceptor : public SymbAutomaton{
    friend class hybridFiniteAutomaton;
    
  public:
    typedef boost::shared_ptr<SymbAutomatonLanguageAcceptor> ptr;
    
    SymbAutomatonLanguageAcceptor(const SymbAutomaton::ptr);
    SymbAutomatonLanguageAcceptor(const string&);
    ~SymbAutomatonLanguageAcceptor(){};
    
    void parse(const SymbAutomaton::ptr);
    void makeDeterministic();
    
    inline BDDFunction getAccept(){return acceptingStates;}
    inline BDDFunction getReachable(){return reachableStates;}
    bool hasStoppedTransitions(void){return containsStopp;}
    
    virtual State_Renderer::ptr getDefaultStateRenderer() const;
    virtual void toDot(ostream& out) const;
    
  private:
    
    struct SALA_State_Renderer : public SymbAutomaton::State_Renderer {
      SALA_State_Renderer(const SymbAutomatonLanguageAcceptor& automaton_) : 
      SymbAutomaton::State_Renderer(automaton_) {}
      
      virtual void render(ostream& out, const BDDFunction& assignment, bool use_x) const;
      virtual bool render_dot_style(ostream& out, const BDDFunction& assignment, bool is_initial, bool is_terminal) const;
    };
    
    void parse();
  	
    inline void fill(SymbAutomatonLanguageAcceptor::ptr tmp, BDDFunction diff);
    inline void eat(const SymbAutomatonLanguageAcceptor::ptr tmp);
    
    string stream_expr;
    
    unsigned long varlevel;
    BDDFunction acceptingStates;
    BDDFunction reachableStates;
    SymbAutomaton::ptr ca;
    bool containsStopp;
  };
  typedef SymbAutomatonLanguageAcceptor::ptr SymbAutomatonLanguageAcceptor_ptr;
}

#endif
