/***************************************************************************
 *   Copyright (C) 2008-2009 by Sascha Klueppelholz                        *
 *   klueppel@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef MCREO_HPP
#define MCREO_HPP

#include <string>
#include <boost/smart_ptr.hpp>

#include "mcreo_setup.hpp"
#include "vereofy/system/symb_automaton/symb_automaton.hpp"
#include "vereofy/modelcheck/checkers/ModelChecker.hpp"
#include "vereofy/modelcheck/specifications/asl/ASL_Formula.hpp"
#include "vereofy/common/streamlistener.hpp"
#include "ASLModelCheckingResult.hpp"

/**
 @author Sascha Klueppelholz
 */

namespace vereofy{
  class mcreo{
  public:
    mcreo(const OptionContainer&, const streamListener::ptr=standardListener);
    ~mcreo();
    
    bool findOrAddRequirement(const std::string&);
    ASLModelCheckingResult::ptr analyse(const SymbAutomaton::ptr);
    
    void getStatus(void) const;
    void clearRequirements(void);
    
  private:
    BDDFunction sat(formula::ptr);
    inline pair <BDDFunction, BDDFunction> preImage(const BDDFunction&, const formula::ptr);
    inline BDDFunction BTSLProduct(const BDDFunction&, const SymbAutomatonLanguageAcceptor_ptr);
    inline BDDFunction ASLProduct(const BDDFunction&, const SymbAutomatonLanguageAcceptor_ptr, const BDDFunction&);
    inline void clear(void);
    inline void clearComputation(void);
    inline SymbAutomatonLanguageAcceptor::ptr doWhatYouHaveToDo(const formula::ptr);
    
    map<std::string, formula::ptr> formulas;
    
    unsigned int preSteps;
    
    unsigned int opt_witness_level;
    bool opt_asl_fairness;
    bool opt_precompute_reachable;
    bool opt_keep_sat;
    bool opt_compute_strategy;
    
	  progressListener::wptr listener;
	  
    double progress;
    
    SymbAutomaton::ptr ca;
    BDDFunction initialStates;
    BDDFunction copies, ports,label_vars;
    BDDFunction reachable;
    BDDFunction controllable, allTransitions, unpreventable;
    BDDFunction strategy;
    
    BDDFunction gfp(const BDDFunction&, const formula::ptr, const bool, const bool=false);
    BDDFunction lfp(const BDDFunction&, const BDDFunction&, const formula::ptr, const bool, const bool=false);
  };
}

#endif
