/***************************************************************************
 *   Copyright (C) 2008-2009 by Sascha Klueppelholz                        *
 *   klueppel@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mcreo_functions.hpp"

/**
 @author Sascha Klueppelholz
 */

using namespace std;
using namespace vereofy;

/**
 * \brief A helper method which finds the position of a boolean operator within an expression
 *
 * This method receives an expression as first parameter and a set of operators given in another string variable.
 * It returns the position where an operator was found or 0 otherwise.
 *
 * \sa mcreo.hpp ea.hpp formula.hpp
*/
size_t vereofy::findOperatorPosition(const string& expr, const string& patterns){
  vector<unsigned int> allOPPos;
  int level1=0;
  int level2=0;
 
  // walk over the expression and count level for each type of brackets
  for (unsigned int i=0;i<expr.length();++i){
    if ((expr[i]=='(')) ++level1;
    else if (expr[i]==')') --level1;
    else if (expr[i]=='[') ++level2;
    else if (expr[i]==']') --level2;

    // the level is never lower than zero
    if (level1<0 || level2<0){
      throw "error parsing expression: " + expr;
    }
    
    // if all levels are zero and an operator is found allOPPos takes the position
    if (expr.substr(i,1).find_first_of(patterns)!=string::npos && !level1 && !level2) allOPPos.push_back(i);
  }

  // return one of the positions found or 0 otherwise
  return (allOPPos.size())?(allOPPos[allOPPos.size()/2]):(0);
}


/**
 * findMatchingClosing returns the position of the matching closing bracket to the first bracket
 * found in s
 * For "xyz[01[345]67]abc[]", it would return the position (with open=='[' and close==']') 
 * of the ] after 7.
 * If no such bracket exits, returns string::npos
 */
size_t vereofy::findMatchingClosing(const string& s, unsigned char open, unsigned char close) {
  size_t pos=0;
  unsigned int open_count=0;
  for (string::const_iterator it=s.begin();
       it!=s.end();
       ++it) {
    if (*it==open) {
      open_count++;
    } else if (*it==close) {
      if (open_count==0) {
        // found closing bracket without seeing an opening first...
        return string::npos;
      }
      --open_count;
      if (open_count==0) {
        // ok, found the closing bracket matching the first open bracket
        return pos;
      }
    } else {
      // not open or close -> ignore
    }

    pos++; // increment position
  }
  return string::npos;
}
