/***************************************************************************
 *   Copyright (C) 2008-2009 by Ilham Kurnia, Sascha Klüppelholz           *  
 *   s4036591@mail.inf.tu-dresden.de  , klueppel@tcs.inf.tu-dresden.de     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef WITNESS_HPP
#define WITNESS_HPP

#include "mcreo_setup.hpp"
#include "vereofy/modelcheck/specifications/asl/ASL_Formula.hpp"
#include "vereofy/modelcheck/checkers/ModelCheckingResult.hpp"
#include "vereofy/modelcheck/checkers/ModelCheckingResultTrace.hpp"
#include "vereofy/system/symb_automaton/symb_automaton.hpp"
#include "vereofy/system/symb_automaton/symb_trace.hpp"
#include "vereofy/system/symb_automaton/symb_bfs.hpp"

#include <boost/smart_ptr.hpp>
#include <stack>

/**
 @author Ilham Kurnia
 @author Sascha Klueppelholz
 */

namespace vereofy{

class formula;
	
class witness : public SymbAutomaton{
friend class formula;
friend class mcreo;

public:
  typedef boost::shared_ptr<witness> ptr;

  witness(boost::shared_ptr<formula>, SymbAutomaton::ptr);
  ~witness(){};
  
  const f_info& getFixpointType(void) const{ return whatAmI.myFixpointType; }
  
private:
  void generateWitness(const BDDFunction&, const unsigned int, const int, ModelCheckingResult&) const;
  void append(const BDDFunction&);
  void setFixpointType(const f_info&);
  
  // @TODO: some of those guys could prop. also be shared....
  inline BDDFunction preImage(const BDDFunction& p) const {return (p.moveDown()&getDelta()).exists(getCopyStateVars() & getTransitionVars());};
  inline BDDFunction postImage(const BDDFunction& p) const {return (p&getDelta()).exists(getOriginalStateVars() & getTransitionVars()).moveUp();};
  
  // @TODO: compare implementation in mcreo with this one. find a good place to share code!
  BDDFunction forwardSet(const BDDFunction&, const BDDFunction&) const;
  BDDFunction sccFind(const BDDFunction&, const BDDFunction&, const BDDFunction&) const;
  	
  std::pair<SymbTrace::ptr,ModelCheckingResultTrace::trace_type_t> computePath(const BDDFunction&) const;

  boost::weak_ptr<SymbAutomaton> ca;
  boost::weak_ptr<formula> representedFormula;
  std::stack<BDDFunction> result;

  BDDFunction terminalStates;
  BDDFunction relevantStateVars;
  BDDFunction terminalLoop;
    
  witnessInfo whatAmI;
};
 typedef witness::ptr witness_ptr;
}

#endif
