/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "BDD_NBA.hpp"
#include "common/varset.h"
#include "vereofy/system/symb_automaton/varmanager.hpp"

namespace vereofy { namespace ltl_check {

BDDFunction BDD_NBA::newStateVariable() {
  BDDFunction proj_var=VarManager::getInstance()->createStateVars(1).X->at(0);

  state_variables.push(proj_var);
  state_vars&=proj_var;

  return proj_var;
}


unsigned int BDD_NBA::nba_i_newState() {
  unsigned int cur=states;
  ++states;

  if (var_count>0 &&
      states <= (1u << var_count) ) {
    // ok
  } else {
    BDDFunction proj_var=newStateVariable();

    BDDFunction not_names=!proj_var;
    BDDFunction not_names_=!proj_var.moveDown();
    BDDFunction combined=not_names&not_names_;

    if (!(delta.isConstant())) {
      delta&=combined;
    }
    if (!(initial.isConstant())) {
      initial&=not_names;
    }
    if (!(accepting.isConstant())) {
      accepting&=not_names;
    }
    if (!(all_states.isConstant())) {
      all_states&=not_names;
    }
    
    ++var_count;
  }

  all_states|=state2bdd(cur, false);
  if (debugLevel>=4) {
    std::cout << "all_states:\n";
    all_states.printPaths();
  }
 
  return cur;
}

void BDD_NBA::nba_i_addEdge(unsigned int from,
			    const APMonom& m,
			    unsigned int to) {
  if (m.isFalse()) {return;}

  BDDFunction m_bdd=monom2bdd(m);
  delta|=state2bdd(from, false) & state2bdd(to, true) & m_bdd;
}

APSet_cp BDD_NBA::nba_i_getAPSet() {
  return apset;
}

void BDD_NBA::nba_i_setFinal(unsigned int state, bool final) {
  if (debugLevel>=4) {
    std::cout << "setFinal(" << state << " = " << final << ") before "<< std::endl;
    accepting.printPaths();
    std::cout << "\n";
  }

  if (final) {
    accepting|=state2bdd(state, false);
  } else {
    accepting&=!state2bdd(state, false);
  }

  if (debugLevel>=4) {
    std::cout << "setFinal(" << state << " = " << final << ") after "<< std::endl;
    accepting.printPaths();
  }
}

void BDD_NBA::nba_i_setStartState(unsigned int state) {
  initial=state2bdd(state, false);
}

void BDD_NBA::nba_i_finished() {

}


}}
