/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_NBA_PRODUCT_HPP
#define SYMB_NBA_PRODUCT_HPP

#include "BDD_NBA.hpp"
#include "vereofy/system/symb_automaton/symb_automaton.hpp"
#include "vereofy/system/symb_automaton/symb_component_instance.hpp"

#include <boost/shared_ptr.hpp>

namespace vereofy {namespace ltl_check {
    
class SymbNBAProductAutomaton : public SymbAutomaton {
public:
  typedef boost::shared_ptr<SymbNBAProductAutomaton> ptr;

  SymbNBAProductAutomaton(SymbComponentInstance::ptr system_automaton,
                          BDD_NBA::ptr nba);
  
  BDDFunction getAcceptingStates() const;

  struct Product_State_Renderer : public SymbAutomaton::State_Renderer {
    Product_State_Renderer(const SymbComponentInstance& sci,
                           const BDD_NBA& nba_,
                           const BDDFunction& highlight_states_=BDDFunction::ZERO()) : 
      SymbAutomaton::State_Renderer(sci), 
      system_renderer(sci.getDefaultStateRenderer()),
      nba(nba_),
      highlight_states(highlight_states_) {}
    
    virtual void render(ostream& out, const BDDFunction& assignment, bool use_x) const;
    virtual bool render_dot_style(ostream& out, const BDDFunction& assignment, bool is_initial, bool is_terminal) const;

    SymbAutomaton::State_Renderer::ptr system_renderer;
    const BDD_NBA& nba;
    BDDFunction highlight_states;
  };

  struct Trace_IO_Renderer : public SymbAutomaton::IO_Renderer {
    Trace_IO_Renderer(const SymbComponentInstance& sci,
                      const BDD_NBA& nba_,
                      const BDDFunction& transition_on_prefix_=BDDFunction::ZERO(), 
                      const BDDFunction& transition_on_cycle_=BDDFunction::ZERO()) : 
      SymbAutomaton::IO_Renderer(sci), 
      system_renderer(sci.getDefaultIORenderer()),
      nba(nba_),
      transition_on_prefix(transition_on_prefix_),
      transition_on_cycle(transition_on_cycle_) {}
    
    virtual void render(ostream& out, const BDDFunction& assignment, const BDDFunction& io_vars) const;
    virtual bool render_dot_style(ostream& out, 
                                  const BDDFunction& assignment, 
                                  const BDDFunction& io_vars,
                                  const BDDFunction& state_pair) const;
  private:
    SymbAutomaton::IO_Renderer::ptr system_renderer;
    const BDD_NBA& nba;
    BDDFunction transition_on_prefix;
    BDDFunction transition_on_cycle;
  };

  virtual State_Renderer::ptr getDefaultStateRenderer() const;
  virtual IO_Renderer::ptr getDefaultIORenderer() const;
  
  void toDotWithTrace(ostream& out, 
                      const BDDFunction& states_on_trace,
                      const BDDFunction& prefix_transitions,
                      const BDDFunction& cycle_transitions) const;
private:
  SymbComponentInstance::ptr system_automaton;
  BDD_NBA::ptr nba;
  
  BDDFunction acceptingStates;
};
    
}
}

#endif
