/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/



#include <boost/algorithm/string.hpp>
#include "SpecParser.hpp"

using namespace vereofy;
using namespace std;

// static map
SpecParser::parser_map SpecParser::parsers;

void SpecParser::registerParser(const string& type, SpecParser::ptr parser) {
  bool success=parsers.insert(parser_map::value_type(type, parser)).second;
  if (!success) {
    throw "There is already a parser registered for this type!";
  }
}

SpecParser::ptr SpecParser::getParser(const string& type) {
  parser_map::iterator it=parsers.find(type);
  if (it==parsers.end()) {
    return SpecParser::ptr();
  }
  return (*it).second;
}

Specification::ptr SpecParser::parse_global(const string& formula_,
                                            SystemContext::ptr system_context, 
                                            ComponentInterface::ptr system_interface) {
  // remove leading and trailing whitespace
  string formula=boost::trim_copy(formula_);

  size_t i=formula.find("<<");
  if (i==string::npos) throw "Invalid formula";

  string type=formula.substr(0, i); 
  SpecParser::ptr parser=getParser(type);
  if (!parser) throw "No parser for formula type";

  if (!boost::ends_with(formula, ">>")) {
    throw "Invalid formula, missing closing >>";
  }

  return parser->parse(formula.substr(i+2,formula.length()-i-4),
                       system_context,
                       system_interface);
}
