/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SPECPARSER_HPP
#define SPECPARSER_HPP

#include <map>
#include <string>

#include <boost/shared_ptr.hpp>
#include "Specification.hpp"

#include "vereofy/system/system_context.hpp"
#include "vereofy/system/component_interface.hpp"

namespace vereofy {

class SpecParser {
public:
  typedef boost::shared_ptr<SpecParser> ptr;

  virtual ~SpecParser() {}
  
  virtual Specification::ptr parse(const std::string& formula,
                                   SystemContext::ptr system_context, 
                                   ComponentInterface::ptr system_interface) const = 0;
  
  static void registerParser(const std::string& type, SpecParser::ptr parser);
  static SpecParser::ptr getParser(const std::string& type);

  static Specification::ptr parse_global(const std::string& formula,
                                         SystemContext::ptr system_context, 
                                         ComponentInterface::ptr system_interface);

private:
  typedef std::map<std::string, SpecParser::ptr> parser_map;
  static parser_map parsers;
};

}

#endif
