/***************************************************************************
 *   Copyright (C) 2008-2009 by Sascha Klueppelholz                        *
 *   klueppel@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

// #define BOOST_SPIRIT_DEBUG
#include "vereofy/reo_parser/spirit_init.hpp"

#include "ASLInfixParser.hpp"
#include "branching_time_logics_parser.hpp"

#include "vereofy/common/errors.hpp"
#include "vereofy/modelcheck/specifications/asl/ASL_Formula.hpp"

using namespace std;
using namespace vereofy;

formula::ptr ASLInfixParser::parse(std::string const& formula,
                                   SystemContext::ptr system_context, 
                                   ComponentInterface::ptr system_interface) {

  symbol_table_stack_t symbol_table_stack;
  symbol_table_stack.push(system_context->getGlobalSymbols());

  branching_time_logics_parser infix_p(symbol_table_stack, system_interface);
  
  const char* formula_c=formula.c_str();
  vparser::iterator_t first(formula_c, formula_c + formula.length());
  vparser::iterator_t last;
  tree_parse_info<vparser::iterator_t, REO_PARSER_NODE_FACTORY > info = ast_parse<REO_PARSER_NODE_FACTORY>(first, last, infix_p >> end_p, space_p);
    
  if (!info.full) {
    throw "Parse error, ASL formula";
  }

#if defined(BOOST_SPIRIT_DUMP_PARSETREE_AS_XML)
  // dump parse tree as XML
  TRACE("ASLInfixParser", "ParseTreeXML", tree_to_xml(cout, info.trees, "", infix_p.getRuleNames()););
#endif
           
  formula::ptr fptr = formula::toASLGraphStructure(info.trees[0], symbol_table_stack);
  return fptr;
}



