/***************************************************************************
 *   Copyright (C) 2008-2009 by Sascha Klueppelholz                        *
 *   klueppel@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ASL_FORMULA_H
#define ASL_FORMULA_H

#include "vereofy/modelcheck/checkers/ModelCheckingResult.hpp"
#include "vereofy/modelcheck/checkers/branching_time_check/SymbAutomatonLanguageAcceptor.hpp"
#include "vereofy/modelcheck/checkers/branching_time_check/mcreo_setup.hpp"
#include "vereofy/modelcheck/checkers/branching_time_check/witness.hpp"
#include "vereofy/common/return_value.hpp"
#include "vereofy/reo_parser/reo_parser_types.hpp"

#include <boost/smart_ptr.hpp>
#include <string>
#include <map>

/**
@author Sascha Klueppelholz
*/

namespace vereofy{

class witness;
  
typedef enum{MIN_UNDEFINED, MIN_NONE, MIN_BRZOZOWSKI, MIN_HOPCROFT} min_type;
  	
class formula{
friend class mcreo;
friend class witness;
  
public:
  typedef boost::shared_ptr<formula> ptr;
  typedef boost::weak_ptr<formula> w_ptr;

  static formula::ptr findOrAdd(const std::string&);
  static formula::ptr toASLGraphStructure(const vparser::pt_node&, const symbol_table_stack_t& symbol_table_stack);
  ~formula(){};

  BTSLFT getOpType() const {return formulaType;}
  const std::string& getSpecification() const {return specAsString;}
  const std::string getParsableSpecification() const;

  void clearComputation(void);
  const f_info& getFixpointType(void) const;
  void setFixpointType(const f_info&);
  void generateWitness(BDDFunction, unsigned int, int, ModelCheckingResult&) const;
  
  static void setDFAMinimizationMethod(const string&);
  
  string newSpec;
  
private:
  formula(const std::string&);
  formula(const string&, const BTSLFT, const vparser::pt_node&, const formula::ptr, const formula::ptr, const bool, const bool, const set<string>);

  std::string parse(const std::string&);
  std::string getRegExpression(void) const {return regExprAsString;}

  SymbAutomatonLanguageAcceptor_ptr buildEA(const SymbAutomaton::ptr, const bool=false);
  SymbAutomatonLanguageAcceptor_ptr composeEA(const SymbAutomaton::ptr, const bool=true);
  
  void createNewWitnessFor(formula::ptr, SymbAutomaton::ptr);
  void appendToWitness(const BDDFunction&);
  void createSecondWitnessFor(formula::ptr, SymbAutomaton::ptr, const f_info&, const BDDFunction&);
  
  BTSLFT formulaType;

  formula::ptr sub1;
  formula::ptr sub2;

  SymbAutomatonLanguageAcceptor_ptr regExpr;
  boost::shared_ptr<vparser::pt_node> regExprRootNode;
  
  boost::shared_ptr<witness> wit_least;
  boost::shared_ptr<witness> wit_greatest;

  std::string specAsString;
  std::string regExprAsString;

  ReturnValue<BDDFunction> satSet;

  bool isASL;
  bool isBTSL;
  set<std::string> controllablePorts;
  
  bool done;
  
  typedef std::map<std::string, formula::w_ptr> formula_map;
  static formula_map* allFormulas;
  static min_type minimazationMethod;
};
  typedef formula::ptr formula_ptr;
  typedef formula::w_ptr weak_formula_ptr;
  typedef std::map<std::string, formula::w_ptr> formula_map;
}

#endif
