/***************************************************************************
 *   Copyright (C) 2008-2009 by Sascha Klueppelholz                        *
 *   klueppel@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef BRANCHING_TIME_LOGICS_PARSER_HPP
#define BRANCHING_TIME_LOGICS_PARSER_HPP

#include "vereofy/reo_parser/spirit_init.hpp"
#include "vereofy/reo_parser/guard_parser.hpp"

#include <boost/bind.hpp>

using namespace std;
using namespace vereofy;
using namespace vereofy::spirit;

struct branching_time_logics_parser : public grammar<branching_time_logics_parser>
{
  enum bt_ids {
    ID_CONST=1,
    ID_ATOM,
    ID_NEG,
    ID_PATH_QUANTIFIERS,
    ID_UNARY_CTL_OP,
    ID_BINARY_CTL_OP,

    // regular expression parser enum elements
    ID_IO_GUARD_ATOM,
    ID_IO_CLOSE_OP,
    ID_IO_GUARD_STARED,
    ID_IO_SEQUENCE,
    ID_IO_TERM_AND,
    ID_IO_REGULAR_IO_EXPRESSION,
    
    ID_EXISTS_PREFIX,
    ID_FORALL_PREFIX,
    ID_PREFIX_QUANTIFIER,
    ID_IO_PORT,
    ID_IO_PORT_LIST,
    ID_ASL_PORT_LIST,
    ID_ASL_STATE_FORMULA_AND,
    ID_ASL_STATE_TERM,
    ID_ASL_STATE_FORMULA
};

  std::map<parser_id,string> getRuleNames() {
    std::map<parser_id,string> result;
    result[ID_CONST]                     = "ctl_const";
    result[ID_ATOM]                      = "ctl_atom";
    result[ID_NEG]                       = "neg_op";
    result[ID_PATH_QUANTIFIERS]          = "path_quantifier";
    result[ID_UNARY_CTL_OP]              = "ctl_unary_operators";
    result[ID_BINARY_CTL_OP]             = "ctl_binary_operators";
    
    // regular expression rule names
    result[ID_IO_GUARD_ATOM]             = "io_guard_atom";
    result[ID_IO_CLOSE_OP]               = "io_close_op";
    result[ID_IO_GUARD_STARED]           = "io_guard_stared";
    result[ID_IO_SEQUENCE]               = "io_sequence";
    result[ID_IO_TERM_AND]               = "io_term_and";
    result[ID_IO_REGULAR_IO_EXPRESSION]  = "reg_io_expr";
    
    result[ID_EXISTS_PREFIX]             = "existential_prefix_quantifier";
    result[ID_FORALL_PREFIX]             = "universal_prefix_quantifier";
    result[ID_PREFIX_QUANTIFIER]         = "btsl_prefix_quantifier";
    result[ID_IO_PORT]                   = "port";
    result[ID_IO_PORT_LIST]              = "port_list";
    result[ID_ASL_PORT_LIST]             = "asl_port_list";
    result[ID_ASL_STATE_FORMULA_AND]     = "asl_state_formula_and";
    result[ID_ASL_STATE_TERM]            = "asl_state_term";
    result[ID_ASL_STATE_FORMULA]         = "asl_state_formula";
    return result;
  }


  branching_time_logics_parser(symbol_table_stack_t& symbol_table_stack_,
                               vereofy::ComponentInterface::ptr system_interface_) :
    symbol_table_stack(symbol_table_stack_),
    system_interface(system_interface_) {}

  symbol_table_stack_t& symbol_table_stack;
  symbols<> reserved_keywords;
  vereofy::ComponentInterface::ptr system_interface;

  template <typename ScannerT>
  struct definition 
  {
    definition(branching_time_logics_parser const& self) :
      guard_grammar(self.symbol_table_stack, self.reserved_keywords, self.system_interface)
    { 
      // CTL specific fragment:
      ctl_const = root_node_d[str_p("true")] | root_node_d[str_p("false")];
            
      ctl_atom = (no_node_d[ch_p('"')] >> token_node_d[ +(~ch_p('"')) ] >> no_node_d[ch_p('"')]) 
               | (no_node_d[ch_p('\'')] >> token_node_d[ +(~ch_p('\'')) ] >> no_node_d[ch_p('\'')]);
      
      neg_op = str_p("!");
      
      path_quantifier = str_p("E") | str_p("A");
    
      ctl_unary_operators = str_p("X") | str_p("F") | str_p("G");
      
      ctl_binary_operators = str_p("U") | str_p("R");
      
      // BTSL specific fragment:
      
      { ///////// @TODO: here comes the grammar for regular expressions over io-guards: move to separate parser
        io_guard_atom = !str_p("!") >> (
                                       (no_node_d[ch_p('"')]  >> token_node_d[ +(~ch_p('"')) ] >> no_node_d[ch_p('"')]      ) 
                                     | (no_node_d[ch_p('\'')] >> token_node_d[ +(~ch_p('\'')) ] >> no_node_d[ch_p('\'')]     )
                                     | (no_node_d[ str_p("(") ] >> reg_io_expr >> no_node_d[ str_p(")")] )
                                      );
        
        io_close_op = (str_p("*") | str_p("+"));
        
        io_guard_stared = io_guard_atom >> !io_close_op;
        
        io_sequence = io_guard_stared >> *(no_node_d[ch_p(';')] >> io_guard_stared);
        
        io_term_and = io_sequence >> *(no_node_d[ch_p('&')] >> io_sequence);
        
        reg_io_expr = io_term_and >> *(no_node_d[ch_p('|')] >> io_term_and);
      }
                                                    
      existential_prefix_quantifier = no_node_d[str_p("<")] >> reg_io_expr >> no_node_d[str_p(">")];
      universal_prefix_quantifier = no_node_d[str_p("[")] >> reg_io_expr >> no_node_d[str_p("]")];
      btsl_prefix_quantifier = existential_prefix_quantifier | universal_prefix_quantifier;
      
      // ASL:
      ENTRY_GRAMMAR(port_name_p, vereofy::vparser::guard_parser, guard_grammar, vereofy::vparser::guard_parser::port_name);
      port_list = port_name_p >> *( no_node_d[ch_p(',')] >> port_name_p);
      asl_port_list = no_node_d[ch_p('{')] >> !port_list >> no_node_d[ch_p('}')];

      asl_state_formula_and = asl_state_term >> *(root_node_d[ str_p("&") ] >> asl_state_term);
      
      asl_state_term = path_quantifier >> !asl_port_list >> 
                       (  
                          ctl_unary_operators >> no_node_d[ch_p('[')] >> asl_state_formula >> no_node_d[ch_p(']')]
                        | ctl_binary_operators >> no_node_d[ch_p('[')] >> asl_state_formula >> no_node_d[ch_p(',')] >> asl_state_formula >> no_node_d[ch_p(']')]
                        | btsl_prefix_quantifier >> asl_state_formula 
                       )
                     | neg_op >> asl_state_formula
                     | no_node_d[ str_p("(") ] >> asl_state_formula >> no_node_d[ str_p(")")]
                     | ctl_const | ctl_atom;

      asl_state_formula = asl_state_formula_and >> *(( root_node_d[ str_p("|") ] | root_node_d[ str_p("^") ]) >> asl_state_formula_and) | asl_state_term;
      
      BOOST_SPIRIT_DEBUG_RULE(asl_state_formula);
    }

    rule<ScannerT, parser_tag<ID_CONST> >                    ctl_const;
    rule<ScannerT, parser_tag<ID_ATOM> >                     ctl_atom;
    rule<ScannerT, parser_tag<ID_NEG> >                      neg_op;
    rule<ScannerT, parser_tag<ID_PATH_QUANTIFIERS> >         path_quantifier;                                                                             
    rule<ScannerT, parser_tag<ID_UNARY_CTL_OP> >             ctl_unary_operators;
    rule<ScannerT, parser_tag<ID_BINARY_CTL_OP> >            ctl_binary_operators;
    
    // regular expressions rules
    rule<ScannerT, parser_tag<ID_IO_GUARD_ATOM> >            io_guard_atom;
    rule<ScannerT, parser_tag<ID_IO_CLOSE_OP> >              io_close_op;
    rule<ScannerT, parser_tag<ID_IO_GUARD_STARED> >          io_guard_stared;
    rule<ScannerT, parser_tag<ID_IO_SEQUENCE> >              io_sequence;
    rule<ScannerT, parser_tag<ID_IO_TERM_AND> >              io_term_and; 
    rule<ScannerT, parser_tag<ID_IO_REGULAR_IO_EXPRESSION> > reg_io_expr;
    
    rule<ScannerT, parser_tag<ID_EXISTS_PREFIX> >            existential_prefix_quantifier;
    rule<ScannerT, parser_tag<ID_FORALL_PREFIX> >            universal_prefix_quantifier;
    rule<ScannerT, parser_tag<ID_PREFIX_QUANTIFIER> >        btsl_prefix_quantifier;
    rule<ScannerT, parser_tag<ID_IO_PORT> >                  port;
    rule<ScannerT, parser_tag<ID_IO_PORT_LIST> >             port_list;
    rule<ScannerT, parser_tag<ID_ASL_PORT_LIST> >            asl_port_list;
    rule<ScannerT, parser_tag<ID_ASL_STATE_FORMULA_AND> >    asl_state_formula_and;
    rule<ScannerT, parser_tag<ID_ASL_STATE_TERM> >           asl_state_term;
    rule<ScannerT, parser_tag<ID_ASL_STATE_FORMULA> >        asl_state_formula;
    
    rule<ScannerT, parser_tag<ID_ASL_STATE_FORMULA> > const& 
    start() const { return asl_state_formula; }

    vereofy::vparser::guard_parser guard_grammar;
  };
};

#endif
