/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef LTL_INFIX_PARSER_HPP
#define LTL_INFIX_PARSER_HPP

// don't use standard spirit_init, as LTLInfixParser.cpp relies on
// collapsed tree semantics
//#include "spirit_init.hpp"

#include <boost/bind.hpp>
#include <boost/spirit/home/classic/core.hpp>
#include <boost/spirit/home/classic/attribute.hpp>
#include <boost/spirit/home/classic/dynamic/if.hpp>
#include <boost/spirit/home/classic/utility/confix.hpp>
#include <boost/spirit/home/classic/utility/distinct.hpp>
#include <boost/spirit/home/classic/utility/lists.hpp>
#include <boost/spirit/home/classic/utility/grammar_def.hpp>
#include <boost/spirit/home/classic/symbols/symbols.hpp>
#include <boost/spirit/home/classic/tree/ast.hpp>
#include <boost/spirit/home/classic/tree/parse_tree.hpp>
#include <boost/spirit/home/classic/tree/tree_to_xml.hpp>
#include <boost/spirit/home/classic/error_handling.hpp>

namespace vereofy {
  namespace spirit = boost::spirit::classic;
};

using namespace std;
using namespace vereofy::spirit;


struct ltl_infix_parser : public grammar<ltl_infix_parser>
{
  enum ltl_ids {
    ID_LTL_const_true,
    ID_LTL_const_false,
    ID_LTL_implies_op,
    ID_LTL_equivalent_op,
    ID_LTL_or_op,
    ID_LTL_xor_op,
    ID_LTL_and_op,
    ID_LTL_not_op,
    ID_LTL_eventually_op,
    ID_LTL_always_op,
    ID_LTL_next_op,
    ID_LTL_until_op,
    ID_LTL_weak_until_op,
    ID_LTL_release_op,

    ID_LTL_ap,
    ID_LTL_io,
    ID_LTL_term,
    ID_LTL_and,
    ID_LTL_implies_equiv,
    ID_LTL_until_release,
    ID_LTL_or,
    ID_LTL_xor,
    ID_LTL_formula
  };

  std::map<parser_id,string> getRuleNames() {
    std::map<parser_id,string> result;
    result[ID_LTL_const_true]="const_true";
    result[ID_LTL_const_false]="const_false";
    result[ID_LTL_implies_op]="implies_op";
    result[ID_LTL_equivalent_op]="equivalent_op";
    result[ID_LTL_or_op]="or_op";
    result[ID_LTL_xor_op]="xor_op";
    result[ID_LTL_and_op]="and_op";
    result[ID_LTL_not_op]="not_op";
    result[ID_LTL_eventually_op]="eventually_op";
    result[ID_LTL_always_op]="always_op";
    result[ID_LTL_next_op]="next_op";
    result[ID_LTL_until_op]="until_op";
    result[ID_LTL_weak_until_op]="weak_until_op";
    result[ID_LTL_release_op]="release_op";
    
    result[ID_LTL_ap]="ltl_ap";
    result[ID_LTL_io]="ltl_io";
    result[ID_LTL_term]="ltl_term";
    result[ID_LTL_and]="ltl_and";
    result[ID_LTL_or]="ltl_or";
    result[ID_LTL_xor]="ltl_xor";
    result[ID_LTL_implies_equiv]="ltl_implies_equiv";
    result[ID_LTL_until_release]="ltl_until_release";
    result[ID_LTL_formula]="ltl_formula";
    return result;
  }

  template <typename ScannerT>
  struct definition 
  {
    definition(ltl_infix_parser const& self_)
    {

      const_true = str_p("true") | str_p("TRUE") | str_p("t") | str_p("1");
      const_false = str_p("false") | str_p("FALSE") | str_p("f") | str_p("0");

      implies_op = str_p("=>") | str_p("->");
      equivalent_op = str_p("<=>") | str_p("<->");
      
      or_op = str_p("||") |
        str_p("|") |
        str_p("\\/");

      xor_op = str_p("^") | str_p("xor");
      
      and_op = str_p("&&") | 
        str_p("&") | 
        str_p("/\\");

      not_op = str_p("!");
      
      eventually_op = str_p("F") | str_p("<>");
      always_op = str_p("G") | str_p("[]");
      next_op = str_p("X") | str_p("()");
      until_op = str_p("U");
      weak_until_op = str_p("W");
      release_op = str_p("R") | str_p("V");
      

      ltl_ap = (lexeme_d[ no_node_d[ch_p('"')] >>
                         token_node_d[ +(~ch_p('"')) ] >>
                         no_node_d[ch_p('"')]
                          ]) 
        |
                (lexeme_d[ no_node_d[ch_p('\'')] >>
                         token_node_d[ +(~ch_p('\'')) ] >>
                         no_node_d[ch_p('\'')]
                          ]);

      ltl_io = no_node_d[ch_p('<')] 
        >> ltl_ap 
        >> no_node_d[ch_p('>')] 
        >> ltl_term;
      
      ltl_term = root_node_d[ltl_ap]
        | root_node_d[ltl_io]
        | root_node_d[const_true]
        | root_node_d[const_false]
        | (no_node_d[ch_p('(')] >> ltl_formula >> no_node_d[ch_p(')')])
        | (root_node_d[not_op] >> ltl_term)
        | (root_node_d[eventually_op] >> ltl_term)
        | (root_node_d[always_op] >> ltl_term)
        | (root_node_d[next_op] >> ltl_term);

      ltl_until_release = 
        ltl_term >> *( ((root_node_d[until_op]) | (root_node_d[release_op]) | (root_node_d[weak_until_op])) >> ltl_term );        

      ltl_implies_equiv =
        ltl_until_release >> *( ((root_node_d[implies_op]) | (root_node_d[equivalent_op])) >> ltl_until_release );

      ltl_and = ltl_implies_equiv >> *( root_node_d[and_op] >> ltl_implies_equiv );
      ltl_xor = ltl_and >> *( root_node_d[xor_op] >> ltl_and );
      ltl_or = ltl_xor >> *( root_node_d[or_op] >> ltl_xor );
          

      ltl_formula= ltl_or;
      BOOST_SPIRIT_DEBUG_RULE(ltl_and);
    }

    rule<ScannerT, parser_tag<ID_LTL_ap> >               ltl_ap;
    rule<ScannerT, parser_tag<ID_LTL_io> >               ltl_io;
    rule<ScannerT, parser_tag<ID_LTL_const_true> >       const_true;
    rule<ScannerT, parser_tag<ID_LTL_const_false> >      const_false;
    rule<ScannerT, parser_tag<ID_LTL_implies_op> >       implies_op;
    rule<ScannerT, parser_tag<ID_LTL_equivalent_op> >    equivalent_op;
    rule<ScannerT, parser_tag<ID_LTL_and_op> >           and_op;
    rule<ScannerT, parser_tag<ID_LTL_or_op> >            or_op;
    rule<ScannerT, parser_tag<ID_LTL_xor_op> >           xor_op;
    rule<ScannerT, parser_tag<ID_LTL_not_op> >           not_op;
    rule<ScannerT, parser_tag<ID_LTL_eventually_op> >    eventually_op;
    rule<ScannerT, parser_tag<ID_LTL_always_op> >        always_op;
    rule<ScannerT, parser_tag<ID_LTL_next_op> >          next_op;
    rule<ScannerT, parser_tag<ID_LTL_until_op> >         until_op; 
    rule<ScannerT, parser_tag<ID_LTL_weak_until_op> >    weak_until_op;
    rule<ScannerT, parser_tag<ID_LTL_release_op> >       release_op;

    rule<ScannerT, parser_tag<ID_LTL_term> >             ltl_term;
    rule<ScannerT, parser_tag<ID_LTL_and> >              ltl_and;
    rule<ScannerT, parser_tag<ID_LTL_or> >               ltl_or;
    rule<ScannerT, parser_tag<ID_LTL_xor> >              ltl_xor;
    rule<ScannerT, parser_tag<ID_LTL_implies_equiv> >    ltl_implies_equiv;
    rule<ScannerT, parser_tag<ID_LTL_until_release> >    ltl_until_release;

    rule<ScannerT, parser_tag<ID_LTL_formula> >          ltl_formula;

    rule<ScannerT, parser_tag<ID_LTL_formula> > const& 
    start() const { return ltl_formula; }
  };
};

#endif
