/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef REO_EXECUTE_EXCEPTION_HPP
#define REO_EXECUTE_EXCEPTION_HPP

#include <exception>
#include <string>
#include <vector>

namespace vereofy {
  class REOExecuteException : public std::exception {
  public:
    REOExecuteException(const std::string& message_) {
      message=message_;
    }

    ~REOExecuteException() throw () {};

    virtual void raise() {throw *this;}

    void addInstantiation(const std::string& instantiation) {
      instantiations.push_back(instantiation);
    }

    void setFile(const std::string& file_) {
      file=file_;
    }

    void setLocation(const std::string& location_) {
      location=location_;
    }

    virtual const char* what() {
      calc_text();
      return text.c_str();
    }

    const std::string& getFile() const {return file;}
    const std::string& getLocation() const {return location;}
    const std::string& getMessage() const {return message;}

  private:
    std::vector<std::string> instantiations;
    std::string message;
    std::string file;
    std::string location;
    
    std::string text;

    void calc_text() {
      text="Execute error";
      if (file!="" && location!="") {
        text+=" in "+file+" at "+location;
      } else if (file!="") {
        text+=" in "+file;
      } else if (location!="") {
        text+=" at "+location;
      }
      text+=":\n";
      text+=message;
      
      if (!instantiations.empty()) {
        text+="\n";
        for (unsigned int i=0;i<instantiations.size();i++) {
          text+=instantiations.at(i);
        }
      }
    }
  };
}


#endif
