/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef REOPARSER_HPP
#define REOPARSER_HPP

#include <string>
#include "spirit_init.hpp"
#include <boost/shared_ptr.hpp>

#include "vereofy/system/system_context.hpp"
#include "reo_parser_types.hpp"
#include "REOParseException.hpp"

using namespace vereofy::spirit;

namespace vereofy {
  class ComponentInterface;

namespace vparser {

class REOParser {
public:
  REOParser(SystemContext::wptr system_context);
  
  bool parse_file(const std::string& filename);
  bool parse(const std::string& text, const std::string& filename="");
  pt_node parse_io_guard(const std::string& guard, 
                         const boost::shared_ptr<ComponentInterface> c_interface,
                         bool allow_state_references) const;
  pt_node parse_state_guard(const std::string& guard,
                            const boost::shared_ptr<ComponentInterface> c_interface) const;

  void parse_and_add_typedef(const std::string& type_def);
  void parse_and_add_constdef(const std::string& const_def);

  static void subtree_to_xml(std::ostream& out, const pt_node& n);

private:
  SystemContext::wptr system_context;

  template <class Node>
  void add_parse_trees(const Node& global);

  pt_node parse_guard(unsigned int parser_type,
                      const std::string& guard,
                      const boost::shared_ptr<ComponentInterface> c_interface,
                      bool allow_state_references_for_io_guard=false) const;
  
  typedef char                    char_t;
  //  typedef file_iterator<char_t>   file_iterator_t;
  typedef position_iterator<const char *>   iterator_t;
};

}
}
#endif

