/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMBOL_TABLE_HPP
#define SYMBOL_TABLE_HPP

#include "vereofy/common/checks.hpp"
#include <boost/shared_ptr.hpp>
#include <string>
#include <map>
#include <iostream>


template <class Symbol_ptr>
struct SymbolTable {
  typedef boost::shared_ptr<SymbolTable> ptr;
  typedef boost::weak_ptr<SymbolTable> wptr;
  typedef Symbol_ptr symbol_ptr;
  
  SymbolTable() : read_only(false) {}
  
  bool add(const std::string& name, Symbol_ptr s) {
    ERROR_CHECK_ASSERT(name!="");
    if (read_only) return false;
    if (find(name)) {
      return false;
    }

    if (debugLevel>=4) {
      std::cerr << "SymbolTable::add("<<this<<", "<<name<<")\n";
    }
    return symbols.insert(typename symbol_map::value_type(name, s)).second;
  }
  
  void replace(const std::string& name, Symbol_ptr new_s) {
    symbols[name]=new_s;
  }

  Symbol_ptr find(const std::string& name) const {
    typename symbol_map::const_iterator it=symbols.find(name);
    if (it==symbols.end()) {
      // no SymbolInfo found
      return Symbol_ptr();
    } else {
      // found, return SymbolInfo::ptr
      return (*it).second;
    }
  }

  typedef std::map<std::string, Symbol_ptr> symbol_map;
  typedef typename symbol_map::iterator iterator;

  typename symbol_map::iterator begin() {return symbols.begin();}
  typename symbol_map::iterator end() {return symbols.end();}

  symbol_map symbols;
  
  std::string name;
  bool read_only;
};



#endif
