/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMBOL_TABLE_STACK_HPP
#define SYMBOL_TABLE_STACK_HPP

#include <string>
#include <vector>

#include <iostream>

#include <boost/shared_ptr.hpp>
#include "vereofy/common/weak_ptr_helper.hpp"

template <class SymbolTable>
struct SymbolTableStack {
  typedef boost::shared_ptr<SymbolTable> SymbolTable_ptr;
  typedef boost::weak_ptr<SymbolTable> SymbolTable_wptr;

  SymbolTableStack() {}
  
  void push(SymbolTable_wptr t) {
    if (debugLevel>=5) {
      std::cerr << "SymbolTableStack::push("<<this<< "," << vereofy::get_shared_ptr_from_weak_ptr(t) <<")\n";
    }
    table_stack.push_back(t);
  }
  
  void pop() {
    if (empty()) {
      throw "Trying to pop on an empty stack";
    }

    if (debugLevel>=5) {
      std::cerr << "SymbolTableStack::pop("<<this<<"," << vereofy::get_shared_ptr_from_weak_ptr(table_stack.back()) <<")\n";    
    }
    table_stack.pop_back();
  }

  void pop_all() {
    while (!empty()) {
      pop();
    }
  }

  bool empty() const {return table_stack.empty();}

  SymbolTable_ptr getTopTable() const {
    if (empty()) {
      return SymbolTable_ptr();
    }
    return vereofy::get_shared_ptr_from_weak_ptr(table_stack.back());
  }

  typename SymbolTable::symbol_ptr find(const std::string& name) const {
    typename SymbolTable::symbol_ptr result;

    for (typename table_stack_t::const_reverse_iterator it=
           table_stack.rbegin();
         it!=table_stack.rend();
         ++it) {
      result=vereofy::access_weak_ptr(*it)->find(name);
      if (result) {
        return result;
      }
    }
    return result;
  }

private:
  typedef std::vector<SymbolTable_wptr> table_stack_t;
  table_stack_t table_stack;
};

#endif
