/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef CIRCUIT_TYPES_HPP
#define CIRCUIT_TYPES_HPP

#include "reo_parser_types.hpp"
#include "vereofy/system/reactive_module/reactive_module.hpp"
#include "vereofy/system/reactive_module/component_instance_rm.hpp"
#include "vereofy/system/circuit_connector/circuit_connector.hpp"
#include "vereofy/system/circuit_connector/component_instance_circuit.hpp"

#include "vereofy/system/reo_node.hpp"
#include "vereofy/system/reo_channel_end.hpp"
#include "vereofy/system/reo_component.hpp"

namespace vereofy {
namespace vparser {

struct BasicChannelInfo : public ComponentData {
  typedef boost::shared_ptr<BasicChannelInfo> ptr;

  BasicChannelInfo(const string& name_) : ComponentData(COMPONENT_BASIC_CHANNEL, name_) {}
};

struct CircuitInfo : public ComponentData {
  typedef boost::shared_ptr<CircuitInfo> ptr;
  
  CircuitInfo(const string& name_) : ComponentData(COMPONENT_CIRCUIT, name_) {}

  boost::shared_ptr<vereofy::CircuitConnector> connector;
};

struct ModuleInfo : public ComponentData {
  typedef boost::shared_ptr<ModuleInfo> ptr;

  ModuleInfo(const string& name_) : ComponentData(COMPONENT_REACTIVE_MODULE, name_) {}

  boost::shared_ptr<vereofy::ReactiveModule> module;
};


struct ComponentRef : public Value {
  typedef boost::shared_ptr<ComponentRef> ptr;

  ComponentRef(ComponentInstance::ptr ci_,
               REO_Component::ptr reo_component_=REO_Component::ptr()) 
  : Value(ValueType::ptr(new ValueType(VALUE_TYPE_COMPONENT_REF))),
    ci(ci_),
    reo_component(reo_component_) {
    if (!reo_component) {
      reo_component=REO_Component::getFor(ci);
    }
  }
  
  virtual Value::ptr clone() const {
    return Value::ptr(new ComponentRef(ci, reo_component));
  }
  
  virtual void print(ostream& out, bool detailed=false) {out << "ComponentRef: " << reo_component;}

  ComponentInstance::ptr ci;
  REO_Component::ptr reo_component;
};


struct NodeRef : public Value {
  typedef boost::shared_ptr<NodeRef> ptr;

  NodeRef() : Value(ValueType::NODE_REF) {}

  virtual Value::ptr clone() const {
    NodeRef::ptr result(new NodeRef());
    result->reo_node=reo_node;
    return result;
  }

  virtual void print(ostream& out, bool detailed=false) {out << "NodeRef: " << reo_node;}

  REO_Node::ptr reo_node;
};

struct ChannelEndRef : public Value {
  typedef boost::shared_ptr<ChannelEndRef> ptr;

  ChannelEndRef() : Value(ValueType::CHANNEL_END_REF) {}

  virtual Value::ptr clone() const {
    ChannelEndRef::ptr result(new ChannelEndRef());
    result->reo_channel_end=reo_channel_end;
    return result;
  }

  virtual void print(ostream& out, bool detailed=false) {out << "ChannelEndRef: " << reo_channel_end;}

  REO_Channel_End::ptr reo_channel_end;

};

}
}

#endif
