/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef CONST_DEF_PARSER_HPP
#define CONST_DEF_PARSER_HPP

#include "spirit_init.hpp"
using namespace vereofy::spirit;

#include <boost/bind.hpp>

#include <boost/lexical_cast.hpp>
#include <boost/shared_ptr.hpp>

#include <stack>
#include <string>


#include "reo_parser_types.hpp"
#include "position_node.hpp"
#include "position_node_factory.hpp"

#include "parser_helper.hpp"

#include "vereofy/system/datadomain.hpp"

#include "identifier_parser.hpp"
#include "int_parser.hpp"

#include "const_def_parser_helper.hpp"

namespace vereofy {
namespace vparser {

struct int_parser;

struct const_def_parser : public grammar<const_def_parser>
{
  static 
  void init_rule_names(std::map<parser_id, std::string>& rule_names) {
    rule_names[ID_CO_const_value_int] = "const_value_int";
    rule_names[ID_CO_const_def]   = "const_def";
  }


  struct s_const_def {
    const_def_parser& self;
    
    s_const_def(const_def_parser& self_) : self(self_) {
    }

    template <class Node, class Iter>
    void operator()(const Node& node, Iter i1, Iter i2) const {
      std::string name=const_def_helper::t_const_def::getName(node);

      SymbolData::ptr sd=self.symbol_table_stack.find(name);
      if (sd) {
        if (sd->type==SYMBOL_TYPE_CONSTANT &&
            (static_cast<ConstantData&>(*sd)).hasPriority()) {
          // the already defined constant has higher priority
          std::cout << "CONST definition for \""<< name << "\" is ignored, has already been defined previously (Value = ";
          std::cout <<  (static_cast<ConstantData&>(*sd)).valueToString() << ")\n";
        } else {
          throw_(i1, "There is already a definition for "+name);
        }
      }

      const Node& value=const_def_helper::t_const_def::getValue(node);

      ConstantData::ptr const_data;
      int v=const_def_helper::t_const_value_int::getIntValue(value, self.symbol_table_stack);

      const_data=ConstantData::ptr(new ConstantData(name, v));
      
      if (debugLevel>=5) {
        cerr << "CONST " << name << " = " << v << "\n";
      }

      const_data->hasPriority(self.has_priority);

      self.symbol_table_stack.getTopTable()->add(name, const_data);
    }
  };


  const_def_parser(symbol_table_stack_t& symbol_table_stack_,
                   const symbols<>& reserved_keywords_,
                   bool has_priority_) :
    symbol_table_stack(symbol_table_stack_),
    reserved_keywords(reserved_keywords_),
    has_priority(has_priority_) 
  {}

  symbol_table_stack_t& symbol_table_stack;
  const symbols<>& reserved_keywords;
  bool has_priority;
  
  template <typename ScannerT>
  struct definition {
    definition(const_def_parser const& self_) :
      identifier_grammar(last_identifier, self_.reserved_keywords),
      integer_grammar(self_.symbol_table_stack)
    {
      const_def_parser& self=const_cast<const_def_parser&>(self_);

      BOOST_SPIRIT_DEBUG_GRAMMAR(identifier_grammar);
      BOOST_SPIRIT_DEBUG_GRAMMAR(integer_grammar);
      
      ENTRY_GRAMMAR(identifier_p, identifier_parser, identifier_grammar, identifier_parser::identifier);
      ENTRY_GRAMMAR(enum_value_p, identifier_parser, identifier_grammar, identifier_parser::enum_value);

      ENTRY_GRAMMAR(const_expression_p, int_parser, integer_grammar, int_parser::const_expression);

      //  Start grammar definition
      const_value_int = 
        const_expression_p; 
      
      const_def = access_node_d[
                                root_node_d[keyword_d[str_p("CONST")]]
                                >> identifier_p
                                >> no_node_d[ch_p('=')]
                                >> const_value_int
                                >> no_node_d[ch_p(';')]
                                ][s_const_def(self)];
      //  End grammar definition

      BOOST_SPIRIT_DEBUG_RULE(const_value_int);
      BOOST_SPIRIT_DEBUG_RULE(const_def);
    }
    rule<ScannerT, parser_tag<ID_CO_const_value_int> >  const_value_int;
    rule<ScannerT, parser_tag<ID_CO_const_def> >   const_def;

    rule<ScannerT, parser_tag<ID_CO_const_def> > const&
    start() const { return const_def; }

    std::string last_identifier;
    identifier_parser identifier_grammar;
    int_parser integer_grammar;
  };
};

}
}
#endif
