/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef IDENTIFIER_PARSER_HPP
#define IDENTIFIER_PARSER_HPP

#include "spirit_init.hpp"
using namespace vereofy::spirit;

#include <boost/bind.hpp>

#include <boost/lexical_cast.hpp>
#include <boost/shared_ptr.hpp>

#include <string>

#include "reo_parser_types.hpp"
#include "position_node.hpp"
#include "position_node_factory.hpp"

#include "vereofy/common/singleton_set.hpp"
#include "parser_helper.hpp"

#include "vereofy/system/reactive_module/reactive_module.hpp"
#include "vereofy/system/datadomain.hpp"

#include <boost/typeof/typeof.hpp>

namespace vereofy {
namespace vparser {

struct identifier_parser : public grammar<identifier_parser>
{
  std::string& last_identifier_ref;
  const symbols<>& reserved_keywords;

  enum {
    identifier = 0,
    enum_value = 1
  };


  static 
  void init_rule_names(map<parser_id, std::string>& rule_names) {
    rule_names[ID_I_identifier] = "identifier";
    rule_names[ID_I_enum_value] = "enum_value";
  }


  identifier_parser(const symbols<>& reserved_keywords_) :
    last_identifier_ref(own_last_identifier),
    reserved_keywords(reserved_keywords_)
  {}

  identifier_parser(std::string& last_identifier_ref_,
                    const symbols<>& reserved_keywords_) :
    last_identifier_ref(last_identifier_ref_),
    reserved_keywords(reserved_keywords_)
  {}


  std::string own_last_identifier;
  
  template <typename ScannerT>
  struct definition :
    public grammar_def< rule<ScannerT, parser_tag<ID_I_identifier> >, 
                        rule<ScannerT, parser_tag<ID_I_enum_value> > > 
  {
    definition(identifier_parser const& self_)
    {
      identifier_parser& self=const_cast<identifier_parser&>(self_);

      //  Start grammar definition
      identifier  =   lexeme_d[
                               ~eps_p(keyword_d[self.reserved_keywords])   // ensure that identifier is not a reserved keyword
                               >> leaf_node_d[(ch_p('_') | alpha_p) >> * (alnum_p | ch_p('_'))]
                               ][assign_a(self.last_identifier_ref)];


      enum_value = (identifier >> str_p("::") >> identifier) 
        | identifier;  
      //  End grammar definition

      this->start_parsers(identifier, enum_value);


      BOOST_SPIRIT_DEBUG_RULE(identifier);
      BOOST_SPIRIT_DEBUG_RULE(enum_value);
    }


    rule<ScannerT, parser_tag<ID_I_identifier> >   identifier;
    rule<ScannerT, parser_tag<ID_I_enum_value> >  enum_value;
  };
};

}
}
#endif
