/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef IDENTIFIER_PARSER_HELPER_HPP
#define IDENTIFIER_PARSER_HELPER_HPP

#include "parser_helper.hpp"
#include "vereofy/common/checks.hpp"

namespace vereofy {
namespace vparser {

struct identifier_helper {


  struct t_identifier {
    static string getIdentifier(const pt_node& n) {
      ID_ASSERT_(n, ID_I_identifier);
      return string(n.value.begin(),
                    n.value.end());
    }
  };


  struct t_enum_value {
    static bool isFullyQualifiedName(const pt_node& n) {
      ID_ASSERT_(n, ID_I_enum_value);
      return n.children.size()==3;
    }

    static string getTypePart(const pt_node& n) {
      ID_ASSERT_(n, ID_I_enum_value);
      ERROR_CHECK_ASSERT(isFullyQualifiedName(n));
      return t_identifier::getIdentifier(n.children.at(0));
    }

    static string getNamePart(const pt_node& n) {
      ID_ASSERT_(n, ID_I_enum_value);
      if (isFullyQualifiedName(n)) {
        return t_identifier::getIdentifier(n.children.at(2));
      } else {
        return t_identifier::getIdentifier(n.children.at(0));
      }
    }

    static string getString(const pt_node& n) {
      ID_ASSERT_(n, ID_I_enum_value);
      if (isFullyQualifiedName(n)) {
        return getTypePart(n) + "::"+ getNamePart(n);
      } else {
        ERROR_CHECK_ASSERT(n.children.size()==1);
        return getNamePart(n);
      }
    }
  };



};



}
}

#endif
