/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef PARSER_HELPER_HPP
#define PARSER_HELPER_HPP

#include "spirit_init.hpp"
#include "vereofy/common/checks.hpp"
#include "vereofy/common/singleton_set.hpp"
#include "reo_parser_types.hpp"

using namespace vereofy::spirit;

// keyword_p for C++
// (for basic usage instead of std_p)
const vereofy::spirit::distinct_parser<> keyword_p("0-9a-zA-Z_");

// keyword_d for C++
// (for more intricate usage, for example together with symbol tables)
const vereofy::spirit::distinct_directive<> keyword_d("0-9a-zA-Z_");

#define REO_PARSER_WITH_MODULES


/** assertion: check that pt_node N either has no ID yes (ID==0) or
 *   matches the given ID(s) */
#define ID_ASSERT_(N, ID) CHECK(1, N.value.id().to_long()==0 || N.value.id() == parser_id(ID), "parse tree mismatch "+boost::lexical_cast<std::string>(N.value.id().to_long()))
#define ID_ASSERT_2(N, ID1, ID2) CHECK(1, N.value.id().to_long()==0 || N.value.id() == parser_id(ID1) || N.value.id() == parser_id(ID2), "parse tree mismatch "+boost::lexical_cast<std::string>(N.value.id().to_long()))
#define ID_ASSERT_3(N, ID1, ID2, ID3) CHECK(1, N.value.id().to_long()==0 || N.value.id() == parser_id(ID1) || N.value.id() == parser_id(ID2), N.value.id() == parser_id(ID3), "parse tree mismatch "+boost::lexical_cast<std::string>(N.value.id().to_long()))


namespace vereofy {
namespace vparser {

  template <class Tree>
  static void tree2xml(ostream& out, Tree& tree, 
                       std::map<parser_id, std::string>& rule_names) {
    tree_to_xml(cout, tree, "", rule_names);
  }

  template <class TreeNode>
  static void treenode2xml(ostream& out, const TreeNode& node,
                       std::map<parser_id, std::string>& rule_names) {
    SingletonSet<const TreeNode> s(node);
    tree_to_xml(cout, s, "", rule_names);
  }

  struct s_int_value_assign {
    template <class Node, class IterT>
    void operator()(Node& n, IterT it1, IterT it2) const {
      IntValue::ptr v(new IntValue(boost::lexical_cast<int>(string(it1, it2))));
      n.value.value(v);
    }
  };

#ifdef REO_PARSER_POSITION_NODES
  template <class FilePosition>
  std::string getFilePositionToString(FilePosition &pos) {
    return pos.file+" ("+
      boost::lexical_cast<string>(pos.line)+
      ","+
      boost::lexical_cast<string>(pos.column)+
      ")";
  }
#endif


  // generic helpers

  struct t_generic_list {
    static unsigned int countChildren(const pt_node& n) {
      return n.children.size();
    }
    
    static const pt_node& get(const pt_node& n, unsigned int i) {
      return n.children.at(i);
    }
  };
  

}
}

#endif
