/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef POSITION_NODE_HPP
#define POSITION_NODE_HPP

#include <boost/shared_ptr.hpp>
#include "spirit_init.hpp"

namespace boost { namespace spirit { namespace classic {

template <typename NodeT, 
          typename PositionIteratorT, 
          typename IteratorT = char const*, 
          typename ValueT = vereofy::spirit::nil_t>
struct position_node : public NodeT {
  typedef typename NodeT::value_type value_type;
  typedef typename NodeT::allocator_type allocator_type;
  typedef typename NodeT::container_t container_t;
  typedef typename NodeT::iterator_t iterator_t;
  typedef typename NodeT::const_iterator_t const_iterator_t;

  typedef typename boost::shared_ptr<file_position> file_position_ptr;

  position_node() {}

  position_node(const file_position& pos_) :
    pos(new file_position(pos_)) {}

  position_node(IteratorT const& _first, IteratorT const& _last) :
    NodeT(_first, _last) {}

  // This constructor is for building text out of iterators
  template <typename IteratorT2>
  position_node(IteratorT2 const& _first, IteratorT2 const& _last , const file_position& pos_) :
    NodeT(_first, _last), pos(new file_position(pos_)) {
    //    cerr << pos->line << ":-:"<< pos->column <<"\n";
    //    cerr << " [" << string(_first, _last) << "]\n";
  }

  template <typename IteratorT2>
    position_node(IteratorT2 const& _first, IteratorT2 const& _last) :
    NodeT(_first, _last) {}

  position_node(PositionIteratorT const& _first, PositionIteratorT const& _last) : 
    NodeT(_first, _last) {
    pos=file_position_ptr(new file_position(_first.get_position()));
    //    cerr << pos->line << "::"<< pos->column <<"\n";
    //    cerr << " [" << string(_first, _last) << "]\n";
  }

  void setPosition(const file_position& fpos) {
    pos=file_position_ptr(new file_position(fpos));
  }

  void swap(position_node& x) {
    NodeT::swap((NodeT)x);
    impl::cp_swap(x.pos);
  }

  file_position_ptr pos;
};

} 
}
}


#endif
