/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef POSITION_NODE_FACTORY_HPP
#define POSITION_NODE_FACTORY_HPP

#include "vereofy/common/checks.hpp"
#include "position_node.hpp"

// adapted from boost 1.34(?)

namespace boost { namespace spirit { namespace classic {

template <typename NodeT>
class position_node_factory 
{
public:
  // This inner class is so that position_node_val_data_factory can simulate
  // a template template parameter
  template <typename IteratorT>
  class factory
  {
  public:
    typedef IteratorT iterator_t;
    typedef NodeT node_t;
    
    static node_t create_node(iterator_t const& first, iterator_t const& last,
                              bool is_leaf_node) {
      if (is_leaf_node) {
        return node_t(first, last);
      } else {
        return node_t(first.get_position());
      }
    }
    
    static node_t empty_node()
    {
      return node_t();
    }
    
    template <typename ContainerT>
    static node_t group_nodes(ContainerT const& nodes) {
      typename node_t::container_t c;
      typename ContainerT::const_iterator i_end = nodes.end();
      // copy all the nodes text into a new one
      for (typename ContainerT::const_iterator i = nodes.begin();
           i != i_end; ++i)
        {
          // See docs: token_node_d or leaf_node_d cannot be used with a
          // rule inside the [].
          ERROR_CHECK_ASSERT(i->children.size() == 0);
          c.insert(c.end(), i->value.begin(), i->value.end());
        }
      return node_t(c.begin(), c.end(), *nodes.begin()->value.pos);
    }
  };
};

}
}
}
#endif
