/***************************************************************************
 *   Copyright (C) 2010 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef REO_PARSER_HELPER_HPP
#define REO_PARSER_HELPER_HPP

#include "vereofy/common/checks.hpp"
#include "identifier_parser_helper.hpp"
#include "type_parser_helper.hpp"

namespace vereofy {
namespace vparser {

struct reo_parser_helper {

  // empty helpers, as the effects of these rules happens during parsing
  struct t_include {};
  struct t_alias_def {};
  struct t_replace_def {};
  struct t_global_ap_def {};
  struct t_global {};
  
  struct t_function_def {
    static std::string getName(const pt_node& n) {
      return identifier_helper::t_identifier::getIdentifier(n.children.at(1));
    }

    static const pt_node& getReturnType(const pt_node& n) {
      return n.children.at(0);
    }

    static bool isReturnTypeBoolean(const pt_node& n) {
      if (type_helper::t_type::isBoolDataType(getReturnType(n))) {
        return true;
      }
      return false;
    }

    static const pt_node& getArgumentList(const pt_node& n) {
      return n.children.at(2);
    }

    static bool isFunctionExpressionBoolean(const pt_node& n) {
      return (getFunctionExpression(n).value.id() == parser_id(ID_G_bool_expression));
    }


    static const pt_node& getFunctionExpression(const pt_node& n) {
      return n.children.at(3);
    }
  };

  struct t_function_arg {
    static std::string getName(const pt_node& n) {
      return identifier_helper::t_identifier::getIdentifier(n.children.at(1));
    }

    static const pt_node& getType(const pt_node& n) {
      return n.children.at(0);
    }
  };

 struct t_function_arg_list {
    static unsigned int getSize(const pt_node& n) {
      return n.children.size();
    }

    static const pt_node& getArgument(const pt_node& n, unsigned int i) {
      CHECK_SUCCESS(i<getSize(n));
      return n.children.at(i);
    }
  };


};

}}

#endif
