/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef REO_PARSER_TYPES_HPP
#define REO_PARSER_TYPES_HPP

#include <string>

#include "vereofy/system/symb_automaton/symb_expression.hpp"
#include "vereofy/system/symb_automaton/symb_automaton.hpp"
#include "vereofy/system/symb_automaton/symb_datadomain.hpp"
#include "vereofy/system/datadomain.hpp"
#include "vereofy/system/types.hpp"

#include "vereofy/common/value_or_undefined.hpp"

#include "SymbolTable.hpp"
#include "SymbolTableStack.hpp"
#include "ExecutionEnvironment.hpp"

//#include <iostream>
#include "spirit_init.hpp"
#include "position_node_factory.hpp"


struct Data {
  typedef boost::shared_ptr<Data> ptr;

  virtual ~Data() {}
  virtual void print(ostream& out, bool detailed=false) const {out << "<unknown>>";}
};


namespace vereofy {
  class ReactiveModule;
  class ComponentInterface;
};

namespace vereofy {
namespace vparser {
  typedef char                    char_t;
  //  typedef file_iterator<char_t>   file_iterator_t;
  typedef vereofy::spirit::position_iterator<const char*>   iterator_t;

#define REO_PARSER_POSITION_NODES 1

#ifdef REO_PARSER_POSITION_NODES
#define REO_PARSER_NODE_TYPE vereofy::spirit::position_node<vereofy::spirit::node_val_data<vereofy::vparser::iterator_t, Data::ptr>, vereofy::vparser::iterator_t>
#define REO_PARSER_NODE_FACTORY vereofy::spirit::position_node_factory<REO_PARSER_NODE_TYPE >
#else
#define REO_PARSER_NODE_TYPE vereofy::spirit::node_val_data<ns_reo_vparser::iterator_t, Data::ptr>
#define REO_PARSER_NODE_FACTORY vereofy::spirit::node_val_data_factory<Data::ptr>
#endif

  /** An AST parse tree node */
  typedef vereofy::spirit::tree_node<REO_PARSER_NODE_TYPE> pt_node;

  
  enum rule_id {
    // REO CIRCUIT_PARSER
    ID_C_null_value = 1,     // the initial ID has to be = 1!
    ID_C_rsl_factor, 
    ID_C_rsl_term,
    ID_C_rsl_expression, 
    
    ID_C_rsl_bool_simple, 
    ID_C_rsl_bool_factor, 
    ID_C_rsl_bool_and, 
    ID_C_rsl_bool_or,
    ID_C_rsl_bool_expression,
    
    ID_C_lhs_var, 
    ID_C_rhs_var, 
    ID_C_array_ref, 
    ID_C_var_expression, 
    
    ID_C_assignment, 
    ID_C_connector_spec, 
    ID_C_instantiation, 
    ID_C_node_instantiation, 
    ID_C_join_expression,

    ID_C_ap_statement,

    ID_C_if_statement, 
    ID_C_for_statement,

    ID_C_statement, 
    ID_C_statements, 
    ID_C_statement_block,
    ID_C_configuration_block,
    ID_C_parameter_list, 
    ID_C_with_pair, 
    ID_C_with_block,
    ID_C_circuit_body,
    ID_C_circuit_def,

    ID_C_include,
    ID_C_alias_def,
    ID_C_replace_def,
    ID_C_global_ap_def,
    ID_C_global,

    ID_C_function_arg,
    ID_C_function_arg_list,
    ID_C_function_def,
    
    ID_C_is_op,  // special ID if rule is an operator, type in value

    // COMMON PARSER
    ID_COM_quoted_string,
    ID_COM_parameter_def,

    // IDENTIFIER PARSER
    ID_I_identifier,
    ID_I_enum_value,

    
    // INT PARSER
    ID_INT_hex_string,
    ID_INT_integer,
    ID_INT_const_ref_int,
    ID_INT_const_ref_enum,
    ID_INT_minus_operator,
    ID_INT_const_function_call,
    ID_INT_const_factor,
    ID_INT_const_term,
    ID_INT_const_expression,

    // TYPE PARSER

    ID_T_int_range,
    ID_T_int_type,
    ID_T_enum_type,
    ID_T_named_type,
    ID_T_flat_type,
    ID_T_array_type,
    ID_T_struct_member_def,
    ID_T_struct_type,
    ID_T_tagged_union_type,
    ID_T_type,
    ID_T_type_def,



    // GUARD PARSER

    ID_G_any_value,
    ID_G_function_call,
    ID_G_bool_function_call,

    ID_G_guard_factor,
    ID_G_guard_term,
    ID_G_guard_expression,
    ID_G_guard_expression_full,

    ID_G_range_def,
    ID_G_tmp_var_def,
    ID_G_iterate_expression,
    ID_G_compare_expression,

    ID_G_bool_expression_term,
    ID_G_bool_expression_ranged,
    ID_G_bool_expression_implies_equiv,
    ID_G_bool_expression_and,
    ID_G_bool_expression_xor,
    ID_G_bool_expression,
    
    ID_G_if_expression,
    ID_G_ite_expression,
    ID_G_ite_bool_expression,

    ID_G_virtual_value,
    ID_G_int_range,

    ID_G_state_var,
    ID_G_simple_state_var,
    ID_G_ap_ref,
    ID_G_simple_ap_ref,
    ID_G_subcomponent_access,
    ID_G_struct_member_access,
    ID_G_array_access,
    ID_G_port,
    ID_G_port_name,
    ID_G_port_data,
    ID_G_active_ports,
    ID_G_compare_op,
    ID_G_port_activity_expression,
    ID_G_port_data_expression,
    ID_G_has_label_expression,
    ID_G_label_id,

    ID_G_state_guard,
    ID_G_io_guard,
    ID_G_state_assignment,
    ID_G_add_label,

    // MODULE PARSER

    ID_M_l_transition,
    ID_M_m_transition,
    ID_M_r_transition,

    ID_M_transitions,
    ID_M_transition,

    ID_M_compact_transition,

    ID_M_var_type,
    ID_M_var_init,
    ID_M_var_def,
    ID_M_label_def,
    ID_M_defs,
    ID_M_ap_def,
    ID_M_ap_defs,
    ID_M_interface_def,
    ID_M_interface_defs,
    ID_M_body,

    ID_M_module_def,

    ID_CO_const_value_int,
    ID_CO_const_def
  };


  struct module_var_type : public Data {
    typedef boost::shared_ptr<module_var_type> ptr;

    module_var_type(DataDomain::ptr type_) :
      type(type_) {}

    vereofy::DataDomain::ptr type;

    virtual void print(ostream& out) {
      out << type->toString() << "\n";
    }
  };


}
}

namespace vereofy {

enum symbol_type {SYMBOL_TYPE_COMPONENT,
                  SYMBOL_TYPE_TYPEDEF,
                  SYMBOL_TYPE_CONSTANT,
                  SYMBOL_TYPE_FUNCTION,
                  SYMBOL_TYPE_VIRTUAL_VALUE};

enum value_type {VALUE_TYPE_NODE_REF,
                 VALUE_TYPE_CHANNEL_END_REF,
                 VALUE_TYPE_COMPONENT_REF,
                 VALUE_TYPE_INT,
                 VALUE_TYPE_BOOL,
                 VALUE_TYPE_UNKNOWN};


struct SymbolData : public Data {
  typedef boost::shared_ptr<SymbolData> ptr;

  SymbolData(symbol_type t) : type(t) {}

  symbol_type type;
};

struct ComponentData : public SymbolData {
  typedef boost::shared_ptr<ComponentData> ptr;

  ComponentData(component_type_t ct, const std::string& name_) :
    SymbolData(SYMBOL_TYPE_COMPONENT), comp_type(ct), name(name_) {}

  component_type_t comp_type;
  std::string name;
};


struct TypeDefData : public SymbolData {
  typedef boost::shared_ptr<TypeDefData> ptr;

  TypeDefData(const std::string& name_, DataDomain::ptr data_domain_) :
    SymbolData(SYMBOL_TYPE_TYPEDEF), name(name_), data_domain(data_domain_) {}

  std::string name;
  DataDomain::ptr data_domain;
};

struct ConstantData : public SymbolData {
  typedef boost::shared_ptr<ConstantData> ptr;

  ConstantData(const std::string& name_, int_value_t value_) :
    SymbolData(SYMBOL_TYPE_CONSTANT), name(name_), int_value(value_), has_priority(false) {
  }

  void hasPriority(bool value) {has_priority=value;}
  bool hasPriority() {return has_priority;}

  std::string valueToString() {return boost::lexical_cast<std::string>(int_value);}
  
  std::string name;
  int_value_t int_value;
  bool has_priority;
};



struct VirtualValueData : public SymbolData {
  typedef boost::shared_ptr<VirtualValueData> ptr;

  VirtualValueData(const std::string& name_, DataDomain::ptr data_domain_) :
    SymbolData(SYMBOL_TYPE_VIRTUAL_VALUE), name(name_), data_domain(data_domain_) {}

  VirtualValueData(const std::string& name_, SymbExpression::ptr expression_) :
    SymbolData(SYMBOL_TYPE_VIRTUAL_VALUE), name(name_), data_domain(expression_->getDataDomain()->getDataDomain()),
    expression(expression_) {}

  const std::string& getName() {return name;}
  SymbExpression::ptr getExpression() {return expression;}
  DataDomain::ptr getDataDomain() {return data_domain;}

  std::string name;
  DataDomain::ptr data_domain;
  SymbExpression::ptr expression;
};


// forward declaration
class SymbFunction;


struct FunctionData : public SymbolData {
  typedef boost::shared_ptr<FunctionData> ptr;

  FunctionData(const std::string& name_, boost::shared_ptr<SymbFunction> function_) :
    SymbolData(SYMBOL_TYPE_FUNCTION), name(name_), function(function_) {}

  std::string name;
  boost::shared_ptr<SymbFunction> function;
};


struct ValueType : public Data {
  typedef boost::shared_ptr<ValueType> ptr;

  ValueType(value_type type_, Data::ptr data_=Data::ptr()) :
    type(type_), data(data_) {};

  std::string typeToString() const {
    switch (type) {
    case VALUE_TYPE_NODE_REF: return "NODE_REF";
    case VALUE_TYPE_COMPONENT_REF: return "COMPONENT_REF";
    case VALUE_TYPE_INT: return "INT";
    case VALUE_TYPE_BOOL: return "BOOL";
    default: return "<<UNKNOWN>>";
    }
  }

  bool isComponent() {
    return 
      (type==VALUE_TYPE_COMPONENT_REF);
  }

  value_type type;
  Data::ptr data;

  static ValueType::ptr INT;
  static ValueType::ptr BOOL;
  static ValueType::ptr NODE_REF;
  static ValueType::ptr CHANNEL_END_REF;
};

struct Value : public Data  {
  typedef boost::shared_ptr<Value> ptr;

  Value(ValueType::ptr type_=ValueType::ptr(new ValueType(VALUE_TYPE_UNKNOWN))) :
    type(type_) {}

  virtual Value::ptr clone() const = 0;

  virtual void print(ostream& out, bool detailed=false) {out << "<unknown>";}

  ValueType::ptr type;
};

struct IntValue : public Value {
  typedef boost::shared_ptr<IntValue> ptr;

  IntValue(int v) : Value(ValueType::INT), value(v) {}
  
  virtual Value::ptr clone() const {
    return Value::ptr(new IntValue(value));
  }

  virtual void print(ostream& out, bool detailed=false) const {out << value;}

  template <class D_ptr>
  static int getIntValue(D_ptr dp) {
    Value::ptr vp=boost::static_pointer_cast<Value>(dp);
    if (vp->type->type!=VALUE_TYPE_INT) {
      throw "Type mismatch!";
    }
    return boost::static_pointer_cast<IntValue>(vp)->value;
  }

  virtual void print(ostream& out, bool detailed=false) {out << "int: " << value;}
  
  int value;
};

typedef SymbolTable<SymbolData::ptr> symbol_table_t;
typedef symbol_table_t::ptr symbol_table_ptr;
typedef symbol_table_t::wptr symbol_table_wptr;
typedef SymbolTableStack<symbol_table_t> symbol_table_stack_t;

typedef ExecutionEnvironment<Value::ptr, symbol_table_t> execution_environment_t;

}
#endif
