/**************************************************************************
*   Copyright (C) 2008-2009 by Johannes Osterholzer                       *
*   s5892660@inf.tu-dresden.de                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef SDE_GRAMMAR_HPP
#define SDE_GRAMMAR_HPP

using namespace boost::spirit::classic;

// Grammar for scheduled data expressions
// Warning: this grammar is known to eat little children
struct sdexp : public grammar<sdexp>
{
  static const int exprID = 1;
  static const int termID = 2;
  static const int factorID = 3;
  static const int dataID = 4;
  static const int guardID = 5;
  static const int guard_conjID = 6;
  static const int guard_litID = 7;
  static const int portnameID = 8;
  static const int portnamesID = 9;
  static const int starsID = 10;
  static const int rhsvalueID = 11;
  static const int valuesID = 12;
  static const int trueID = 13;
  static const int falseID = 14;

  template <typename ScannerT>
  struct definition
  {
    definition(sdexp const& /*self*/)
      {
	expr = term >> *(root_node_d[ch_p('|')] >> term);
	term = stars >> *(root_node_d[ch_p(';')] >> stars);
	stars = factor >> leaf_node_d[ch_p('^')] | factor >> leaf_node_d[ch_p('~')] | factor;
	factor = leaf_node_d[str_p("epsilon")] | 
	  discard_node_d[ch_p('(')] >> expr >> discard_node_d[ch_p(')')] | 
	  data;
	data = ch_p('<') >> portnames >> 
	  ch_p(',') >> guard >> ch_p('>');

	guard = guard_conj >> *(root_node_d[str_p("||")] >> guard_conj);
	guard_conj = guard_lit >> *(root_node_d[str_p("&&")] >> guard_lit);
	guard_lit = root_node_d[ch_p('!')] >> guard_lit | 
	  inner_node_d['(' >> guard >> ')'] | 
	  atomtrue | 
	  atomfalse | 
	  portname >> root_node_d[ch_p('=')] >> rhsvalue | 
	  portname >> root_node_d[ch_p('=')] >> portname |
	  portname >> root_node_d[str_p("<-")] >> values;
	portname = leaf_node_d[upper_p];
	portnames = ch_p('{') >> ch_p('}') |
	  ch_p('{') >> portname >> *(ch_p(',') >> portname) >> ch_p('}');
	values = ch_p('{') >> rhsvalue >> *(ch_p(',') >> rhsvalue) >> ch_p('}');
	rhsvalue = leaf_node_d[int_p];
	atomtrue = leaf_node_d[str_p("true")];
	atomfalse = leaf_node_d[str_p("false")];

// 	BOOST_SPIRIT_DEBUG_RULE(expr);
// 	BOOST_SPIRIT_DEBUG_RULE(term);
// 	BOOST_SPIRIT_DEBUG_RULE(stars);
// 	BOOST_SPIRIT_DEBUG_RULE(factor);
// 	BOOST_SPIRIT_DEBUG_RULE(data);
// 	BOOST_SPIRIT_DEBUG_RULE(guard);
// 	BOOST_SPIRIT_DEBUG_RULE(guard_conj);
// 	BOOST_SPIRIT_DEBUG_RULE(guard_lit);
// 	BOOST_SPIRIT_DEBUG_RULE(portname);
// 	BOOST_SPIRIT_DEBUG_RULE(portnames);

      }

    rule<ScannerT, parser_context<>, parser_tag<exprID> > expr;
    rule<ScannerT, parser_context<>, parser_tag<termID> > term;
    rule<ScannerT, parser_context<>, parser_tag<factorID> > factor;
    rule<ScannerT, parser_context<>, parser_tag<dataID> > data;
    rule<ScannerT, parser_context<>, parser_tag<guardID> > guard;
    rule<ScannerT, parser_context<>, parser_tag<guard_conjID> > guard_conj;
    rule<ScannerT, parser_context<>, parser_tag<guard_litID> > guard_lit;
    rule<ScannerT, parser_context<>, parser_tag<portnameID> > portname;
    rule<ScannerT, parser_context<>, parser_tag<portnamesID> > portnames;
    rule<ScannerT, parser_context<>, parser_tag<starsID> > stars;
    rule<ScannerT, parser_context<>, parser_tag<rhsvalueID> > rhsvalue;
    rule<ScannerT, parser_context<>, parser_tag<valuesID> > values;
    rule<ScannerT, parser_context<>, parser_tag<trueID> > atomtrue;
    rule<ScannerT, parser_context<>, parser_tag<falseID> > atomfalse;    

    rule<ScannerT, parser_context<>, parser_tag<exprID> > const&
    start() const { return expr; }
  };
};
#endif
