/***************************************************************************
 *   Copyright (C) 2008-2009 by Tobias Blechmann                           *
 *   blechman@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 @author Tobias Blechmann
*/

#ifndef AUTOMATON_H
#define AUTOMATON_H

#include "automaton_channel_end.hpp"

#include <string>
#include <iostream>
#include <boost/smart_ptr.hpp>


namespace vereofy {
  
  class Automaton {
  public:
    typedef boost::shared_ptr<Automaton> ptr;
// derived automata classes have to implement this methods    
    virtual ~Automaton() {};
    virtual void clear() = 0;
    virtual void product(const Automaton&) = 0;
    virtual Automaton::ptr copyProduct(const Automaton&) const = 0;
    virtual Automaton::ptr copyUnite(const Automaton&) const = 0;
    
//TODO: this could also be a set of Automaton_Channel_End to hide all at the same time
    virtual void hide(const Automaton_Channel_End::ptr&) = 0;
    
    virtual operator std::string() const = 0;
    virtual std::string toDotString() const = 0;
    virtual void toDot(std::ostream& out) const = 0;
    virtual double getNumberOfReachableStates() const = 0;

// all derived classes inherit these methods
    virtual void product(const Automaton::ptr);
    virtual Automaton::ptr copyProduct(const Automaton::ptr) const;
    virtual Automaton::ptr copyUnite(const Automaton::ptr) const;
  };
} //end namespace

#endif

