/***************************************************************************
 *   Copyright (C) 2008-2009 by Tobias Blechmann                           *
 *   blechman@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef AUTOMATONFACTORY_H
#define AUTOMATONFACTORY_H

#include "vereofy/setup.hpp"
#include "vereofy/common/functions.hpp"
#include "automaton_channel_end.hpp"
#include "automaton_info.hpp"
#include "automaton.hpp"

#include <string>
#include <vector>
#include <map>
#include <stdexcept>

#include <boost/utility.hpp>
#include <boost/smart_ptr.hpp>
#include <boost/any.hpp>

namespace vereofy {
  template <class DataDomainT>
  class AutomatonFactory {
  public:
    typedef Automaton::ptr (*CreateComponentCallback) (Automaton_Info&, DataDomainT);
    typedef std::map<std::string, CreateComponentCallback> ComponentCallbackMap;
    
    virtual ~AutomatonFactory() {};
    bool registerAutomaton(const std::string& automatonType, CreateComponentCallback createFn) {
      debugOut(3, factoryName + "::registerAutomaton "+ automatonType);
      bool result = componentCallbacks.insert(typename ComponentCallbackMap::value_type(automatonType, createFn)).second;
      return result;
    }

    virtual Automaton::ptr createAutomaton(const std::string& automatonType, Automaton_Info& auto_info, DataDomainT data_domain) {
      typename ComponentCallbackMap::const_iterator it = componentCallbacks.find(automatonType);
      if (it == componentCallbacks.end()) {
	throw runtime_error(factoryName + " Unknown component automatonType: " + automatonType);
      } 
    return (it->second)(auto_info, data_domain);
    }
  private:
    ComponentCallbackMap componentCallbacks;
  protected:
    std::string factoryName;
  };
  
} //end namespace

#endif

