/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "builtin_functions.hpp"
#include "vereofy/system/symb_automaton/symb_function_arithmetic.hpp"
#include "vereofy/system/symb_automaton/symb_function_seq_number_arithmetic.hpp"
#include "vereofy/system/symb_automaton/symb_function_boolean.hpp"
#include "vereofy/reo_parser/reo_parser_types.hpp"

namespace vereofy {
  void loadBuiltinFunctions(symbol_table_t& symbol_table) {
    symbol_table.add("uint_add", SymbolData::ptr(new FunctionData("uint_add", 
                                                                  SymbFunction::ptr(new SymbFunction_uint_add()))));

    symbol_table.add("uint_sub", SymbolData::ptr(new FunctionData("uint_sub", 
                                                                  SymbFunction::ptr(new SymbFunction_uint_sub()))));


    symbol_table.add("uint_mult", SymbolData::ptr(new FunctionData("uint_mult", 
                                                                   SymbFunction::ptr(new SymbFunction_uint_mult()))));

    symbol_table.add("uint_div", SymbolData::ptr(new FunctionData("uint_div", 
                                                                  SymbFunction::ptr(new SymbFunction_uint_div()))));

    symbol_table.add("uint_mod", SymbolData::ptr(new FunctionData("uint_mod", 
                                                                  SymbFunction::ptr(new SymbFunction_uint_mod()))));
    
    symbol_table.add("uint_mod_bits", SymbolData::ptr(new FunctionData("uint_mod_bits", 
                                                                       SymbFunction::ptr(new SymbFunction_uint_mod_bits()))));
    
    symbol_table.add("uint_equal", SymbolData::ptr(new FunctionData("uint_equal", 
                                                                    SymbFunction::ptr(new SymbFunction_uint_equal()))));
    
    symbol_table.add("uint_not_equal", SymbolData::ptr(new FunctionData("uint_not_equal", 
                                                                        SymbFunction::ptr(new SymbFunction_uint_not_equal()))));
    
    symbol_table.add("uint_less", SymbolData::ptr(new FunctionData("uint_less", 
                                                                   SymbFunction::ptr(new SymbFunction_uint_less()))));
    
    symbol_table.add("uint_less_equal", SymbolData::ptr(new FunctionData("uint_less_equal", 
                                                                         SymbFunction::ptr(new SymbFunction_uint_less_equal()))));
    
    symbol_table.add("uint_greater", SymbolData::ptr(new FunctionData("uint_greater", 
                                                                      SymbFunction::ptr(new SymbFunction_uint_greater()))));

    symbol_table.add("uint_greater_equal", SymbolData::ptr(new FunctionData("uint_greater_equal", 
                                                                            SymbFunction::ptr(new SymbFunction_uint_greater_equal()))));


    symbol_table.add("uint_max", SymbolData::ptr(new FunctionData("uint_max", 
                                                                  SymbFunction::ptr(new SymbFunction_uint_max()))));
    
    symbol_table.add("uint_min", SymbolData::ptr(new FunctionData("uint_min", 
                                                                  SymbFunction::ptr(new SymbFunction_uint_min()))));


    symbol_table.add("uint_pow2", SymbolData::ptr(new FunctionData("uint_pow2", 
                                                                   SymbFunction::ptr(new SymbFunction_uint_pow2()))));
    symbol_table.add("uint_bits_needed", SymbolData::ptr(new FunctionData("bits_needed", 
                                                                          SymbFunction::ptr(new SymbFunction_uint_bits_needed()))));

    symbol_table.add("uint_ones_complement", SymbolData::ptr(new FunctionData("uint_ones_complement", 
                                                                              SymbFunction::ptr(new SymbFunction_uint_ones_complement()))));
    symbol_table.add("uint_twos_complement", SymbolData::ptr(new FunctionData("uint_twos_complement", 
                                                                              SymbFunction::ptr(new SymbFunction_uint_twos_complement()))));




    symbol_table.add("bool_and", SymbolData::ptr(new FunctionData("bool_and", 
                                                                  SymbFunction::ptr(new SymbFunction_bool_and()))));
    symbol_table.add("bool_or", SymbolData::ptr(new FunctionData("bool_or", 
                                                                  SymbFunction::ptr(new SymbFunction_bool_or()))));
    symbol_table.add("bool_xor", SymbolData::ptr(new FunctionData("bool_xor", 
                                                                  SymbFunction::ptr(new SymbFunction_bool_xor()))));
    symbol_table.add("bool_implies", SymbolData::ptr(new FunctionData("bool_implies", 
                                                                  SymbFunction::ptr(new SymbFunction_bool_implies()))));
    symbol_table.add("bool_equivalent", SymbolData::ptr(new FunctionData("bool_equivalent", 
                                                                  SymbFunction::ptr(new SymbFunction_bool_equivalent()))));
    symbol_table.add("bool_not", SymbolData::ptr(new FunctionData("bool_not", 
                                                                  SymbFunction::ptr(new SymbFunction_bool_not()))));


    symbol_table.add("seq_number_add", SymbolData::ptr(new FunctionData("seq_number_add",
                                                                        SymbFunction::ptr(new SymbFunction_seq_number_add()))));
    symbol_table.add("seq_number_less", SymbolData::ptr(new FunctionData("seq_number_less", 
                                                                         SymbFunction::ptr(new SymbFunction_seq_number_less()))));
    symbol_table.add("seq_number_less_equal", SymbolData::ptr(new FunctionData("seq_number_less_equal", 
                                                                               SymbFunction::ptr(new SymbFunction_seq_number_less_equal()))));
    symbol_table.add("seq_number_greater", SymbolData::ptr(new FunctionData("seq_number_greater", 
                                                                            SymbFunction::ptr(new SymbFunction_seq_number_greater()))));
    symbol_table.add("seq_number_greater_equal", SymbolData::ptr(new FunctionData("seq_number_greater_equal", 
                                                                                  SymbFunction::ptr(new SymbFunction_seq_number_greater_equal()))));
    symbol_table.add("seq_number_max", SymbolData::ptr(new FunctionData("seq_number_max", 
                                                                        SymbFunction::ptr(new SymbFunction_seq_number_max()))));


  }
}
