/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/



#include "builtin_lib.hpp"

namespace vereofy {
namespace builtin {
  
  const char *SYNC = "                      \
MODULE SYNC {                                   \
 in: A;                                         \
 out: B;                                        \
                                                \
  -[ {A,B} & #A==#B ]->;                        \
}                                               \
";

  const char *SYNC_NONDET_DATA = "\
MODULE SYNC_NONDET_DATA {                       \
 in: A;                                         \
 out: B;                                        \
                                                \
  -[ {A,B} ]->;                                 \
}                                               \
";

  const char *SYNC_DRAIN = " \
MODULE SYNC_DRAIN {          \
 in: A;                      \
 in: B;                      \
                             \
  -[ {A,B} ]->;              \
}                            \
";

  const char *SYNC_DRAIN_EQUAL_DATA = "\
MODULE SYNC_DRAIN_EQUAL_DATA {         \
 in: A;                                \
 in: B;                                \
                                       \
  -[ {A,B} & #A==#B]->;                \
}                                      \
";

  const char *ASYNC_DRAIN = " \
MODULE ASYNC_DRAIN {         \
 in: A;                      \
 in: B;                      \
                             \
  -[ {A} ]->;                \
  -[ {B} ]->;                \
}                            \
";


  const char *SYNC_SPOUT = " \
MODULE SYNC_SPOUT {          \
 out: A;                     \
 out: B;                     \
                             \
  -[ {A,B} ]->;              \
}                            \
";

  const char *ASYNC_SPOUT = " \
MODULE ASYNC_SPOUT {         \
 out: A;                     \
 out: B;                     \
                             \
  -[ {A} ]->;                \
  -[ {B} ]->;                \
}                            \
";


  // ----------------- Synchronous filters ------------------------------
  const char *FILTER_EQUAL = " \
MODULE FILTER_EQUAL<d> {       \
 in : A;                       \
 out : B;                      \
                                                \
  -[ {A,B} & #A==#B & #A == d ]->;              \
  -[ {A} & #A != d ]->;                          \
}                                               \
";

  const char *FILTER_NOT_EQUAL = " \
MODULE FILTER_NOT_EQUAL<d> {       \
 in : A;                           \
 out : B;                          \
                                   \
  -[ {A,B} & #A==#B & #A != d ]->; \
  -[ {A} & #A == d ]->;            \
}                                  \
";
  
  const char *FILTER_LESS_EQUAL = "  \
MODULE FILTER_LESS_EQUAL<d> {                   \
 in : A;                                        \
 out : B;                                       \
                                                \
  -[ {A,B} & #A==#B & #A <= d ]->;              \
  -[ {A} & #A > d ]->;                          \
}                                               \
";

  const char *FILTER_LESS = " \
MODULE FILTER_LESS<d> {                         \
 in : A;                                        \
 out : B;                                       \
                                                \
  -[ {A,B} & #A==#B & #A < d ]->;               \
  -[ {A} & #A >= d ]->;                         \
}                                               \
";
  
  const char *FILTER_GREATER_EQUAL = " \
MODULE FILTER_GREATER_EQUAL<d> {          \
 in : A;                                  \
 out : B;                                 \
                                          \
  -[ {A,B} & #A==#B & #A >= d ]->;        \
  -[ {A} & #A < d ]->;                    \
}                                         \
";
  
  const char *FILTER_GREATER = " \
MODULE FILTER_GREATER<d> {                      \
 in : A;                                        \
 out : B;                                       \
                                                \
  -[ {A,B} & #A==#B & #A > d ]->;               \
  -[ {A} & #A <= d ]->;                         \
}                                               \
";

  // ------ FIFOs ---------------------------

  const char* FIFO1 = "\
MODULE FIFO1 {                                  \
 in: A;                                         \
 out: B;                                        \
                                                \
 var : enum {EMPTY, FULL} state := EMPTY;       \
 var : Data value;                              \
                                                                   \
 state==EMPTY  -[  {A}             ]-> state:=FULL & value:=#A;	   \
 state==FULL   -[  {B} & #B==value ]-> state:=EMPTY & value:=ANY;  \
}                                                                  \
";
    
  const char* FIFO1_FULL = "                                            \
    MODULE FIFO1_FULL<initial_value> {                                  \
    in: A;                                                              \
    out: B;                                                             \
                                                                        \
    var : enum {EMPTY, FULL} state := FULL;                             \
    var : Data value := initial_value;                                  \
                                                                        \
    state==EMPTY  -[  {A}             ]-> state:=FULL & value:=#A;      \
    state==FULL   -[  {B} & #B==value ]-> state:=EMPTY & value:=ANY;    \
    }                                                                   \
    ";


  const char* FIFO1_FULL_NONDET_VALUE = "                               \
    MODULE FIFO1_FULL_NONDET_VALUE {                                    \
    in: A;                                                              \
    out: B;                                                             \
                                                                        \
    var : enum {EMPTY, FULL} state := FULL;                             \
    var : Data value;                                                   \
                                                                        \
    state==EMPTY  -[  {A}             ]-> state:=FULL & value:=#A;      \
    state==FULL   -[  {B} & #B==value ]-> state:=EMPTY & value:=ANY;    \
    }                                                                   \
    ";


  const char* FIFO1_NONDET_DATA = "\
MODULE FIFO1_NONDET_DATA {                      \
 in: A;                                         \
 out: B;                                        \
                                                \
 var : enum {EMPTY, FULL} state := EMPTY;       \
                                                \
 state==EMPTY  -[  {A} ]-> state:=FULL;         \
 state==FULL   -[  {B} ]-> state:=EMPTY;        \
}                                               \
";



  const char* FIFO1_FULL_NONDET_DATA = "\
MODULE FIFO1_FULL_NONDET_DATA {                 \
 in: A;                                         \
 out: B;                                        \
                                                \
 var : enum {EMPTY, FULL} state := FULL;        \
                                                \
 state==EMPTY  -[  {A} ]-> state:=FULL;         \
 state==FULL   -[  {B} ]-> state:=EMPTY;        \
}                                               \
";

  const char* LOSSY_SYNC_ND = "\
MODULE LOSSY_SYNC_ND {            \
  in: A;                          \
  out: B;                         \
                                  \
  -[ {A,B} & #A==#B ]->;          \
  -[ {A} ]->;                     \
}                                 \
";

  const char* READER_INF = " \
MODULE READER_INF {          \
  in: A;                     \
                             \
  -[ {A} ]->;                \
}                            \
";
  
  const char* WRITER_INF = "                      \
MODULE WRITER_INF<value> {                        \
  out: A;                                         \
                                                  \
  -[ {A} & #A==value ]->;                         \
}                                                 \
";

  const char* WRITER_INF_NONDET_VALUE = "\
MODULE WRITER_INF_NONDET_VALUE {         \
  out: A;                                \
                                         \
  -[ {A} ]->;                            \
}                                        \
";

  const char* WRITER = "                      \
MODULE WRITER<k,value> {                      \
  out: A;                                     \
                                              \
  var : int 0..k count := 0;                  \
                                              \
  count!=k -[ {A} & #A==value ]-> count:=count+1; \
}                                             \
";

  const char* WRITER_NONDET_VALUE = "         \
MODULE WRITER_NONDET_VALUE<k> {               \
  out: A;                                     \
                                              \
  var : int 0..k count := 0;                  \
                                              \
  count!=k -[ {A} ]-> count:=count+1;         \
}                                             \
";

  const char* READER = "              \
MODULE READER<k> {                    \
  in: A;                              \
                                      \
  var : int 0..k count := 0;          \
                                      \
  count!=k -[ {A} ]-> count:=count+1; \
}                                     \
";

  const char* SEQUENCER = "\
    CIRCUIT SEQUENCER<k>{                       \
      F[0] = new FIFO1_FULL<0>(N[0];N[1]);      \
                                                \
      for (i=1;i<k;i=i+1){                      \
        F[i] = new FIFO1(N[i];N[i+1]);          \
      }                                         \
                                                \
      new SYNC(N[k]; N[0]);                     \
                                                \
     for (i=0;i<k;i=i+1) {                      \
       new SYNC(N[i+1]; sink[i]);               \
     }                                          \
   }                                            \
  ";

  const char* SEQUENCING_READER = "\
    CIRCUIT SEQUENCING_READER<k>{               \
      F[0]    = new FIFO1_FULL<0>(A[0];B[0]);   \
      SD[0]   = new SYNC_DRAIN(C[0],D[0]);      \
      Node[0] = join(B[0],D[0]);                \
                                                \
      for (i=1;i<k;i=i+1){                      \
        F[i]      = new FIFO1(A[i];B[i]);       \
        SD[i]     = new SYNC_DRAIN(C[i],D[i]);  \
        Node[i]   = join(B[i],D[i]);            \
        Node[i-1] = join(Node[i-1],A[i]);       \
      }                                         \
                                                \
      Node[k-1] = join(Node[k-1],A[0]);         \
                                                \
                                                \
      for (i=0;i<k;i=i+1) {                     \
        source[i] = C[i];                       \
      }                                         \
    }                                           \
    ";

  
  const char* EXROUTER = "\
CIRCUIT EXROUTER<k> {                           \
  if(k > 1) {                                   \
    drain = new SYNC_DRAIN;                     \
    source[0] = drain.source[0];                \
                                                \
    for(i = 0; i < k; i = i+1) {                \
      lossy[i] = new LOSSY_SYNC_ND;             \
      drainSync[i] = new SYNC;                  \
      sinkSync[i] = new SYNC;                   \
                                                \
      source[0]=join(source[0], lossy[i].source);  \
      join(lossy[i].sink, drainSync[i].source); \
      join(lossy[i].sink, sinkSync[i].source);  \
      join(drainSync[i].sink, drain.source[1]); \
                                                \
      sink[i] = sinkSync[i].sink;               \
    }                                           \
  } else {                                      \
    if(k == 1) {                                \
      sync = new SYNC;                          \
      source = sync.source;                     \
      sink = sync.sink;                         \
    }                                           \
  }                                             \
}                                               \
";

  const char* FIFO_k="\
CIRCUIT FIFO<k> {                               \
  if (k==0) {                                   \
    f[0]=new SYNC(source;sink);                 \
  }                                             \
                                                \
  if (k==1){                                    \
    f[0]=new FIFO1(source;sink);                \
  }                                             \
                                                \
  if (k>1){                                     \
    for (i=0;i<k;i=i+1){                        \
      f[i]=new FIFO1(A[i];A[i+1]);              \
    }                                           \
    source = A[0];                              \
    sink   = A[k];                              \
  }                                             \
}                                               \
";


  // ----------------- Generics ---------------------------------
  const char *GENERIC_NONDET = "\
CIRCUIT GENERIC_NONDET<in_count, out_count> {           \
  for (i=0;i<in_count;i=i+1) {                          \
    r[i] = new READER_INF(source[i]);                   \
  }                                                     \
  for (i=0;i<out_count;i=i+1) {                         \
    w[i] = new WRITER_INF_NONDET_VALUE(;sink[i]);       \
  }                                                     \
}                                                       \
";

  const char *GENERIC_SYNC = "\
CIRCUIT GENERIC_SYNC<in_count, out_count> {             \
  N=NODE;                                               \
  w=new WRITER_INF_NONDET_VALUE(;N);                    \
                                                        \
  for (i=0;i<in_count;i=i+1) {                          \
     d[i]=new SYNC_DRAIN(source[i], N);                 \
  }                                                     \
                                                        \
  for (i=0;i<out_count;i=i+1) {                         \
    s[i]=new SYNC_NONDET_DATA(N; sink[i]);              \
  }                                                     \
}                                                       \
";

  const char *GENERIC_SYNC_DATA_EQUAL = "\
CIRCUIT GENERIC_SYNC_DATA_EQUAL<in_count, out_count> {  \
  N=NODE;                                               \
  w=new WRITER_INF_NONDET_VALUE(;N);                    \
                                                        \
  for (i=0;i<in_count;i=i+1) {                          \
     d[i]=new SYNC_DRAIN_EQUAL_DATA(source[i], N);      \
  }                                                     \
                                                        \
  for (i=0;i<out_count;i=i+1) {                         \
    s[i]=new SYNC(N; sink[i]);                          \
  }                                                     \
}                                                       \
";


  const char *GENERIC_ASYNC = "\
CIRCUIT GENERIC_ASYNC<in_count, out_count> {            \
  N=ROUTE_NODE;                                         \
  w=new WRITER_INF_NONDET_VALUE(;N);                    \
                                                        \
  for (i=0;i<in_count;i=i+1) {                          \
    s_in[i]=new SYNC_DRAIN(source[i], N);               \
  }                                                     \
                                                        \
  for (i=0;i<out_count;i=i+1) {                         \
    s_out[i]=new SYNC(N; sink[i]);                      \
  }                                                     \
}                                                       \
";

}


  std::string getBuiltinLib() {
    return 
    std::string(builtin::SYNC)+
      builtin::SYNC_NONDET_DATA+
      builtin::SYNC_DRAIN+
      builtin::SYNC_DRAIN_EQUAL_DATA+
      builtin::ASYNC_DRAIN+
      builtin::SYNC_SPOUT+
      builtin::ASYNC_SPOUT+
      builtin::FILTER_EQUAL+
      builtin::FILTER_NOT_EQUAL+
      builtin::FILTER_LESS+
      builtin::FILTER_LESS_EQUAL+
      builtin::FILTER_GREATER+
      builtin::FILTER_GREATER_EQUAL+
      builtin::FIFO1+
      builtin::FIFO1_FULL+
      builtin::FIFO1_FULL_NONDET_VALUE+
      builtin::FIFO1_NONDET_DATA+
      builtin::FIFO1_FULL_NONDET_DATA+
      builtin::LOSSY_SYNC_ND+
      builtin::WRITER_INF+
      builtin::WRITER_INF_NONDET_VALUE+
      builtin::READER_INF+
      builtin::READER+
      builtin::WRITER+
      builtin::WRITER_NONDET_VALUE+
      
      builtin::EXROUTER+
      builtin::SEQUENCER+
      builtin::SEQUENCING_READER+
      builtin::FIFO_k+

      builtin::GENERIC_NONDET+
      builtin::GENERIC_SYNC+
      builtin::GENERIC_SYNC_DATA_EQUAL+
      builtin::GENERIC_ASYNC;
  }

}
