/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "builtin_mapper.hpp"

#include "vereofy/common/xml_helpers.hpp"
#include <boost/algorithm/string.hpp>

using namespace std;

namespace vereofy {

  BuiltinMapper::BuiltinMapper() {
    init();
  }

  void BuiltinMapper::init() {
    simple_channels["SYNC"]="channels:Sync";
    simple_channels["SYNC_DRAIN"]="channels:SyncDrain";
    simple_channels["ASYNC_DRAIN"]="channels:AsyncDrain";
    simple_channels["SYNC_SPOUT"]="channels:SyncSpout";
    simple_channels["ASYNC_SPOUT"]="channels:AsyncSpout";
    simple_channels["LOSSY_SYNC_ND"]="channels:LossySync";
  }

  BuiltinMapper::mapped::ptr BuiltinMapper::mapToReoXML(std::ostream& out, REO_Component::ptr component) const {
    string type_name=component->getComponentInstance()->getTypeName();
    
    if (simple_channels.count(type_name)) {
      xml_tag_open(out, "primitives",
                   xml_parameter("xmi:type", simple_channels.find(type_name)->second)+
                   xml_parameter("xmi:id", component.get()));

      return mapped::ptr(new mapped("primitives", "channel"));
    }
    
    if (boost::algorithm::starts_with(type_name, "FIFO1") ||
        type_name=="FIFO<1>") {
      string full_flag = 
        (boost::algorithm::starts_with(type_name, "FIFO1_FULL")
         || type_name=="FIFO1_FULL_NONDET_VALUE") 
        ? "true" : "false";

      string cell_name=component->getComponentInstance()->getName();
      if (boost::algorithm::starts_with(cell_name, "$")) {
        cell_name="";
      }      

      xml_tag_open(out, "primitives",
                   xml_parameter("xmi:type", "channels:FIFO")+
                   xml_parameter("full", full_flag)+
                   xml_parameter("cellName", cell_name)+
                   xml_parameter("xmi:id", component.get()));

      return mapped::ptr(new mapped("primitives", "channel"));
    }


    
    /*   builtin::FILTER_EQUAL+
         builtin::FILTER_NOT_EQUAL+
         builtin::FILTER_LESS+
         builtin::FILTER_LESS_EQUAL+
         builtin::FILTER_GREATER+
         builtin::FILTER_GREATER_EQUAL+
         builtin::WRITER_INF+
         builtin::WRITER_INF_NONDET_VALUE+
         builtin::READER_INF+
         builtin::READER+
         builtin::WRITER+
         builtin::WRITER_NONDET_VALUE+
         
         builtin::EXROUTER+
         builtin::SEQUENCER+
         builtin::SEQUENCING_READER+
         builtin::FIFO_k;*/
    
    // no mapping
    return mapped::ptr();
  }


}
