/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef CIRCUIT_LEVEL_HPP
#define CIRCUIT_LEVEL_HPP

#include "component_instance_circuit.hpp"

namespace vereofy {

  class CircuitLevel {
  public:
    typedef boost::shared_ptr<CircuitLevel> ptr;
    
    typedef execution_environment_t::value_table_ptr value_table_ptr;
    
    CircuitLevel::ptr clone(unsigned int id) const;
    void resetValueTable(const value_table_ptr& vt);

    CircuitLevel(CircuitConnector& connector, value_table_ptr vt);

    REO_Circuit::ptr getREO_Circuit();
    value_table_ptr getValueTable();

    void execute(const vparser::pt_node& statement_block,
                 symbol_table_ptr local_symbols);
    void set_parent(ComponentInstance *parent);
    void build_name_map();
    void plug(bool plug);
    void build_interface(ComponentInterface::ptr c_interface, bool strict);
    void build_interface_for(ComponentInterface::ptr c_interface, 
                             REO_Graph_Object::ptr rgo,
                             bool strict);

    void toDot(ostream& out,
               bool is_subgraph=false);
    void toReoXML(ostream& out, const std::string& name, const std::vector<std::string>& input_files, bool is_subgraph=false);


    bool is_anonymous(REO_Graph_Object::ptr);
    bool is_exported_sink(REO_Graph_Object::ptr);
    bool is_exported_source(REO_Graph_Object::ptr);


    void export_circuit(ostream& stream) const;

    void calculatePortPartitions(PortPartitions& port_partitions);

  private:
    unsigned int id;

    bool was_cloned;
    REO_Graph_Object::rgo_map mapping;
    
    typedef shared_ptr_set<REO_Graph_Object::ptr>::type exported_set;
    typedef std::vector<REO_Graph_Object::ptr> exported_rgos;

    CircuitConnector& connector;

    REO_Circuit::ptr reo_circuit;
    value_table_ptr values;

    REO_Circuit::reo_to_names_map reo_to_names;
    exported_set exported_sources;
    exported_set exported_sinks;

    exported_rgos sources;
    exported_rgos sinks;
    

    unsigned int anonymous_counter;

    ComponentInstance* parent;

    value_table_ptr clone_and_map_value_table(const value_table_ptr& vt,
                                              const REO_Graph_Object::rgo_map& clone_mapping) const;
  };
}

#endif
