/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef COMPONENT_INSTANCE_CIRCUIT_HPP
#define COMPONENT_INSTANCE_CIRCUIT_HPP

#include "vereofy/common/shared_ptr_map.hpp"
#include "vereofy/system/component_instance.hpp"
#include "vereofy/system/circuit_connector/circuit_connector.hpp"
#include "vereofy/reo_parser/reo_parser_types.hpp"

#include "vereofy/system/reo_circuit.hpp"
#include "vereofy/system/symb_automaton/var_positions.hpp"

namespace vereofy {

// forward
class CircuitLevel;

class ComponentInstanceCircuit : public ComponentInstance {
public:
  typedef boost::shared_ptr<ComponentInstanceCircuit> ptr;
  typedef boost::shared_ptr<CircuitLevel> CircuitLevel_ptr;

  ComponentInstanceCircuit(CircuitConnector::ptr circuit_connector,
                           const std::vector<Value::ptr>& parameters=std::vector<Value::ptr>(),
                           bool strict=true);
  
  virtual boost::shared_ptr<SymbComponentInstance> getSymbAutomaton(ComponentInstance::ptr self, Symb_DataDomain::ptr s_dd);

  CircuitConnector::ptr getCircuitConnector();
  REO_Circuit::ptr getREO_Circuit();

  void toDot(ostream& out);
  void toReoXML(ostream& out, const std::vector<std::string>& input_files);
  void export_circuit(ostream& stream) const;

  CircuitLevel_ptr getBaseLevel();
  unsigned int countConfigurations() const;
  CircuitLevel_ptr getConfiguration(unsigned int index);

  StateDescription::ptr getConfigState() const;
  PortDescription::ptr getConfigPort() const;

  virtual bool hasInitialConfiguration() const;
  virtual unsigned int getInitialConfiguration() const;
  
  virtual bool hasReconfigurationPort() const;
  virtual unsigned int getReconfigurationPortSourceIndex() const;

  virtual void calculatePortPartitions(PortPartitions& port_partitions);

private:
  CircuitLevel_ptr base;
  vector<CircuitLevel_ptr> configurations;
  StateDescription::ptr config_state;
  PortDescription::ptr config_port;

  execution_environment_t::value_table_ptr values;
  CircuitConnector::ptr circuit_connector;
  std::vector<Value::ptr> parameters;


  void execute();
  void build_name_map();
  void plug(bool strict);  
  void build_interface(bool strict);
};


}

 
#endif
