/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/




#include "symb_configurator.hpp"
#include "vereofy/common/bdd_helpers.hpp"

using namespace vereofy;

SymbConfigurator::SymbConfigurator(Symb_DataDomain::ptr dataDomain,
                                   PortDescription::ptr config_port_,
                                   StateDescription::ptr config_state_,
                                   Symb_Channel_End::ptr config_port_ce) :
  SymbComponentInstance(ComponentInterface::ptr(new ComponentInterface("$configurator")), dataDomain),
  config_port(config_port_), config_state(config_state_) {

  if (config_port_ce) {
    addPort(config_port, config_port_ce);
  } else {
    addPort(config_port);
  }
  addState(config_state);
  
  state=getState(config_state);
  ce_port=get_symb_ce_for_port(config_port);
  
  BDDFunction no_config_state_change=pairwiseEQUIV(*state->vars.X, 
                                                   *state->vars.Y);
    
  BDDFunction config_change_depends_on_config_port=
    ce_port->getPortActive() & pairwiseEQUIV_with_fill(*state->vars.Y, ce_port->getDCVars());
  //  | (!ce_port->getPortActive() & no_config_state_change);
  
  delta=config_change_depends_on_config_port;
}

Symb_Channel_End::ptr SymbConfigurator::getConfigPort() const {
  return ce_port;
}

SymbolicState::ptr SymbConfigurator::getConfigState() const {
  return state;
}
