/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   klein@tcs.inf.tu-dresden.de                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
@author Joachim Klein
*/

#ifndef SYMB_CONFIGURATOR_H
#define SYMB_CONFIGURATOR_H

#include <string>
#include <vector>
#include <set>
#include <iostream>
#include <boost/smart_ptr.hpp>

#include "vereofy/system/symb_automaton/symb_automaton.hpp"
#include "vereofy/system/symb_automaton/symb_component_instance.hpp"
//#include "var_positions.hpp"


namespace vereofy {
  class SymbConfigurator : public SymbComponentInstance {
  public:
    typedef boost::shared_ptr<SymbConfigurator> ptr;

    SymbConfigurator(Symb_DataDomain::ptr dataDomain,
                     PortDescription::ptr config_port,
                     StateDescription::ptr config_state,
                     Symb_Channel_End::ptr config_port_ce=Symb_Channel_End::ptr());

    Symb_Channel_End::ptr getConfigPort() const;
    SymbolicState::ptr getConfigState() const;

    virtual void build_automaton(BuildContext::ptr build_context) {throw "Configurator is special, build_automaton not implemented";};
  private:
    PortDescription::ptr config_port;
    StateDescription::ptr config_state;

    Symb_Channel_End::ptr ce_port;
    SymbolicState::ptr state;
  };
  
} //end namespace

#endif
