/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef COMPONENT_INSTANCE_HPP
#define COMPONENT_INSTANCE_HPP

#include "vereofy/system/types.hpp"
#include "vereofy/reo_parser/reo_parser_types.hpp"
#include "vereofy/system/system_context.hpp"
#include "vereofy/system/symb_automaton/symb_automaton.hpp"
#include "vereofy/system/symb_automaton/symb_maps.hpp"
#include "vereofy/system/component_interface.hpp"
#include "vereofy/system/port_equivalence_relation.hpp"
#include "vereofy/system/port_partitions.hpp"
#include "vereofy/system/port_id.hpp"
#include "vereofy/system/state_id.hpp"
#include "vereofy/system/symb_automaton/var_positions.hpp"
#include "vereofy/common/vereofy_id.hpp"

#include <boost/shared_ptr.hpp>
#include <boost/enable_shared_from_this.hpp>

namespace vereofy {
// fwd
class SymbComponentInstance;

class ComponentInstance : public Data, public VereofyID, public boost::enable_shared_from_this<ComponentInstance> {
public:
  typedef boost::shared_ptr<ComponentInstance> ptr;
  typedef boost::weak_ptr<ComponentInstance> wptr;

  ComponentInstance(component_type_t component_type_,
		    SystemContext::wptr system_context,
		    ComponentInterface::ptr c_interface_=ComponentInterface::ptr());
  virtual ~ComponentInstance() {}

  component_type_t getType() const;
  ComponentInterface::ptr getInterface() const;
  SystemContext::wptr getSystemContext() const;

  void print_interface_xml(std::ostream& out) const;

  virtual boost::shared_ptr<SymbComponentInstance> getSymbAutomaton(ComponentInstance::ptr self,
								    Symb_DataDomain::ptr s_dd) = 0;

  class lookup_failure_exception : public std::runtime_error {
  public:
    lookup_failure_exception(const std::string& message) : std::runtime_error(message) {}
  };

  PortID  lookupPort(const std::string& fq_port_name, bool exact=true) const;
  StateID lookupState(const std::string& fq_state_name) const;
  ComponentInstance::ptr lookupComponent(const std::string& fq_component_name) const;

  void setName(const string& name);
  const string& getName() const;
  
  // get the fully qualified name (including parents' names)
  std::string getFQName() const;   
  
  void setTypeName(const string& name);
  const string& getTypeName() const;

  void setParent(ComponentInstance* parent);
  ComponentInstance* getParent() const;

  typedef std::map<std::string, std::string> with_params_map;
  void addWithParameter(const std::string& key, const std::string& value);
  std::string getWithParam(const std::string& key) const;

  virtual bool hasReconfigurationPort() const;
  virtual unsigned int getReconfigurationPortSourceIndex() const;


  virtual void calculatePortPartitions(PortPartitions& port_partitions) = 0;

  virtual void attachAtomicPropositions(const AtomicPropositionSpec::vector& atomic_propositions);
  virtual void attachAtomicPropositionsForType();

  void registerSubcomponent(ComponentInstance::ptr ci);
  ComponentInstance::ptr getSubcomponent(const std::string& name) const;
protected:
  void setInterface(ComponentInterface::ptr c_interface_);
  string getParametersAsString(const std::vector<Value::ptr>& parameters) const;

private:
  component_type_t component_type;
  SystemContext::wptr system_context;
  ComponentInterface::ptr c_interface;
  string name;
  string type_name;

  with_params_map with_params;

  typedef std::map<std::string, ComponentInstance::ptr> subcomponent_map_t;
  subcomponent_map_t subcomponents;

  // backward pointer to parent
  ComponentInstance* parent;
};

}

 
#endif
