/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "component_type_identifier.hpp"
#include <boost/algorithm/string.hpp>

using namespace vereofy;
using namespace std;

ComponentTypeIdentifier::ComponentTypeIdentifier(const std::string& type_name_,
                                                 const std::vector<Value::ptr>& parameters_) :
  type_name(type_name_), parameters(parameters_) {

  size_t param_start=type_name.find_first_of('<');
  if (param_start!=string::npos) {
    throw type_name+" has <> parameters, but parameters where specified separately";
  }
}


ComponentTypeIdentifier::ComponentTypeIdentifier(const std::string& type_name_,
                                                 const std::vector<std::string>& parameters_) :
  type_name(type_name_) {
  parameter_strings_to_value_vector(parameters_);

  size_t param_start=type_name.find_first_of('<');
  if (param_start!=string::npos) {
    throw type_name+" has <> parameters, but parameters where specified separately";
  }
}

ComponentTypeIdentifier::ComponentTypeIdentifier(const std::string& name_) {
  string name=name_;
  boost::trim(name);
  
  size_t param_start=name.find_first_of('<');
  if (param_start==string::npos) {
    // has no parameters
    type_name=name;
    return;
  }

  // has parameters
  if (name.at(name.length()-1)!='>') {
    throw "Component type name has < but not closing >";
  }
  
  string param_string=name.substr(param_start);
  CHECK_SUCCESS(param_string.length()>=2);
  param_string=param_string.substr(1, param_string.length()-2);
  
  if (param_string.find_first_of('<')!=string::npos) {
    throw "Component type name has multiple <";
  }

  if (param_string.find_first_of('>')!=string::npos) {
    throw "Component type name has multiple >";
  }
  
  type_name=name.substr(0, param_start);
  
  vector<string> params;
  split(params, param_string, boost::is_any_of(","));

  if (params.size()>0) {
    parameter_strings_to_value_vector(params);
  }
}

std::string ComponentTypeIdentifier::toString() const {
  if (parameters.size()==0) {
    return type_name;
  } else {
    stringstream out;
    out << type_name;
    out << "<";
    
    for (unsigned int i=0;i<parameters.size();i++) {
      if (i>0) {
        out << ",";
      }
      Value::ptr v=parameters.at(i);
      
      if (!v) {
        throw "NULL value in parameters!";
      }
      
      switch (v->type->type) {
      case VALUE_TYPE_INT:
        out << IntValue::getIntValue(v);
        break;
      case VALUE_TYPE_BOOL: 
        THROW_UNIMPLEMENTED();
      default:
        THROW_UNIMPLEMENTED();
      }
    }
    out << ">";

    return out.str();
  }
}

void ComponentTypeIdentifier::parameter_strings_to_value_vector(const std::vector<std::string>& parameter_strings) {
  for (unsigned int i=0;i<parameter_strings.size();i++) {
    string p=parameter_strings.at(i);
    trim(p);
    
    try {
      parameters.push_back(IntValue::ptr(new IntValue(boost::lexical_cast<int>(p))));
    } catch (boost::bad_lexical_cast&) {
      throw "The parameter with index "+boost::lexical_cast<string>(i)+", '"+p+"' is not a valid parameter";
    }
  }
}

