/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "port_equivalence_relation.hpp"
#include "component_instance.hpp"

using namespace vereofy;

void PortEquivalenceRelation::add(const PortID& pd) {
  port_partition.make_set(pd);
}


void PortEquivalenceRelation::make_equivalent(const PortID& pd1, const PortID& pd2) {
  port_partition.union_set(pd1, pd2);
}

bool PortEquivalenceRelation::are_equivalent(const PortID& pd1, const PortID& pd2) {
  return (port_partition.find_set(pd1) == port_partition.find_set(pd2));
}

PortID PortEquivalenceRelation::getRepresentative(const PortID& pd) {
  return port_partition.find_set(pd);
}

void PortEquivalenceRelation::print(std::ostream& out) {
  for (std::map<PortID,PortID>::const_iterator it=port_partition.parent_map_begin();
       it!=port_partition.parent_map_end();
       ++it) {
    const PortID& id=it->first;

    out << id << " -> " << getRepresentative(id) << "\n";
  }
}
