/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef PORT_ID_HPP
#define PORT_ID_HPP

#include <boost/shared_ptr.hpp>

namespace vereofy {
  
  // fwd
  class ComponentInstance;
  class PortDescription;

  class PortID {
  public:
    PortID() :
      component_instance(0), port_description(0) {}

    PortID(boost::shared_ptr<ComponentInstance> component_instance_,
           boost::shared_ptr<PortDescription> port_description_) : 
      component_instance(component_instance_.get()),
      port_description(port_description_.get()) {}

    PortID(ComponentInstance* component_instance_,
           boost::shared_ptr<PortDescription> port_description_) : 
      component_instance(component_instance_),
      port_description(port_description_.get()) {}

    bool operator<(const PortID& other) const;
    bool operator==(const PortID& other) const;
    bool operator!=(const PortID& other) const {return !(*this==other);}

    ComponentInstance& getComponentInstance() const {return *component_instance;}
    PortDescription& getPortDescription() const {return *port_description;}

    void print(std::ostream& out) const;

  private:
    ComponentInstance* component_instance;
    PortDescription* port_description;
  };

  inline std::ostream& operator<<(std::ostream& out, const PortID& id) {
    id.print(out);
    return out;
  }

}

#endif

