/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef PORT_PARTITIONS_HPP
#define PORT_PARTITIONS_HPP

#include "port_equivalence_relation.hpp"
#include "port_symmetric_relation.hpp"
#include <boost/shared_ptr.hpp>

namespace vereofy {

  struct PortPartitions {
    typedef boost::shared_ptr<PortPartitions> ptr;
    
    // identical_ports groups those PortIDs that behave absolutely identical,
    // i.e. are either all active or not and have the same data value
    PortEquivalenceRelation identical_ports;

    // mixed_data_ports groups those ports that are not identical but where
    // the data values may be related when more than one of the grouped ports
    // is synchronously active
    PortEquivalenceRelation mixed_data_ports;

    // if_active_same_data_ports is a symmetric, areflexive relation
    // relating ports that, if active at the same time, are guaranteed to have
    // the same data value
    PortSymmetricRelation if_active_same_data_ports;
  };


}

#endif
