/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "port_symmetric_relation.hpp"
#include "component_instance.hpp"

using namespace vereofy;
using namespace std;

void PortSymmetricRelation::make_related(const PortID& pd1, const PortID& pd2) {
  relation[pd1].insert(pd2);
  relation[pd2].insert(pd1);
}

void PortSymmetricRelation::make_all_related(const std::set<PortID>& port_ids) {
  for (std::set<PortID>::const_iterator it_1=port_ids.begin();
       it_1!=port_ids.end();
       ++it_1) {
    for (std::set<PortID>::const_iterator it_2=port_ids.begin();
         it_2!=port_ids.end();
         ++it_2) {
      if (*it_1==*it_2) {continue;}

      make_related(*it_1, *it_2);      
    }
  }
}


bool PortSymmetricRelation::are_related(const PortID& pd1, const PortID& pd2) const {
  relation_map_t::const_iterator it=relation.find(pd1);
  if (it==relation.end()) return false;
  
  return (it->second.count(pd2) > 0);
}


const std::set<PortID>& PortSymmetricRelation::get_related(const PortID& port_id) const {
  relation_map_t::const_iterator it=relation.find(port_id);
  if (it==relation.end()) return empty_set;
  
  return it->second;
}

PortSymmetricRelation PortSymmetricRelation::lifted(PortEquivalenceRelation& equiv) const {
  PortSymmetricRelation result;

  for (relation_map_t::const_iterator it=relation.begin();
       it!=relation.end();
       ++it) {
    const PortID& id_1=it->first;
    PortID id_1_representative=equiv.getRepresentative(id_1);

    for (set<PortID>::const_iterator it_2=it->second.begin();
         it_2!=it->second.end();
         ++it_2) {
      const PortID& id_2=*it_2;
      PortID id_2_representative=equiv.getRepresentative(id_2);

      // avoid reflexivity
      if (id_1_representative==id_2_representative) {continue;}
      
      result.make_related(id_1_representative, id_2_representative);
    }
  }  

  return result;
}


void PortSymmetricRelation::print(std::ostream& out) const {
  for (relation_map_t::const_iterator it=relation.begin();
       it!=relation.end();
       ++it) {
    const PortID& id=it->first;

    out << id << " : ";
    for (set<PortID>::const_iterator it_2=it->second.begin();
         it_2!=it->second.end();
         ++it_2) {
      if (it_2!=it->second.begin()) {out << ", ";}
        
      out << *it_2;
    }
    out << "\n";
  }
}

