/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef PORT_SYMMETRIC_RELATION_HPP
#define PORT_SYMMETRIC_RELATION_HPP

#include "port_description.hpp"
#include "port_id.hpp"

#include <iostream>

#include "port_equivalence_relation.hpp"
#include <boost/shared_ptr.hpp>
#include <set>
#include <map>

namespace vereofy {

  /** A symmetric, areflexive relation over PortIDs */

  class PortSymmetricRelation {
  public:
    typedef boost::shared_ptr<PortSymmetricRelation> ptr;

    void make_related(const PortID& port_id_1, const PortID& port_id_2);
    void make_all_related(const std::set<PortID>& port_ids);
    bool are_related(const PortID& port_id_1, const PortID& port_id_2) const;

    const std::set<PortID>& get_related(const PortID& port_id) const;

    PortSymmetricRelation lifted(PortEquivalenceRelation& equiv) const;

    void print(std::ostream& out) const;
  private:
    typedef std::map<PortID, std::set<PortID> > relation_map_t;
    
    relation_map_t relation;

    std::set<PortID> empty_set;
  };
}


#endif
