/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_CA_REACTIVE_MODULE_HPP
#define SYMB_CA_REACTIVE_MODULE_HPP

#include "vereofy/common/shared_ptr_map.hpp"
#include "vereofy/setup.hpp"
#include "vereofy/system/component_instance.hpp"
#include "vereofy/system/reactive_module/component_instance_rm.hpp"
#include "vereofy/system/symb_automaton/symb_component_instance.hpp"
#include "reactive_module.hpp"

namespace vereofy {
  class Symb_CA_ReactiveModule : public SymbComponentInstance {
  public:
    typedef boost::shared_ptr<Symb_CA_ReactiveModule> ptr;

    ComponentInstanceReactiveModule::ptr ci_module;

    Symb_CA_ReactiveModule(ComponentInstanceReactiveModule::ptr ci_module_,
                           Symb_DataDomain::ptr dataDomain);

    virtual void build_automaton(BuildContext::ptr build_context);

  private:
    void perform_assignments(const vector<bdd_assignment_vector>& assignments, 
                             BDDFunction assignment,
                             BDDFunction csTargets,
                             unsigned int index,
                             vector<BDDFunction>& result) const;

    void handle_initialization(const vparser::pt_node& body, BuildContext::ptr build_context);
    void handle_atomic_proposition_defs(const vparser::pt_node& body, BuildContext::ptr build_context);
    void handle_transitions(const vparser::pt_node& body, BuildContext::ptr build_context);

    BDDFunction get_no_generic_labels() const;

    const BDDFunctionContainer& getRMVariables();

    bool have_rm_variables;
    BDDFunctionContainer rm_variables;
  };
}
#endif
