/***************************************************************************
 *   Copyright (C) 2008-2009 by Tobias Blechmann                           *
 *   blechman@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 @author Tobias Blechmann
 */

#include <iostream>
#include <stdexcept>
#include "reo_channel_end.hpp"
#include "reo_component.hpp"
#include "vereofy/common/errors.hpp"
#include "vereofy/common/weak_ptr_helper.hpp"

namespace vereofy {

  REO_Channel_End::REO_Channel_End(REO_Component::ptr _my_component, PortDescription::ptr _pd_in_my_component) :
  REO_Graph_Object(REO_CHANNEL_END), 
  my_component(_my_component), 
  pd_in_my_component(_pd_in_my_component),
  plugged(false) {

    TRACE("REO_Channel_End", "constructor", cout << this << endl);

    switch (pd_in_my_component->getType()) {
    case PortDescription::IN_PORT:
      ce_type=CE_SOURCE;
      break;
    case PortDescription::OUT_PORT:
      ce_type=CE_SINK;
      break;
    default:
      THROW_UNIMPLEMENTED();
    }
  };

  
  REO_Channel_End::ce_type_t REO_Channel_End::get_CE_Type() const {
    return ce_type;
  }
  
  
  std::string REO_Channel_End::get_Description() {
    string result=(get_CE_Type()==CE_SOURCE) ? "SOURCE" : "SINK";
    if (pd_in_my_component && pd_in_my_component->getName()!="") {
      result+=" "+pd_in_my_component->getName();
    }
    if (pd_in_circuit) {
      result+=" pd="+pd_in_circuit->getName();
    }
    return "\""+result+"\"";
  }

  void REO_Channel_End::setPlug(bool plugged_) {
    TRACE("REO_Channel_End", "plug", cout << this << ": Plug=" << plugged_ << endl);
    plugged=plugged_;
  }

  bool REO_Channel_End::isPlugged() const {
    return plugged;
  }

  PortDescription::ptr REO_Channel_End::get_PortDescription_Circuit() {
    return pd_in_circuit;
  }

  void REO_Channel_End::set_PortDescription_Circuit(PortDescription::ptr pd) {
    pd_in_circuit=pd;
  }

  PortDescription::ptr REO_Channel_End::get_PortDescription_Component() {
    return pd_in_my_component;
  }

  // clones this REO_Graph_Object, adds itself to the mapping
  REO_Graph_Object::ptr REO_Channel_End::clone(REO_Graph_Object::ptr self, REO_Graph_Object::rgo_map& mapping) const {
    CHECK_SUCCESS(this==self.get());

    // We require that the component has already been cloned!
    CHECK_SUCCESS(mapping.count(access_weak_ptr(my_component))==1);

    // We can't clone if we already have a PortDescription in the circuit...
    CHECK_SUCCESS(!pd_in_circuit);
    
    // clone, share pd_in_my_component
    REO_Component::ptr cloned_component=boost::dynamic_pointer_cast<REO_Component>(mapping[access_weak_ptr(my_component)]);
    CHECK_SUCCESS(cloned_component);

    REO_Channel_End::ptr result(new REO_Channel_End(cloned_component, pd_in_my_component));
    result->setClonedFrom(self);
    result->plugged=plugged;

    // insert into mapping
    CHECK_SUCCESS(mapping.insert(make_pair(self, result)).second == true);
    
    return result;
  }
  

}
