/***************************************************************************
 *   Copyright (C) 2008-2009 by Tobias Blechmann                           *
 *   blechman@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 @author Tobias Blechmann
 */

#ifndef REO_CHANNEL_END_HPP
#define REO_CHANNEL_END_HPP

#include <string>
#include "reo_graph_object.hpp"
#include "port_description.hpp"

namespace vereofy {
  
  class REO_Component;
  
  class REO_Channel_End : public REO_Graph_Object {
  public:
    enum ce_type_t {CE_SINK, CE_SOURCE};
    typedef boost::shared_ptr<REO_Channel_End> ptr;
    REO_Channel_End(boost::shared_ptr<REO_Component>, PortDescription::ptr);
    ce_type_t get_CE_Type() const;
    virtual std::string get_Description();

    void setPlug(bool plugged=true);
    bool isPlugged() const;
    
    PortDescription::ptr get_PortDescription_Circuit();
    void set_PortDescription_Circuit(PortDescription::ptr pd);
    PortDescription::ptr get_PortDescription_Component();

    // clones this REO_Graph_Object, adds itself to the mapping
    virtual REO_Graph_Object::ptr clone(REO_Graph_Object::ptr self, rgo_map& mapping) const;
  private:
    ce_type_t ce_type;
    boost::weak_ptr<REO_Component> my_component;
    PortDescription::ptr pd_in_my_component;
    PortDescription::ptr pd_in_circuit;

    bool plugged;
    friend class REO_Circuit;
  };
  
} //end namespace

#endif 
