/***************************************************************************
 *   Copyright (C) 2008-2009 by Tobias Blechmann                           *
 *   blechman@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 @author Tobias Blechmann
 */

#ifndef REO_COMPONENT_HPP
#define REO_COMPONENT_HPP

#include <string>
#include <iostream>
#include <boost/smart_ptr.hpp>
#include "vereofy/common/shared_ptr_map.hpp"
#include "component_instance.hpp"
#include "port_description.hpp"
#include "reo_graph_object.hpp"
#include "reo_channel_end.hpp"

namespace vereofy {
  
  class REO_Component : public REO_Graph_Object {
  public:
    typedef boost::shared_ptr<REO_Component> ptr;
    virtual ~REO_Component() {};

    virtual std::string get_Label();
    virtual std::string get_Description();

    unsigned int getSourceCECount() const;
    REO_Channel_End::ptr getSourceCE(unsigned int) const;

    unsigned int getSinkCECount() const;
    REO_Channel_End::ptr getSinkCE(unsigned int) const;

    // return a name for the channel end (not efficient)
    std::string getNameForCE(REO_Channel_End::ptr) const;

    //Call this static method to create a REO_Component for a ComponentInstance
    static REO_Component::ptr getFor(ComponentInstance::ptr);

    ComponentInstance::ptr getComponentInstance();

    // clones this REO_Graph_Object, adds itself to the mapping
    virtual REO_Graph_Object::ptr clone(REO_Graph_Object::ptr self, rgo_map& mapping) const;
  private:
    REO_Component(ComponentInstance::ptr _my_component);
    
    ComponentInstance::ptr my_component;
    typedef shared_ptr_map<PortDescription::ptr, REO_Channel_End::ptr>::type pd_to_ce_map;
    pd_to_ce_map pd_to_ce;
    vector<REO_Channel_End::ptr> source_channel_ends;
    vector<REO_Channel_End::ptr> sink_channel_ends;
  };
  
}

#endif
