/***************************************************************************
 *   Copyright (C) 2008-2009 by Tobias Blechmann                           *
 *   blechman@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 @author Tobias Blechmann
 */

#ifndef REO_GRAPH_OBJECT_HPP
#define REO_GRAPH_OBJECT_HPP

#include <string>
#include <boost/graph/adjacency_list.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/weak_ptr.hpp>

#include "vereofy/common/shared_ptr_map.hpp"
#include "vereofy/common/tracing.hpp"

#define TRACE_REO_OBJECT(scope, name, rgo)                              \
  TRACE(scope, name,                                                    \
        cout << "Trace: " << scope << "::" << name << " " << #rgo <<  " = " \
        << (rgo)->getBOOST_Vertex() << endl) 

namespace vereofy {

  // forward
  class REO_Graph_Object;

  // lightweight version for boost graphs 
  struct REO_Graph_Object_Ref {
    boost::shared_ptr<REO_Graph_Object> ptr;
  };

  struct REO_Graph_Object_Ref;
  
  typedef boost::adjacency_list<boost::listS, boost::listS, boost::undirectedS, REO_Graph_Object_Ref, boost::no_property> graph_t;
  typedef boost::shared_ptr<graph_t> graph_ptr_t;
  typedef boost::graph_traits< graph_t >::vertex_descriptor vertex_t;
  typedef boost::graph_traits< graph_t >::vertex_iterator vertex_iter_t;
  typedef boost::graph_traits< graph_t >::out_edge_iterator out_edge_iter_t;
  typedef boost::graph_traits< graph_t >::edge_descriptor edge_descriptor_t;

  struct vertex_property {vertex_t vertex;};

  class REO_Graph_Object {
  public:
    typedef boost::shared_ptr<REO_Graph_Object> ptr;
    typedef shared_ptr_map<REO_Graph_Object::ptr, REO_Graph_Object::ptr>::type rgo_map;

    enum Graph_Object_Type {REO_COMPONENT, REO_CHANNEL_END, REO_NODE};

    REO_Graph_Object(Graph_Object_Type _type) : graph_object_type(_type) {};
    virtual ~REO_Graph_Object() {};
    vertex_t getBOOST_Vertex();
    void setBOOST_Vertex(vertex_t new_vertex);
    virtual std::string get_Description();
    Graph_Object_Type get_graph_object_type();

    // clones this REO_Graph_Object, adds itself to the mapping
    virtual REO_Graph_Object::ptr clone(REO_Graph_Object::ptr self, rgo_map& mapping) const = 0;

    bool wasCloned() const {return (bool)clonedFrom;}
    void setClonedFrom(REO_Graph_Object::ptr clonedFrom_) {clonedFrom=clonedFrom_;}
    REO_Graph_Object::ptr getClonedFrom() const {return clonedFrom;}
  private:
    Graph_Object_Type graph_object_type;
    vertex_t BOOST_Vertex;
    REO_Graph_Object::ptr clonedFrom;
  };


}

#endif
