/***************************************************************************
 *   Copyright (C) 2008-2009 by Tobias Blechmann                           *
 *   blechman@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 @author Tobias Blechmann
 */

#include "vereofy/common/errors.hpp"
#include "vereofy/common/tracing.hpp"
#include "reo_node.hpp"
#include "reo_circuit.hpp"
#include <iostream>
#include <boost/lexical_cast.hpp>

#include <boost/foreach.hpp>
#define foreach BOOST_FOREACH

using namespace std;

#include <iostream>

namespace vereofy {

  REO_Node::REO_Node(node_semantics_t node_semantic_) 
    : REO_Graph_Object(REO_NODE), 
      hidden(false), sink_plug(false), source_plug(false),
      node_semantic(node_semantic_)
  {
    TRACE("REO_Node", "constructor", cout << this << endl);
  }

  REO_Node::REO_Node(const REO_Node& other) 
    : REO_Graph_Object(REO_NODE) {
    hidden=other.hidden;
    sink_plug=other.sink_plug;
    source_plug=other.source_plug;
    node_semantic=other.node_semantic;
    
    // can't clone if we already have a port description
    CHECK_SUCCESS(!pd);
    // can't clone if this is an invalid node
    CHECK_SUCCESS(!parentNode);
  }

  std::string REO_Node::get_Description() {
    string result="REO_Node";
    if (pd) {
      result+=" pd="+pd->getName();
    }
    return "\""+result+"\"";
  }


  void REO_Node::setHidden(bool _hidden) {
    hidden = _hidden;
  }

  
  bool REO_Node::getHidden() const {
    return hidden; 
  }

  void REO_Node::setNodeSemantic(node_semantics_t node_semantic_) {
    node_semantic=node_semantic_;
  }

  REO_Node::node_semantics_t REO_Node::getNodeSemantic() const {
    return node_semantic;
  }


  bool REO_Node::hasParent() const {
    if (parentNode) {
      return true;
    } else {
      return false;
    }
  }
    
  
  REO_Node::ptr REO_Node::getParent() const {
    return parentNode;
  }
  
  
  void REO_Node::setParent(REO_Node::ptr _parentNode) {
    parentNode = _parentNode;
  }
  
  void REO_Node::setSinkPlug(bool plugged) {
    TRACE("REO_Node", "plug", cout << this << ": SinkPlug=" << plugged << endl);
    sink_plug=plugged;
  }

  void REO_Node::setSourcePlug(bool plugged) {
    TRACE("REO_Node", "plug", cout << this << ": SourcePlug=" << plugged << endl);
    source_plug=plugged;
  }

  bool REO_Node::isSinkPlugged() const {
    return sink_plug;
  }

  bool REO_Node::isSourcePlugged() const {
    return source_plug;
  }

  PortDescription::ptr REO_Node::getPortDescription() const {
    return pd;
  }
  
  
  void REO_Node::setPortDescription(PortDescription::ptr pd_) {
    pd=pd_;
  }

  REO_Graph_Object::ptr REO_Node::clone(REO_Graph_Object::ptr self, REO_Graph_Object::rgo_map& mapping) const {
    CHECK_SUCCESS(this==self.get());
    
    // copy node
    REO_Node::ptr result(new REO_Node(*this));
    result->setClonedFrom(self);

    // insert into mapping
    CHECK_SUCCESS(mapping.insert(make_pair(self, result)).second==true);

    return result;
  }

}
