/***************************************************************************
 *   Copyright (C) 2008-2009 by Tobias Blechmann                           *
 *   blechman@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 @author Tobias Blechmann
 */


#ifndef REO_NODE_HPP
#define REO_NODE_HPP

#include <string>
#include <iostream>
#include <boost/smart_ptr.hpp>
#include "reo_graph_object.hpp"
#include "reo_channel_end.hpp"

namespace vereofy {
  class REO_Circuit;

  class REO_Node : public REO_Graph_Object {
  public:
    enum node_type_t {NODE_FREE, NODE_SINK, NODE_SOURCE, NODE_MIXED};
    enum node_semantics_t {NODE_REPLICATE, NODE_ROUTE};

    typedef boost::shared_ptr<REO_Node> ptr;
    REO_Node(node_semantics_t node_semantic=NODE_REPLICATE);
    virtual ~REO_Node() {};
    virtual std::string get_Description();
    void setHidden(bool);
    bool getHidden() const;
    bool hasParent() const;
    REO_Node::ptr getParent() const;
    void setParent(REO_Node::ptr _parentNode);    

    void setSinkPlug(bool plugged=true);
    void setSourcePlug(bool plugged=true);
    bool isSinkPlugged() const;
    bool isSourcePlugged() const;

    PortDescription::ptr getPortDescription() const;
    void setPortDescription(PortDescription::ptr);

    void setNodeSemantic(node_semantics_t node_semantic);
    node_semantics_t getNodeSemantic() const;

    static node_semantics_t getDefaultSemantic() {
      return NODE_REPLICATE;
    }

    virtual REO_Graph_Object::ptr clone(REO_Graph_Object::ptr self, rgo_map& mapping) const;
  private:
    // copy constructor
    REO_Node(const REO_Node& other);

    bool hidden;
    bool sink_plug, source_plug;
    node_semantics_t node_semantic;
    PortDescription::ptr pd;
    REO_Node::ptr parentNode;
  };

}

#endif
