/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/



#ifndef RESTRICTION_MANAGER_HPP
#define RESTRICTION_MANAGER_HPP

#include <string>
#include <vector>
#include <map>
#include <iostream>
#include <boost/shared_ptr.hpp>
#include "vereofy/common/setup.hpp"

namespace vereofy {
  // fwd 
  class SymbComponentInstance;

  class RestrictionManager {
  public:
    typedef boost::shared_ptr<RestrictionManager> ptr;
    typedef std::vector<std::string> restriction_vector_t;
    typedef restriction_vector_t::const_iterator restriction_iterator_t;

    RestrictionManager();

    bool restrict_delta_x_with_initial;
    bool restrict_delta_x;
    bool restrict_delta_y;

    void addStateRestriction(const std::string& component_instance_name, 
                             const std::string& restriction);

    bool hasStateRestrictions(const std::string& component_instance_name) const;
    std::pair<restriction_iterator_t, restriction_iterator_t> getStateRestrictions(const std::string& component_instance_name) const;

    void print(std::ostream& out) const;

    BDDFunction getStateRestrictions(SymbComponentInstance& sci, bool states_x) const;
  private: 
    typedef std::map<std::string, restriction_vector_t> restriction_map_t;

    restriction_map_t state_restrictions;
    bool initial_states_restriction;
  };

}


#endif
