/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef STATE_DESCRIPTION_HPP
#define STATE_DESCRIPTION_HPP

#include <string>
#include <boost/shared_ptr.hpp>

#include "datadomain.hpp"
#include "vereofy/common/vereofy_id.hpp"
#include "vereofy/common/xml_helpers.hpp"
#include "symb_automaton/symb_datadomain.hpp"


namespace vereofy {
  class StateDescription : public VereofyID {
  public:
    typedef boost::shared_ptr<StateDescription> ptr;

    StateDescription(const std::string& name_,
		     DataDomain::ptr type_) : 
      name(name_), type(type_) {}

    // get a name for this state
    const std::string& getName() const {return name;}

    DataDomain::ptr getType() const {
      return type;
    }

    void print_xml(std::ostream& out) {
      xml_tag_open(out, "statedescription", 
                   xml_parameter("name", getName())+
                   xml_parameter("shorttype", getType()->toString()));
      getType()->to_xml(out);
      xml_tag_close(out, "statedescription");
    }


  private:
    std::string name;
    DataDomain::ptr type;
  };
} 


#endif
