/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef STATE_ID_HPP
#define STATE_ID_HPP

#include <boost/shared_ptr.hpp>

namespace vereofy {
  
  // fwd
  class ComponentInstance;
  class StateDescription;

  class StateID {
  public:
    StateID() :
      component_instance(0), state_description(0) {}

    StateID(boost::shared_ptr<ComponentInstance> component_instance_,
            boost::shared_ptr<StateDescription> state_description_) : 
      component_instance(component_instance_.get()),
      state_description(state_description_.get()) {}

    StateID(ComponentInstance* component_instance_,
            boost::shared_ptr<StateDescription> state_description_) : 
      component_instance(component_instance_),
      state_description(state_description_.get()) {}

    bool operator<(const StateID& other) const;
    bool operator==(const StateID& other) const;
    bool operator!=(const StateID& other) const {return !(*this==other);}

    ComponentInstance& getComponentInstance() const {return *component_instance;}
    StateDescription& getStateDescription() const {return *state_description;}

    void print(std::ostream& out) const;

  private:
    ComponentInstance* component_instance;
    StateDescription* state_description;
  };

  inline std::ostream& operator<<(std::ostream& out, const StateID& id) {
    id.print(out);
    return out;
  }

}

#endif

