/***************************************************************************
 *   Copyright (C) 2008-2009 by Johannes Osterholzer                       *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


/**
   @author Johannes Osterholzer
*/

#ifndef BDDSUBSETITERATOR_HPP
#define BDDSUBSETITERATOR_HPP

#include "vereofy/setup.hpp"
#include <vector>

namespace vereofy {

  // Class intended to iterate through the subsets of a set of states
  // in a symbolic Constraint Automaton.
  // Note that the "elements" of the provided set must be dependent on the
  // same set of variables, e.g.
  // (xa & !xb & !xc) | (xa & xb & xc) | (!xa & xb & !xc)
  // is a valid input set while
  // (xa & xb) | (xc & !xd & xe) | (xf & !xg) | !xh
  // is *not*!
  class BDDSubsetIterator
  {
  public:
    BDDSubsetIterator(BDDFunction set, BDDFunction csVars);
    ~BDDSubsetIterator();

    BDDFunction operator*(); // access current subset
    BDDSubsetIterator operator++(); // go to next subset
    //BDDSubsetIterator operator++(int);
    bool iterated() {return end;} // used to check whether all subsets have been iterated
    void forbidSet(); // used to skip sets containing the current set in the following

  private:
    void nextSet();
    bool isForbidden();

    std::vector<unsigned long> bitmask;
    std::vector<BDDFunction> elements;
    std::vector< std::vector<unsigned long> > forbiddenSets;
    bool end;
  };

}

#endif //BDDSUBSETITERATOR_HPP
