/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                  x *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef BUILD_CONTEXT_HPP
#define BUILD_CONTEXT_HPP

#include <boost/shared_ptr.hpp>
#include "port_id_to_symb_ce_map.hpp"
#include "state_id_to_symbolic_state_map.hpp"
#include "vereofy/system/port_equivalence_relation.hpp"
#include "vereofy/system/restriction_manager.hpp"

namespace vereofy {

class BuildContext {
public:
  typedef boost::shared_ptr<BuildContext> ptr;

  BuildContext() :
    port_id_to_symb_ce(new PortID2SymbCEMap()),
    port_id_to_deserialized_symb_ce(new PortID2SymbCEMap()),
    state_id_to_deserialized_symbolic_state(new StateID2SymbolicStateMap()),
    param_restrict_x(false), param_restrict_y(false){}
  
  bool param_constrain_to_reachable_components;
  unsigned int param_constrain_to_reachable_circuits;

  bool param_retain_hidden_ports;
  bool param_retain_top_level_ports;

  bool param_abstract_data_domain;

  bool param_no_shared_data;

  bool try_loading_serialized_varorder;
  std::string varorder_dir;

  // global port_id to symb_ce map (identical ports)
  PortID2SymbCEMap::ptr port_id_to_symb_ce;


  // port_id to symb_ce map for deserialization
  PortID2SymbCEMap::ptr port_id_to_deserialized_symb_ce;

  // state_id to SymbolicState map for deserialization
  StateID2SymbolicStateMap::ptr state_id_to_deserialized_symbolic_state;

  // PortID partitions
  PortPartitions::ptr port_partitions;

  RestrictionManager::ptr restriction_manager;
  bool param_restrict_x, param_restrict_y;
};

}
#endif
