/***************************************************************************
 *   Copyright (C) 2011 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "extract_explicit.hpp"

namespace vereofy {
  
  struct ExtractExplicitFunctor 
    : public SymbComponentInstance::ExtractExplicitComponentInstanceFunctor {
    
    ExtractExplicitFunctor(SymbComponentInstance::ptr sci,
                           std::ostream& out_) : 
      SymbComponentInstance::ExtractExplicitComponentInstanceFunctor(sci),
      out(out_) {}
    
    virtual void start() {
      out << "Explicit automaton:\n";
    }
    
    virtual void finished() {
    }


    // state_id: the ID of the state
    // state: a string representation of the state valuation
    // true_aps: a vector of the atomic propositions that are true in this state
    // is_initial: true if state is initial
    // is_terminal: true if state is terminal
    virtual void explicit_state(unsigned long state_id,
                                const std::string& state,
                                const std::vector<std::string>& true_aps,
                                bool is_initial,
                                bool is_terminal) {
      out << "State: " << state_id;
      if (is_initial) {
        out << " (initial)";
      }
      out << "\n";
      out << "\"" << state << "\"\n";
      if (true_aps.size()>0) {
        out << "APs:";
        for (unsigned int i=0;i<true_aps.size();i++) {
          out << " \"" << true_aps.at(i) << "\"";
        }
        out << "\n";
      }
      out << "\n";
    }
    
    // state_id_from: the ID of the from state of this transition
    // state_id_to:   the ID of the to state of this transition
    // transition: a string representation of the transition (active ports and port data values)
    // transition_labels: a vector of the labels attached to the transition
    virtual void explicit_transition(unsigned long state_id_from,
                                     unsigned long state_id_to,
                                     const std::string& transition,
                                     const std::vector<std::string>& transition_labels) {
      out << "Transition: " << state_id_from << " -> " << state_id_to << "\n";
      out << "\"" << transition << "\"\n";
      if (transition_labels.size()>0) {
        out << "Labels:";
        for (unsigned int i=0;i<transition_labels.size();i++) {
          out << " \"" << transition_labels.at(i) << "\"";
        }
        out << "\n";
      }
      out << "\n";
    }

    std::ostream& out;
  };
  
  void extract_explicit(SymbComponentInstance::ptr sci,
                        std::ostream& out) {
    ExtractExplicitFunctor f(sci, out);

    sci->extractExplicitAutomaton(f);
      
  }

}
