/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef PORT_ID_TO_SYMB_CE_MAP_HPP
#define PORT_ID_TO_SYMB_CE_MAP_HPP

#include "vereofy/common/checks.hpp"
#include "vereofy/common/tracing.hpp"
#include "vereofy/system/port_id.hpp"
#include "symb_channel_end.hpp"

namespace vereofy {

  class PortID2SymbCEMap {
  public:
    typedef boost::shared_ptr<PortID2SymbCEMap> ptr;

    void add(const PortID& id, Symb_Channel_End::ptr symb_ce) {
      CHECK(1, (!get(id)) || symb_ce==get(id), ""); // either no symb_ce or the same

      TRACE("port_id_to_symb_ce", "add", std::cout << "Adding: " << id << ":" << symb_ce << "\n");
      
      port_id_2_symb_ce.insert(make_pair(id, symb_ce));
    }

    bool has_symb_ce(const PortID& id) const {
      return (bool)get(id);
    }
    
    Symb_Channel_End::ptr get(const PortID& id) const {
      map_t::const_iterator it=port_id_2_symb_ce.find(id);
      if (it==port_id_2_symb_ce.end()) {
        return Symb_Channel_End::ptr();
      }
      return it->second;
    }

  private:
    typedef map<PortID, Symb_Channel_End::ptr> map_t;
    map_t port_id_2_symb_ce;
  };

}

#endif
