/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SERIALIZED_VARORDER_HPP
#define SERIALIZED_VARORDER_HPP

#include "port_id_to_symb_ce_map.hpp"
#include "state_id_to_symbolic_state_map.hpp"
#include "vereofy/system/component_instance.hpp"
#include "symb_component_instance.hpp"

#include <map>
#include <string>

namespace vereofy {
  
  class SerializedVarorder {
  public:

    class malformed_varorder_exception : public std::runtime_error {
    public:
      malformed_varorder_exception(const string &message);
    };

    class varorder_mismatch_exception : public std::runtime_error {
    public:
      varorder_mismatch_exception(const string &message);
    };
    
    static void writeVarorder(std::ostream& out,
                              SymbComponentInstance::ptr system);
    static void readVarorder(std::istream& in,
                             ComponentInstance::ptr system,
                             BuildContext::ptr build_context,
                             bool verify_only);

    static bool findAndReadVarorder(ComponentInstance::ptr system,
                                    BuildContext::ptr build_context);
    
    static std::string getFilename(const std::string& type_name);

  private:
    typedef std::map<std::string, Symb_Channel_End::ptr> symb_ce_map_t;
    
    std::vector<BDDFunction> vars;
    symb_ce_map_t vars_to_symb_ce;
    symb_ce_map_t data_vars_to_symb_ce;

    void generateVariables(const std::string& var_codes, bool prepend=true);
    BDDFunctionContainer getVars(const std::string& variables);
    bool readItem(std::istream& in,
                  ComponentInstance::ptr system,
                  BuildContext::ptr build_context,
                  bool verify_only=false);

    static void writeComponents(std::ostream& out, SymbComponentInfo::ptr info);
    static void writeStates(std::ostream& out, SymbComponentInfo::ptr info);
    static void writePorts(std::ostream& out, SymbComponentInfo::ptr info);
    static unsigned int counter;
  };
}


#endif

