/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef STATE_ID_TO_SYMBOLIC_STATE_MAP_HPP
#define STATE_ID_TO_SYMBOLIC_STATE_MAP_HPP

#include "vereofy/system/state_id.hpp"
#include "symb_maps.hpp"

namespace vereofy {

  class StateID2SymbolicStateMap {
  public:
    typedef boost::shared_ptr<StateID2SymbolicStateMap> ptr;

    void add(const StateID& id, SymbolicState::ptr symbolic_state) {
      CHECK(1, (!get(id)) || symbolic_state==get(id), ""); // either none or the same

      TRACE("state_id_to_symbolic_state", "add", std::cout << "Adding: " << id << ":" << symbolic_state << "\n");
      
      state_id_2_symbolic_state.insert(make_pair(id, symbolic_state));
    }

    bool has_state(const StateID& id) const {
      return (bool)get(id);
    }
    
    SymbolicState::ptr get(const StateID& id) const {
      map_t::const_iterator it=state_id_2_symbolic_state.find(id);
      if (it==state_id_2_symbolic_state.end()) {
        return SymbolicState::ptr();
      }
      return it->second;
    }

  private:
    typedef map<StateID, SymbolicState::ptr> map_t;
    map_t state_id_2_symbolic_state;
  };

}

#endif
