/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_ARRAY_DATADOMAIN_HPP
#define SYMB_ARRAY_DATADOMAIN_HPP 

#include "vereofy/setup.hpp"
#include <boost/shared_ptr.hpp>
#include "symb_datadomain.hpp"
#include "symb_flat_datadomain.hpp"

#include <string>
using namespace std;

namespace vereofy {

  class Symb_ArrayDataDomain : public Symb_DataDomain {
  public:
    typedef boost::shared_ptr<Symb_ArrayDataDomain> ptr;
    
    Symb_ArrayDataDomain(ArrayDataDomain::ptr data_domain);
    
    virtual unsigned int getLogSize() const;
    virtual double countValidValues() const;
    virtual BDDFunction restrictToValidValues(const BDDFunctionContainer& encoding) const;

    ArrayDataDomain::ptr getArrayDataDomain() const;
    Symb_DataDomain::ptr getSymbBase() const;
    Symb_FlatDataDomain::ptr getIndexDomain() const;

    BDDFunctionContainer getEncoding(const BDDFunctionContainer& encoding,
				     unsigned int index) const;
    virtual string reverseEncoding(const BDDFunction& assignment, const BDDFunctionContainer& vars) const;
    virtual void reverseEncodingToXML(std::ostream& out, 
                                      const BDDFunction& assignment,
                                      const BDDFunctionContainer& vars) const;
    virtual void reverseEncodingToGuards(const BDDFunction& assignment, 
                                         const BDDFunctionContainer& vars,
                                         const std::string& prefix,
                                         std::vector<std::string>& result) const;


    virtual BDDFunction getEquivalent(const BDDFunction& assignments, 
                                      const BDDFunctionContainer& encoding) const;

    virtual boost::shared_ptr<SymbFunction_Constructor> getConstructorFunction();

    virtual BDDFunction performComparison(comp_op_t compare_op,
                                          const SymbExpression& left, 
                                          const SymbExpression& right) const;

    virtual SymbExpression castExpression(const SymbExpression& expression) const;

  private:
    Symb_DataDomain::ptr symb_base;
    Symb_FlatDataDomain::ptr symb_index;
  };


}

#endif
