/***************************************************************************
 *   Copyright (C) 2008-2009 by Tobias Blechmann                           *
 *   blechman@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 @author Tobias Blechmann
 */

#ifndef SYMB_CHANNEL_END_H
#define SYMB_CHANNEL_END_H

#include "vereofy/common/setup.hpp"
#include "vereofy/common/functions.hpp"
#include "vereofy/common/shared_ptr_map.hpp"
#include "vereofy/system/automaton_channel_end.hpp"
#include "symb_flat_datadomain.hpp"
#include "vereofy/system/symb_automaton/var_positions.hpp"

#include "vereofy/common/vereofy_id.hpp"

namespace vereofy {

  /* Assumption: There are no two Symb_Channel_Ends with the same portActive variable */  
  class Symb_Channel_End : public Automaton_Channel_End, public VereofyID {
  public:
    typedef boost::shared_ptr<Symb_Channel_End> ptr;
    typedef boost::weak_ptr<Symb_Channel_End> weak_ptr;
    typedef shared_ptr_id_set<Symb_Channel_End::ptr>::type ptr_set;
    
    enum channel_end_state_t {CE_NORMAL, CE_HIDDEN, CE_SEMI_HIDDEN};

    Symb_Channel_End(Symb_DataDomain::ptr dataDomain);
    Symb_Channel_End(Symb_DataDomain::ptr dataDomain, VarPositions::ptr var_positions_in_out);
    Symb_Channel_End(Symb_DataDomain::ptr dataDomain, const BDDFunctionContainer& variables);

    Symb_Channel_End::ptr clone() const;
    Symb_Channel_End::ptr getSharedDataChannelEnd() const;
    Symb_Channel_End::ptr getSharedDataChannelEnd(const BDDFunction& portActive) const;

    const BDDFunction& getPortActive() const {
      if (isHidden()) {throw "Trying to access hidden Channel End!";}
      return portActive;
    }
    
    const BDDFunctionContainer& getDCVars() const {
      if (isHidden()) {throw "Trying to access hidden Channel End!";}
      return *vecDCVars;
    }

    Symb_DataDomain::ptr getDataDomain() const {
      return dataDomain;
    }
    
    BDDFunction equals(const Symb_Channel_End&) const;
    BDDFunction synched(const Symb_Channel_End&) const;
    BDDFunction data_equal(const BDDFunctionContainer& vecDCVars) const;
    
    string getVariableName() const;

    bool isSamePort(const Symb_Channel_End& other) const;
    
    channel_end_state_t getState() const;
    bool isHidden() const;
    bool isSemiHidden() const;
    bool isVisible() const;

    bool mayShareDataVariables() const;
    bool areDataVariablesShared() const;
    bool areDataVariablesSharedWith(const Symb_Channel_End::ptr& other) const;

    void hide();
    void semi_hide();

    void print_vars(std::ostream& out, std::string name) const;
    void serialize_vars(std::ostream& out) const;

    // returns those Symb_Channel_Ends from ces that share the data variables with 
    // this (excluding this, if it is contained in ces)
    ptr_set getSharedDataCEsFromSet(const ptr_set& ces) const;

  private:
    typedef boost::shared_ptr<BDDFunctionContainer> dc_vars_vector;
    Symb_Channel_End(Symb_DataDomain::ptr dataDomain, BDDFunction portActive, dc_vars_vector vecDCVars);
    Symb_Channel_End(const Symb_Channel_End& other);

    Symb_DataDomain::ptr dataDomain;
    channel_end_state_t state;
    static int nr_of_channel_ends;
    BDDFunction portActive;

    dc_vars_vector vecDCVars;
  };
  
}

#endif
