/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_COMPONENT_INFO_HPP
#define SYMB_COMPONENT_INFO_HPP

#include <boost/shared_ptr.hpp>

#include "vereofy/system/component_interface.hpp"
#include "vereofy/system/component_instance.hpp"
#include "symb_maps.hpp"


namespace vereofy {

class SymbComponentInfo {
public:
  typedef boost::shared_ptr<SymbComponentInfo> ptr;

  SymbComponentInfo(ComponentInterface::ptr ci) : c_interface(ci), abstract_port_data_domain(false) {}

  boost::shared_ptr<ComponentInstance> component_instance; // may be NULL
  ComponentInterface::ptr c_interface;

  const SymbolicState::ptr getState(const string& name) const;
  const SymbolicState::ptr getState(StateDescription::ptr state_d) const;

  const Symb_Transition_Label_Var::ptr getTransitionLabel(const string& name) const;
  const Symb_Transition_Label_Var::ptr getTransitionLabel(TransitionLabelDescription::ptr transition_label_d) const;

  ComponentInterface::ptr getComponentInterface() {return c_interface;}

  void addSubcomponentAutomaton(const string& name, boost::shared_ptr<SymbComponentInstance> sci);
  void addSubcomponentInfo(const string& name, SymbComponentInfo::ptr info);

  boost::shared_ptr<SymbComponentInstance> getSubcomponentAutomaton(const string& name) const;
  boost::shared_ptr<SymbComponentInfo> getSubcomponentInfo(const string& name) const;
  unsigned int getSubcomponentAutomataCount() const {return subcomponent_automata.size();}
  unsigned int getSubcomponentInfoCount() const {return subcomponent_infos.size();}

  subcomponent_info_map::const_iterator subcomponent_infos_begin() const;
  subcomponent_info_map::const_iterator subcomponent_infos_end() const;

  typedef std::map<std::string, BDDFunction> atomic_proposition_map;

  const BDDFunction& getAtomicProposition(const std::string& ap_name) const;
  bool hasAtomicProposition(const std::string& ap_name) const;

  void addAtomicProposition(const std::string& name, const BDDFunction& ap_function);
  
  atomic_proposition_map::const_iterator atomic_propositions_begin() const;
  atomic_proposition_map::const_iterator atomic_propositions_end() const;

  port_to_bdd_map port_to_bdd;
  ce_to_port_map ce_to_port;
  state_to_bdd_map state_to_bdd;
  transition_label_to_bdd_map transition_label_to_bdd;

  bool abstract_port_data_domain;

  void print_var_info(std::ostream& out) const;

private:
  subcomponent_automaton_map subcomponent_automata;
  subcomponent_info_map subcomponent_infos;

  atomic_proposition_map atomic_propositions;
};

}
#endif
