/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_DATADOMAIN_HPP
#define SYMB_DATADOMAIN_HPP 

#include "vereofy/setup.hpp"
#include <boost/shared_ptr.hpp>
#include <boost/enable_shared_from_this.hpp>
#include "vereofy/system/datadomain.hpp"
#include "vereofy/system/types.hpp"

#include <string>

namespace vereofy {

  // forward declarations
  class SymbExpression;
  class SymbFunction_Constructor;

  class Symb_DataDomain : public boost::enable_shared_from_this<Symb_DataDomain> {
  public:
    typedef boost::shared_ptr<Symb_DataDomain> ptr;

    Symb_DataDomain(DataDomain::ptr data_domain);
    virtual ~Symb_DataDomain() {}
    
    virtual unsigned int getLogSize() const = 0;
    virtual double countValidValues() const = 0;
    virtual BDDFunction restrictToValidValues(const BDDFunctionContainer& encoding) const = 0;
    DataDomain::data_domain_type getType() const;
    const DataDomain::ptr getDataDomain() const;

    static 
    Symb_DataDomain::ptr getFor(DataDomain::ptr data_domain);

    virtual std::string reverseEncoding(const BDDFunction& assignment, const BDDFunctionContainer& vars) const = 0;
    virtual void reverseEncodingToXML(std::ostream& out, 
                                      const BDDFunction& assignment,
                                      const BDDFunctionContainer& vars) const = 0;

    virtual void reverseEncodingToGuards(const BDDFunction& assignment, 
                                         const BDDFunctionContainer& vars,
                                         const std::string& prefix,
                                         std::vector<std::string>& result) const = 0;

    virtual SymbExpression castExpression(const SymbExpression& expression) const;

    virtual BDDFunction performComparison(comp_op_t compare_op,
                                          const SymbExpression& left, 
                                          const SymbExpression& right) const;

    virtual BDDFunction performAssignment(const BDDFunctionContainer& targets,
                                          const SymbExpression& right) const;

    virtual BDDFunction getEquivalent(const BDDFunction& assignments, 
                                      const BDDFunctionContainer& encoding) const;

    // get a valid function container with undefined value
    virtual BDDFunctionContainer getNULL() const;

    virtual boost::shared_ptr<SymbFunction_Constructor> getConstructorFunction() = 0;

  private:
    DataDomain::ptr data_domain;
  };
}

#endif
