/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_EXPRESSION_HPP
#define SYMB_EXPRESSION_HPP

#include "symb_datadomain.hpp"
#include "symb_flat_datadomain.hpp"
#include "vereofy/common/checks.hpp"
#include "vereofy/common/errors.hpp"

#include <boost/shared_ptr.hpp>

namespace vereofy {

  class SymbExpression {
  public:
    typedef boost::shared_ptr<SymbExpression> ptr;

    SymbExpression(Symb_DataDomain::ptr data_domain_,
                   const BDDFunctionContainer& values_,
                   BDDFunction condition_=BDDFunction::ONE()) :
      data_domain(data_domain_),
      values(values_),
      condition(condition_) {
      CHECK(1, values.size() == data_domain->getLogSize(), "");
    }

    SymbExpression(unsigned int value) :
      condition(BDDFunction::ONE()) {
      IntRangeDataDomain::ptr int_data_domain(new IntRangeDataDomain(0,value));
      Symb_FlatDataDomain::ptr symb_data_domain=Symb_FlatDataDomain::ptr(new Symb_FlatDataDomain(int_data_domain));

      data_domain=symb_data_domain;
      values=symb_data_domain->getEncoding(value);
    }
    
    SymbExpression() {}

    Symb_DataDomain::ptr getDataDomain() const {
      return data_domain;
    }
    
    const BDDFunctionContainer& getValues() const {
      return values;
    }

    const BDDFunction& getCondition() const {
      return condition;
    }

    void setCondition(const BDDFunction& condition_) {
      condition=condition_;
    }

    bool isConstant() const {
      if (condition!=BDDFunction::ONE()) 
        return false;

      for (unsigned int i=0;i<values.size();i++) {
        if (values.at(i)!=BDDFunction::ZERO() &&
            values.at(i)!=BDDFunction::ONE()) {
          return false;
        }
      }
      return true;
    }

    void restrictToValidValues() {
      condition&=data_domain->restrictToValidValues(values);
    }

  private:
    Symb_DataDomain::ptr data_domain;
    BDDFunctionContainer values;
    BDDFunction condition;
  };


}

#endif
