/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_FLAT_DATADOMAIN_HPP
#define SYMB_FLAT_DATADOMAIN_HPP 

#include "vereofy/setup.hpp"
#include <boost/shared_ptr.hpp>
#include "symb_datadomain.hpp"


#include <string>
using namespace std;

namespace vereofy {

  class Symb_FlatDataDomain : public Symb_DataDomain {
  public:
    typedef boost::shared_ptr<Symb_FlatDataDomain> ptr;
    
    Symb_FlatDataDomain(FlatDataDomain::ptr data_domain);
    
    virtual unsigned int getLogSize() const;
    virtual double countValidValues() const;
    virtual BDDFunction restrictToValidValues(const BDDFunctionContainer& encoding) const;

    BDDFunctionContainer getEncoding(unsigned long value) const;
    BDDFunctionContainer getEncoding(const string& element) const;

    unsigned int reverseConstExpression(const SymbExpression& const_expression) const;
    virtual string reverseEncoding(const BDDFunction& assignment, const BDDFunctionContainer& vars) const;
    virtual void reverseEncodingToXML(std::ostream& out, 
                                      const BDDFunction& assignment,
                                      const BDDFunctionContainer& vars) const;
    virtual void reverseEncodingToGuards(const BDDFunction& assignment, 
                                         const BDDFunctionContainer& vars,
                                         const std::string& prefix,
                                         std::vector<std::string>& result) const;


    virtual SymbExpression castExpression(const SymbExpression& expression) const;

    virtual BDDFunction performComparison(comp_op_t compare_op,
                                          const SymbExpression& left, 
                                          const SymbExpression& right) const;

    BDDFunction performAssignment(const BDDFunctionContainer& targets,
                                  const SymbExpression& right) const;

      
    const FlatDataDomain::ptr getFlatDataDomain() const;

    virtual boost::shared_ptr<SymbFunction_Constructor> getConstructorFunction();
  private:
    int getIntValueForEnumElement(const string& element) const;
    SymbExpression cast_up(const SymbExpression& expression) const;
    SymbExpression cast_down(const SymbExpression& expression) const;

  };

}

#endif
