/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "symb_function.hpp"
#include "vereofy/common/errors.hpp"

using namespace vereofy;
using namespace std;

SymbFunction::SymbFunction(unsigned int arity_, return_value_t return_value_type_, bool is_const_evaluatable_) :
  arity(arity_), return_value_type(return_value_type_), is_const_evaluatable(is_const_evaluatable_) {
}

SymbFunction::~SymbFunction() {}

unsigned int SymbFunction::getArity() const {
  return arity;
}

bool SymbFunction::isConstEvaluatable() const {
  return is_const_evaluatable;
}


void SymbFunction::checkArguments(SymbFunctionArgumentVector& arguments) const {
  if (arguments.size() != getArity()) {
    throw "Function call: Argument size mismatch!";
  }

  for (unsigned int i=0;i<arguments.size();i++) {
    if (requiresConstArgumentAt(i) && !arguments.at(i).isConstValue()) {
      throw "Functin call: Argument "+boost::lexical_cast<std::string>(i)+" should be constant!";
    }
  }
}

SymbFunction::return_value_t SymbFunction::getReturnValueType() const {
  return return_value_type;
}


SymbExpression::ptr SymbFunction::calculate(SymbFunctionArgumentVector& arguments) const {
  THROW_UNIMPLEMENTED();
}

unsigned int SymbFunction::const_evaluate(SymbFunctionArgumentVector& arguments) const {
  THROW_UNIMPLEMENTED();
}

