/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_FUNCTION_HPP
#define SYMB_FUNCTION_HPP

#include <boost/shared_ptr.hpp>
#include "symb_expression.hpp"
#include "symb_function_argument.hpp"

namespace vereofy {

  class SymbFunction {
  public:
    typedef boost::shared_ptr<SymbFunction> ptr;

    enum return_value_t {RETURN_VALUE_INTEGER, RETURN_VALUE_BOOLEAN, RETURN_VALUE_COMPLEX};


    SymbFunction(unsigned int arity_, return_value_t return_value_type_=RETURN_VALUE_INTEGER, bool is_const_evaluatable_=true);
    virtual ~SymbFunction();

    unsigned int getArity() const;
    bool isConstEvaluatable() const;

    virtual bool requiresConstArgumentAt(unsigned int index) const {
      //default
      return false;
    }

    return_value_t getReturnValueType() const;

    /* Return expected type of argument at index, empty ::ptr if not known or no preference
     * In the vector previous_arguments, the already processed arguments from 0..index-1 are passed
     * to allow different responses depending on these arguments
     */
    virtual Symb_DataDomain::ptr getExpectedArgumentType(unsigned int index, 
                                                         const SymbFunctionArgumentVector& previous_arguments) const {
      return Symb_DataDomain::ptr();
    }

    virtual SymbExpression::ptr calculate(SymbFunctionArgumentVector& arguments) const;
    virtual unsigned int const_evaluate(SymbFunctionArgumentVector& arguments) const;

    void checkArguments(SymbFunctionArgumentVector& arguments) const;

  private:
    unsigned int arity;
    return_value_t return_value_type;
    bool is_const_evaluatable;
  };

}

#endif
