/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_FUNCTION_ARGUMENT_HPP
#define SYMB_FUNCTION_ARGUMENT_HPP

#include <vector>

#include "vereofy/common/errors.hpp"
#include "symb_expression.hpp"

namespace vereofy {
  class SymbFunctionArgument {
  public:
    SymbFunctionArgument(SymbExpression::ptr expression_) :
      is_valid(true),
      argument_type(EXPRESSION),
      expression(expression_), const_value(0) {}
    
    SymbFunctionArgument(unsigned int const_value_) :
      is_valid(true),
      argument_type(CONST_VALUE),
      expression(),
      const_value(const_value_) {}
    
    SymbFunctionArgument() : 
      is_valid(false),
      expression() {}

    bool isConstValue() const {
      CHECK_SUCCESS(is_valid);
      return argument_type==CONST_VALUE;
    }  

    bool isSymbExpression() const {
      CHECK_SUCCESS(is_valid);
      return argument_type==EXPRESSION;
    }
    
    SymbExpression::ptr getSymbExpression() const {
      CHECK_SUCCESS(is_valid);
      if (isConstValue()) {
	return SymbExpression::ptr(new SymbExpression(getConstValue()));
      } else {
	return expression;
      }
    }
    
    unsigned int getConstValue() const {
      CHECK_SUCCESS(is_valid);
      CHECK_SUCCESS(isConstValue());
      return const_value;
    }
    
    IntRangeDataDomain::ptr getIntRangeDataDomain() const {
      CHECK_SUCCESS(is_valid);
      Symb_DataDomain::ptr s_dd=getSymbExpression()->getDataDomain();
      
      DataDomain::ptr dd=s_dd->getDataDomain();
      if (dd->getType()==DataDomain::DD_FLAT) {
	FlatDataDomain::ptr f_dd=boost::static_pointer_cast<FlatDataDomain>(dd);
	if (f_dd->getFlatType()==FlatDataDomain::DD_INT) {
	  return boost::static_pointer_cast<IntRangeDataDomain>(f_dd);
	}
      }
      // wrong type:
      throw "Argument is not of integer type";
    }

  private:
    enum argument_type_t {EXPRESSION, CONST_VALUE};

    bool is_valid;
    argument_type_t argument_type;
    SymbExpression::ptr expression;
    unsigned int const_value;
  };
  
  typedef std::vector<SymbFunctionArgument> SymbFunctionArgumentVector;
  
}
#endif
