/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_FUNCTION_ARITHMETIC_HPP
#define SYMB_FUNCTION_ARITHMETIC_HPP

#include "symb_function.hpp"

namespace vereofy {

  class SymbFunction_Arithmetic : public SymbFunction {
  public:
    SymbFunction_Arithmetic(unsigned int arity,
                            SymbFunction::return_value_t return_value=SymbFunction::RETURN_VALUE_INTEGER) :
      SymbFunction(arity, return_value) {}

    IntRangeDataDomain::ptr getDataDomainForArgument(unsigned int index);
  };

  class SymbFunction_uint_add : public SymbFunction_Arithmetic {
  public:
    SymbFunction_uint_add() : SymbFunction_Arithmetic(2) {}
    SymbExpression::ptr calculate(SymbFunctionArgumentVector& arguments) const;
    unsigned int const_evaluate(SymbFunctionArgumentVector& arguments) const;
  };

  class SymbFunction_uint_sub : public SymbFunction_Arithmetic {
  public:
    SymbFunction_uint_sub() : SymbFunction_Arithmetic(2) {}
    SymbExpression::ptr calculate(SymbFunctionArgumentVector& arguments) const;
    unsigned int const_evaluate(SymbFunctionArgumentVector& arguments) const;
  };

  class SymbFunction_uint_mult : public SymbFunction_Arithmetic {
  public:
    SymbFunction_uint_mult() : SymbFunction_Arithmetic(2) {}
    SymbExpression::ptr calculate(SymbFunctionArgumentVector& arguments) const;
    unsigned int const_evaluate(SymbFunctionArgumentVector& arguments) const;
  };

  class SymbFunction_uint_div : public SymbFunction_Arithmetic {
  public:
    SymbFunction_uint_div() : SymbFunction_Arithmetic(2) {}
    SymbExpression::ptr calculate(SymbFunctionArgumentVector& arguments) const;
    unsigned int const_evaluate(SymbFunctionArgumentVector& arguments) const;
  };

  class SymbFunction_uint_mod : public SymbFunction_Arithmetic {
  public:
    SymbFunction_uint_mod() : SymbFunction_Arithmetic(2) {}
    SymbExpression::ptr calculate(SymbFunctionArgumentVector& arguments) const;
    unsigned int const_evaluate(SymbFunctionArgumentVector& arguments) const;
  };
  

  // uint_mod_bits(a,k) = a % (1<<k)
  class SymbFunction_uint_mod_bits : public SymbFunction_Arithmetic {
  public:
    SymbFunction_uint_mod_bits() : SymbFunction_Arithmetic(2) {}

    bool requiresConstArgumentAt(unsigned int index) const {return index==1;}

    SymbExpression::ptr calculate(SymbFunctionArgumentVector& arguments) const;
    unsigned int const_evaluate(SymbFunctionArgumentVector& arguments) const;
  };

  class SymbFunction_uint_compare : public SymbFunction_Arithmetic {
  public:
    SymbFunction_uint_compare(comp_op_t compare_op_) : 
      SymbFunction_Arithmetic(2, SymbFunction::RETURN_VALUE_BOOLEAN),
      compare_op(compare_op_) {
    }

    SymbExpression::ptr calculate(SymbFunctionArgumentVector& arguments) const;
    unsigned int const_evaluate(SymbFunctionArgumentVector& arguments) const;

  private:
    comp_op_t compare_op;
  };

  class SymbFunction_uint_equal : public SymbFunction_uint_compare {
  public:
    SymbFunction_uint_equal() : SymbFunction_uint_compare(COMPARE_EQUAL) {}
  };

  class SymbFunction_uint_not_equal : public SymbFunction_uint_compare {
  public:
    SymbFunction_uint_not_equal() : SymbFunction_uint_compare(COMPARE_NOTEQUAL) {}
  };

  class SymbFunction_uint_less : public SymbFunction_uint_compare {
  public:
    SymbFunction_uint_less() : SymbFunction_uint_compare(COMPARE_LESS) {}
  };

  class SymbFunction_uint_less_equal : public SymbFunction_uint_compare {
  public:
    SymbFunction_uint_less_equal() : SymbFunction_uint_compare(COMPARE_LESSEQUAL) {}
  };

  class SymbFunction_uint_greater : public SymbFunction_uint_compare {
  public:
    SymbFunction_uint_greater() : SymbFunction_uint_compare(COMPARE_GREATER) {}
  };

  class SymbFunction_uint_greater_equal : public SymbFunction_uint_compare {
  public:
    SymbFunction_uint_greater_equal() : SymbFunction_uint_compare(COMPARE_GREATEREQUAL) {}
  };

  class SymbFunction_uint_max : public SymbFunction_Arithmetic {
  public:
    SymbFunction_uint_max() : SymbFunction_Arithmetic(2) {}

    SymbExpression::ptr calculate(SymbFunctionArgumentVector& arguments) const;
    unsigned int const_evaluate(SymbFunctionArgumentVector& arguments) const;
  };
    
  class SymbFunction_uint_min : public SymbFunction_Arithmetic {
  public:
    SymbFunction_uint_min() : SymbFunction_Arithmetic(2) {}

    SymbExpression::ptr calculate(SymbFunctionArgumentVector& arguments) const;
    unsigned int const_evaluate(SymbFunctionArgumentVector& arguments) const;
  };



  class SymbFunction_uint_pow2 : public SymbFunction_Arithmetic {
  public:
    SymbFunction_uint_pow2() : SymbFunction_Arithmetic(1) {}

    bool requiresConstArgumentAt(unsigned int index) const {return index==0;}

    SymbExpression::ptr calculate(SymbFunctionArgumentVector& arguments) const;
    unsigned int const_evaluate(SymbFunctionArgumentVector& arguments) const;
  };

  class SymbFunction_uint_bits_needed : public SymbFunction_Arithmetic {
  public:
    SymbFunction_uint_bits_needed() : SymbFunction_Arithmetic(1) {}

    bool requiresConstArgumentAt(unsigned int index) const {return index==0;}

    SymbExpression::ptr calculate(SymbFunctionArgumentVector& arguments) const;
    unsigned int const_evaluate(SymbFunctionArgumentVector& arguments) const;
  };

  class SymbFunction_uint_ones_complement : public SymbFunction_Arithmetic {
  public:
    SymbFunction_uint_ones_complement() : SymbFunction_Arithmetic(2) {}

    bool requiresConstArgumentAt(unsigned int index) const {return index==0;}

    SymbExpression::ptr calculate(SymbFunctionArgumentVector& arguments) const;
    unsigned int const_evaluate(SymbFunctionArgumentVector& arguments) const;
  };

  class SymbFunction_uint_twos_complement : public SymbFunction_Arithmetic {
  public:
    SymbFunction_uint_twos_complement() : SymbFunction_Arithmetic(2) {}

    bool requiresConstArgumentAt(unsigned int index) const {return index==0;}

    SymbExpression::ptr calculate(SymbFunctionArgumentVector& arguments) const;
    unsigned int const_evaluate(SymbFunctionArgumentVector& arguments) const;
  };


}

#endif
