/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "symb_function_boolean.hpp"

namespace vereofy {

  BDDFunction SymbFunction_Boolean::getArgument(SymbFunctionArgumentVector& arguments, unsigned int index) const {
    SymbExpression::ptr v=arguments.at(index).getSymbExpression();
    Symb_DataDomain::ptr s_dd=v->getDataDomain();
    IntRangeDataDomain::ptr dd=arguments.at(index).getIntRangeDataDomain();

    BDDFunction is_null=s_dd->performComparison(COMPARE_EQUAL, 
                                                *v,
                                                SymbExpression(0));
    return !is_null;
  }

  SymbExpression::ptr SymbFunction_Boolean::calculate(SymbFunctionArgumentVector& arguments) const {
    checkArguments(arguments);

    BDDFunction a=(getArity()>=1) ? getArgument(arguments, 0) : BDDFunction::ZERO();
    BDDFunction b=(getArity()>=2) ? getArgument(arguments, 1) : BDDFunction::ZERO();

    BDDFunction result=calculate(a,b);
    BDDFunctionContainer result_vector;
    result_vector.push(result);

    IntRangeDataDomain::ptr bool_dd(new IntRangeDataDomain(0,1));

    return SymbExpression::ptr(new SymbExpression(Symb_DataDomain::getFor(bool_dd),
                                                  result_vector));
  }

  unsigned int SymbFunction_Boolean::const_evaluate(SymbFunctionArgumentVector& arguments) const {
    unsigned int a=arguments.at(0).getConstValue();
    unsigned int b=arguments.at(1).getConstValue();

    bool a_bool=(a!=0);
    bool b_bool=(b!=0);

    return (const_evaluate(a_bool, b_bool) ? 1 : 0);
  }


  BDDFunction SymbFunction_bool_and::calculate(const BDDFunction &a, const BDDFunction& b) const {
    return a & b;
  }

  bool SymbFunction_bool_and::const_evaluate(bool a, bool b) const {
    return a & b;
  }


  BDDFunction SymbFunction_bool_or::calculate(const BDDFunction &a, const BDDFunction& b) const {
    return a | b;
  }

  bool SymbFunction_bool_or::const_evaluate(bool a, bool b) const {
    return a | b;
  }

  BDDFunction SymbFunction_bool_xor::calculate(const BDDFunction &a, const BDDFunction& b) const {
    return a ^ b;
  }

  bool SymbFunction_bool_xor::const_evaluate(bool a, bool b) const {
    return a ^ b;
  }

  BDDFunction SymbFunction_bool_implies::calculate(const BDDFunction &a, const BDDFunction& b) const {
    return (!a) | b;
  }

  bool SymbFunction_bool_implies::const_evaluate(bool a, bool b) const {
    return (!a) | b;
  }

  BDDFunction SymbFunction_bool_equivalent::calculate(const BDDFunction &a, const BDDFunction& b) const {
    return !(a ^ b);
  }

  bool SymbFunction_bool_equivalent::const_evaluate(bool a, bool b) const {
    return !(a ^ b);
  }

  BDDFunction SymbFunction_bool_not::calculate(const BDDFunction &a, const BDDFunction& b) const {
    return !a;
  }

  bool SymbFunction_bool_not::const_evaluate(bool a, bool b) const {
    return !a;
  }


}
