/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_FUNCTION_BOOLEAN_HPP
#define SYMB_FUNCTION_BOOLEAN_HPP

#include "symb_function.hpp"

namespace vereofy {

  class SymbFunction_Boolean : public SymbFunction {
  public:
    SymbFunction_Boolean(unsigned int arity) : SymbFunction(arity, SymbFunction::RETURN_VALUE_BOOLEAN) {}


    SymbExpression::ptr calculate(SymbFunctionArgumentVector& arguments) const;
    unsigned int const_evaluate(SymbFunctionArgumentVector& arguments) const;

    virtual BDDFunction calculate(const BDDFunction& a, const BDDFunction& b) const = 0;
    virtual bool const_evaluate(bool a, bool b) const = 0;

  private:
    BDDFunction getArgument(SymbFunctionArgumentVector& arguments, unsigned int index) const;
  };

  class SymbFunction_bool_and : public SymbFunction_Boolean {
  public:
    SymbFunction_bool_and() : SymbFunction_Boolean(2) {}
    virtual BDDFunction calculate(const BDDFunction& a, const BDDFunction& b) const;
    virtual bool const_evaluate(bool a, bool b) const;
  };

  class SymbFunction_bool_or : public SymbFunction_Boolean {
  public:
    SymbFunction_bool_or() : SymbFunction_Boolean(2) {}
    virtual BDDFunction calculate(const BDDFunction& a, const BDDFunction& b) const;
    virtual bool const_evaluate(bool a, bool b) const;
  };

  class SymbFunction_bool_xor : public SymbFunction_Boolean {
  public:
    SymbFunction_bool_xor() : SymbFunction_Boolean(2) {}
    virtual BDDFunction calculate(const BDDFunction& a, const BDDFunction& b) const;
    virtual bool const_evaluate(bool a, bool b) const;
  };

  class SymbFunction_bool_implies : public SymbFunction_Boolean {
  public:
    SymbFunction_bool_implies() : SymbFunction_Boolean(2) {}
    virtual BDDFunction calculate(const BDDFunction& a, const BDDFunction& b) const;
    virtual bool const_evaluate(bool a, bool b) const;
  };

  class SymbFunction_bool_equivalent : public SymbFunction_Boolean {
  public:
    SymbFunction_bool_equivalent() : SymbFunction_Boolean(2) {}
    virtual BDDFunction calculate(const BDDFunction& a, const BDDFunction& b) const;
    virtual bool const_evaluate(bool a, bool b) const;
  };

  class SymbFunction_bool_not : public SymbFunction_Boolean {
  public:
    SymbFunction_bool_not() : SymbFunction_Boolean(1) {}
    virtual BDDFunction calculate(const BDDFunction& a, const BDDFunction& b) const;
    virtual bool const_evaluate(bool a, bool b) const;

  };

}

#endif
