/***************************************************************************
 *   Copyright (C) 2008-2009 by Tobias Blechmann                           *
 *   blechman@tcs.inf.tu-dresden.de                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
@author Tobias Blechmann
*/

#ifndef SYMB_NODE_H
#define SYMB_NODE_H

#include <string>
#include <vector>
#include <set>
#include <iostream>
#include <boost/smart_ptr.hpp>

#include "vereofy/system/reo_node.hpp"
#include "symb_automaton.hpp"
#include "symb_component_instance.hpp"
#include "var_positions.hpp"

using namespace boost;

namespace vereofy {
  class SymbNode : public SymbComponentInstance {
  public:
    typedef boost::shared_ptr<SymbNode> ptr;


    SymbNode(SymbComponentInstance::ptr parent,
             Symb_DataDomain::ptr dataDomain,
             REO_Node::node_semantics_t node_semantics,
             const vector<PortDescription::ptr>& in,
             const vector<PortDescription::ptr>& out,
             PortDescription::ptr node_port,
             bool share_channel_ends,           
             VarPositions::ptr var_positions_,
             BuildContext::ptr build_context);
    
    Symb_Channel_End::ptr getCEforPort() const;

    void addAugmentation(BDDFunction augmentation);

    virtual void build_automaton(BuildContext::ptr build_context) {throw "Node is special, build_automaton not implemented";};
  private:
    Symb_Channel_End::ptr ce_port;

    void build_merge_copy_node(SymbComponentInstance::ptr parent,
                               Symb_DataDomain::ptr dataDomain,
                               const vector<PortDescription::ptr>& in,
                               const vector<PortDescription::ptr>& out,
                               PortDescription::ptr node_port,
                               bool share_channel_ends,           
                               VarPositions::ptr var_positions_,
                               BuildContext::ptr build_context);

    void build_route_node(SymbComponentInstance::ptr parent,
                          Symb_DataDomain::ptr dataDomain,
                          const vector<PortDescription::ptr>& in,
                          const vector<PortDescription::ptr>& out,
                          PortDescription::ptr node_port,
                          bool share_channel_ends,           
                          VarPositions::ptr var_positions_,
                          BuildContext::ptr build_context);
  };
  
} //end namespace

#endif
