/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/



#ifndef SYMB_STATE_HPP
#define SYMB_STATE_HPP

#include "vereofy/setup.hpp"
#include "vereofy/common/bdd_helpers.hpp"
#include "vereofy/common/bdd_checks.hpp"
#include <boost/shared_ptr.hpp>

namespace vereofy {
  class SymbState {
  public:
    typedef boost::shared_ptr<SymbState> ptr;

    SymbState(const BDDFunction& state_, 
              BDDFunction csStateVars_) :
      state(state_), csStateVars(csStateVars_) {
      BDD_CHECK2(3, BDDChecks::isAssignment(state, csStateVars), "", state, csStateVars);
    }

    operator const BDDFunction&() const {
      return getX();
    }

    bool operator==(const SymbState& other) {
      if (csStateVars != other.csStateVars)
        return false;

      bool rv = (getX() & csStateVars) == (other.getX() & csStateVars);
      return rv;
    }

    const BDDFunction& getX() const {
      return state;
    }

    BDDFunction getY() const {
      return state.moveDown();
    }

    const BDDFunction& getCsStateVars() const {
      return csStateVars;
    }

    static SymbState pickState(const BDDFunction& state_set,
                               const BDDFunction& csStateVars) {
      BDDFunction state_set_=state_set;
      return popState(state_set_, csStateVars);
    }

    static SymbState popState(BDDFunction& state_set,
                              const BDDFunction& csStateVars) {
      BDDFunction state=popAssignment(state_set, csStateVars);
      return SymbState(state, csStateVars);
    }
    
  private:
    BDDFunction state;
    BDDFunction csStateVars;
  };
}
#endif
