/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/



#ifndef SYMB_STATESET_HPP
#define SYMB_STATESET_HPP

#include "symb_state.hpp"

namespace vereofy {
  class SymbStateSet {
  public:
    typedef boost::shared_ptr<SymbStateSet> ptr;

    SymbStateSet(const BDDFunction& state_set_, 
                 BDDFunction csStateVars_) :
      state_set(state_set_), csStateVars(csStateVars_) {
      BDD_CHECK(3, BDDChecks::isCubeSet(csStateVars), "", csStateVars);
    }

    operator const BDDFunction&() const {
      return getX();
    }

    bool operator==(const SymbStateSet& other) {
      if (csStateVars != other.csStateVars)
        return false;

      bool rv = (getX() & csStateVars) == (other.getX() & csStateVars);
      return rv;
    }

    const BDDFunction& getX() const {
      return state_set;
    }

    BDDFunction getY() const {
      return state_set.moveDown();
    }

    const BDDFunction& getCsStateVars() const {
      return csStateVars;
    }

    bool isEmpty() const {
      return state_set==BDDFunction::ZERO();
    }

    bool contains(const SymbState& state) {
      ERROR_CHECK_ASSERT(state.getCsStateVars() == csStateVars);
      return bdd_sets_have_intersection(state_set, state.getX());
    }

    SymbState pickState() {
      BDDFunction state_set_=state_set;  // copy
      BDDFunction state=popAssignment(state_set_, csStateVars);
      return SymbState(state, csStateVars);
    }

    SymbState popState() {
      BDDFunction state=popAssignment(state_set, csStateVars);
      return SymbState(state, csStateVars);
    }

    SymbState randomState() const {
      ERROR_CHECK_ASSERT(state_set!=BDDFunction::ZERO());
      return SymbState(state_set.getRandomAssignment(csStateVars),
                       csStateVars);
    }
    
  private:
    BDDFunction state_set;
    BDDFunction csStateVars;
  };
}
#endif
