/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_STRUCT_DATADOMAIN_HPP
#define SYMB_STRUCT_DATADOMAIN_HPP 
#include "vereofy/setup.hpp"
#include <boost/shared_ptr.hpp>
#include "symb_datadomain.hpp"

#include <string>
using namespace std;

namespace vereofy {

  class Symb_StructDataDomain : public Symb_DataDomain {
  public:
    typedef boost::shared_ptr<Symb_StructDataDomain> ptr;
    
    Symb_StructDataDomain(StructDataDomain::ptr data_domain);
    
    virtual unsigned int getLogSize() const;
    virtual double countValidValues() const;
    virtual BDDFunction restrictToValidValues(const BDDFunctionContainer& encoding) const;

    StructDataDomain::ptr getStructDataDomain() const;
    Symb_DataDomain::ptr getSymbDataDomainForMember(unsigned int index) const;

    BDDFunctionContainer getEncodingForMember(const BDDFunctionContainer& encoding,
                                              unsigned int index) const;
    virtual string reverseEncoding(const BDDFunction& assignment, const BDDFunctionContainer& vars) const;
    virtual void reverseEncodingToXML(std::ostream& out, 
                                      const BDDFunction& assignment,
                                      const BDDFunctionContainer& vars) const;
    virtual void reverseEncodingToGuards(const BDDFunction& assignment, 
                                         const BDDFunctionContainer& vars,
                                         const std::string& prefix,
                                         std::vector<std::string> &result) const;

    virtual BDDFunction getEquivalent(const BDDFunction& assignments, 
                                      const BDDFunctionContainer& encoding) const;

    virtual boost::shared_ptr<SymbFunction_Constructor> getConstructorFunction();
  private:
    // vector of pairs (start_bit, symb_data_domain) per member
    struct encoding_info {
      unsigned int start_bit;
      unsigned int length;
      Symb_DataDomain::ptr symb_datadomain;
    };
    
    std::vector< encoding_info > encodings;
    unsigned int logsize;
  };


}



#endif
