/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_TAGGED_UNION_DATADOMAIN_HPP
#define SYMB_TAGGED_UNION_DATADOMAIN_HPP 
#include "vereofy/setup.hpp"
#include <boost/shared_ptr.hpp>
#include "symb_datadomain.hpp"
#include "symb_flat_datadomain.hpp"

#include <string>
using namespace std;

namespace vereofy {

  class Symb_TaggedUnionDataDomain : public Symb_DataDomain {
  public:
    typedef boost::shared_ptr<Symb_TaggedUnionDataDomain> ptr;
    
    Symb_TaggedUnionDataDomain(TaggedUnionDataDomain::ptr data_domain);
    
    virtual unsigned int getLogSize() const;
    virtual double countValidValues() const;
    virtual BDDFunction restrictToValidValues(const BDDFunctionContainer& encoding) const;

    TaggedUnionDataDomain::ptr getTaggedUnionDataDomain() const;
    Symb_FlatDataDomain::ptr getSymbDataDomainForTag() const;
    Symb_DataDomain::ptr getSymbDataDomainForMember(unsigned int index) const;

    BDDFunctionContainer getEncodingForTag(const BDDFunctionContainer& encoding) const;
    BDDFunctionContainer getEncodingForMember(const BDDFunctionContainer& encoding,
                                              unsigned int index) const;

    // returns cubeset with the variables that are not used for a given member
    BDDFunction getUnusedEncodingForMember(const BDDFunctionContainer& encoding, unsigned int index) const;

    SymbExpression accessMember(const std::string& name, const BDDFunctionContainer& encoding) const;
    BDDFunction isMember(unsigned int index, const BDDFunctionContainer& encoding) const ;

    virtual string reverseEncoding(const BDDFunction& assignment, const BDDFunctionContainer& vars) const;
    virtual void reverseEncodingToXML(std::ostream& out, 
                                      const BDDFunction& assignment,
                                      const BDDFunctionContainer& vars) const;
    virtual void reverseEncodingToGuards(const BDDFunction& assignment, 
                                         const BDDFunctionContainer& vars,
                                         const std::string& prefix,
                                         std::vector<std::string>& result) const;


    virtual BDDFunction getEquivalent(const BDDFunction& assignments, 
                                      const BDDFunctionContainer& encoding) const;

    virtual boost::shared_ptr<SymbFunction_Constructor> getConstructorFunction();
  private:
    Symb_FlatDataDomain::ptr tag_domain;
    std::vector<Symb_DataDomain::ptr> encodings;
    unsigned int logsize;
  };


}



#endif
