/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_TRACE_HPP
#define SYMB_TRACE_HPP

#include <boost/shared_ptr.hpp>

#include "vereofy/setup.hpp"
#include "symb_transition.hpp"

namespace vereofy {

  // fwd
  class SymbComponentInstance;


  class SymbTrace {
  public:
    typedef boost::shared_ptr<SymbTrace> ptr;
    
    SymbTrace();

    void appendTrace(const SymbTrace& trace);
    void appendTransition(const SymbTransition& transition);
    void appendState(const SymbState& state);

    unsigned int countStates() const;
    unsigned int countLabels() const;
    unsigned int countTransitions() const;

    const SymbState& getState(unsigned int index) const;
    const SymbState& getLastState() const;
    const SymbTransitionLabel& getLabel(unsigned int index) const;
    SymbTransition getTransition(unsigned int index) const;

    BDDFunction getAllStates() const;
    BDDFunction getAllTransitions() const;

    void print_xml(SymbComponentInstance& system, std::ostream& out) const;

  private:
    typedef std::vector<SymbState> state_vector;
    typedef std::vector<SymbTransitionLabel> label_vector;

    state_vector states;
    label_vector labels;
  };
  
}
#endif
