/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_TRACE_EXPLORER_AUTOMATON_HPP
#define SYMB_TRACE_EXPLORER_AUTOMATON_HPP

#include "symb_automaton.hpp"
#include "symb_trace_explorer.hpp"

namespace vereofy {

  /**
   * SymbTraceExplorerAutomaton provides a SymbTraceExplorer for
   * all traces in a SymbAutomaton.
   * If the underlying automaton is changed during exploration, the
   * results of function calls are undefined.
   */
  class SymbTraceExplorerAutomaton : public SymbTraceExplorer {
  public:
    typedef boost::shared_ptr<SymbTraceExplorerAutomaton> ptr;

    SymbTraceExplorerAutomaton(SymbAutomaton::ptr automaton_);
    
    /** Get a state set of possible starting states */
    virtual SymbStateSet getInitialStates() const;

    /** Get the set of possible successor states from the current state */
    virtual SymbStateSet getSuccessorStates() const;

    /** Get the set of possible successor states from the current state at depth steps 
     *  (all states reachable via a path of length steps) */
    virtual SymbStateSet getSuccessorStates(unsigned int steps) const;
    
    /** Get the set of transitions between from and to
     *  If canonical_transitions is true, the transition set will contain
     *  exactly one assignment per transition
     */
    virtual SymbTransitionSet getTransitionSetToState(const SymbState& to, 
                                                      bool canonical_transitions) const;

    /** Returns true if the transition set may change depending on the 
     *  history of the current trace. */
    virtual bool isHistoryDependant() const;

    /** Get the set of all valid transitions */
    virtual SymbTransitionSet getTransitionSet() const;

    /** Get an infinite trace starting from state from.
     *  This function currently throws not_supported_exception for SymbTraceExplorerAutomaton */
    virtual ReturnValue<SymbTraceLasso::ptr> getInfiniteTrace(const SymbState& from) const;

    /** Get an infinite trace extending the current trace.
     *  This function currently throws not_supported_exception for SymbTraceExplorerAutomaton */
    virtual ReturnValue<SymbTraceLasso::ptr> extendToInfiniteTrace() const;


  private:
    SymbAutomaton::ptr automaton;
  };

}

#endif
