/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "vereofy/system/symb_automaton/symb_trace_for_java.hpp"
#include "vereofy/system/symb_automaton/symb_trace.hpp"
#include "vereofy/system/symb_automaton/symb_component_instance.hpp"

#include "vereofy/common/weak_ptr_helper.hpp"

using namespace vereofy;

unsigned int SymbTrace_for_java::countStates() const {
  return access_weak_ptr(trace)->countStates();
}

unsigned int SymbTrace_for_java::countLabels() const {
  return access_weak_ptr(trace)->countLabels();
}

bool SymbTrace_for_java::isNull() const {
  return !(get_shared_ptr_from_weak_ptr(trace));
}

std::string SymbTrace_for_java::getState_xml(unsigned int index) const {
  const SymbState& state=access_weak_ptr(trace)->getState(index);
  std::stringstream out_xml;

  SymbComponentInstance::ptr system_automaton=access_weak_ptr(context)->getSystemAutomaton();
  if (!system_automaton) {
    throw "Can not access system automaton";
  }

  system_automaton->print_xml(out_xml, state);

  return out_xml.str();
}

std::string SymbTrace_for_java::getLabel_xml(unsigned int index) const {
  const SymbTransitionLabel& label=access_weak_ptr(trace)->getLabel(index);
  std::stringstream out_xml;

  SymbComponentInstance::ptr system_automaton=access_weak_ptr(context)->getSystemAutomaton();
  if (!system_automaton) {
    throw "Can not access system automaton";
  }

  system_automaton->print_xml(out_xml, label);

  return out_xml.str();
}

