/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "symb_trace_lasso.hpp"
#include "symb_component_instance.hpp"

using namespace vereofy;

void SymbTraceLasso::print(SymbComponentInstance& system, std::ostream& out) {
  if (prefix && prefix->countStates()>0) {
    out << "Prefix:\n";
    system.print(out, *prefix);
  }

  if (cycle && cycle->countStates()>0) {
    out << "\nCycle:\n";
    system.print(out, *cycle);
  }
}

void SymbTraceLasso::print_xml(SymbComponentInstance& system, std::ostream& out) const {
  if (prefix && prefix->countStates()>0) {
    xml_tag_open(out, "tracefragment", xml_parameter("type", "prefix"));
    out << "\n";
    system.print_xml(out, *prefix);
    xml_tag_close(out, "tracefragment");
    out << "\n";
  }

  if (cycle && cycle->countStates()>0) {
    xml_tag_open(out, "tracefragment", xml_parameter("type", "cycle"));
    out << "\n";
    system.print_xml(out, *cycle);
    xml_tag_close(out, "tracefragment");
    out << "\n";
  }
}

SymbTrace::ptr SymbTraceLasso::merge() const {
  SymbTrace::ptr trace(new SymbTrace(*prefix));
  
  if (cycle && cycle->countStates()>0) {
    trace->appendTrace(*cycle);
  }

  return trace;
}
