/***************************************************************************
 *   Copyright (C) 2011 Joachim Klein                                      *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_TRANSITION_LABEL_VAR_HPP
#define SYMB_TRANSITION_LABEL_VAR_HPP
#include "vereofy/common/setup.hpp"
#include "vereofy/common/functions.hpp"
#include "vereofy/common/shared_ptr_map.hpp"
#include "vereofy/system/transition_label_description.hpp"
#include "symb_flat_datadomain.hpp"
#include "vereofy/system/symb_automaton/var_positions.hpp"

#include "vereofy/common/vereofy_id.hpp"

namespace vereofy {

  class Symb_Transition_Label_Var : public VereofyID {
  public:
    typedef boost::shared_ptr<Symb_Transition_Label_Var> ptr;
    typedef boost::weak_ptr<Symb_Transition_Label_Var> weak_ptr;
    typedef shared_ptr_id_set<Symb_Transition_Label_Var::ptr>::type ptr_set;
    
    Symb_Transition_Label_Var(TransitionLabelDescription::wptr label_description_);
    Symb_Transition_Label_Var(TransitionLabelDescription::wptr label_description_,
                              VarPositions::ptr var_positions_in_out);

    const BDDFunction& getHasLabelVar() const {
      return label_var;
    }
    
    const BDDFunctionContainer& getValueVars() const {
      return *vecValueVars;
    }

    const BDDFunction getVars() const;
    
    Symb_DataDomain::ptr getDataDomain() const {
      return dataDomain;
    }
    
    BDDFunction has_label() const;
    BDDFunction value_equal(const BDDFunctionContainer& vecValueVars) const;
    bool hasValueDomain() const {return (bool)dataDomain;}
    
    string getVariableName() const;

    TransitionLabelDescription::wptr getTransitionLabelDescription() const { return label_description;}

    void print_vars(std::ostream& out, std::string name) const;
    void serialize_vars(std::ostream& out) const;

  private:
    typedef boost::shared_ptr<BDDFunctionContainer> value_vars_vector;
    Symb_Transition_Label_Var(TransitionLabelDescription::wptr label_description_,
                              const BDDFunctionContainer& variables) ;
    Symb_Transition_Label_Var(TransitionLabelDescription::wptr label_description_,
                              BDDFunction label_var_, 
                              value_vars_vector vecValueVars);

    TransitionLabelDescription::wptr label_description;
    Symb_DataDomain::ptr dataDomain;

    BDDFunction label_var;
    value_vars_vector vecValueVars;

    static int nr_of_labels;
  };
  
}

#endif
