/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_TRANSITION_SET_HPP
#define SYMB_TRANSITION_SET_HPP 

#include "symb_state.hpp"
#include "symb_transition.hpp"
#include "symb_transition_label.hpp"

namespace vereofy {
  
  class SymbTransitionSet {
  public:
    typedef boost::shared_ptr<SymbTransitionSet> ptr;
    
    SymbTransitionSet(const BDDFunction& delta_,
                      BDDFunction csStateVars_,
                      BDDFunction csLabelVars_) :
      delta(delta_),
      csStateVars(csStateVars_),
      csLabelVars(csLabelVars_) {
      BDD_CHECK(3, BDDChecks::isCubeSet(csStateVars),  "", csStateVars);
      BDD_CHECK(3, BDDChecks::isCubeSet(csLabelVars),  "", csLabelVars);
    }

    const BDDFunction& getDelta() const {
        return delta;
    }

    const BDDFunction& getCsStateVars() const {
      return csStateVars;
    }

    const BDDFunction& getCsLabelVars() const {
      return csLabelVars;
    }
    
    const BDDFunction getCsTransitionVars() const {
      return csStateVars & csLabelVars & csStateVars.moveDown();
    }

    bool isEmpty() const {
      return delta==BDDFunction::ZERO();
    }

    bool contains(const SymbTransition& transition) {
      ERROR_CHECK_ASSERT(transition.getCsStateVars()==csStateVars);
      ERROR_CHECK_ASSERT(transition.getCsLabelVars()==csLabelVars);

      return bdd_sets_have_intersection(delta, transition.getTransitionFunction());
    }

    SymbTransition pickTransition(const SymbState& from,
                                  const SymbState& to) {
      return SymbTransition::pickTransition(delta, from, to, csLabelVars);
    }

    SymbTransition popTransition(const SymbState& from,
                                 const SymbState& to) {
      SymbTransition transition=pickTransition(from, to);
      delta&=!transition.getTransitionFunction();
      return transition;
    }                                      

    SymbTransition randomTransition() const {
      ERROR_CHECK_ASSERT(delta!=BDDFunction::ZERO());
      return SymbTransition(delta.getRandomAssignment(getCsTransitionVars()),
                            csStateVars, csLabelVars);
    }


  private:
    BDDFunction delta;
    BDDFunction csStateVars;
    BDDFunction csLabelVars;
  };
}

#endif
