/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYMB_VARIABLE_ID_HPP
#define SYMB_VARIABLE_ID_HPP

#include <boost/shared_ptr.hpp>
#include <vector>

namespace vereofy {
  
  class SymbVariableID {
  public:
    typedef boost::shared_ptr<SymbVariableID> ptr;
    typedef std::vector<SymbVariableID> vector;

    enum vid_type_t {VID_PORT, VID_STATE, VID_GLOBAL};

    SymbVariableID(vid_type_t type_, unsigned int bit_=0);
    virtual ~SymbVariableID() {};

    vid_type_t getType() const;
    unsigned int getBit() const;

    virtual bool equals(const SymbVariableID& other) const;
    virtual bool less(const SymbVariableID& other) const;

    bool operator==(const SymbVariableID& other) const;
    bool operator<(const SymbVariableID& other) const;

    boost::shared_ptr<SymbVariableID> getNextBit() const;
    void setNextBit(boost::shared_ptr<SymbVariableID> nextBit_);

    boost::shared_ptr<SymbVariableID> getPrevBit() const;
    void setPrevBit(boost::shared_ptr<SymbVariableID> prevBit_);

  private:
    vid_type_t type;
    unsigned int bit;

    SymbVariableID::ptr nextBit;
    SymbVariableID::ptr prevBit;
  };

}


#endif 
