/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "symb_variable_id_port.hpp"

namespace vereofy {
  
  SymbVariableID_Port::SymbVariableID_Port(ComponentInstance::ptr ci_,
                                           PortDescription::ptr port_,
                                           unsigned int bit_) :
    SymbVariableID(VID_PORT, bit_),
    ci(ci_),
    port(port_) {}
  

  bool SymbVariableID_Port::equals(const SymbVariableID& other_) const {
    if (!SymbVariableID::equals(other_))
      return false;

    const SymbVariableID_Port& other=static_cast<const SymbVariableID_Port&>(other_);
   
    if (ci.get()!=other.ci.get())
      return false;

    if (port.get()!=other.port.get())
      return false;

    return true;
  }

  bool SymbVariableID_Port::less(const SymbVariableID& other_) const {
    if (SymbVariableID::less(other_))
      return true;

    if (other_.SymbVariableID::less(*this))
      return false;

    // SymbVariableID parts are equal

    const SymbVariableID_Port& other=static_cast<const SymbVariableID_Port&>(other_);
    
    if (ci.get()<other.ci.get()) 
      return true;

    if (ci.get()>other.ci.get()) 
      return false;

    if (port.get()<other.port.get()) 
      return true;

    if (port.get()>other.port.get()) 
      return false;

    return false;
  }

  ComponentInstance::ptr SymbVariableID_Port::getComponentInstance() const {
    return ci;
  }

  PortDescription::ptr SymbVariableID_Port::getPortDescription() const {
    return port;
  }
  

}

