/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "symb_variable_id_state.hpp"

namespace vereofy {
  
  SymbVariableID_State::SymbVariableID_State(ComponentInstance::ptr ci_,
                                             StateDescription::ptr state_,
                                             unsigned int bit_) :
    SymbVariableID(VID_STATE, bit_),
    ci(ci_),
    state(state_) {}
  

  bool SymbVariableID_State::equals(const SymbVariableID& other_) const {
    if (!SymbVariableID::equals(other_))
      return false;
    
    const SymbVariableID_State& other=static_cast<const SymbVariableID_State&>(other_);
    
    if (ci.get()!=other.ci.get())
      return false;
    
    if (state.get()!=other.state.get())
      return false;
    
    return true;
  }

  bool SymbVariableID_State::less(const SymbVariableID& other_) const {
    if (SymbVariableID::less(other_))
      return true;

    if (other_.SymbVariableID::less(*this))
      return false;

    // SymbVariableID parts are equal

    const SymbVariableID_State& other=static_cast<const SymbVariableID_State&>(other_);
    
    if (ci.get()<other.ci.get()) 
      return true;

    if (ci.get()>other.ci.get()) 
      return false;

    if (state.get()<other.state.get()) 
      return true;

    if (state.get()>other.state.get()) 
      return false;
    
    return false;
  }
  
  ComponentInstance::ptr SymbVariableID_State::getComponentInstance() const {
    return ci;
  }
  
  StateDescription::ptr SymbVariableID_State::getStateDescription() const {
    return state;
  }
  

}

