/***************************************************************************
*   Copyright (C) 2008-2009 by Tobias Blechmann                           *
*   vereofy@blechmann.de                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef VARMANAGER_H
#define VARMANAGER_H

#include "vereofy/setup.hpp"
#include "vereofy/system/datadomain.hpp"

#include <iostream>
#include <utility>
#include <vector>
#include <list>
#include <string>
#include <map>

#include <boost/utility.hpp>
#include <boost/smart_ptr.hpp>

namespace vereofy {

  struct StateVars {
    boost::shared_ptr<BDDFunctionContainer> X;
    boost::shared_ptr<BDDFunctionContainer> Y;
    boost::shared_ptr<BDDFunctionContainer> Z;
  };

  //Singleton
  class VarManager: boost::noncopyable {
  public:
    static VarManager* getInstance();

    StateVars findOrAddGlobalState(const std::string& name, unsigned int bits);
    BDDFunctionContainer findOrAddGlobalBits(const std::string& name, unsigned int bits);

    StateVars createStateVars(unsigned int number, bool prepend=true);
    // if var_positions_in_out is too short, append new positions
    StateVars createStateVars(unsigned int number, BDDFunctionContainer& var_positions_in_out);

    BDDFunctionContainer createVars(const std::string& prefix, unsigned int number, bool prepend=true);
    BDDFunctionContainer createVars(const std::string& prefix, unsigned int number, BDDFunctionContainer& var_positions_in_out);



    BDDFunction getEncoding(unsigned long, const BDDFunctionContainer&);

    void enableDynamicReordering();
    void disableDynamicReordering();
    void setDynamicReorderingMaxGrowth(float maxGrowth);
    void setDynamicReorderingTimeout(unsigned int timeout_ms);

    void enableSelectiveReordering(unsigned int level);
    void disableSelectiveReordering();
    void selectiveReorderingHook(unsigned int level, bool force_reordering=false);
    
    struct ReorderLock {
      ReorderLock();
      ~ReorderLock();
    };

    void writeVarOrder(std::ostream& out);
    void readVarOrder(std::istream& in);

    void reset();
  private:
    VarManager();
    void init();
    BDDFunctionContainer createVarsInGroup(const std::vector<std::string>& names,
                                           unsigned int position);
    void setReorderingState();

    void acquireReorderingLock();
    void releaseReorderingLock();

    unsigned long numberOfStateVars;

    std::map<std::string, StateVars> globalStateVars;
    std::map<std::string, BDDFunctionContainer> globalBits;
    //    DataDomain* dataDomain;

    typedef boost::shared_ptr< BDDGroup > BDDGroup_ptr;
    typedef std::list< BDDGroup_ptr > BDDGroupList;

    BDDGroupList group_list;

    bool dynamic_reorder_enabled;
    unsigned int reorder_lock;
    unsigned int selective_reorder_level;
  };

} //end namespace

#endif
