/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SYSTEM_CONTEXT_HPP
#define SYSTEM_CONTEXT_HPP

#include <boost/shared_ptr.hpp>
#include <boost/enable_shared_from_this.hpp>
#include "vereofy/reo_parser/reo_parser_types.hpp"
#include "vereofy/system/datadomain.hpp"
#include "vereofy/system/replacement_manager.hpp"
#include "vereofy/system/type_modification_registry.hpp"
#include "vereofy/common/return_value.hpp"
#include "vereofy/common/weak_ptr_helper.hpp"
#include "vereofy/system/component_type_identifier.hpp"

#include <map>

namespace vereofy {

// fwd
class ComponentInstance;

class SystemContext : public boost::enable_shared_from_this<SystemContext> {
public:
  typedef boost::shared_ptr<SystemContext> ptr;
  typedef boost::weak_ptr<SystemContext> wptr;

  typedef std::map<std::string, bool> flag_map;

  static SystemContext::ptr newSystemContext();

  void addBasicChannel(const string& name);
  void loadBuiltins();

  boost::shared_ptr<ComponentInstance> instantiateSystem(const ComponentTypeIdentifier& component_type_id,
                                                         bool strict);

  void requireDataDomain();

  symbol_table_t::ptr getGlobalSymbols();
  DataDomain::ptr getDataDomain();

  TypeModificationRegistry::ptr getTypeModificationRegistry();

  void addIncludeDirs(const std::vector<string>& include_dirs);
  ReturnValue<string> findFile(const std::string& filename) const;

  void markFileAsIncluded(const std::string& filename);
  bool isFileMarkedAsIncluded(const std::string& filename);

  bool ignoreMultipleIncludes() const;
  void setIgnoreMultipleIncludes(bool value=true);

  void setFlag(const std::string& flag, bool enabled);
  bool getFlag(const std::string& flag) const;
  const flag_map& getFlagMap() const;

  ReplacementManager::ptr getReplacementManager();
private:
  SystemContext();

  symbol_table_t::ptr global_symbols;
  std::vector<string> include_dirs;
  std::set<string> included_files;
  bool builtins_were_loaded;

  flag_map flags;

  bool ignore_multiple_includes;

  ReplacementManager::ptr replacement_manager;

  TypeModificationRegistry::ptr type_modification_registry;

  void initStandardDefinitions();
};

}
#endif
