/***************************************************************************
 *   Copyright (C) 2010 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "vereofy/system/system_statistics.hpp"
#include "vereofy/system/symb_automaton/symb_datadomain.hpp"

namespace vereofy {
  SystemStatistics::SystemStatistics(ComponentInstance::ptr system,
                                     DataDomain::ptr default_port_dd) :
    ci_count(0), dataflow_count(0), state_bits(0), distinct_state_values(1) {

    Symb_DataDomain::ptr sdd=Symb_DataDomain::getFor(default_port_dd);
    port_data_bits=sdd->getLogSize();
    port_distinct_values=sdd->countValidValues();
    run(system);
  }

  void SystemStatistics::printReport(std::ostream& out) {
    out << "Statistics:\n";
    out << "Number of component instances: " << ci_count << "\n";
    out << "Number of different component types: " << type_names.size() << "\n";
    out << "Number of total dataflow locations: " << dataflow_count << "\n";
    out << "Number of bits needed to encode the data at each dataflow location: " << port_data_bits <<
      " (up to " << port_distinct_values << " distinct values) \n";

    out << "Number of bits needed to encode the state variables of the system: " << state_bits <<
      " (up to " << distinct_state_values << " distinct values) \n";
  }
  
  void SystemStatistics::run(ComponentInstance::ptr ci) {
    std::string type_name=ci->getTypeName();
    type_names.insert(type_name);

    ci_count++;
    
    if (!ci->getParent()) {
      // the in/out ports are already counted on the higher level
      dataflow_count+=
        ci->getInterface()->getInPortCount()
        +ci->getInterface()->getOutPortCount();
    }
    
    dataflow_count+=
      +ci->getInterface()->getInternalPortCount()
      +ci->getInterface()->getAnonymousPortCount();

    for (unsigned int i=0;i<ci->getInterface()->getStateDescriptionCount();i++) {
      Symb_DataDomain::ptr sdd=Symb_DataDomain::getFor(ci->getInterface()->getStateDescription(i)->getType());
      state_bits+=sdd->getLogSize();
      distinct_state_values*=sdd->countValidValues();
    }

    for (unsigned int i=0;i<ci->getInterface()->getSubcomponentCount();i++) {
      run(ci->getSubcomponent(ci->getInterface()->getSubcomponentName(i)));
    }
  }
}

