/***************************************************************************
 *   Copyright (C) 2011 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/**
 @author Joachim Klein
*/

#ifndef TRANSITION_LABEL_DESCRIPTION_H
#define TRANSITION_LABEL_DESCRIPTION_H

#include "vereofy/system/datadomain.hpp"
#include "vereofy/common/vereofy_id.hpp"
#include "vereofy/common/xml_helpers.hpp"
#include <boost/smart_ptr.hpp>

namespace vereofy {

  class TransitionLabelDescription : public VereofyID {
  public:
    typedef boost::shared_ptr<TransitionLabelDescription> ptr;
    typedef boost::weak_ptr<TransitionLabelDescription> wptr;
    
    enum label_type_t {GENERIC, MUTEX};

    TransitionLabelDescription(label_type_t label_type_,
                               std::string label_name_,
                               DataDomain::ptr value_domain_ = DataDomain::ptr() ) : 
      label_type(label_type_), 
      label_name(label_name_),
      value_domain(value_domain_) {
      
    }
    
    virtual ~TransitionLabelDescription() {}
    
    label_type_t getType() const {return label_type;}
    virtual const std::string getName() const {return label_name;}

    DataDomain::ptr getValueDomain() const {return value_domain;}
    bool hasValueDomain() const {return (bool)value_domain;}

    std::string getTypeString() const {
      switch (label_type) {
      case GENERIC: return "generic";
      case MUTEX: return "mutex";
      }
      THROW_UNIMPLEMENTED();
    }

    void print(std::ostream& out) {
      out << getName();
      if (hasValueDomain()) {
        out << " (" << getValueDomain()->toString() << ")";
      }
    }
    
    void print_xml(std::ostream& out, unsigned int index) {
      xml_tag_open(out, "labeldescription", 
                   xml_parameter("name", getName())+xml_parameter("type", getTypeString())+
                   xml_parameter("index", boost::lexical_cast<std::string>(index)));
      
      if (hasValueDomain()) {
        getValueDomain()->to_xml(out);
      }
      xml_tag_close(out, "labeldescription");
    }



  private:
    label_type_t label_type;
    std::string label_name;
    DataDomain::ptr value_domain;
  };
} //end namespace

#endif
