/***************************************************************************
 *   Copyright (C) 2009 by Joachim Klein                                   *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "type_modification_registry.hpp"
#include <boost/algorithm/string.hpp>
#include "vereofy/common/errors.hpp"

using namespace vereofy;

void TypeModificationRegistry::addAtomicProposition(const std::string& type_specifier,
                                                    const std::string& ap_name,
                                                    const std::string& ap_formula) {
  APSpecAttachement a;
  a.type_specifier=type_specifier;
  a.ap_spec.name=ap_name;
  a.ap_spec.specification=ap_formula;

  ap_attachements.push_back(a);
}
    
AtomicPropositionSpec::vector
TypeModificationRegistry::getAtomicPropositionsForType(const std::string& type_name) const {
  AtomicPropositionSpec::vector result;

  for (std::vector<APSpecAttachement>::const_iterator it=ap_attachements.begin();
       it!=ap_attachements.end();
       ++it) {
    if (matchesTypeSpecifier(type_name, it->type_specifier)) {
      result.push_back(it->ap_spec);
    }
  }

  return result;
}


bool TypeModificationRegistry::matchesTypeSpecifier(const std::string& type_name, const std::string& type_specifier) {
  if (boost::algorithm::ends_with(type_specifier, "<>")) {
    // match all type_names of the form  TYPE<...>
    std::string type_specifier_base=type_specifier.substr(0, type_specifier.length()-2); // strip <>
    std::size_t langle_pos = type_name.find("<");
    if (langle_pos!=std::string::npos) {
      std::string type_name_base=type_name.substr(0, langle_pos);
      
      return (type_specifier_base == type_name_base);
    }
  }

  // complete match between specifier and type
  return type_name==type_specifier;
}
