#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <stdlib.h>
#include <set>
#include <boost/shared_ptr.hpp>
#include <boost/foreach.hpp>
#include <ctime>
#define  foreach BOOST_FOREACH
using namespace std;

// Settings
/////////////////////////////////////////////////
int data_ranges[]	= {6};
int port_ranges[]	= {6};
int nbo_atoms[]		= {20};
int seed_start		= 0;
int nbo_runs		= 10;
string path_to_vereofy_run ="./";
string outputfile_name ="result.csv";
string csv_separator = ",";

// Code
//////////////////////////////////////////////////
#define NBO_OPER	6
int a_data_range = data_ranges[0]+1;
int a_port_range = port_ranges[0]+1;
int a_nbo_atoms	 = nbo_atoms[0];

string minimize[] = {"none","brzozowski","hopcroft"};

struct pia{
	int ver;
	int nboop;
	string op;
	pia(int v,int o, string ope): ver(v), nboop(o), op(ope){};
};
pia	operatoren[]	={
		pia(5,1,"*"),
		pia(5,1,"+"),
		pia(10,1,"!"),
		pia(10,2,";"),
		pia(10,2,"|"),
		pia(10,2,"&")
};

string randomSet(){
	string atom = "\"{";
	char a[255];
	int range = rand()%a_port_range;
	int num = rand()%a_port_range;
	sprintf(a, "%d", num);
	atom += "N[" + (string)a + "]";
	set<int>	reached;
	reached.insert(num);
	for(int i=0; i<range;i++){
		while (reached.find(num) != reached.end())
			num = rand()%a_port_range;
		reached.insert(num);
		sprintf(a, "%d", num);
		atom += ",N[" + (string)a + "]";
	}
	return atom + "}\"";
}

string randomCompare(){
	char a[255];
	string atom ="\"";
	string ops[] = {"==", "<=", "!="};
	int port1 = rand()%a_port_range;
	sprintf(a, "%d", port1);
	atom += "#N[" + (string)a + "]";
	int op = rand()%3;
	atom += ops[op];
	int da = rand()%2;
	if (da == 0){
		int port2 = rand()%a_port_range;
		while (port1 == port2)
			port2 = rand()%a_port_range;
		sprintf(a, "%d", port2);
		atom += "#N[" + (string)a + "]";	
	} else {
		int data = rand()%a_port_range;
		sprintf(a, "%d", data);
		atom += (string)a;	
	}	
	return atom + "\"";
}

string randomAtom(){
	int ran = rand()%2;
	if (ran == 0)
		return "(" + randomSet() +")";
	return "(" + randomCompare() + ")";
}

int	count_op[NBO_OPER];
string randomExpression(){
	int sum = 0;
	for (int i=0;i<NBO_OPER; i++){
		sum += operatoren[i].ver;
		count_op[i] = 0;
	}
	vector<boost::shared_ptr<string> > expressions;
	boost::shared_ptr<string> act;
	for(int j=0; j<a_nbo_atoms; j++){
		act.reset(new string(randomAtom()));
		expressions.push_back( act );
	}
	while (expressions.size() > 1){
		int zufall = rand()%sum + 1;
		int i=0;
		int help=operatoren[0].ver;
		while (zufall > help){
			i++;
			help += operatoren[i].ver;
		}
		string op = operatoren[i].op;
		count_op[i]++;
		if (operatoren[i].nboop == 1){
			int pos1 = rand()%expressions.size();
			if (op == "!")
				*expressions[pos1] = "(" + op + *expressions[pos1] + ")";
			else
				*expressions[pos1] = "(" + *expressions[pos1] + op + ")";
		}else{
			int pos1 = rand()%expressions.size();
			int pos2;
			if (expressions.size() > 2)
				pos2 = (pos1 + rand()%(expressions.size()-2) + 1)%expressions.size();
			else
				pos2 = (pos1 + 1)%expressions.size();
			string op2 = *expressions[pos2];
			expressions.erase( expressions.begin() + pos2 );
			string a= "(" + *expressions[pos1] + op + op2 + ")";
			*expressions[pos1] = a;
		}
	}
	return *expressions[0];
}

int main(void)
{
	//cout.precision(4); 
	srand(seed_start);
	char a[255], b[255];
	ofstream outputfile;
	outputfile.open(outputfile_name.data());
	int data_ranges_size = sizeof(data_ranges) / sizeof(int);
	int port_ranges_size = sizeof(port_ranges) / sizeof(int);
	int nbo_atoms_size = sizeof(nbo_atoms) / sizeof(int);
	int gesamt = data_ranges_size*port_ranges_size*nbo_atoms_size*nbo_runs;	
	int runs = 0;
	for (int data=0; data<data_ranges_size; data++){
		a_data_range = data_ranges[data]+1;
		sprintf(a, "%d", a_data_range);
		for (int port=0; port<port_ranges_size; port++){
			a_port_range = port_ranges[port]+1;
			sprintf(b, "%d", a_port_range);
			for (int atoms=0; atoms<nbo_atoms_size; atoms++){
				a_nbo_atoms = nbo_atoms[atoms];
				for (int run=0; run<nbo_runs; run++){
					runs++;
					string exp = randomExpression();	
					cout << "Runs: " << runs << "/" << gesamt << "    Data: " << data_ranges[data] << "    Ports: " << port_ranges[port] << "    Atoms: " << nbo_atoms[atoms] << "    Run: " << run << endl << flush;
					cout << "-------------------------------------------------------------------------------" << endl << flush;
					foreach(string min, minimize){
						string command = path_to_vereofy_run + "vereofy-run --load-builtins  --data-domain='int(0,"+ (string)a + ")' --system='SEQUENCER<" + (string)b + ">' --formula='ASL<< E[" + exp + "]false >>' --minimize-dfa=" + min;
						char path[5000];
						time_t start,end;
						time (&start);		
						
						//system(command.data());
						FILE * fp = popen(command.data(), "r");
						while (fgets(path, 5000, fp) != NULL);
							//cout << path;
						pclose(fp);
			
						time (&end);
						double elapsedTime = difftime(end,start);
						cout << elapsedTime <<  "      " << flush;
						outputfile << elapsedTime << csv_separator;
					}
					cout << endl << "-------------------------------------------------------------------------------" << endl << endl << flush;
					outputfile << "\n";
				}
			}
		}
	}
	outputfile.close();
	return 0;
}
