/***************************************************************************
*   Copyright (C) 2008-2009 by Johannes Osterholzer                       *
*   s5892660@inf.tu-dresden.de                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <iostream>
#include <ctype.h>
#include <boost/program_options.hpp>
#include "vereofy/synthesis/sdexp/scheduleddataexpression.hpp"

using namespace std;
using namespace boost;
using namespace boost::program_options;

int main(int argc, char **argv)
{
  try {
    int lo, hi;
    options_description desc("Arguments");
    desc.add_options()
      ("help", "show this help")
      ("datadomainmin,l", value<int>(&lo)->default_value(0), "bottom end of data domain")
      ("datadomainmax,h", value<int>(&hi)->default_value(1), "top end of data domain")
      ("sources,i", value<string>(), "source nodes, uppercase chars separated by commata (or something) (required)")
      ("sinks,o", value<string>(), "sink nodes, uppercase chars separated by commata (or something) (required)")
      ("expression,e", value<string>(), "scheduled data expression, (required)")
      ;

    variables_map vm;
    store(parse_command_line(argc, argv, desc), vm);
    notify(vm);

    if(vm.count("help")) {
      cout << desc << "\n";
      return 1;
    }

    if(vm.count("sources") != 1) {
      cout << "No sources specified\n";
      cout << desc << "\n";
      return 1;
    }

    if(vm.count("sinks") != 1) {
      cout << "No sinks specified\n";
      cout << desc << "\n";
      return 1;
    }

    if(vm.count("expression") != 1) {
      cout << "No Expression specified\n";
      cout << desc << "\n";
      return 1;
    }

    set<string> sources;
    string srcstr = vm["sources"].as<string>();
    for(string::iterator c = srcstr.begin(); c != srcstr.end(); ++c) {
      if(isupper(*c))
	sources.insert(string(1, *c));
    }

    set<string> sinks;
    string snkstr = vm["sinks"].as<string>();
    for(string::iterator c = snkstr.begin(); c != snkstr.end(); ++c) {
      if(isupper(*c))
	sinks.insert(string(1, *c));
    }

    string str = vm["expression"].as<string>();

    cerr << "parsing string \"" << str << "\"\n";
 
    vector<string> sourcesVec;
    for(set<string>::iterator it = sources.begin(); it != sources.end(); ++it)
      sourcesVec.push_back(*it);
    ScheduledDataExpression::setSources(sourcesVec);

    vector<string> sinksVec;
    for(set<string>::iterator it = sinks.begin(); it != sinks.end(); ++it)
      sinksVec.push_back(*it);
    ScheduledDataExpression::setSinks(sinksVec);

    ScheduledDataExpression::setDataDomain(pair<int,int>(lo,hi));

    ScheduledDataExpression::ptr exp = ScheduledDataExpression::readFromString(str);
    cerr << "\n --- Parsed as: ---\n";
    exp->printTree();

    cerr << "\n --- Canonical Form: ---\n";
  
    exp->canonicalForm()->printTree();
  
    cerr << "\n\n";

    cout << exp->toREOCircuit();

    return 0;
  } catch(const char *err) {
    cerr << "Exception: " << err << endl;
  }
}
