/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef VEREOFY_CONTEXT_HPP
#define VEREOFY_CONTEXT_HPP

#include <string>

#include "vereofy/modelcheck/checkers/ModelCheckingResult.hpp"
#include "vereofy/system/system_context.hpp"
#include "vereofy/common/OptionContainer.hpp"
#include "vereofy/common/stopwatch.hpp"


// forward declarations

namespace vereofy {
  class ComponentInstance;
  class SymbComponentInstance;
  class RestrictionManager;
  class CSVFile;
};

class VereofyContext {
public:
  typedef boost::shared_ptr<VereofyContext> ptr;

  VereofyContext();
  ~VereofyContext();

  void new_parameter_section();
  void add_parameters(unsigned int argc, char **argv);
  void add_parameter(const std::string& parameter);

  bool apply_parameters();
  //  void handle_parameters(const string& parameters);

  void parse(const std::string& content);
  void parse_file(const std::string& file_name);

  void instantiate_system(const std::string& name);
  void print_interface(ostream& out);
  void print_interface_xml(ostream& out);
  void print_circuit_dot(ostream& out);
  void export_as_reo_xml(ostream& out, const std::vector<std::string>& input_files);

  void build_system(boost::shared_ptr<vereofy::RestrictionManager> restriction_manager=boost::shared_ptr<vereofy::RestrictionManager>());
  void print_automaton_dot(ostream& out);

  // handle varorder writing if --write-varorder parameters are set
  void varorder_write();

  unsigned int random_trace(unsigned int steps, bool xml_format, std::ostream& out);

  vereofy::ModelCheckingResult::ptr model_check(const std::string& formula);
  vereofy::ModelCheckingResult::ptr bisimulation_check(const std::string& other_system);

  int run(unsigned int argc, char**argv);
  
  boost::shared_ptr<vereofy::SymbComponentInstance> getSystemAutomaton();

private:
  boost::shared_ptr<vereofy::SystemContext> system_context;
  boost::shared_ptr<vereofy::SymbComponentInstance> automaton;
  boost::shared_ptr<vereofy::ComponentInstance> system;
  boost::shared_ptr<vereofy::CSVFile> csv_file;

  vereofy::StopWatch build_phase_stopwatch;
  vereofy::StopWatch check_phase_stopwatch;
  
  void initialize();
  void enableDynamicReordering();
  void disableDynamicReordering();

  void print_xml_vereofy_trace_open(std::ostream& out, bool output_interface);
  void print_xml_vereofy_trace_close(std::ostream& out);

  int random_trace_iterative(unsigned int steps, bool xml_format, std::ostream& out);

  static bool initialized;

  // options
  vector< vector<string> > cmd_line_parameters;

  bool bisim_check_mode;

  vereofy::OptionContainer modelchecker_options;
  string system_name;
  string check_equivalence_with;
  string formula;
  vector<string> param_input_files;
  string dot_file;
  string dot_circuit;
  string extract_explicit_file;
  string trace_xml_file;
  string export_circuit_reo_xml;
  string export_circuit;
  string stat_file;
  string data_domain;
  string print_trace_to;
  bool print_varorder;
  bool param_print_interface;
  bool param_automatic_includes;
  vector<string> parameters;
  vector<string> trace_points;
  vector<string> include_dirs;
  vector<string> param_constant_defs;
  vector<string> flags;
  bool dynamic_reordering;
  float dynamic_reordering_max_growth;
  unsigned int dynamic_reordering_timeout;
  bool dynamic_reordering_during_build;
  bool dynamic_reordering_during_modelchecking;
  bool dynamic_reordering_reverse;
  double unique_table_load_factor;
  unsigned int selective_reorder_level;
  bool without_restrict;
  bool strict;
  bool help;
  bool param_load_builtins;
  bool param_parse_only;
  unsigned int param_gcd_building;
  unsigned int param_gcd_checking;
  string read_varorder_file;
  string read_varorder_file_old;
  string varorder_dir;
  string write_varorder_file;
  string write_varorder_file_old;
  signed int random_trace_steps;
  signed int random_trace_steps_iterative;
  unsigned int param_constrain_reachable_components;
  unsigned int param_constrain_reachable_circuits;
  bool param_retain_hidden_ports;
  bool param_retain_top_level_ports;
  bool param_abstract_data_domain;
  bool param_show_all_ports;
  bool param_optimize_ports;
  bool param_no_shared_data;
  bool param_hide_internals;
  bool param_ignore_multiple_includes;
  bool param_show_system_stats;
  bool param_check_result;
};


#endif
