/***************************************************************************
 *   Copyright (C) 2008-2009 by Joachim Klein                              *
 *   vereofy@tcs.inf.tu-dresden.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef VEREOFY_CONTEXT_FOR_JAVA_HPP
#define VEREOFY_CONTEXT_FOR_JAVA_HPP

#include <string>
#include <vector>
#include <boost/shared_ptr.hpp>

#include "vereofy/common/interruptible.hpp"

#include "vereofy/reo_parser/REOExecuteException.hpp"
#include "vereofy/reo_parser/REOParseException.hpp"

#include "vereofy/modelcheck/checkers/ModelCheckingResult_for_java.hpp"
#include "vereofy/system/symb_automaton/symb_trace_for_java.hpp"

// forward

class VereofyContext;
namespace vereofy {
  class ModelCheckingResult;
  class SymbTrace;
};

class VereofyContextForJava {
public:
  VereofyContextForJava();
  ~VereofyContextForJava();

  void new_parameter_section();
  void add_parameter(const std::string& parameter);
  void apply_parameters();

  void parse(const std::string& content);
  void parse_file(const std::string& file_name);

  void instantiate_system(const std::string& name);
  std::string print_interface();
  std::string print_interface_xml();
  void print_circuit_dot(const std::string& filename);

  // has to have instantiated system
  void export_to_reo_xml(const std::string& filename, const std::string& system_filename);

  void build_system();
  void print_automaton_dot(const std::string& filename);

  unsigned int model_check(const std::string& formula);
  unsigned int bisimulation_check(const std::string& system_1, const std::string& system_2);
  bool model_checking_passed(unsigned int index);
  std::string model_checking_info(unsigned int index);

  ModelCheckingResult_for_java model_checking_result(unsigned int index);
  SymbTrace_for_java get_random_trace(unsigned int length);

  std::string get_random_trace_xml(unsigned int steps);

  std::string get_statistics();

  static std::string get_version();

  void asyncRaiseInterruptFlag();

private:
  boost::shared_ptr<VereofyContext> context;
  std::vector<boost::shared_ptr<vereofy::ModelCheckingResult> > modelchecking_results;
  boost::shared_ptr<vereofy::SymbTrace> random_trace;
};


#endif
