%module vereofy

%{
#include "vereofy/ui/vereofy_context_for_java.hpp"
#include "vereofy/modelcheck/checkers/ModelCheckingResult_for_java.hpp"
#include "vereofy/system/symb_automaton/symb_trace_for_java.hpp"
%}

%include "std_string.i"

%exception {
  try {
     $action
  } catch (const char* err) {
    jclass clazz = jenv->FindClass("de/vereofy/jni/VereofyException");
    jenv->ThrowNew(clazz, err);
    return $null;
  } catch (std::string& err) {
    jclass clazz = jenv->FindClass("de/vereofy/jni/VereofyException");
    jenv->ThrowNew(clazz, err.c_str());
    return $null;
  } catch (vereofy::interrupt_exception& interrupted) {
    jclass clazz = jenv->FindClass("de/vereofy/jni/VereofyInterruptedException");
    jenv->ThrowNew(clazz, "Interrupted");
    return $null;
  } catch (vereofy::REOParseException& rpe) {
    jstring js_type=jenv->NewStringUTF("parse");
    jstring js_filename=jenv->NewStringUTF(rpe.getFile().c_str());
    jstring js_message=jenv->NewStringUTF(rpe.getMessage().c_str());
    jstring js_location=jenv->NewStringUTF(rpe.getLocation().c_str());

    jclass clazz = jenv->FindClass("de/vereofy/jni/VereofyExceptionWithLocation");
    jmethodID jmethodid_constructor = 
      jenv->GetMethodID(clazz, "<init>", "(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V" );
    jobject j_exception =      
      jenv->NewObject(clazz, jmethodid_constructor, js_type, js_message, js_filename, js_location);

    jenv->Throw((jthrowable)j_exception);
    return $null;
  } catch (vereofy::REOExecuteException& ree) {
    jstring js_type=jenv->NewStringUTF("execute");
    jstring js_filename=jenv->NewStringUTF(ree.getFile().c_str());
    jstring js_message=jenv->NewStringUTF(ree.getMessage().c_str());
    jstring js_location=jenv->NewStringUTF(ree.getLocation().c_str());

    jclass clazz = jenv->FindClass("de/vereofy/jni/VereofyExceptionWithLocation");
    jmethodID jmethodid_constructor = 
      jenv->GetMethodID(clazz, "<init>", "(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V" );
    jobject j_exception =      
      jenv->NewObject(clazz, jmethodid_constructor, js_type, js_message, js_filename, js_location);

    jenv->Throw((jthrowable)j_exception);
    return $null;
  } catch (std::exception& err) {
    jclass clazz = jenv->FindClass("de/vereofy/jni/VereofyException");
    jenv->ThrowNew(clazz, err.what());
    return $null;
  }
}

%include "vereofy/ui/vereofy_context_for_java.hpp"
%include "vereofy/modelcheck/checkers/ModelCheckingResult_for_java.hpp"
%include "vereofy/modelcheck/checkers/ModelCheckingResultTrace_for_java.hpp"
%include "vereofy/system/symb_automaton/symb_trace_for_java.hpp"
