#! /usr/bin/perl -w

use strict;

my $csv_file=shift;
my $stat_file=shift;
my @key_value_pairs;

while (@ARGV) {
  my $key=shift;
  my $value=shift;
  push @key_value_pairs, [$key, $value];
}

my @csv;
if (-e $csv_file) {
  my $csv_stream;
  open ($csv_stream, "< $csv_file") or die;
  @csv=read_csv($csv_stream);
  close($csv_stream);
} 

if (!@key_value_pairs) {
  if (! @csv) {
    push @key_value_pairs, ["Key", 0];
  } else {
    push @key_value_pairs, ["Key", scalar @{ $csv[1] }];  # key is index+1
  }
}

my $stat_stream;
open($stat_stream, "< $stat_file") or die;
my @stat_csv=read_csv($stat_stream);


foreach my $kv (reverse @key_value_pairs) {
  unshift @{$stat_csv[0]}, $kv->[0];
  unshift @{$stat_csv[1]->[0]}, $kv->[1];
}

if (! @csv) {
  @csv=@stat_csv;
} else {
  # add stat rows
  push @{$csv[1]}, @{$stat_csv[1]};
}

my $csv_out;
open($csv_out, "> $csv_file") or die;
write_csv($csv_out, @csv);
close($csv_out);


sub read_csv {
  my $file=shift;

  my @headers;
  my @rows;

  my $line;
  if (defined($line=<$file>)) {
    chomp($line);
    @headers=split(/;/, $line);
  }

  while (defined($line=<$file>)) {
    chomp($line);
    my @row=split(/;/, $line);
    push @rows, \@row;
  }

  return (\@headers, \@rows);
}

sub write_csv {
  my $file=shift;
  my $header=shift;
  my $rows=shift;

  print $file join(';', @$header), "\n";
  my $r;
  foreach $r (@$rows) {
    print $file join(';', @$r), "\n";
  }
}
