#! /usr/bin/perl -w

use strict;

die "Please call script in reo_parser directory!" if (!-d '../reo_parser');

open(IN, "< reo_parser_types.hpp") or die;
my @reo_parser_types_hpp = <IN>;
close(IN);

my $state = 0;

my $id = 0;
my $first_id = 0;

my @parser_id_idx;
my %parser_ids;

foreach (@reo_parser_types_hpp) {
  chomp;
  s{//.+$}{};

  if ($state == 0) {
    if (/enum rule_id/) {
      $state=1;
      next;
    }
  } elsif ($state == 1) {
    if (/}/) {
      if (/\s*}/) {
        last;
      } else {
        die "enum rule_id { } extraction: } not on individual line!\n";
      }
    }

    if (/^\s*(ID[a-zA-Z_]+)\s*(=\s*(\d+))?,?\s*$/) {
      die "Multiple definitions of $1!" if defined $parser_ids{$1};

      if (defined $3) {
        # initial enum value
        if ($3 == 1 && $id == 0) {
          $first_id = $id = $3;
        } else {
          die "Can not handle enum initializer not equal to one";
        }
      }

      $parser_id_idx[$id] = $1;
      $parser_ids{$1} = $id;

      $id++;
      next;
    }    

    if (/^\s*$/) { 
      # only whitespace
      next;
    }

    print STDERR "Garbage: $_\n";
  }

}


print "Reo Parser rule IDs:\n";
print "--------------------\n";

for (my $i=$first_id;$i<@parser_id_idx;$i++) {
  my $rule_id = $parser_id_idx[$i];
  print $parser_ids{$rule_id}, ": ", $rule_id, "\n";
}

print "\n\n";


foreach my $file (glob("*_parser.hpp")) {
  $|=1;
  print "* Checking $file: ";

  open(IN, "< $file") or die;
  my @lines=<IN>;
  close(IN);
  chomp(@lines);


  my %id_to_rule;
  my %rule_to_id;
  my $error = 0;

  # scan rule<  > for id -> rule mapping  
  foreach (@lines) {
    s{//.+$}{};
    
    if (/^\s*rule<ScannerT,\s*parser_tag<([A-Za-z_0-9]+)>\s+>\s+([a-zA-Z_0-9]+)\s*;\s*$/) {
      my $id = $1;
      my $rule = $2;

      my @rule_names = grep(/rule_names\[$id\]\s*=\s*".+"/, @lines);
      if (@rule_names == 0) {
        my $probable_rule_name = $id;
        $probable_rule_name =~ s/ID_[A-Z]+_//;
        print "\n   rule_names[$id] = \"$probable_rule_name\";";
        $error=1;
      } elsif (@rule_names >1) {
        print "\n   multiple 'rule_names[$id]'";
        $error=1;
      } else {
        if ($rule_names[0] =~ /rule_names\[$id\]\s*=\s*"(.+)"/) {
          $id_to_rule{$id} = $1;
          $rule_to_id{$1}=$id;
        } else {
          die;
        }
      }
    }
  }

  foreach my $rule (sort { $parser_ids{$rule_to_id{$a}} <=>  $parser_ids{$rule_to_id{$b}} } values %id_to_rule) {
    if (!grep(/BOOST_SPIRIT_DEBUG_RULE\($rule\)/, @lines)) {
        print "\n   BOOST_SPIRIT_DEBUG_RULE($rule);";
        $error=1;
    }
  }

  my $helper_file = $file;
  $helper_file=~s/_parser.hpp/_parser_helper.hpp/;
  if (!-f $helper_file) {
    print "\n   No $helper_file!";
    $error=1;
    goto helper_ende;
  }
  open(IN, "< $helper_file") or die;
  my @helper=<IN>;
  close(IN);
  chomp(@helper);
  foreach (@helper) {s{//}{}};

  foreach my $rule (sort { $parser_ids{$rule_to_id{$a}} <=>  $parser_ids{$rule_to_id{$b}} }  values %id_to_rule) {
    if (!grep(/struct t_$rule/, @helper)) {
      print "\n   Missing t_$rule helper";
      $error=1;
    }
  }

 helper_ende:


  if (!$error) {
    print " [OK] \n";
  } else {
    print "\n";
  }
  
}
