#! /usr/bin/perl -w

use strict;


# usage:
# list_trace_points.pl DIRS|FILES

my @files;

if (!@ARGV) {
  $ARGV[0]='.';
}

foreach (@ARGV) {
  if (-d $_) {
    my @f=`find $_ -name '*pp'`;
    push @files, @f;
  } elsif (-f $_) {
    push @files, $_;
  } else {
    print "Illegal argument: ", $_, "\n";
    exit;
  }
}

my %trace_points;

foreach (@files) {
  chomp;

  open(IN, $_) or die;
  while (<IN>) {
    next if (/#define/); # skip macro definitions

    if (/TRACE(.*?)\((.+?),(.+?),/) {
      my $trace_type=$1;
      my $scope=$2;
      my $name=$3;

      # remove whitespace
      $scope=~s/ //g;
      $name=~s/ //g;
      
      # remove quotes
      $scope=~s/"//g;
      $name=~s/"//g;
      
      $trace_type=~s/^_//;
      $trace_type="TRACE" if !$trace_type;

      push @{$trace_points{$scope."::".$name} }, $trace_type;
    }
  }
}

foreach (sort keys %trace_points) {
  print $_;
  print "\t\t(";

  my %types;
  foreach (@{ $trace_points{$_} }) {
    $types{$_}=1;
  }

  my $first=1;
  foreach (sort keys %types) {
    print "," if !$first;
    $first=0;
    
    print $_;
  }
  print ")\n";
}
