#! /bin/bash

if test "$1" = "--no-ltl" ; then
  FORMULA_TYPE='--formulas=ASL';
  echo $FORMULA_TYPE;
  shift;
else
  FORMULA_TYPE='';
fi

if test "$1" = "--stats" ; then
  STATS='--stats=stats.csv';
  rm -f stats.csv;
  shift;
else
  STATS='';
fi
  

EXTRA_ARGS=""
while test ! -z "$1" ; do
  EXTRA_ARGS="${EXTRAARGS} --extra-arg=${1}";
  shift;
done

VEREOFY_EXECUTABLE='';
for I in ./vereofy-run ./vereofy-run.exe */vereofy-run ; do
  if test -x "$I" ; then 
    VEREOFY_EXECUTABLE=$I;
    break;
  fi;
done

if test ! -x "$VEREOFY_EXECUTABLE" ; then
  echo "Can't find vereofy-run!";
  exit 1;
fi

echo "Using executable :" "$VEREOFY_EXECUTABLE"

## run property tests
mkdir -p test_results

FAILED_GLOBALLY=0;

for FILE in `find ../test_input/bsn/ -name '*.properties'` ; do
    echo "Verifying " $FILE;
    DIR=`dirname $FILE`;
    BASEFILE=`basename $FILE`;
    perl ../tools/verify-properties.pl $FORMULA_TYPE $STATS $EXTRA_ARGS --extra-arg=--check-level=10 --negate --xml=test_results/$BASEFILE --tap $VEREOFY_EXECUTABLE $DIR $FILE
    FAILED=$?;
    if [ $FAILED -ne 0 ]; then
	echo "# FAILED: " $FAILED;
    fi
    FAILED_GLOBALLY=$[$FAILED_GLOBALLY + $FAILED];
done

if [ $FAILED_GLOBALLY -eq 0 ]; then
  echo "# ALL TESTS OK";
else
  echo "#" $FAILED_GLOBALLY " TESTS FAILED!";
fi
