#! /bin/bash

FORMULA_TYPE='';
STATS='';
EXTRA_ARGS='';
LOG_FAILURES='';
PARALLEL=1;

while test ! -z "$1"; do 
  if [[ "$1" == --formulas* ]] ; then
    FORMULA_TYPE="${FORMULA_TYPE} $1";
    shift;
  elif [[ "$1" == --procs=* ]] ; then
    PARALLEL=${1#*=};
    shift;
  elif test "$1" = "--stats" ; then
    STATS='--stats=stats.csv';
    rm -f stats.csv;
    shift;
  elif [[ "$1" == --log-failures ]] ; then
    LOG_FAILURES="--log-failures";
    rm -f failures.log;
    shift;
  else
    EXTRA_ARGS="${EXTRAARGS} --extra-arg=${1}";
    shift;
  fi;
done

VEREOFY_EXECUTABLE='';
for I in ./vereofy-run ./vereofy-run.exe */vereofy-run ; do
  if test -x "$I" ; then 
    VEREOFY_EXECUTABLE=$I;
    break;
  fi;
done

if test -n "$FORMULA_TYPE" ; then
  echo "Selected formulas: $FORMULA_TYPE";
fi

if test ! -x "$VEREOFY_EXECUTABLE" ; then
  echo "Can't find vereofy-run!";
  exit 1;
fi

echo "Using executable :" "$VEREOFY_EXECUTABLE"

## run property tests
mkdir -p test_results


if [ $PARALLEL -ge 2 ] ; then
    if [ -z `type -P parallel` ]; then
        echo "Could not find 'GNU parallel' in PATH, executing single thread instead."
        PARALLEL=1
    else 
        echo "Executing in parallel, with $PARALLEL threads."
    fi
fi


# the arguments for the verify-properties.pl script
VERIFY_PROPERTIES_STD_ARGS="$FORMULA_TYPE $STATS $LOG_FAILURES $EXTRA_ARGS --extra-arg=--check-result=1 --extra-arg=--check-level=10 --extra-arg=--btsl-witness-level=10 --extra-arg=--compute-asl-strategy=1 --negate --tap"

# gather the property files
PROPERTY_FILES=`find ../test_input ../examples -name '*.properties'`

# perl scriptlet to add the number of failed tests
EXTRACT_GLOBAL_FAILURES_PERL='BEGIN {$failed=0;} $failed+=$1 if /^\# FAILED: *(\d+)/; END {if ($failed>0) {print "# $failed  TESTS FAILED!\n";} else {print "# ALL TESTS OK\n"}}'

if [ $PARALLEL -gt 1 ] ; then
 # run in parallel by piping the commands to the GNU parallel program
  ( for FILE in $PROPERTY_FILES ; do
     CMD="echo 'Verifying ' $FILE;";
     DIR=`dirname $FILE`;
     BASEFILE=`basename $FILE`;
     CMD="$CMD perl ../tools/verify-properties.pl $VERIFY_PROPERTIES_STD_ARGS --xml=test_results/$BASEFILE  $VEREOFY_EXECUTABLE $DIR $FILE";
     echo $CMD
    done ) | \
   parallel -k -j$PARALLEL | \
   perl -p -e "$EXTRACT_GLOBAL_FAILURES_PERL"

else
 # run single threaded
  ( for FILE in $PROPERTY_FILES ; do
     echo "Verifying " $FILE;
     DIR=`dirname $FILE`;
     BASEFILE=`basename $FILE`;
     perl ../tools/verify-properties.pl $VERIFY_PROPERTIES_STD_ARGS --xml=test_results/$BASEFILE  $VEREOFY_EXECUTABLE $DIR $FILE
    done ) | \
   perl -p -e "$EXTRACT_GLOBAL_FAILURES_PERL"

fi
